;; -*- Mode:Lisp; Syntax:Common-Lisp; Package: FUG5 -*-
;;; -----------------------------------------------------------------------
;;; File:         morphology.l
;;; Description:  
;;; Author:       Yael Dahan Netzer
;;; Created:       7 Apr 1997
;;; Modified:     
;;; Package:      FUG5
;;; -----------------------------------------------------------------------
(in-package "FUG5")

;;; linearizer returns a list of FDs with the relevant features for
;;; morphology only. 
;;; 
;;; inflect prep + pronoun

;;;===============================================================	   
(defun Get-sentence (pattern)
  (print-sentence
	   (punctuate 
	    (get-all-lex
	     (scan-pattern
	      pattern)) '())))

 
(defun get-all-lex (fd-list)
   (remove-if #'null (mapcar #'(lambda (fd) (if (eq fd ':fail) "" (incr-gdp 'lex fd {}))) fd-list)))
			    
(defun SCAN-PATTERN (list &optional (acc '()))
   (if (not (null list))
       
   (let* (
	  (first-word (first list)) 
	  (second-word (second list))
	  (lex-first (get-lex first-word))
	  (lex-second (get-lex second-word))
	  (cat-first (get-cat first-word))
	  (cat-second (get-cat second-word)))
;;; ha + noun/adj/... -> ha-noun
;;; prep + ha  -> definite-prep
;;; prep + pronoun -> inflect-pronoun-to-prep
;;; noun-nis + pronoun -> inflect-pronoun-to-prep
;;; v + word -> inflect-vv
  (cond (;(eq cat-first 'conj)
	 ;(scan-pattern (update-list second-word 
		;		    (form-conj-inflection lex-first lex-second)
		;		    cat-second list) acc))
	 
	  (eq cat-first 'article)
	  
	   (cond ((prefixed-article? lex-first) 
	       ;; form a new lex, unify the words and call scan-pattern
	       ;; again.
;; The problems:
;; The category now should be the same, and also the lex, so this will have
;; to be updated.
		  (if (null second-word) acc
                 (scan-pattern (update-list second-word 
					    (form-prefixed-lex lex-second lex-first)
                                            cat-second list) acc)))
		(t (scan-pattern (rest list) (if (empty-word lex-first) acc
					       (add-to-acc first-word acc))))))
	((eq cat-second 'personal-pronoun)
	 (cond ((objective-case? second-word)
                (scan-pattern (update-list first-word
                                           (form-pronoun-inflection 
                                            lex-first 
                                            (get-number first-word)
                                            cat-first
                                            lex-second)
                                           cat-second list) acc))
		
	       (t (scan-pattern (rest list) (if (empty-word lex-first) acc
					      (add-to-acc first-word acc))))))
	((or (eq cat-first 'prep) (eq cat-first 'conj))
	 (cond ((and (eq cat-second 'article)
		     (prefixed-article? lex-second))
		
		(if (prefixed-to-word lex-first)
		    (scan-pattern (update-list first-word 
					       (form-definite-prep lex-first lex-second)
                                               cat-first list) acc)
		  (scan-pattern (rest list) (add-to-acc first-word acc))))
	       ((prefixed-to-word lex-first)
		(scan-pattern
		 (update-list second-word
			      (form-prep-noun lex-first
					       lex-second)
					      cat-second list) acc))
	       (t (scan-pattern (rest list) (add-to-acc first-word acc)))))
	(t (scan-pattern (rest list) 
			 (if (empty-word lex-first) acc
			     (add-to-acc first-word acc))))))
   acc))
  
	

(defun empty-word (word) (or (equal word "nil") (equal word 'none)))
(defun rest-pattern (list) (cddr list))

(defun get-number (fd) (incr-gdp 'number fd {}))
(defun get-lex (fd) (incr-gdp 'lex fd {}))
(defun get-cat (fd) (incr-gdp 'cat fd {}))

(defun prefixed-article? (word) (member word *prefixed-articles* :test #'equal))

(defvar *prefixed-articles* nil)

(setq *prefixed-articles*
  '("ha"))


(defun form-prefixed-lex (word article)
  (concatenate 'string article word))

(defun update-list (word lex cat list)
  (cons (update-word word cat lex)
	(rest-pattern list)))

(defun update-word (fd cat lex)
  (setf (second (top-gdpp fd (make-path :l '(cat)))) cat)
  (setf (second (top-gdpp fd (make-path :l '(lex)))) lex)
  fd)

(defun add-to-acc (elt acc)
  (append acc (list elt)))

(defun objective-case? (fd)
  (eq 'objective (incr-gdp 'case fd {}))) 

(defun form-prep-noun (prep noun)
  (concatenate 'string prep noun))

(defun prefixed-to-word (prep)
  (member prep *preps-prefixed-to-words* :test #'equal))

(defun form-conj-inflection (lex1 lex2)
  (if (equal lex1 "V")
    (concatenate 'string lex1 lex2)
    lex1))

(defun form-definite-prep (prep article)
  (cond ((or (equal prep "m") (equal prep "V"))

	
	 (concatenate 'string prep article))

	((member prep '("b" "l" "c" "Vb" "Vc" "Vl") :test #'equal)
	 (concatenate 'string prep "a"))
	(t prep)))

;; Glinert chap. 42 p.497
(defun form-pronoun-inflection (lex1 number lex1-cat plex)
  (case lex1-cat
	(prep
	 (cond ((member lex1 *preps-with-objective* :test #'equal) 
		(concatenate 'string lex1  plex))
	       ((member lex1 '("b" "l") :test #'equal)
		(cond ((equal plex "xem") (setf plex "axem"))
		      ((equal plex "xen") (setf plex "axen"))
		      ((equal plex "am") (setf plex "ahem"))
		      ((equal plex "an") (setf plex "ahen"))
		      (t lex1))
		(concatenate 'string lex1 plex))
	       ((equal lex1 "wim")
		(form-pronoun-inflection "It" number lex1-cat plex))
	       ((equal lex1 "Et")
		(form-pronoun-inflection "Oot" number lex1-cat plex))
	       ;;((equal lex1 "cmo") : preposition?
	       ((equal lex1 "m")
		(cond ((equal plex "y") (setf plex "meny"))
		      ((equal plex "x") (setf plex "mx"))
		      ((equal plex "ex") (setf plex "mex"))
		      ((equal plex "enu") (setf plex "menu"))
		      ((equal plex "am") (setf plex "hem"))
		      ((equal plex "an") (setf plex "hen"))
		      ((equal plex "o") (setf plex "meno"))
		      ((equal plex "a") (setf plex "menah"))
		      ((equal plex "xem") (setf plex "cem"))
		      ((equal plex "xen") (setf plex "cen"))
		      (t plex))
		(concatenate 'string lex1 plex))
	       ((member lex1 '("AHar" "Oodot" "lifney" "mipney" "biydey" "ER"
			       "clapey" "legabey" "meAHorey" "mewal" "wal"
			       "wal-gabey" "bly" "El") :test #'equal)
		(cond ((equal plex "y") (setf plex "ay"))
		      ((equal plex "x") (setf plex "ex"))
		      ((equal plex "ex") (setf plex "ayix"))
		      ((equal plex "o") (setf plex "ayV"))
		      ((equal plex "a") (setf plex "eyha"))
		      ((equal plex "xem") (setf plex "eyxem"))
		      ((equal plex "xen") (setf plex "eyxen"))
		      ((equal plex "am") (setf plex "ehem"))
		      ((equal plex "an") (setf plex "ehen"))
		      (t plex))
		(when (equal lex1 "bly") (setf lex1 "biwad"))
		(concatenate 'string lex1 plex))
	       (t (concatenate 'string lex1 plex))))
	(t (case number
		 (plural (cond ((equal plex "y") (setf plex "ay")
				(setf lex1 (fix-plural-end lex1)))
			       ((equal plex "x") (setf plex "ex"))
			       ((equal plex "ex") (setf plex "ayx")
				(setf lex1 (fix-plural-end lex1)))
			       ((equal plex "o") (setf plex "ayV")
				(setf lex1 (fix-plural-end lex1)))
			       ((equal plex "ah") (setf plex "eyah"))
			       ((equal plex "xem") (setf plex "eyxem")
				(setf lex1 (fix-plural-end lex1)))
			       ((equal plex "xen") (setf plex "yxen")
				(setf lex1 (fix-plural-end lex1)))
			       ((equal plex "am") (setf plex "hem"))
			       ((equal plex "an") (setf plex "hen"))
			       (t plex))
			 (concatenate 'string lex1 plex)) 	
		 (t (concatenate 'string lex1 plex))))))

(defun fix-plural-end (word)
  (let ((n (length word)))
    (if (eq (char word (1- n)) '#\y)
	    (subseq word 0 (1- n))
      word)))
	
	

;;; morph-noun updates the lex
;;; and creates the FD needed
;;; for the scan-pattern
;;; procedure.
(defun make-pronoun-fd (lex cat number person gender case)
  `((cat ,cat)
    (lex ,lex)
    (number ,number)
    (person ,person)
    (gender ,gender)
    (case ,case)))

(defun make-noun-fd (cat lex gender number smixut)
   `((cat ,cat)
     (lex ,lex)
     (number ,number)
     (gender ,gender)
     (nismax ,smixut)))
;; YD changed 5/9
(defun make-adj-fd (lex number gender smixut adj-type)
     `((cat adj)
       (lex ,lex)
       (number ,number)
      (adj-type ,adj-type)
       (gender ,gender)
       (nismax ,smixut)))

;; 9.4.97 This morph adjust new morphology to Hebrew.
(defun MORPH (cat lex fd path &key (cat-attribute *cat-attribute*))
  "Dispatch according to cat to specialized morphology routine.
       Handles verbs, nouns, pronouns and dets."
  (declare (special *unknown-cat*))
  (unless (stringp lex)
    (setq lex (string-downcase (format nil "~s" lex))))
  
  ;; Normalize subsumed categories to their normal representative
  (let ((normal-cat (member-if #'(lambda (c) (subsume c cat))
			       '(adv conj modal prep relpro punctuation
				     phrase noun verb pronoun article ordinal
				     cardinal fraction quantifier))))
    (if normal-cat
	(setf cat (car normal-cat))))
			    
  (list-if-atom 
   (case cat
     ((adv conj modal prep relpro punctuation phrase) 
      ;;; morph a simple fd
      (morph-simple-fd cat lex))
     ;;; nouns need the features: gender, number, smixut, definite
     ;;; Now - definite is handled now as a DET separately.
     ;;; so perhaps it should be given also not as a feature of the lex itself.
     
     (noun
      (morph-noun
	    lex
	    (check-number (incr-gdp 'number fd path))
	    (incr-gdp 'feature fd path)
	    (check-smixut (incr-gdp 'nismax fd path))
	    (check-definite (incr-gdp 'mark-definite fd path))
	    (check-gender (incr-gdp 'gender fd path))
	    (check-animate (incr-gdp 'animate fd path))))
	   
     (adj (morph-adj
	   lex
	   (check-number (incr-gdp 'number fd path))
	   (check-smixut (incr-gdp 'nismax fd path))
	   (check-definite (incr-gdp 'mark-definite fd path))
	   (check-gender (incr-gdp 'gender fd path))
	   (check-adj-type (incr-gdp 'adj-type fd path))))
     
     (verb (morph-verb
	    lex
	    (incr-gdp 'ending fd path)
	    (check-number (incr-gdp 'number fd path))
	    (check-person (incr-gdp 'person fd path))
	    (check-tense (incr-gdp 'tense fd path))
            (check-gender (incr-gdp 'gender fd path))))
     (pronoun
      (morph-pronoun
       lex
       (incr-gdp 'pronoun-type fd path)
       (check-case (incr-gdp 'case fd path))
       (check-gender (incr-gdp 'gender fd path))
       (check-number (incr-gdp 'number fd path))
       (check-distance (incr-gdp 'distance fd path))
       (check-animate (incr-gdp 'animate fd path))
       (check-person (incr-gdp 'person fd path))
       (check-restrictive (incr-gdp 'restrictive fd path))))
     ((article det)
      (morph-determiner
       cat
       lex
       (incr-gdp 'number fd path)))
     ((ordinal cardinal)
      (morph-numeric lex cat 
		     (incr-gdp 'value fd path)
		     (incr-gdp 'digit fd path)
		     (check-gender (incr-gdp 'gender fd path))
		     (check-definite (incr-gdp 'mark-definite fd path))
		     (check-number (incr-gdp 'number fd path))
		     (check-smixut (incr-gdp 'nismax fd path))))
     (fraction
      (morph-fraction lex 
		      (incr-gdp 'num fd path) 
		      (incr-gdp 'den fd path)
		      (incr-gdp 'digit fd path)))
		      
     (t (cond ((member cat (categories-not-unified cat-attribute)) lex)
	      (t
	       (push cat *unknown-cat*)
	       (format nil "<unknown cat ~s: ~a>" cat lex)))))))




(defun MORPH-NOUN (word number feature smixut definite gender animate) 
  ;; FOR NOW: just fetch from
  ;; lexicon and do not do any
  ;; morphology.
  (setq word (morph-gender word number gender animate))
  (setq word (morph-number word number gender animate))
  (setq word (morph-smixut word number gender smixut))
  (setq word (or (lexfetch word (if (eq smixut 'yes)
		     (case number 
			   (plural 'plural-nis)
			   ( t 'sing-nis))
		   number))
		 word))
  
  (make-noun-fd 'noun word gender number smixut)) 
(defun MORPH-adj (word number smixut definite gender adj-type) 
  ;; FOR NOW: just fetch from
  ;; lexicon and do not do any
  ;; morphology.
  
  (setq word (morph-adj-gender word number gender adj-type))
  (setq word (morph-number word number gender 'yes))
  (setq word (morph-smixut word number gender smixut))
  (setf word (or (lexfetch word (if (eq smixut 'yes)
		     (case number 
			   (plural (case gender
					 (masculine 'plural-masc-nis)
					 (t 'plural-fem-nis)))
			   (t (case gender
				    (masculine 'sing-masc-nis)
				    (t 'sing-fem-nis))))
		   (case number
			 (singular (case gender 
					 (feminine 'sing-fem)
					 (t 'sing-masc)))
			 (t (case gender 
				  (feminine 'plural-fem)
				  (t 'plural-masc))))))
		 word))
  (make-adj-fd word gender number smixut adj-type))


(defun morph-determiner (cat lex number)
  `((cat ,cat)
    (lex ,lex)
    (number ,number)))


(defun MORPH-simple-fd (cat lex)
  `((cat ,cat) (lex ,lex)))

(defun call-linearizer (list &key (cat-attribute *cat-attribute*))
  (let ((*input* list)
	(*unknown-cat* nil))
    (declare (special *input* *unknown-cat*))
    (if (leaf-p list)
	(list "<fail>")
      (values
       	(punctuate (get-all-lex
		    (scan-pattern
		     (linearize list (make-path) :cat-attribute cat-attribute)))
		   (subsume 'interrogative (second (safe-assoc 'mood list))))
	       *unknown-cat*))))

(defun linearize (fd path &key (cat-attribute *cat-attribute*))
  "Linearizes fd (found at level path from the root of the total
       fd) and returns a list of strings.
       Handles gaps and punctuation.
       If a feature (gap xxx) where xxx is not none is found in a
       constituent, the constituent does not appear in the result.
       The features punctuation have a special effect described
       in documentation of function list-punctuation."
  (cond 
    ((leaf-p fd) nil)
    (t
     (let* ((pattern     (incr-gdp 'pattern fd path))
	    (gap         (incr-gdp 'gap fd path))
	    (punctuation (incr-gdp 'punctuation fd path))
	    (punct-path  (path-extend path 'punctuation))
            (lin-const
	      (cond 
	        ((and gap (not (eq gap 'none)))
	        ;; constituent is gapped - don't put it in linearized 
	          nil)
	        (pattern  ;; recurse on all constituents
	          (let* ((order (clean-pattern pattern))
		    ;;YD 25/8
			 (punctuated 
			  (list-punctuation punctuation punct-path 
			   (mapcan 
			    #'(lambda (constituent)
				(let ((new-path (safe-path-extend path constituent)) 
				      (new-fd 
				       (if (attr-p constituent)
					   (incr-gdp constituent fd path)
					 (gdp *input* 
					      (absolute-path 
					       constituent 
					       (path-extend path 'pattern))))))
				  (linearize new-fd new-path)))
			   order))))
			 (if (changed-punct punctuated) (car punctuated)
			   punctuated)))
	        (t  ;; simple constituent: call morphology
	           (let* ((cat (incr-gdp cat-attribute fd path))
		         (lex (incr-gdp 'lex fd path))
		         (punctuated 
			   (list-punctuation punctuation punct-path
					(morph cat lex fd path
					       :cat-attribute
					       cat-attribute)
					'yes)))
		     (if (changed-punct punctuated) (car punctuated)
		       (list punctuated))))))
	    (html        (incr-gdp 'html fd path))
            (hpath       (path-extend path 'html)))
	(if (and html
	         (not (eq html 'none)))
            (cons (html-tag html hpath :start)
                  (append  lin-const
                           (list (html-tag html hpath :end))))
          lin-const)))))


;; List-punctuation
;; ------------------------------------------------------------
;; adjust this procedure to the Hebrew morphology - should return the
;; punctuation in the fd.



(defun list-punctuation (punctuation path lstring &optional flat)
  "Punctuation is an fd with 5 relevant features: before, after,
       capitalize, append-before and append-after.
       The value of before, after, append-before and append-after must be a
       string of punctuation. Capitalize can be yes or no.  Default is no.
       The string in append-before is appended to the first word in lstring.
       The string in append-after is appended to the last word in lstring.
       The strings in before and after are added as new punctuation words 
       before and after lstring.
       Path is the path from the root of the total fd to punctuation.
       Lstring is a list of string.
       Returns a list of strings with punctuation before and
       after lstring as indicated by punctuation."
  ;(setq lstring (trim-lstring lstring))
  (cond ((null punctuation) lstring) 
	(t 
	 (let ((before (incr-gdp 'before punctuation path))
	       (after  (incr-gdp 'after  punctuation path))
	       add-punct)
	       ;; I do not use these features, do not take care now. YD
	       ;(append-before (incr-gdp 'append-before punctuation path))
	       ;(append-after  (incr-gdp 'append-after  punctuation path)))
	       ; no capitalize in Hebrew YD
	       ;(capitalize (incr-gdp 'capitalize punctuation path)))
	;(unless (or (null lstring) ;; comments - YD
        ;            (null capitalize) 
        ;            (eq 'none capitalize)
	;	       (eq 'no capitalize))
	;     (setf (car lstring) (string-capitalize (car lstring) :end 1)))
           (when (null lstring)
             (setf lstring '(()))) ;; empty fd
        ;   (when (stringp append-before)
        ;     (setf (car lstring)
        ;           (concatenate 'string append-before (car lstring))))
        ;   (when (stringp append-after)
        ;     (setf (car (last lstring))
        ;           (concatenate 'string (car (last lstring)) append-after)))
	   (when (stringp before)
		 (setq lstring 
		       (if flat (cons (morph-simple-fd 'punctuation before)
				      (list lstring))
			 (cons (list (morph-simple-fd 'punctuation before))
				 lstring)))
		 (setq add-punct 'yes))
	   (when (stringp after)
		 (setq lstring 
		       (if flat (append lstring (list (morph-simple-fd 'punctuation
								 after)))
			 (append lstring 
				 (list (morph-simple-fd 'punctuation
							after)))))
		 (setq add-punct 'yes))
	   (if add-punct (cons lstring '*stam*) lstring)))))

(defun changed-punct (aa) (equal (cdr aa) '*stam*))

;;==============


(defun MORPH-PRONOUN (lex pronoun-type case gender number
			  distance animate person restrictive)
  "Returns the correct pronoun given the features."
  (cond ((and (stringp lex)
	      (not (string= lex "nil"))
	      (not (string= lex "none"))) lex)
	((or (eq pronoun-type 'personal)
	     (and (eq pronoun-type 'demonstrative)
		  (eq distance 'far)))
	 ;; start with "he" then augment by person, then by
	 ;; number, gender and finally by case.
	 ;(when (eq animate 'no) (setq gender 'neuter))
	 (setq lex "hu'")
	 (unless (eq person 'third)
	   (setq lex (cond ((lexfetch lex person)) (t lex))))
	 (unless (eq gender 'masculine)
	   (setq lex (cond ((lexfetch lex gender)) (t lex))))
	 (unless (eq number 'singular)
	   (setq lex (cond ((lexfetch lex number)) (t lex))))
	 (unless (eq case 'subjective)
	   (setq lex (cond ((lexfetch lex case)) (t lex))))
	 (make-pronoun-fd lex 'personal-pronoun number person gender case)) 
	 
	((eq pronoun-type 'demonstrative)
	 ;; distance near 
	 (setq lex "zeh")
	 (unless (eq gender 'masculine)
	   (setq lex (cond ((lexfetch lex gender)) (t lex))))
	 (unless (eq number 'singular)
	   (setq lex (cond ((lexfetch lex number)) (t lex))))
	 (unless (eq case 'subjective)
	   (setq lex (cond ((lexfetch lex case)) (t lex))))
	 (make-pronoun-fd lex 'demonstrative-pronoun number person gender case)) 
	 
	       
	      
	((eq pronoun-type 'relative)
	 (when (eq animate 'no) (setq gender 'neuter))
	 (cond ((eq restrictive 'yes) "that")
	       ((eq case 'possessive) "whose")
	       ((eq case 'objective) 
		(if (eq gender 'neuter) "which" "whom"))  
	       ((eq gender 'neuter) "which")
	       ((eq animate 'no) "which")
	       (t "who")))
	((eq pronoun-type 'question)
	 (when (eq animate 'no) (setq gender 'neuter))
	 (cond ((eq restrictive 'yes) "which")
	       ((eq case 'possessive) "whose")
	       ((eq case 'objective) 
		(if (eq gender 'neuter) "what" "whom"))
	       ((eq gender 'neuter) "what")
	       ((eq animate 'no) "what")
	       (t "who")))
	((eq pronoun-type 'quantified)
	 lex)))


(defun MORPH-NUMERIC (lex ord-or-card value digit gender definite number nismax)
  "Return a string given a number - either ordinal or cardinal"
  ;; No Luck common-lisp does not do it all for us!!!
  (setf lex 
	(cond ((and (stringp lex)
	      (not (numberp value))     ;; if value is given use it
	      (not (string= lex "nil"))
	      (not (string= lex "none"))) lex)
	((integerp value)
	 (case digit
	   (no (case ord-or-card
		 (ordinal (form-ordinal value gender definite number))
		 (cardinal (form-cardinal value gender nismax))
		 (t (format nil "~r" value))))
	   (roman (format nil "~@R" value))
	   (t  (case ord-or-card
		 (ordinal (format-ordinal value))
		 (t (format nil "~s" value))))))
	((numberp value)
	 (format nil "~s" value))
	((stringp value) value)
	(t (format nil "<unknown-number: ~s>" value))))
  (make-numeric-fd ord-or-card lex number gender nismax digit))

(defun MORPH-FRACTION (lex num den digit)
  "Return a string for a fraction"
  (let ((nismax (if (= num 2) 'yes 'no)))
    (setf lex       
	(cond ((and (integerp num) (integerp den))
	       (cond ((= den 1) 
		      (case digit 
			    (no (form-cardinal num 'feminine 'no)) 
			    (roman (format nil "~@R" num))
			    (t (format nil "~s" num))))
		     ((and (> den 4) (<= den 10))
		      (cond ((= num 1)
			     (case digit
			       (no (form-ordinal den 'feminine 'no 'singulr))
			       (t (format nil "~s/~s" num den))))
			    (t (case digit
				     (no (concatenate 'string 
					 (form-cardinal num 'feminine nismax)
					  "-"
					  (form-ordinal den 'feminine 'no
							'plural)))))))
		     ((= den 2)
		      (case digit
		       (no (if (= num 1) "HeZi"
			     (concatenate 'string (form-cardinal num 'masculine
								      nismax)
					  "-HaZ'iym")))
		       (t (format nil "~s/2" num))))
		     ((= den 3)
		      (case digit
		       (no (if (= num 1) "SlyS"
			 (concatenate 'string (form-cardinal num 'masculine
							     nismax)
				      "-SlySym")))
		       (t (format nil "~s/3" num))))
		     ((= den 4)
		      (case digit
		       (no (if (= num 1) "revaw"
			 (concatenate 'string (form-cardinal num 'masculine nismax)
				      "-revawim")))
		       (t (format nil "~s/4" num))))
		     ;; this should be in the lexicon: me'it alpit milionit..
		     ((= den 100)
		      (case digit
			(no (if (= num 1) "me'yt"
			 (concatenate 'string (form-cardinal num 'feminine nismax)
				      "-me'iot")))
			(t (format nil "~s/100" num))))
		     ((= den 1000)
			 (case digit
			   (no (if (= num 1) "Alpyt"
			     (concatenate 'string (form-cardinal num 'feminine
							    nismax)
					  "-Alpiyot")))
			   (t (format nil "~s/1000" num))))
		      ((= den 1000000)
		       (case digit
		         (no (if (= num 1) "miyliyonyt"
			       (concatenate 'string (form-cardinal num 'feminine
							    nismax)
				     "-miyliyoniyt")))
			 (t (format "~s/1000000" num))))
		      (t
		       (case digit
			     (no (concatenate 'string 
					      (form-cardinal num 'feminine nismax)
					      "-Helkey-"
					      (form-cardinal den 'feminine 'no)))
			     (t (format nil "~s/~s" num den))))))
	       ((and (stringp lex)
		     (not (string= lex "nil"))
		     (not (string= lex "none"))) lex)
	       (t (format nil "~s/~s" num den))))
  (make-fraction-fd lex)))

(defun make-fraction-fd (lex)
  `((cat fraction)
    (lex ,lex)))

(defun make-numeric-fd (cat lex number gender nismax digit)
  `((cat ,cat)
    (lex ,lex)
    (number ,number)
    (gender ,gender)
    (nismax ,nismax)
    (digit ,digit)))


(defun morph-smixut (word number gender smixut)
  "morphs a nismax suffix to word if smixut is yes"
  (cond ((null smixut) word)
	((null word) word)
	((subsume 'yes smixut)
	 (cond ((lexfetch word
                          (case number 
                            (plural 'plural-nis)
                            (t 'sing-nis))))
               (t (form-smixut word number gender))))
        (t word)))


(defun form-smixut (word number gender)
  (let* ((n (length word))
         (next-to-final (elt word (- n 2)))
         (final (elt word (1- n))))
    (case number
	  (singular (cond ((vowel final)
			   (concatenate 'string word "t"))
			  ((equal final #\h)
			   (concatenate 'string 
					(subseq word 0 (1- n)) "t"))
			  (t word)))
	  (t ;;plural
	   (cond ((and (equal final #\m) (equal next-to-final #\y))
		  (subseq word 0 (1- n)))
		  (t word))))))
;;; define types of adjectives both for morphology and order:
;; type are: (adj-type participle) ;;beynony (magwil)
;; (adj-type typical) one of list in glinert481, atias..
;; (adj-type noun-based)

;; YD new 5/9/97
(defun morph-adj-gender (word number gender adj-type)
  "adds the female suffix to word if gender is feminine"
    (cond ((null gender) word)
	((null word) word)
	((subsume 'feminine gender)
	 (cond ((lexfetch word 'feminine))
	        (t (form-adj-feminine word adj-type))))
	(t word)))



(defun check-adj-type (adj-type)
  "Returns adj-type, defualt is Typical"
  (cond ((member adj-type '(typical noun-based participle)) adj-type)
	(t 'typical)))

(defun check-definite (definite)
  "Returns definite, DEFAULT is NO"
  (cond ((member definite '(yes no)) definite)
	(t 'no)))

(defun check-smixut (nismax)
  "Returns smixut feature, DEFAULT is NO"
  (cond ((member nismax '(yes no)) nismax)
	(t 'no)))


(defun form-adj-feminine (word adj-type)
  (let* ((n (length word))
         (next-to-final (elt word (- n 2)))
         (final (elt word (1- n))))
    ;;;;;I think its quite impossible..
    ;; do just the simple 'a'
    (case adj-type
	  (noun-based
	   (concatenate 'string word "t"))
	  (participle ;; it is true except for : hifwil binyan
	   (if (equal final '#\H) (concatenate 'string word "at")
               (concatenate 'string word "et")))
	  (typical ;; except for CiCeC (giben-gibenet) 
	   (if (and (equal final '#\h) (equal next-to-final '#\e))
	       (concatenate 'string (subseq word 0 (- n 2)) "ah")
	     (if (equal final '#\e)
		 (concatenate 'string (subseq word 0 (1- n)) "ah")
	       (if (or (equal final '#\i) (equal final '#\y))
		   (concatenate 'string word "t")
		 (concatenate 'string word "ah"))))))))
	      

(defun morph-gender (word number gender animate)
  "adds the female suffix to word if gender is feminine
   and animate"
  (cond ((null gender) word)
	((null word) word)
	((and (subsume 'feminine gender)
	      (subsume 'yes animate))
	 (cond ((lexfetch word 'feminine))
	        (t (form-feminine word))))
	(t word)))

(defun form-feminine (word)
  (let* ((n (length word))
         (next-to-final (elt word (- n 2)))
         (final (elt word (1- n))))
    ;;;;;I think its quite impossible..
    ;; do just the simple 'a'
    (if (and (equal final '#\n) (equal next-to-final '#\a))
	(concatenate 'string word "yt")
      (if (and (equal final '#\H) (equal next-to-final '#\a))
	(concatenate 'string word "yt")
        (if (or (equal final '#\i) (equal final '#\y))
	  (concatenate 'string word "t")
	(if (and (equal final '#\h) (equal next-to-final '#\e))
	    (concatenate 'string (subseq word 0 (- n 2)) "ah")
	  (if (equal final '#\e)
	      (concatenate 'string (subseq word 0 (1- n)) "ah")
	    (concatenate 'string word "ah"))))))))





(defun morph-number (word number gender animate)
  "adds the plural suffix to word if number is plural
   default is singular."
     (cond ((null number) word)
	   ((null word) word)
	   ((subsume 'plural number)
	    (cond ((lexfetch word 'plural))
		  (t (pluralize word gender))))
	   (t word)))



(defun pluralize (word gender)
 "handles word string endings.."
 (let* ((n (length word))
	(next-to-final (elt word (- n 2)))
	(final (elt word (1- n))))
   ;; most general
   (cond ((subsume 'masculine gender)
	  (case final
		((#\h #\e) 
		 (cond ((vowel next-to-final)
			(concatenate 'string (subseq word 0 (- n 2))
					"ym"))
			(t (concatenate 'string (subseq word 0 (1- n))
					"ym"))))
                ((#\y #\i) (concatenate 'string word "m"))
		(t (concatenate 'string word "ym"))))
	 ((subsume 'feminine gender)
	  (case final
		((#\h #\a) 
		 (cond ((vowel next-to-final)
			(concatenate 'string (subseq word 0 (- n 2))
					"ot"))
			(t (concatenate 'string (subseq word 0 (1- n))
					"ot"))))
		(#\t (concatenate 'string (subseq word 0 (1- n)) 
		     (if (equal next-to-final #\y) "ot" "yot")))
		(t (concatenate 'string word "ot")))))))


	    


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Lexicon and morphology:
(defvar *pronouns-person* nil )

(setq *preps-prefixed-to-words* '("c" "b" "l" "m" "ca" "ba" "la" "mha" "V" "Vha"))
;(setq *preps-prefixed-to-words* '("c" "b" "l" "m"  "V"))
(setq *preps-without-pronoun-inflections*
 '("Agav" "bemeSex" "cegon" "lelo'" "lamrot" "wal-Af" "meAz" "meHamat" "miSum"
   "tox-cedey"))
(setq *preps-with-objective*
 '("biSbyl" "derex" "bekerev" "le'Orex" "neged"))



(setq *pronouns-person*
      ;;(key(third) first second)
      '(("hu'" "Any" "Atah")))

(setq *pronouns-gender*
      ;;(key(masculine) feminine neuter)
      '(("hu'" "hy'")
	("Atah" "At")
	("zeh" "zOt")))

(setq *pronouns-number*
      ;;(key(sing) plural)
      '(("Any" "AnaHnu")
	("hu'" "hem")
	("hy'" "hen")
	("Atah" "Atem")
	("At" "Aten")
	("zeh" "Eleh")
	("zOt" "Eleh")))

	



(setq *pronouns-case*
 ;;(key(subjective) objective possessive reflexive)
      '(("Any" "y" "y" "waZmi")
	("Atah" "x" "x" "waZmex")
	("At" "ex" "ax" "waZmex")
	("Atem" "xem" "axem" "waZmexem")
	("Aten" "xen" "axen" "waZmexen")
	("hu'" "o" "o" "waZmo")
	("hy'" "ah" "ah" "waZmah")
	("AnaHnu" "enu" "anu" "waZmenu")
	("hem" "am" "ahem" "waZmam")
	("hen" "an" "ahen" "waZman")))



;;;===================================
;; Take care of lexicon.

(defvar *noun-inflection-in-Hebrew*)
(defvar *adjectivs*)

(setq *noun-inflection-in-Hebrew*
 ;; (key(sing) plural sing-nis plural-nis
      '(("dunam" "dunam" "dunam" "dunamey")
        ("Av" "Avot" "Avy" "Avot")
        ("dov" "dubym" "dov" "dubey")
        ("IyS" "AnaSym" "IyS" "AnSey")
        ("IySah" "naSym" "ESet" "neSot")
        ("rav" "rabanym" "rav" "rabaney")
        ("rabanyt" "rabanyot" "rabat" "rabanyot")))

   
;; should make a list of irregular feminine inflection.
(setq *adjectivs*
      ;; (key(sig-mas sig-fem pl-mas pl-fem sig-mas-nis sig-fem-nis
      ;;                                      pl-mas-nis pl-fem-nis
      '(("rav" "rabah" "rabym" "rabot" "rav" "rabat" "rabey" "rabot")
	("AHer" "AHeret" "AHerym" "AHerot")
	("yoter" "yoter")
	("kZat" "kZat")
	("mewat" "mewat")
	("AHer" "AHeret" "AHerim" "AHerot")))




(defun store-Hebrew-nouns (list)
  (mapc #'(lambda (x)
	    (let ((key (car x)))
	      (lexstore key 'singular key)
	      (lexstore key 'plural (second x))
	      (lexstore key
			'sing-nis  (third x))
	      (lexstore key
			'plural-nis
                        (fourth x))))
	list))

(defun store-hebrew-adj (list)
  (mapc #'(lambda (x)
	    (let ((key (car x)))
	      (lexstore key 'sing-masc key)
	      (lexstore key 'sing-fem (second x))
	      (lexstore key 'plural-masc (third x))
	      (lexstore key 'plural-fem (fourth x))
	      (lexstore key 'sing-masc-nis (fifth x))
	      (lexstore key 'sing-fem-nis (sixth x))
	      (lexstore key 'plural-masc-nis (seventh x))
	      (lexstore key 'plural-fem-nis (eighth x))))
	list))

(defun store-pronouns ()
;; first and second
  (mapc #'(lambda (x)
	    (let ((key (car x)))
	      (lexstore key 'first (second x))
	      (lexstore key 'second (third x))))
		*pronouns-person*)
  (store-plurals *pronouns-number*)
  (mapc #'(lambda (x)
	    (let ((key (car x)))
	      (lexstore key 'feminine (second x))
	      (lexstore key 'neuter (third x))))
	*pronouns-gender*)
  (mapc #'(lambda (x) 
	    (let ((key (car x)))
	      (lexstore key 'objective (second x))
	      (lexstore key 'possessive (third x))
	      (lexstore key 'reflexive (fourth x))))
	*pronouns-case*))



(defun intitialize-lexicon (&optional size)
  (setq *dictionary* (make-hash-table :test #'equal
				      :size (cond (size) (t '200))))
  (store-Hebrew-nouns *noun-inflection-in-Hebrew*)
  (store-hebrew-adj *adjectivs*)
  (store-pronouns)
  (store-numbers-1-10 *numbers-1-10*)
  (store-numbers-11-19 *numbers-11-19*)
  (store-numbers-tens *numbers-tens*)
  (store-ordinals *ordinals*))


;;;;;Try to do some morphology...

;; yeled - yalda, ish-isa, baHur-baHura, namer-nmera

;;; morph number and gender
;;; 'alef: ',A,E   vav: v,o,u     caf: c xaf- x      wayin - w  Sin -S
;;; beyt: b  V     zayin: z       lamed: l           pe' -p     tav -t
;;; gymel: g       Het: H         mem: m             Zadi - Z
;;; dalet: d       Tet: T         nun: n             quf -q
;;; he' - h        yod: y         samex: s           reys - r
(defun PREPARE-to-TCL (word-list &optional acc)
  (cond ((null word-list) acc)
	((failed (first word-list)) 'fail)
	(t (let ((word (car word-list)))
	     (PREPARE-to-TCL (cdr word-list) 
			     (append acc (parse word
						(length word))))))))

(defun failed (sentence)
  (equal sentence "<fail>"))
       
(defun parse (word length &optional (acc "") (count 0))
  (if (= length count) (list acc)
    (parse word length 
	   (concatenate 'string acc 
			(print-if-not-null  
			 (get-sign (char word count) count word)))
	   (1+ count))))

(defun print-if-not-null (char)
  (when char (princ-to-string char)))


(defun last-in-word (num length word)
  (or (= (1- length) num)
      (equal (char word (1+ num)) #\Space))) ;; it should be more than
					     ;; #\space (also, - )

(defun first-in-word (num word)
  (or (= num 0)
      (equal (char word (- num 1)) #\Space)))

(defun get-sign (char count word)
  (let ((length (length word)))
    (case char
	  ((#\a #\e) (if (last-in-word count length word) #\h
		       (if (first-in-word count word) #\a nil))) 
	  (#\i (if (last-in-word count length word) #\y
		 (if (first-in-word count word) #\a #\y))) ;;?
	  ((#\' #\A #\E #\I #\O #\U) #\a)
	  ((#\b #\v) #\b)
	  ((#\g #\j) #\g)
	  ((#\o #\u #\V) #\v)
	  ((#\c #\x) (if (last-in-word count length word) #\q #\c))
	  (#\m (if (last-in-word count length word) #\o #\m))
	  (#\n (if (last-in-word count length word) #\i #\n))
	  ((#\p #\f) (if (last-in-word count length word) #\f #\p))
	  (#\Z (if (last-in-word count length word) #\= #\Z))
	  ((#\k #\q) #\k)
	  (#\Space '_)
	  (t char))))

;; uni with preparation to TCL hebrew input.
(defun uni-TCL (input &key grammar non-interactive (limit 10000)
		       (cat-attribute *cat-attribute*)
		       (cset-attribute *cset-attribute*))
  "Generate an English sentence represented by the FD input. 
       Unify input with the grammar then linearize the result.
       If output is just <fail>, that means the unification has
       failed.
       cf. also UNI-FD and UNIF."
  (let ((*from-top* t))   ;; know that we are calling from uni toplevel
    (declare (special *from-top*))
    (multiple-value-bind (sentence unknown-cats)
	(call-linearizer 
	 (unify-top input (or grammar *u-grammar*) 
		    :non-interactive non-interactive
		    :limit limit
		    :cat-attribute cat-attribute
		    :cset-attribute cset-attribute)
	 :cat-attribute cat-attribute)
	(if (and unknown-cats (not non-interactive))
	    (morphology-help unknown-cats))
	(PREPARE-to-TCL sentence)
      )))
      


(defun PRINT-SENTENCE-to-file (file sentence) 
  (with-open-file (ofile file
		   :direction :output 
			:if-does-not-exist :create)
          (setq *position* 0)
	  (mapc #'(lambda (s) (format ofile "~a" s)) sentence)
	  (terpri)   (setq *position* 0) 
	  (values)))



 
;;;;;;these are for ALLEGRO  - UNIX
;; uni with preparation to TCL hebrew input.
(defun uni-TCL (input &key grammar non-interactive (limit 10000)
		       (cat-attribute *cat-attribute*)
		       (cset-attribute *cset-attribute*))
  "Generate an English sentence represented by the FD input. 
       Unify input with the grammar then linearize the result.
       If output is just <fail>, that means the unification has
       failed.
       cf. also UNI-FD and UNIF."
  (let ((*from-top* t))   ;; know that we are calling from uni toplevel
    (declare (special *from-top*))
    (multiple-value-bind (sentence unknown-cats)
	(call-linearizer 
	 (unify-top input (or grammar *u-grammar*) 
		    :non-interactive non-interactive
		    :limit limit
		    :cat-attribute cat-attribute
		    :cset-attribute cset-attribute)
	 :cat-attribute cat-attribute)
	(if (and unknown-cats (not non-interactive))
	    (morphology-help unknown-cats))
	(PREPARE-to-TCL sentence)
      )))
      
;; for allegro
(defun system (call)
  (COMMON-LISP-USER::run-shell-command call)
  (values))
  

(defun PRINT-SENTENCE-to-file (file sentence) 
  (with-open-file (ofile file
		   :direction :output :if-exists :supersede 
			:if-does-not-exist :create)
          (setq *position* 0)
	  (mapc #'(lambda (s) (format ofile "~a" s)) sentence)
	  (terpri)   (setq *position* 0) 
	  (values)))

;; temporary ..
(defun CALL-GDTCL (ifile &optional (ofile "tempfile.gif"))
  ;; make a file for the input
  (system (concatenate 'string "show-sentence" " " ifile " " ofile))
  (values))

(defun uni-H (fd &optional (ifile "tempfile")  ofile)
  (let* ((sentence (uni-tcl fd))
	 (success (not (eq sentence 'fail))))
    (unless ofile (setf ofile (concatenate 'string ifile ".gif")))
    (cond (success (PRINT-SENTENCE-to-file ifile sentence)
		   (call-gdtcl ifile ofile))
	  (t (cformat "<fail>"))))
  
  (values))
  


 

;;; -*- Mode:Lisp; Syntax:Common-Lisp; Package: FUG5 -*-
;;; -----------------------------------------------------------------------
;;; File:         numbers.l
;;; Description:  generate hebrew number system.
;;; Author:       Yael Dahan Netzer
;;; Created:      17 Mar 1997
;;; Modified:     
;;; Package:      FUG5
;;; -----------------------------------------------------------------------

(in-package "FUG5")

#+ignore(defun MORPH-NUMERIC (lex ord-or-card value digit gender definite
number nismax)
  "Return a string given a number - either ordinal or cardinal"
  ;; No Luck common-lisp does not do it all for us!!!
  (cond ((and (stringp lex)
	      (not (numberp value))     ;; if value is given use it
	      (not (string= lex "nil"))
	      (not (string= lex "none"))) lex)
	((integerp value)
	 (case digit
	   (no (case ord-or-card
		 (ordinal (form-ordinal value gender definite number))
		 (cardinal (form-cardinal value gender nismax))
		 (t (format nil "~r" value))))
	   (roman (format nil "~@R" value))
	   (t  (case ord-or-card
		 (ordinal (format-ordinal value))
		 (t (format nil "~s" value))))))
	((numberp value)
	 (format nil "~s" value))
	((stringp value) value)
	(t (format nil "<unknown-number: ~s>" value))))

;; AUX
(defun MASCULINE? (gender)
  (eq gender 'masculine))
(defun NISMAX? (nismax)
  (eq nismax 'yes))
(defun DEFINITE? (definite)
  (eq definite 'yes))

(defun space-or-and (rest)
  (if (and (<= rest 20) (/= 0 rest)) " V" " "))

(defun form-gender-nismax (gender nismax)
  (cond ((masculine? gender) 
	 (if (nismax? nismax) 'masculine-nismax 'masculine))
	(t (if (nismax? nismax) 'feminine-nismax 'feminine))))
	
(defun form-gender-number (gender number)
  (case gender
	(masculine (case number
			 (plural 'masculine-plural)
			 (t gender)))
	(feminine (case number
			(plural 'feminine-plural)
			(t gender)))))

(defun FORM-CARDINAL (value &optional (gender 'feminine) (nismax 'no))
  (cond ((<= value 10) (lexfetch value (form-gender-nismax gender nismax)))
	((< value 20) (lexfetch value gender))
	((< value 100) (multiple-value-bind 
			(tens ones)
			(floor value 10)
			(string-append (lexfetch tens 'tens) " V"
				       (form-cardinal ones gender 'no))))
	((< value 1000)	(multiple-value-bind 
			 (hundreds rest)
			 (floor value 100)
			 (string-append 
			  (form-hundreds hundreds) (space-or-and rest)
 			  (form-cardinal rest gender 'no))))
	((< value 1000000) (multiple-value-bind 
			    (thousands rest)
			    (floor value 1000)
			    (string-append 
			     (form-thousands thousands)
			     (space-or-and rest)
			     (form-cardinal rest gender 'no))))
	 ((< value 100000000) (multiple-value-bind 
			       (millions rest)
			       (floor value 1000000)
			      (string-append
			       (form-millions millions) (space-or-and rest)
			       (form-cardinal rest gender 'no))))
	(t (multiple-value-bind (milliards rest)
				(floor value 100000000)
				(string-append
				 (form-millions milliards) (space-or-and rest)
				 (form-cardinal rest gender 'no))))))

(defun FORM-ORDINAL (value gender definite number)       
 (if (<= value 10)
     (let ((key (list value 'ord)))
     (lexfetch key (form-gender-number gender number)))
    
	   (form-cardinal value gender 'no)))

				     
(defun FORM-HUNDREDS (value)
  (cond ((= value 1) "meAh")
	((= value 2) "mAtayim")
	(t (string-append (lexfetch value 'feminine-nismax) "-meOot"))))

(defun FORM-THOUSANDS (value)
  (cond ((= value 1) "Elef")
	((= value 2) "Alpayim")
	((<= value 10)
	 (string-append (lexfetch value 'masculine-nismax) "-Alafym"))
	(t (string-append (form-cardinal value 'masculine 'no) " " "Elef")))) 

(defun FORM-MILLIONS (value)
  (if (= 1 value) "myliyon"
    (string-append (form-cardinal value 'masculine 'no) " " "myliyon"))) 

(setq *numbers-1-10*
      ;;(value masc masc-nis fem fem-nis)
     '((1 "EHad" "EHad" "AHat" "AHat")
       (2 "Snayim" "Sney" "Stayim" "Stey")
       (3 "SloSah" "SloSet" "SaloS" "SloS")
       (4 "arbawah" "arbawat" "arbaw" "arbaw")
       (5 "HamiSah" "HameSet" "HameS" "HameS")
       (6 "SiSah" "SeSet" "SeS" "SeS")
       (7 "Sivwah" "Sivawat" "Sevaw" "Sevaw")
       (8 "Smonah" "Smonat" "Smoneh" "Smoneh")
       (9 "tiSwah" "tiSawat" "teSaw" "teSaw")
       (10 "waSarah" "waSeret" "weSer" "weSer")))

(setq *numbers-11-19*
      '((11 "EHad-waSar" "AHat-weSreh")
	(12 "Sneym-waSar" "Stem-weSreh")
	(13 "SloSah-waSar" "SloS-weSreh")
	(14 "arbawah-waSar" "arba-weSreh")
	(15 "HamiSah-waSar" "HameS-weSreh")
	(16 "SiSah-waSar" "SeS-weSreh")
	(17 "Sivwah-waSar" "Svaw-weSreh")
	(18 "Smonah-waSar" "Smoneh-weSreh")
	(19 "tiSwah-aSar" "tSaw-wesreh")))

(setq *numbers-tens*
      ;;
      '((2 "weSrim")
	(3 "SloSim")
	(4 "arbawym")
	(5 "HamiSym")
        (6 "SiSym")
	(7 "Sivwym")
	(8 "Smonym")
	(9 "tiSwim"))) 


(setq *ordinals*
      '((1 "rISon" "rISonim" "rISonah" "rISonot")
	(2 "Seny" "Snyym" "Sniyah" "Sniyot") 
	(3 "SlySy" "SlySyim" "SlySyit" "SlySyot")
	(4 "reviwy" "revyiwyym" "revyiwyt" "reviwyot")
	(5 "HamySy" "HamySiyym" "HamyiSyt" "HamySyot")
	(6 "SySy" "SySiyim" "SySyt" "SiSyot")
	(7 "Svywy" "Svywym" "Svywyt" "Svywyot")
	(8 "Smyny" "Smyniyim" "Smynyt" "Smynyot")
	(9 "tSywy" "tSyiwyim" "tSyiwyt" "tSyiwyot")
	(10 "wasyiry" "wasyriyim" "wasyryt" "wasyiryot")))




(defun store-numbers-1-10 (list)
  (mapc #'(lambda (x)
	    (let ((key (car x)))
	      (lexstore key 'masculine (second x))
	      (lexstore key 'masculine-nismax (third x))
	      (lexstore key 'feminine (fourth x))
	      (lexstore key 'feminine-nismax (fifth x))))
	list))

(defun store-numbers-11-19 (list)
  (mapc #'(lambda (x)
	    (let ((key (car x)))
	      (lexstore key 'masculine (second x))
	      (lexstore key 'feminine (third x))))
	list))

(defun store-numbers-tens (list)
  (mapc #'(lambda (x)
	    (let ((key (car x))) (lexstore key 'tens (second x))))
	list))

(defun store-ordinals (list)
  (mapc #'(lambda (x)
	    (let ((key (list (car x) 'ord)))
	      (lexstore key 'masculine (second x))
	      (lexstore key 'masculine-plural (third x))
	      (lexstore key 'feminine (fourth x))
	      (lexstore key 'feminine-plural (fifth x))))
	list))


(intitialize-lexicon)








