;;; -*- Mode:Lisp; Syntax:Common-Lisp; Package: FUG5 -*-
;;; -----------------------------------------------------------------------
;;; File:         misc.l
;;; Description:  all sorts of little fractions of grammar
;;; Author:       Yael Dahan Netzer
;;; Created:      11 Aug 1997
;;; Modified:     
;;; Package:      FUG5
;;; -----------------------------------------------------------------------

(in-package "FUG5")


;; here there are some good examples why DEF should be a feature handled by
;; morphology and not by the grammar.

;;;unmarked definite titles: mar, profesor, doktor, gveret, military..
;; for gveret, samal you can say: hagveret me'ir. 

;; Haym cadury
;; harabanim cadury vehabAbA saly
;; cadury
;; hayarkan naHum mordexay
;; Haled da'ud el wazraq
;; yaron Zidkiyahu ("zidkyahu mehaHamuZim")
;; fuAd ben Eliwezer
;; rOS hamemSalah binyamin netanyahu
;; H"c  Efrayim sneh
;; boduAn melex belgyah
;; hacomer vehaHayal hasfarady IygneZyus loyolah

;; first-name last-name               Aytan dahAn
;; first-name                         Aytan
;; last-name                          dahAn
;; title first-name last-name         mar Ely dahAn
;; title last-name                    d"r galil
;; title first-name                   ?
;; first-name nickname last-name      bynyamyn "byby" netanyahu
;; first-name middle-name last-name   yosef Slomoh neZer
;; nickname last-name                 fuad ben-Eliwezer
;; nickname                           raful
;; occupation                         haSofeTet dorner


(def-conj person-name

  (cat #(under person-name))
  (pattern (occupation-pat title first-name middle-name nickname last-name))
  (opt ((title given)
	(title ((alt (((cat noun))
		      ((cat noun-compound))
		      ((cat list))))
		(gender {^2 gender})
		(animate yes)))))
  (opt ((occupation given)
	(occupation ((alt (((cat noun))
                           ((cat noun-compound))
                           ((cat list))))
                     (gender {^2 gender})
                     (animate yes)
                     (definite yes)
                     (mark-definite yes)))
        (alt (((occupation ((cat noun-compound)))
               (occupation-pat {^ occupation}))
              ((occupation-pat ((article occupation)))
               (article ((cat article)
                         (definite yes))))))))
                     	      
  

  (alt  (((nickname none)
	  (middle-name none)
	  (alt (((first-name given)
		 (first-name ((lex given)
			      (cat noun)))
		 (opt ((last-name given)
		       (last-name ((cat noun))))))
		((last-name given)
		 (opt ((last-name ((lex given)
				   (cat noun)))))))))
	 ((nickname given)
	  (nickname ((cat noun)
		     (lex given)))
	  (middle-name none)
	  (alt (((first-name given)
		 (last-name given)
		 (first-name ((cat noun)
			      (lex given)))
		 (last-name ((cat noun)
			     (lex given)))
		 (nickname ((punctuation ((before "(") (after ")"))))))
		((last-name given)
		 (last-name ((cat noun)
			     (lex given))))
		((last-name none)
		 (first-name none)))))
	 
	 ((middle-name ((lex given) (cat noun)))
	  (first-name ((cat noun)
		       (lex given)))
	  (opt ((last-name ((cat noun)
			     (lex given)))))))))



#+ignore(def-conj address
  (cat #(under address))
  
  ;; reHov agripas
  ;; tel-aviv
  ;; maHaneh haplitim Awidah Sebenapat beyt-leHem
  ;; cfar yonah
  ;; 
  )

#+ignore(def-conj date
  (cat #(under date))
  
  ;; Snat 1975
  ;; 1967
  ;; boker yom SiSy, ha-22 benoVember '68
  ;; 29.6.78
  ;; rESit Snot ha-90
  ;; 8 beyanu'ar haSana
  ;; '79
  ;; April '92
  ;; yuny 1967
  ;; yom HamiSy habA, ha-7 leAogust
)

;; examples:
(def-test name1
  "roS hamemSala mar binyamin (bibi) netanyahu."
  ((cat person-name)
   (first-name ((lex "binyamin")))
   (last-name ((lex "netanyahu")))
   (nickname ((lex "bibi")))
   (title ((lex "mar")))
   (occupation ((cat noun-compound) (head ((lex "roS"))) (classifier ((lex
								  "memSala")))))))


(def-test name2
  "danyEl mordexay."
  ((cat person-name)
   (first-name ((lex "danyEl")))
   (middle-name ((lex "mordexay")))))

