;;; -*- Mode:Lisp; Syntax:Common-Lisp; Package: FUG5 -*-
;;; -----------------------------------------------------------------------
;;; File:         verb-morph.l
;;; Description:  basic morph for verbs
;;; Author:       Yael Netzer
;;; Created:       3 Feb 1998
;;; Modified:     
;;; Package:      FUG5
;;; -----------------------------------------------------------------------

(in-package "FUG5")


(defun MORPH-VERB (word ending number person tense) 
  "Adds the proper suffix to the verb root taking into account
   ending, number, person, and tense." 
  (cond ((eq ending 'root) word)
        ((eq ending 'infinitive) (list "to" word)) 
        ((equal word "be") (morph-be number person (or ending tense))) 
        ((eq ending 'present-participle)
         (cond ((lexfetch word 'present-participle))
               (t (form-ing word))))
        ((eq ending 'past-participle) 
         (cond ((lexfetch word 'past-participle))
               (t (form-past word)))) 
        ((lexfetch word (if (and (eq tense 'present)
                            (eq person 'third)
                            (eq number 'singular))
                       'present-third-person-singular
                       tense)))
        ((eq tense 'present) (form-present-verb word number person))
        ((eq tense 'past) (form-past word)) 
        (t nil)))
;; Assume lex for verb is the letters of root.
;; handle only with lexfetch:
;; no real morphology, no real tense - just past -
;; third person - sing/plural masc/fem
(defun MORPH-VERB (word ending number person tense gender) 
  "Adds the proper suffix to the verb root taking into account
   ending, number, person, and tense." 
  (morph-simple-fd 'verb
		   (if (eq number 'plural) 
		       (lexfetch word 'plural)
		     (if (eq gender 'feminine)
			 (lexfetch word 'singular-feminine)
		       (lexfetch word 'singular-masculine)))))

(defvar *verbs-in-Hebrew-past-tense*)
(setq *verbs-in-Hebrew-past-tense*
      ;;key(root) - sing-masc sing-fem plural
      '(("nSv" "naSav" "naSvah" "naSvu")
	("hyh" "hayah" "haytah" "hayu")
	("yrd" "yarad" "yardah" "yardu")))

(defun store-Hebrew-verbs (list)
  (mapc #'(lambda (x)
	    (let ((key (car x)))
	      (lexstore key 'singular-masculine (second x))
	      (lexstore key 'singular-feminine (third x))
	      (lexstore key 'plural (fourth x))))
	list))
		
(store-Hebrew-verbs *verbs-in-Hebrew-past-tense*)



