;;; -*- Mode:Lisp; Syntax:Common-Lisp; -*-
;;; ------------------------------------------------------------
;;; File        : IR11.L
;;; Description : Sample inputs to test GR11.
;;; Author      : Michael Elhadad and Jacques Robin
;;; Created     : Aug 14 91
;;; Modified    : Sep 17 91 Added relative examples
;;; Language    : Common Lisp
;;; ------------------------------------------------------------

(defmacro define (var value) `(setq ,var ,value))
(load (concatenate 'string *cfuf-common-path* "testsuit/ir11.scm"))

(defun ir11_1 ()
  (format t "ir11_1  --> This car is expensive~%")
  (setq ir11_1 _ir11_1))

(ir11_1)

(defun ir11_2 ()
  (format t "ir11_2  --> John gives a blue book to Mary.~%")
  (setq ir11_2 _ir11_2))


(ir11_2)

(defun ir11_3 ()
  (format t "ir11_3  --> A science book is given by John to Mary.~%")
  (setq ir11_3 _ir11_3))

(ir11_3)

(defun ir11_4 ()
  (format t "ir11_4  --> John likes Mary.~%")
  (setq ir11_4 _ir11_4))

(ir11_4)

(defun ir11_5 ()
  (format t "ir11_5  --> John is the teacher.~%")
  (setq ir11_5 _ir11_5))

(ir11_5)

(defun ir11_6 ()
  (format t "ir11_6  --> The president is John.~%")
  (setq ir11_6 _ir11_6))

(ir11_6)


;; Really difficult: throw is classified as "locative" by Fawcett.
;; However, throw allows for dative move, which we prefer to leave
;; as a characteristic unique to possessive processes.
;; We could say that throw is a combination of material/locative AND
;; possessive, that would require us to change the grammar to support
;; composite processes of 3 processes.  That would be a Good thing to do in
;; any case, to account for sentences like [4 participants]:
;; I paid the hotel $2 for the room.
;; No! This is not a 4 participants sentence, sorry...
(defun ir11_7 ()
  (format t "ir11_7  --> Mary is thrown a heavy ball by John.~%")
  (setq ir11_7 _ir11_7))

(ir11_7)

(defun ir11_8 ()
  (format t "ir11_8  --> John's book is beautiful.~%")
  (setq ir11_8 _ir11_8))

(ir11_8)

(defun ir11_9 ()
  (format t "ir11_9  --> The SIG-display marker moves.~%")
  (setq ir11_9 _ir11_9))

(ir11_9)

(defun ir11_10 ()
  (format t "ir11_10 --> The SIG-display marker moves to the right.~%")
  (setq ir11_10 _ir11_10))

(ir11_10)

(defun ir11_11 ()
  (format t "ir11_11 --> The SIG-display marker moves from the left.~%")
  (setq ir11_11 _ir11_11))

(ir11_11)

(defun ir11_12 ()
  (format t
	  "ir11_12 --> The SIG-display marker moves from the left to the right.~%")  
  (setq ir11_12 _ir11_12))

(ir11_12)

(defun ir11_13 ()
  (format t "ir11_13 --> the power level increasing causes the sig-display ~
	   ~%        marker to move to the right.~%")
  (setq ir11_13 _ir11_13))
;; JR comment: check Talmy's analysis of such explicitly causative clauses

(ir11_13)

(defun ir11_14 ()
  (format t "ir11_14 --> For her to do it.~%")
  (setq ir11_14 _ir11_14))

(ir11_14)

(defun ir11_15 ()
  (format t "ir11_15 --> For her to do it is a bold statement.~%")
  (setq ir11_15 _ir11_15))

(ir11_15)

(defun ir11_16 ()
  (format t "ir11_16 --> The man whom I know.~%")
  (setq ir11_16 _ir11_16))


(ir11_16)

(defun ir11_17 ()
  (format t "ir11_17 --> The man that I know.~%")
  (setq ir11_17 _ir11_17))

(ir11_17)

(defun ir11_18 ()
  (format t "ir11_18 --> The man that knows me.~%")
  (setq ir11_18 _ir11_18))

(ir11_18)

(defun ir11_19 ()
  (format t "ir11_19 --> The position to which the marker moves.~%")
  (setq ir11_19 _ir11_19))

(ir11_19)

(defun ir11_20 ()
  (format t "ir11_20 --> The position of the marker.~%")
  (setq ir11_20 _ir11_20))

(ir11_20)

(defun ir11_21 ()
  (format t "ir11_21 --> My wife, Carol.~%")
  (setq ir11_21 _ir11_21))

(ir11_21)

(defun ir11_22 ()
  (format t "ir11_22 --> My brother Steve.~%")
  (setq ir11_22 _ir11_22))

(ir11_22)

(defun ir11_23 ()
  (format t "ir11_23 --> An intelligent person.~%")
  (setq ir11_23 _ir11_23))

(ir11_23)


(defun ir11_24 ()
  ;; New treatment of lists: complex can be apposition or conjunction
  (format t "ir11_24 --> My brother Steve and my wife, Carol.~%")
  (setq ir11_24 _ir11_24))


(ir11_24)

(defun ir11_25 ()
  (format t "ir11_25 --> My wife, Carol, my brother Steve and John.~%")
  (setq ir11_25 _ir11_25))

(ir11_25)

(defun ir11_26 ()
  (format t "ir11_26 --> Take the hammer and hit the nail.~%")
  (setq ir11_26 _ir11_26))

(ir11_26)		   

(defun ir11_27 ()
  (format t "ir11_27 --> First, take the hammer and then, hit the nail.~%")
  (setq ir11_27 _ir11_27))

(ir11_27)		   

(defun ir11_28 ()
  ;; ir11_28 fails because epistemic-modality is not compatible with imperative.
  (format t "ir11_28 --> <fail>~%")
  (setq ir11_28 _ir11_28))

(ir11_28)

(defun ir11_29 ()
  (format t "ir11_29 --> In order to install the battery, clean the box.~%")
  (setq ir11_29 _ir11_29))
		      
(ir11_29)

(defun ir11_30 ()
  (format t "ir11_30 --> The SIG-display marker can move to the right.~%")
  (setq ir11_30 _ir11_30))

(ir11_30)

(defun ir11_31 ()
  (format t "ir11_31 --> First, take the hammer and then, hit the nail.~%")
  (setq ir11_31 _ir11_31))

(ir11_31)

(defun ir11_32 ()
  ;; ir11_32 fails because epistemic-modality is not compatible with imperative.
  (format t "ir11_32 --> <fail>~%")
  (setq ir11_32 _ir11_32))

(ir11_32)

(defun ir11_33 ()
  (format t "ir11_33 --> Hit the nail with the hammer.~%")
  (setq ir11_33 _ir11_33))

(ir11_33)		   

(defun ir11_34 ()
  (format t "ir11_34 --> Hit the nail using a hammer.~%")
  (setq ir11_34 _ir11_34))

(ir11_34)		   

(defun ir11_35 ()
  (format t "ir11_35 --> Take the nail off using a screwdriver.~%")
  (setq ir11_35 _ir11_35))

(ir11_35)		   

(defun ir11_36 ()
  (format t "ir11_36 --> The nail is taken off using a screwdriver.~%")
  (setq ir11_36 _ir11_36))

(ir11_36)		   

(defun ir11_37 ()
  ;; another way of specifying the prep for the role instrument.
  (format t "ir11_37 --> Take the nail off using a screwdriver.~%")
  (setq ir11_37 _ir11_37))

(ir11_37)		   

(defun ir11_38 ()
  ;; Add an irregular verb to the lexicon with store-verbs
  ;; Treatment of non-movable particles as a single string (sort of hack)
  ;; Test accompaniment role
  (format t "ir11_38 --> The battery lines up with the compartment.~%")
  (store-verbs '( ("line up" "lines up" "lined up" "lining up" "lined up")) )
  (setq ir11_38 _ir11_38))

(ir11_38)		   

(defun ir11_39 ()
  ;; Test reason role
  (format t "ir11_39 --> He does it because of you.~%")
  (setq ir11_39 _ir11_39))

(ir11_39)

(defun ir11_40 ()
  ;; Test reason role with a clause
  (format t "ir11_40 --> He does it because he likes you.~%")
  (setq ir11_40 _ir11_40))

(ir11_40)

(defun ir11_41 ()
  ;; Test reason role
  (format t "ir11_41 --> He scored thirty-nine points for the Lakers.~%")
  (setq ir11_41 _ir11_41))

(ir11_41)


(defun ir11_42 ()
  (format t "ir11_42 --> The holding battery cover plate~%")
  (format t "    --> covers the holding battery compartment.~%")
  (setq ir11_42 _ir11_42))

(ir11_42)

(defun ir11_43 ()
  (format t "ir11_43 --> The holding battery compartment is~%")
  (format t "    --> in the back of the RT, at the bottom left corner.~%")
  (setq ir11_43 _ir11_43))

(ir11_43)

(defun ir11_44 ()
  ;; TEST: try your own syntactic analysis of this one and come up with 3
  ;; reasons why it is inferior to this one.
  ;; Send answers to djk@cs.columbia.edu
  (format t "ir11_44 --> The holding battery compartment contains~%")
  (format t "    --> the old holding battery shown in the cut-away view.~%")
  (setq ir11_44 _ir11_44))
		
(ir11_44)

(defun ir11_45 ()
  ;; Show that conjunction of different cats fails
  (format t "ir11_45 --> <fail>~%")
  (setq ir11_45 _ir11_45))

(ir11_45)


(defun ir11_46 ()
  ;; Show conjunction of low-level constituents (noun)
  (format t "ir11_46 --> My brother and friend, Steve.~%")
  (setq ir11_46 _ir11_46))

(ir11_46)


(defun ir11_47 ()
  ;; Test ordinal
  (format t "ir11_47 --> My first two years were happy and uneventful.~%")
  (setq ir11_47 _ir11_47))

(ir11_47)


(defun ir11_48 ()
  (format t "ir11_48 --> Old McDonald had a farm.~%")
  (setq ir11_48 _ir11_48))

(ir11_48)

(defun ir11_49 ()
  (format t "ir11_49 --> Old McDonald owned a farm.~%")
  (setq ir11_49 _ir11_49))

(ir11_49)
  
(defun ir11_50 ()
  (format t "ir11_50 --> A farm was owned by Old McDonald.~%")
  (setq ir11_50 _ir11_50))

(ir11_50)


(defun ir11_51 ()
  ;; Test reason role
  (format t "ir11_51 --> To be innocent, you must do it for him to eat.~%")
  (setq ir11_51 _ir11_51))

(ir11_51)

;; ======================================================================
;; Test questions 
;; ======================================================================

;; ----------------------------------------------------------------------
;; YES-NO
;; ----------------------------------------------------------------------

(defun ir11_52 ()
  (format t "ir11_52  --> Is this car expensive?~%")
  (setq ir11_52 _ir11_52))

(ir11_52)

(defun ir11_53 ()
  (format t "ir11_53  --> Will John give a blue book to Mary?~%")
  (setq ir11_53 _ir11_53))

(ir11_53)

(defun ir11_54 ()
  (format t "ir11_54  --> John did give a blue book to Mary.~%")
  (setq ir11_54 _ir11_54))

(ir11_54)


;; ----------------------------------------------------------------------
;; WH
;; ----------------------------------------------------------------------

(defun ir11_55 ()
  (format t "ir11_55  --> What is expensive?~%")
  (setq ir11_55 _ir11_55))

(ir11_55)

(defun ir11_56 ()
  (format t "ir11_56  --> What is a classic?~%")
  (setq ir11_56 _ir11_56))

(ir11_56)

;; JR Note: The scope can't simply be "attribute", since then it would be
;; impossible to choose the question pronoun (e.g. in ir11_55 it would be "How"
;; or "How expensive", in ir11_56 it would be "What" or "What kind of")
;; Also note the ambiguity of ir11_56.
(defun ir11_57 ()
  (format t "ir11_57  --> Who is your father?~%")
  (setq ir11_57 _ir11_57))

(ir11_57)		       

(defun ir11_58 ()
  (format t "ir11_58  --> Which is your father?~%")
  (setq ir11_58 _ir11_58))

(ir11_58)

(defun ir11_59 ()
  (format t "ir11_59  --> Where is your mother?~%")
  (setq ir11_59 _ir11_59))

(ir11_59)

(defun ir11_60 ()
  (format t "ir11_60  --> Who is in your house?~%")
  (setq ir11_60 _ir11_60))

(ir11_60)

(defun ir11_61 ()
  (format t "ir11_61  --> What covers the opening~%")
  (setq ir11_61 _ir11_61))

(ir11_61)

(defun ir11_62 ()
  (format t "ir11_62  --> What does the seal cover~%")
  (setq ir11_62 _ir11_62))

(ir11_62)

(defun ir11_63 ()
  (format t "ir11_63  --> When is the game?~%")
  (setq ir11_63 _ir11_63))

(ir11_63)


(defun ir11_64 ()
  (format t "ir11_64  --> What happens then?~%")
  (setq ir11_64 _ir11_64))

(ir11_64)

;; JR note: equative and circumstance-as-process flavors seem difficult
;; without specifying a time unit in the question (e.g. "What day is the
;; 10th?" "How long does the fair last?")

(defun ir11_65 ()
  (format t "ir11_65 --> How is your sister?~%")
  (setq ir11_65 _ir11_65))

(ir11_65)

(defun ir11_66 ()
  (format t "ir11_66 --> With whom is your sister?~%")
  (setq ir11_66 _ir11_66))

(ir11_66)

(defun ir11_67 ()
  (format t "ir11_67  --> Who has a PhD.?~%")
  (setq ir11_67 _ir11_67))

(ir11_67)

(defun ir11_68 ()
  (format t "ir11_68  --> What does she have?~%")
  (setq ir11_68 _ir11_68))
(ir11_68)

(defun ir11_69 ()
  (format t "ir11_69  --> Who owns this book?~%")
  (setq ir11_69 _ir11_69))

(ir11_69)

(defun ir11_70 ()
  (format t "ir11_70  --> Which does she own?~%")
  (setq ir11_70 _ir11_70))

(ir11_70)

(defun ir11_71 ()
  (format t "ir11_71  --> Who gives a blue book to Mary?~%")
  (setq ir11_71 _ir11_71))

(ir11_71)

(defun ir11_72 ()
  (format t "ir11_72  --> What will John give to Mary?~%")
  (setq ir11_72 _ir11_72))

(ir11_72)

(defun ir11_73 ()
  (format t "ir11_73  --> To whom will John give a blue book?~%")
  (setq ir11_73 _ir11_73))
;; Note problem with scope being DOUBLE role, e.g. here Af/Pr
;; since af and pr unify into affected-carrier.
;; Always choose the most specific in your input.

(ir11_73)

(defun ir11_74 ()
  (format t "ir11_74  --> What does Deborah prefer?~%")
  (setq ir11_74 _ir11_74))

(ir11_74)

(defun ir11_75 ()
  (format t "ir11_75  --> Who worships Baal?~%")
  (setq ir11_75 _ir11_75))

(ir11_75)
;; JR Question: How to make generic reference like "Who worships FIRE" or
;; "They want MONEY" ?

(defun ir11_76 ()
  ;; Test reason role with a clause
  (format t "ir11_76  --> Why does he do it?~%")
  (setq ir11_76 _ir11_76))


(ir11_76)

(defun ir11_77 ()
  (format t
    "ir11_77  --> From where does the SIG-display marker move to the right?~%")  
  (setq ir11_77 _ir11_77))

(ir11_77)



;; ======================================================================
;; test RELATIVE CLAUSES
;; ======================================================================

(defun ir11_78 ()
  (format t "ir11_78  --> The box which is expensive.~%")
  (setq ir11_78 _ir11_78))

(ir11_78)

(defun ir11_79 ()
  (format t "ir11_79  --> The box which is a classic.~%")
  (setq ir11_79 _ir11_79))

(ir11_79)

(defun ir11_80 ()
  (format t "ir11_80  --> The man who is your father.~%")
  (setq ir11_80 _ir11_80))

(ir11_80)

(defun ir11_81 ()
  (format t "ir11_81  --> The man who your father is.~%")
  (setq ir11_81 _ir11_81))

(ir11_81)


(defun ir11_82 ()
  (format t "ir11_82  --> The house where your mother is.~%")
  (setq ir11_82 _ir11_82))

(ir11_82)


(defun ir11_83 ()
  (format t "ir11_83  --> The man who is in your house.~%")
  (setq ir11_83 _ir11_83))

(ir11_83)


(defun ir11_84 ()
  (format t "ir11_84  --> The plate which covers the opening.~%")
  (setq ir11_84 _ir11_84))

(ir11_84)


(defun ir11_85 ()
  (format t "ir11_85  --> The hole which the seal covers.~%")
  (setq ir11_85 _ir11_85))

(ir11_85)

(defun ir11_86 ()
  (format t "ir11_86  --> The time when the game starts.~%")
  (setq ir11_86 _ir11_86))

(ir11_86)

(defun ir11_87 ()
  (format t "ir11_87  --> The thing that happens then.~%")
  (setq ir11_87 _ir11_87))

(ir11_87)


(defun ir11_88 ()
  (format t
    "ir11_88  --> The position from which the SIG-display marker moves to the right.~%")  
  (setq ir11_88 _ir11_88))

(ir11_88)


(defun ir11_89 ()
  (format t "ir11_89 --> The way which your sister is.~%")
  (setq ir11_89 _ir11_89))

(ir11_89)


(defun ir11_90 ()
  (format t "ir11_90 --> The person with whom your sister is.~%")
  (setq ir11_90 _ir11_90))

(ir11_90)


(defun ir11_91 ()
  (format t "ir11_91  --> The person who has a PhD.~%")
  (setq ir11_91 _ir11_91))

(ir11_91)


(defun ir11_92 ()
  (format t "ir11_92  --> The box which she has.~%")
  (setq ir11_92 _ir11_92))
(ir11_92)


(defun ir11_93 ()
  (format t "ir11_93  --> The person who owns this book.~%")
  (setq ir11_93 _ir11_93))

(ir11_93)


(defun ir11_94 ()
  (format t "ir11_94  --> The box that she owns.~%")
  (setq ir11_94 _ir11_94))

(ir11_94)


(defun ir11_95 ()
  (format t "ir11_95  --> The person who gives Mary a blue book.~%")
  (setq ir11_95 _ir11_95))

(ir11_95)


(defun ir11_96 ()
  (format t "ir11_96  --> The box which John will give to Mary.~%")
  (setq ir11_96 _ir11_96))

(ir11_96)


(defun ir11_97 ()
  (format t "ir11_97  --> The person to whom John will give a blue book.~%")
  (setq ir11_97 _ir11_97))

(ir11_97)


(defun ir11_98 ()
  (format t "ir11_98  --> The box which Deborah prefers.~%")
  (setq ir11_98 _ir11_98))

(ir11_98)


(defun ir11_99 ()
  (format t "ir11_99  --> The person who worships Baal.~%")
  (setq ir11_99 _ir11_99))

(ir11_99)


;; ======================================================================
;; test of ALL clause patterns of the new transitvity system:
;; ======================================================================
;; simple process
;; event
;; material
;; agentive non-effective: Ag
(defun ir11_100 ()
  (format t "ir11_100 --> Bo runs~%")
  (store-verbs '( ("run" "runs" "ran" "running" "run")) )
  (setq ir11_100 _ir11_100))

(ir11_100)

;; agentive dispositive: Ag + Af
(defun ir11_101 ()
  (format t "ir11_101 --> Bo drinks protein shakes~%")
  (setq ir11_101 _ir11_101))

(ir11_101)

;; agentive creative: Ag + Cr
(defun ir11_102 ()
  (format t "ir11_102 --> Bo cooks diner~%")
  (setq ir11_102 _ir11_102))

(ir11_102)

;; agentive with range: Ag + Rg
(defun ir11_103 ()
  (format t "ir11_103 --> Bo takes a walk~%")
  (setq ir11_103 _ir11_103))

(ir11_103)

(defun ir11_104 ()
  (format t "ir11_104 --> Bo climbs the mountain~%")
  (setq ir11_104 _ir11_104))

(ir11_104)

;; non-agentive dispositive: Af
(defun ir11_105 ()
  (format t "ir11_105 --> The window opens~%")
  (setq ir11_105 _ir11_105))

(ir11_105)

;; non-agentive creative: Cr
(defun ir11_106 ()
  (format t "ir11_106 --> The window pops~%")
  (setq ir11_106 _ir11_106))

(ir11_106)

;; mental non-transitive: Pr
(defun ir11_107 ()
  (format t "ir11_107 --> Bo knows~%")
  (setq ir11_107 _ir11_107))

(ir11_107)

;; mental transitive: Pr + Ph
(defun ir11_108 ()
  (format t "ir11_108 --> Bo knows FUF~%")
  (setq ir11_108 _ir11_108))

(ir11_108)

;; verbal: still on the todo list

;; relation
;; ascriptive attributive: Ca + At
(defun ir11_109 ()
  (format t "ir11_109 --> Bo is strong~%")
  (setq ir11_109 _ir11_109))

(ir11_109)


;; ascriptive equative: Id + Ir
(defun ir11_110 ()
  (format t "ir11_110 --> Bo is the greatest~%")
  (setq ir11_110 _ir11_110))

(ir11_110)

;; possessive attributive: Pr + Pd
(defun ir11_111 ()
  (format t "ir11_111 --> Bo has sneakers~%")
  (setq ir11_111 _ir11_111))

(ir11_111)

;; possesive equative: Id + Ir
(defun ir11_112 ()
  (format t "ir11_112 --> Bo owns sneakers~%")
  (setq ir11_112 _ir11_112))

(ir11_112)

;; locative spatial atributive: Ld + Ln
(defun ir11_113 ()
  (format t "ir11_113 --> Bo is here~%")
  (setq ir11_113 _ir11_113))

(ir11_113)

;; locative spatial equative: Id + Ir
(defun ir11_114 ()
  (format t "ir11_114 --> Bo occupies the left flank~%")
  (setq ir11_114 _ir11_114))

(ir11_114)

;; locative temporal attributive: Ld + Tm
(defun ir11_115 ()
  (format t "ir11_115 --> The race is now~%")
  (setq ir11_115 _ir11_115))

(ir11_115)

;; locative spatial equative: Id + Ir
(defun ir11_116 ()
  (format t "ir11_116 --> The steeplechase follows the high jump~%") 
  (setq ir11_116 _ir11_116))

(ir11_116)

;; locative accompaniment (always equative): Ld/Id + Ac/Ir
(defun ir11_117 ()
  (format t "ir11_117 --> Bo is with Mo~%")
  (setq ir11_117 _ir11_117))

(ir11_117)

(defun ir11_118 ()
  (format t "ir11_118 --> Bo accompanies Mo~%")
  (setq ir11_118 _ir11_118))

(ir11_118)

;; locative existential: Ld
(defun ir11_119 ()
  (format t "ir11_119 --> There is a bug~%")
  (setq ir11_119 _ir11_119))

(ir11_119)

;; locative natural-phenom: no participant
(defun ir11_120 ()
  (format t "ir11_120 --> It rains~%")
  (setq ir11_120 _ir11_120))

(ir11_120)


;; composite process
;; agentive dispositive ascriptive attributive: Ag + Af/Ca + At
(defun ir11_121 ()
  (format t "ir11_121 --> Nike made Bo rich~%")
  (setq ir11_121 _ir11_121))

(ir11_121)
       
;; agentive dispositive ascriptive equative: Ag + Af/Id + At
(defun ir11_122 ()
  (format t "ir11_122 --> The deal makes Bo the richest~%")
  (setq ir11_122 _ir11_122))

(ir11_122)

;; agentive dispositive possessive attributive: Ag + Af/Pr + Pd
(defun ir11_123 ()
  (format t "ir11_123 --> Bo gave the Raiders the victory~%")
  (setq ir11_123 _ir11_123))

(ir11_123)

;; agentive dispositive locative attributive: Ag + Af/Ld + Ln
(defun ir11_124 ()
  (format t "ir11_124 --> Bo lifted the Raiders to a victory~%")
  (setq ir11_124 _ir11_124))

(ir11_124)

;; agentive creative locative attributive: Ag + Cr/Ld + Ln
(defun ir11_125 ()
  (format t "ir11_125 --> It pops a window on the screen~%")
  (setq ir11_125 _ir11_125))

(ir11_125)

;; agentive non-effective ascriptive attributive: Ag/Ca + At
(defun ir11_126 ()
  (format t "ir11_126 --> Bo got tough~%")
  (setq ir11_126 _ir11_126))

(ir11_126)

;; agentive non-effective ascriptive equative: Ag/Id + Ir
(defun ir11_127 ()
  (format t "ir11_127 --> Bo became the best~%")
  (setq ir11_127 _ir11_127))

(ir11_127)

;; agentive non-effective possessive attributive: Ag/Pr + Pd
(defun ir11_128 ()
  (format t "ir11_128 --> Bo bought sneakers~%")
  (setq ir11_128 _ir11_128))

(ir11_128)

;; agentive non-effective locative attributive: Ag/Ld + Ln
(defun ir11_129 ()
  (format t "ir11_129 --> Bo ran home~%")
  (setq ir11_129 _ir11_129))

(ir11_129)

;; non-agentive dispositive ascriptive attributive: Af/Ca + At
(defun ir11_130 ()
  (format t "ir11_130 --> Bo grew old ~%")
  (setq ir11_130 _ir11_130))

(ir11_130)

;; non-agentive dispositive possessive attributive: Af/Pr + Pd
(defun ir11_131 ()
  (format t "ir11_131 --> Bo received the ball~%")
  (setq ir11_131 _ir11_131))

(ir11_131)

;; non-agentive dispositive locative attributive: Af/Ld + Ln
(defun ir11_132 ()
  (format t "ir11_132 --> Bo fell down~%")
  (setq ir11_132 _ir11_132))

(ir11_132)

;; non-agentive creative asciptive attributive: Cr/Ca + At
(defun ir11_133 ()
  (format t "ir11_133 --> The window popped wide~%")
  (setq ir11_133 _ir11_133))

(ir11_133)

;; non-agentive creative locative attributive: Cr/Ld + Ln
(defun ir11_134 ()
  (format t "ir11_134 --> The window popped on the screen~%")
  (setq ir11_134 _ir11_134))

(ir11_134)

(defun ir11_135 ()
  (format t "ir11_135 --> The SIG-display marker moves to the right.~%")
  (setq ir11_135 _ir11_135))

(ir11_135)

(defun ir11_136 ()
  (format t "ir11_136 --> For her to do it must be a bold statement.~%")
  (setq ir11_136 _ir11_136))
(ir11_136)

(defun ir11_137 ()
  (format t "ir11_137 --> The man whom I may know.~%")
  (setq ir11_137 _ir11_137))

(ir11_137)

(defun ir11_138 ()
  (format t "ir11_138 --> Take the hammer.~%")
  (setq ir11_138 _ir11_138))

(ir11_138)		   
		   
(defun ir11_139 ()
  (format t "ir11_139  --> The game happened then.~%")
  (store-verbs '( ("happen" "happens" "happened" "happening" "happened")) )
  (setq ir11_139 _ir11_139))

(ir11_139)

;; agentive dispositive possessive attributive: Ag + Af/Pr + Pd
(defun ir11_140 ()
  (format t "ir11_140 --> Bo gave the victory to the Raiders~%")
  (setq ir11_140 _ir11_140))

(ir11_140)
       
;; ----------------------------------------------------------------------
;; TEST LEXICAL ROLES
;; ----------------------------------------------------------------------

;; PROBLEMS WITH PASSIVE... CHECK HPSG's TREATMENT OF PASSIVE
;; AS A DIFFERENT LEXICAL ITEM GENERATED BY A LEXICAL RULE...

(defun ir11_141 ()
  (format t "ir11_141 --> The customer persuaded the programmer ~
                      to revise the code.~%")
  (setq ir11_141 _ir11_141))


(ir11_141)

;; With a more general approach ala HPSG/S&P91
;; Roles influence/influencer/soa	     
(defun ir11_142 ()
  (format t "ir11_142 --> The customer wants him to do it.~%")
  (setq ir11_142 _ir11_142))

(ir11_142)


;; ----------------------------------------------------------------------
;; TEST BK-CLASS 
;; ----------------------------------------------------------------------

(defun ir11_143 ()
  (format t "ir11_143 --> The Denver Nuggets edged the Celtics.~%")
  (setq ir11_143 _ir11_143))

(ir11_143)

(defun ir11_144 ()
  (store-verbs '(("beat" "beats" "beat" "beating" "beaten")))
  (format t "ir11_144 --> The Denver Nuggets narrowly beat the Celtics.~%")
  (setq ir11_144 _ir11_144))

(ir11_144)

(defun ir11_145 ()
  (format t "ir11_145 --> The hapless Denver Nuggets beat the Celtics.~%")
  (setq ir11_145 _ir11_145))


(ir11_145)


(defun ir11_146 ()
  (format t "ir11_146 --> The Denver Nuggets surprisingly beat the Celtics.~%")
  (setq ir11_146 _ir11_146))


(ir11_146)

(defun ir11_147 ()
  (format t "ir11_147 --> The Denver Nuggets stunned the Celtics.~%")
  (setq ir11_147 _ir11_147))

(ir11_147)


(defun ir11_148 ()
  (format t "ir11_148 --> The hapless Denver Nuggets nipped the Celtics.~%")
  (setq ir11_148 _ir11_148))


(ir11_148)

(defun ir11_149 ()
  (format t "ir11_149 --> The Denver Nuggets beat the Celtics 101-99.~%")
  (setq ir11_149 _ir11_149))

(ir11_149)


(defun ir11_150 ()
  (format t "ir11_150 --> The Denver Nuggets nipped the Celtics 101-99.~%")
  (setq ir11_150 _ir11_150))

(ir11_150)


(defun ir11_151 ()
  (format t "ir11_151 --> The Denver Nuggets narrowly beat the Celtics 101-99.~%")
  (setq ir11_151 _ir11_151))

(ir11_151)

(defun ir11_152 ()
  (format t "ir11_152 --> The Denver Nuggets stunned the Celtics 101-99.~%")
  (setq ir11_152 _ir11_152))

(ir11_152)


(defun ir11_153 ()
  (format t "ir11_153 --> The hapless Denver Nuggets beat the Celtics 101-99.~%")
  (setq ir11_153 _ir11_153))

(ir11_153)


(defun ir11_154 ()
  (format t "ir11_154 --> The Denver Nuggets surprisingly beat the Celtics 101-99.~%")
  (setq ir11_154 _ir11_154))

(ir11_154)


(defun ir11_155 ()
  (format t "ir11_155 --> The hapless Denver Nuggets edged the Celtics 101-99.~%")
  (setq ir11_155 _ir11_155))

(ir11_155)

(defun ir11_156 ()
  (format t "ir11_156 --> The customer persuaded the programmer ~
                      that there is a bug.~%")
  (setq ir11_156 _ir11_156))


(ir11_156)

;; ----------------------------------------------------------------------
;; TESTING FUNCTIONS
;; ----------------------------------------------------------------------

(defun test11 (&key (from 1) (to 156) (timed nil))
  (do* ((i from (1+ i))
	(func (find-symbol (format nil "IR11_~s" i)) 
	      (find-symbol (format nil "IR11_~s" i))))
       ((> i to) 
	(values))
    (when (fboundp func)
      (format t "~%=======================~%")
      (if timed
	(time (uni (funcall func) :limit 1000))
	(uni (funcall func) :limit 1000)))))

(setq *ir-list* (list (cons ir11_1 "ir11_1") (cons ir11_2 "ir11_2") (cons ir11_3 "ir11_3") (cons ir11_4 "ir11_4") (cons ir11_5 "ir11_5")
                      (cons ir11_6 "ir11_6") (cons ir11_7 "ir11_7") (cons ir11_8 "ir11_8") (cons ir11_9 "ir11_9")
                      (cons ir11_10 "ir11_10") (cons ir11_11 "ir11_11") (cons ir11_12 "ir11_12") (cons ir11_13 "ir11_13")
                      (cons ir11_14 "ir11_14") (cons ir11_15 "ir11_15") (cons ir11_16 "ir11_16") (cons ir11_17 "ir11_17")
                      (cons ir11_18 "ir11_18") (cons ir11_19 "ir11_19") (cons ir11_20 "ir11_20") (cons ir11_21 "ir11_21")
                      (cons ir11_22 "ir11_22") (cons ir11_23 "ir11_23") (cons ir11_24 "ir11_24") (cons ir11_25 "ir11_25")
                      (cons ir11_26 "ir11_26") (cons ir11_27 "ir11_27") (cons ir11_28 "ir11_28") (cons ir11_29 "ir11_29")
                      (cons ir11_30 "ir11_30") (cons ir11_31 "ir11_31") (cons ir11_32 "ir11_32") (cons ir11_33 "ir11_33")
                      (cons ir11_34 "ir11_34") (cons ir11_35 "ir11_35") (cons ir11_36 "ir11_36") (cons ir11_37 "ir11_37")
                      (cons ir11_38 "ir11_38") (cons ir11_39 "ir11_39") (cons ir11_40 "ir11_40") (cons ir11_41 "ir11_41")
                      (cons ir11_42 "ir11_42") (cons ir11_43 "ir11_43") (cons ir11_44 "ir11_44") (cons ir11_45 "ir11_45")
                      (cons ir11_46 "ir11_46") (cons ir11_47 "ir11_47") (cons ir11_48 "ir11_48") (cons ir11_49 "ir11_49")
                      (cons ir11_50 "ir11_50") (cons ir11_51 "ir11_51") (cons ir11_52 "ir11_52") (cons ir11_53 "ir11_53")
                      (cons ir11_54 "ir11_54") (cons ir11_55 "ir11_55") (cons ir11_56 "ir11_56") (cons ir11_57 "ir11_57")
                      (cons ir11_58 "ir11_58") (cons ir11_59 "ir11_59") (cons ir11_60 "ir11_60") (cons ir11_61 "ir11_61")
                      (cons ir11_62 "ir11_62") (cons ir11_63 "ir11_63") (cons ir11_64 "ir11_64") (cons ir11_65 "ir11_65")
                      (cons ir11_66 "ir11_66") (cons ir11_67 "ir11_67") (cons ir11_68 "ir11_68") (cons ir11_69 "ir11_69")
                      (cons ir11_70 "ir11_70") (cons ir11_71 "ir11_71") (cons ir11_72 "ir11_72") (cons ir11_73 "ir11_73")
                      (cons ir11_74 "ir11_74") (cons ir11_75 "ir11_75") (cons ir11_76 "ir11_76") (cons ir11_77 "ir11_77")
                      (cons ir11_78 "ir11_78") (cons ir11_79 "ir11_79") (cons ir11_80 "ir11_80") (cons ir11_81 "ir11_81")
                      (cons ir11_82 "ir11_82") (cons ir11_83 "ir11_83") (cons ir11_84 "ir11_84") (cons ir11_85 "ir11_85")
                      (cons ir11_86 "ir11_86") (cons ir11_87 "ir11_87") (cons ir11_88 "ir11_88") (cons ir11_89 "ir11_89")
                      (cons ir11_90 "ir11_90") (cons ir11_91 "ir11_91") (cons ir11_92 "ir11_92") (cons ir11_93 "ir11_93")
                      (cons ir11_94 "ir11_94") (cons ir11_95 "ir11_95") (cons ir11_96 "ir11_96") (cons ir11_97 "ir11_97")
                      (cons ir11_98 "ir11_98") (cons ir11_99 "ir11_99") (cons ir11_100 "ir11_100") (cons ir11_101 "ir11_101")
                      (cons ir11_102 "ir11_102") (cons ir11_103 "ir11_103") (cons ir11_104 "ir11_104") (cons ir11_105 "ir11_105")
                      (cons ir11_106 "ir11_106") (cons ir11_107 "ir11_107") (cons ir11_108 "ir11_108") (cons ir11_109 "ir11_109")
                      (cons ir11_110 "ir11_110") (cons ir11_111 "ir11_111") (cons ir11_112 "ir11_112") (cons ir11_113 "ir11_113")
                      (cons ir11_114 "ir11_114") (cons ir11_115 "ir11_115") (cons ir11_116 "ir11_116") (cons ir11_117 "ir11_117")
                      (cons ir11_118 "ir11_118") (cons ir11_119 "ir11_119") (cons ir11_120 "ir11_120") (cons ir11_121 "ir11_121")
                      (cons ir11_122 "ir11_122") (cons ir11_123 "ir11_123") (cons ir11_124 "ir11_124") (cons ir11_125 "ir11_125")
                      (cons ir11_126 "ir11_126") (cons ir11_127 "ir11_127") (cons ir11_128 "ir11_128") (cons ir11_129 "ir11_129")
                      (cons ir11_130 "ir11_130") (cons ir11_131 "ir11_131") (cons ir11_132 "ir11_132") (cons ir11_133 "ir11_133")
                      (cons ir11_134 "ir11_134") (cons ir11_135 "ir11_135") (cons ir11_136 "ir11_136") (cons ir11_137 "ir11_137")
                      (cons ir11_138 "ir11_138") (cons ir11_139 "ir11_139") (cons ir11_140 "ir11_140") (cons ir11_141 "ir11_141")
                      (cons ir11_142 "ir11_142") (cons ir11_143 "ir11_143") (cons ir11_144 "ir11_144") (cons ir11_145 "ir11_145")
                      (cons ir11_146 "ir11_146") (cons ir11_147 "ir11_147") (cons ir11_148 "ir11_148") (cons ir11_149 "ir11_149")
                      (cons ir11_150 "ir11_150") (cons ir11_151 "ir11_151") (cons ir11_152 "ir11_152") (cons ir11_153 "ir11_153")
                      (cons ir11_154 "ir11_154") (cons ir11_155 "ir11_155") (cons ir11_140 "ir11_156")))
