;; Usage:
;; runfuf -l runfd.scm                   - to pretty print fds in tricky.scm from the testsuit and place them in directory "1"
;; runfuf -l runfd.scm test-file dst-dir - to pretty print fds in test-file and place them in directory dst-dir
(require 'common-list-functions)

(define (get-env-var var-name)
  (let ((var-value (getenv var-name)))
    (if (not var-value)
        (error (string-append "The " var-value " environment variable must be set!")))
    var-value))

(define *cfuf-toplevel-path* (get-env-var "CFUF_TOPLEVEL_PATH"))
(define *surge-path* (get-env-var "SURGE_PATH"))
(define *cfuf-common-path* (get-env-var "CFUF_COMMON_PATH"))

(define (load-all)
  (let ((cwd (getcwd)))
    (chdir *cfuf-toplevel-path*)
    (load "cfuf.scm")
    (chdir *surge-path*)
    (load "surge.scm")
    (chdir cwd)))

(define (fail-load)
  (error (string-append "Usage: runfuf -l runfd.scm [testfile dstdir]")))

(define testfile #f)
(define dstdir #f)

(define (parse-args)
  (let* ((args (if *interactive* () (cdddr *argv*)))
         (nargs (length args)))
    (case nargs
      ('2 (set! testfile (car args))
          (set! dstdir (cadr args)))
      ('0 (set! testfile (string-append (get-env-var "CFUF_TESTSUIT_PATH") "tricky.scm"))
          (set! dstdir "1"))
      (else (fail-load)))
    (if (not (access testfile "r"))
        (error (string-append "testfile " testfile " must be readable")))
    (let ((cwd (getcwd)))
      (if (not (or (chdir dstdir) (mkdir dstdir 511)))
          (error (string-append "Failed to access the " dstdir " directory")))
      (chdir cwd))))

(define (fun cfd name)
  (let ((file-name (format nil "~a/_~s.scm" dstdir name)))
    (write-to-file cfd file-name '*PPRINT-FD*)
    (format t "The result is in ~a~%" file-name)
    #t))

(load-all)
(parse-args)

;; (load-grammar *surge-info* 'linearizer 'types)
(load testfile)
(test :timed t :fun fun :linearize #f)
