;;; -----------------------------------------------------------------------
;;; File:         functional.l
;;; Description:  functional represenation of modifiers.
;;; Author:       Yael Dahan Netzer
;;; Created:      22 May 1997
;;; Modified:     
;;; Package:      FUG5
;;; -----------------------------------------------------------------------

;All features under: (modifiers ((...........)))
;Feature to determine the mapping: (function-realize-as
;				   qualifer/classifierdescriber)
;Suggestion for nominalizations: (simple and not functionalist..)
;  object: hakraAt hmavo' - mavo' is under ((modifiers ((object ((lex
;								 "mavo'"))))))
;  agent: 

; ATTRIBUTE: (as in attributive mode of ascriptive process): describer
;           Examples: i22f, i23f, i35f, i52f, i56f, i58f, i63f, i70f, i128f
; IDENTIFIER: BE -John hamore - apposition
;                hanasi' vaiZman (occupation?)(title?)
; CAUSE: yatus kadaHat (function-realize-as qualifer/relative classifier)
; CAUSED-BY: makat HaSmal

; PRODUCT (Make1): ASaf haqsamim (function-realize-as classifier
;						      qualifer/relative)
; PRODUCER (MAKE2) - kurey wacaviS 
;  Examples: i42f
; MANUFACTURE (sirtey Agfa', manwul yeyl)
; AUTHOR
;      Examples: i44f

; CONTENT (HAVE1): bakbuk yayin, tA hacfafot
;  Examples: i55f, i90f
; MATERIAL (make2/BE): ner SawaVa
;  Examples: i54f, i63f, i90f
; CONFIGURATION (make2): zer praHim. (which is the head?) somex in plural
;               feature is: (config-units...)
;  Examples: i27f
;POSSESSIVE : nawa'ley bo, nawalayv Sel bo
; Possession: hamora Seyes la hascala -
;** According to Levi: psanter canaf - psanter SyeS lo canf
;   but it seems wrong to say: morat haascala
;   hascalat hamorah        
;   So it seems to be a more 'physical' relation (Hadar Ambatya)
;   But again, on the other side, what about: seret metaH? (Levi - can be
;   also CAUSE
;   So again, it seems to be somehow a relation of 'content'.

; OWNER Examples: i25f (Sel)
; HAS-PART: capot yadaym,
; PART-OF: sirat mifras

; TYPE (BE) - realize-as: construct
;                          pp - prep Sel
;  Levi BE Genus-specious) eZ Oren. (eZ Sehu misug Oren)
;  Examples: i22f
;     -metaphorical equvalence - maZevat maHat (prep: cmo) - comparison?
;       - other: GUR arieh


; Represented: phrases like: semel hayokra, aqord Sel horada veharama -
;              the somex is abstrat


; LOCATION: needs feature (locative-type in/on/under/on-top/near/...)
;          map-to: qualifier/pp
;          relation IN can be mapped to classifier
;          Examples: i25f (prep Sel), i25f, i28f, i46f, i52f (in), i64f, i66f

; PURPOSE: FOR RDP  carTis nesiwa, kley biTuy.. (paraphrase: prep "l") 

; MATTER: miSpat reZaH (function-realize-as qualifier/pp (wal), classifier)
;  Examples: i42f, i49f (Sel)
; INSTRUMENT (levi USE): magheZ edim.? (instr-polarity ??)
;  Examples: i57f (adj)


;; Other relations:
; DIRECTION:?? darom,Zafon, yamin, SmOl? (function-realize-as describer)
;          hayam haZfony
;            Examples: i23f

; ORIGION: (function-realize-as qualifier/pp) prep: m
;  Examples: i90f
; DESTINATION: (function-realize-as qualifier/pp) prep: l
;  Examples: i28f

; DURATION: ? Ahava wad clot.? cama pewamim bemeSex haSanim 
; FREQUENCY: ?

; TIME:? describer: waxSaVi.. pp: haHalv milifney Savuw
;  Examples: i42f (classifier)
; INCLUSION: nawal wim sroxym
; DISTANCE:
; REASON:(function-realize-as qualifier/pp) beSel, biglal?

; EXEPTION: HuZ m.. 
; SUBSTITUTION: cise' bimkom SulHan??
; MANNER: ? Halixa zehira? (function-realize-as describer)
;  Examples: i49f, i66f
; COMPARISON: yalda yafa cmo Halom
;  Examples: i70f
; CONDITION:?

; CONTRAST:?
; ACCOMPANIMENT: positiive- wim negtive: bli (nawal bli sroxim)
;  Examples: i45f, i54f, i58f, i59f, i100f (polarity -)
; MEANS:?
; PRESPECTIVE:?
;EFFECT:?

;; mapping:
;; create a feature (realize-modifiers ((owner construct) (purpose
;; qualifier) ...)
;; After going through all possibilities of modifiers, unify this feature.
;; If more than one qualifier is available, map all of them to list.
;; Same to describers and Shel-phrases.
;(define-feature-type qualifier (shel-phrase))

(define :drealize ':drealize)
(define :dprep ':dprep)
(define :dont-realize ':dont-realize)

(define (_make-modifier-alternation modifier optional)
  (let ((drealize 'classifier)
        (dprep "Sel")
        (dont-realize 'none))
    (let ((key-alist (list (cons :drealize (lambda (val) (set! drealize val)))
                           (cons :dprep (lambda (val) (set! dprep val)))
                           (cons :dont-realize (lambda (val) (set! dont-realize val)))
                           )))
      (letrec ((eval-keys (lambda (keys)
                            (if (null? keys)
                                'done
                                (begin
                                 ((aget-value key-alist (car keys)) (cadr keys))
                                 (eval-keys (cddr keys)))))))
              (eval-keys optional)))
    
    (let ((path-elts (list '^ '^ 'modifiers modifier))
          (qualifier-alt (string->symbol
                          (string-append "make-" (symbol->string modifier) "-pp")))
          (shel-alt (string->symbol
                     (string-append "make-" (symbol->string modifier) "-shel-phrase")))
          (fd `((cat pp)
                (np ,(make-path '^ '^ '^ 'modifiers modifier))
                (restrictive ,(make-path '^ '^ '^ 'modifiers modifier 'restrictive))
                (prep ((lex ,(make-path '^ '^ '^ '^ 'modifiers modifier 'functional-prep)))))))
      (let ((path (apply make-path path-elts)))
        
        `(def-alt ,(string->symbol
                    (string-append "realize-" (symbol->string modifier)))
           (((possessive-function none)
             (realize-function-as classifier)
             
             (alt (((lex given))
                   ((lex {^ adj-derivation lex}))))
             (,path {^2 classifier})) ;; was classifier YD 18/9
            
            ((realize-function-as qualifier)
             (possessive-function none)
             (cat np)
             (functional-prep ((alt (,dprep given))))
             
             (alt (((functional-prep "Sel")
                    (alt ,(string->symbol
                           (string-append "make-" (symbol->string modifier) "-shel-phrase"))
                         ((({^2 shel-phrases shel-phrase2} given) ({^2 shel-phrases shel-phrases} ,fd))
                          (({^2 shel-phrases shel-phrase1} given) ({^2 shel-phrases shel-phrase2} ,fd))
                          (({^2 shel-phrase} given) ({^2 shel-phrases shel-phrases1} ,fd))
                          (({^2 shel-phrase} 
                                ((cat pp)
                                 (np ,(make-path '^ '^  'modifiers modifier))
                                 (prep ((lex ,(make-path '^ '^ '^ 'modifiers modifier 'functional-prep))))))))))
                   
                   ((realize-function-as qualifier)
                    (alt ,(string->symbol
                           (string-append  (symbol->string modifier) "-pp"))
                         ((({^2 qualifiers qualifier2}  given) ({^2 qualifiers qualifier3} ,fd))
                          (({^2 qualifiers qualifier1}  given) ({^2 qualifiers qualifier2} ,fd))
                          (({^2 qualifier} given) ({^2 qualifiers qualifier1} ,fd)
                           ({^2 qualifier-list} yes))
                          (({^2 qualifier} 
                                ((cat pp)
                                 (np ,(make-path '^ '^  'modifiers modifier))
                                 (prep ((lex ,(make-path '^ '^ '^ 'modifiers modifier 'functional-prep)))))))))))))
            
            ((possessive-function #(under yes))
             (cat np)
             ;; (,(apply make-path (append path-elts '(realize-function-as))) {^ possessive-style})
             (,path {^2 possessor})
             ({^2 possessor possessive-style} ,(apply make-path (append path-elts '(realize-function-as)))))
            
            ((realize-function-as describer)
             (alt (((lex given))
                   ((lex {^ adj-derivation}))))
             (alt ,(string->symbol
                    (string-append "make-" (symbol->string modifier) "-describer"))
                  ((({^2 describers describer1} given) 
                    ({^2 describers describer2} ,path)
                    ({^2 describer-list} yes))
                   (({^2 describer} given)
                    ({^2 describers describer1} ,path)
                    ({^2 describers-list} yes))
                   (({^2 describer} ,(make-path '^ 'modifiers modifier))))))))))))

(defmacro make-modifier-alternation (modifier . optional) `(_make-modifier-alternation ',modifier ',optional))

(def-alt functional-modifiers
  (((modifiers none))
   ((modifiers given)
    (modifiers
     ((opt attribute
	   ;; HERE handles list of describers
	   ((attribute given)
	    (attribute ((cat ap)
			(realize-function-as describer)
;; ADD here constrain
			;; (:! realize-attribute) ;; UNDEFINED!!!
                        ))))
      ;; FOR RDP
      (opt purpose 
	   ((purpose given)
	    (purpose ((cat np)
		      ;; (:! realize-purpose) ;; UNDEFINED!!!
                      ))))
      ;; MAKE RDP
      (opt material 
	   ((material given) ;i54f
	    (material ((cat np)
		       (countable no)
		       ;; (:! realize-material) ;; UNDEFINED!!!
                       ))))
      ;; HAVE RDP
      (opt owner ((owner given)
                  (owner ((cat np)
                         (possessive-function yes)
                         ;; (:! realize-owner) ;; UNDEFINED!!!
                         ))))

      ;; HUMAN_RELATOR
      (opt human-relator 
	   ((human-relator given) ;;AH, AV..
	    (human-relator ((possessive-function yes)
                            ;; (:! realize-human-relator) ;; UNDEFINED!!!
                            ))))

      ;; HAVE RDP
      (opt part-of ((part-of given)
                    (part-of (;(alt (((realize-function-as qualifier))
			      ;((realize-function-as classifier))))
			      ;; (:! realize-part-of) ;; UNDEFINED!!!
                              ))))
      ;;; TODO build possibility for :
      ;;; Yeled adom panim. part-of(yeled,panim) color(panim, adom)
      ;; HAVE RDP                              
      (opt has-part ((has-part given) 
		     (has-part ((cat np) 
				(possessive-function yes) 
				;; (:! realize-has-part) ;; UNDEFINED!!!
                                ))))

      ;; HAVE RDP      
      (opt content ((content given) (content ((cat np)
                                              ;; (:! realize-content) ;; UNDEFINED!!!
                                              ))))
      ;; MAKE RDP
      (opt config-units ((config-units given) 
			 (config-units ((number plural) (cat np)
					;; (:! realize-config-units) ;; UNDEFINED!!!
                                        ))))
      ;; BE RDP
      (opt type ((type given)
                 (type ((alt (((cat #(under ap))
                               (realize-function-as describer))
                              ((cat np)
                               (realize-function-as ((alt (classifier qualifier)))))))
                        ;; (:! realize-type) ;; UNDEFINED!!!
                        ))))
      ;; MAKE RDP
      (opt author ((author given)
                   (author ((cat proper)
                            (realize-function-as ((alt (classifier qualifier))))
                            ;; (:! realize-author) ;; UNDEFINED!!!
                            ))))
      ;; MAKE RDP
      (opt manufacturer 
	   ((manufacturer given) ;; i39f
	    (manufacturer ((cat proper)
                           (realize-function-as ((alt (classifier qualifier))))
			   ;; (:! realize-manufacturer) ;; UNDEFINED!!!
                           ))))
      ;; MAKE RDP
      (opt producer 
	   ((producer given)
            (producer ((alt (((cat #(under pronoun))
                              (possessive-function yes))
                             ((cat np))))
                       ;; (:! realize-producer) ;; UNDEFINED!!!
                       ))))
      ;; IN RDP
      (opt location 
	   ((location given)
	    (location
	     ((alt locative-relation
		   (((locative-relation none)
                     (alt (((cat #(under ap)))
                           ((cat #(under adj))
                            (definite no))))
                     (realize-function-as describer))
                    ((locative-relation in)
		     (functional-prep ((alt ("b" given))))
		     (alt (((realize-function-as qualifier))
			   ((realize-function-as classifier))))) ;; i25f
		    ((locative-relation on)
		     (realize-function-as qualifier)
		     (functional-prep "wal"))
		    ((locative-relation under)
		     (realize-function-as qualifier)
		     (functional-prep "mtaHat"))
		    ((locative-relation near)
		     (realize-function-as qualifier)
		     (functional-prep "leyad"))
		    ((locative-relation before)
		     (realize-function-as qualifier)
		     (functional-prep "lfney"))
		    ((locative-relation behind)
		     (realize-function-as qualifier)
		     (functional-prep "AHarey"))
                    ((locative-relation distance)
                     (realize-function-as describer))))
                    
	      ;; (:! realize-location) ;; UNDEFINED!!!
              ))))
      ;; INSTRUMENT RDP
      (opt instrument 
	   ((instrument given) ;; cira HaSmalit i57f
	    (instrument ((alt (((cat #(under ap))
                                (realize-function-as classifier))
			       ((realize-function-as qualifier))))
			 ;; (:! realize-instrument) ;; UNDEFINED!!!
                         ))))
      ;; CAUSE RDP
      (opt cause
	   ((cause given)
	    (cause ((cat np)
		    (alt (((realize-function-as qualifier))
			  ((realize-function-as classifier))))
		    ;; (:! realize-cause) ;; UNDEFINED!!!
                    ))))
      ;; CAUSE RDP           
      (opt caused-by
	   ((caused-by given)
	    (caused-by ((cat np)
			(alt (((realize-function-as qualifier))
			      ((realize-function-as classifier))))
			;; (:! realize-caused-by) ;; UNDEFINED!!!
                        ))))
      ;; INCLUSION
      (opt inclusion
	   ((inclusion given)
	    (inclusion
	     ((cat np) 
	      (realize-function-as qualifier)
	      (alt (((polarity +) 
		     (functional-prep ((alt ("wm" given)))))
		    ((polarity -)
		     (functional-prep ((alt ("bly" given)))))))
		     ;; (:! realize-inclusion) ;; UNDEFINED!!!
                     ))))
       ;; ACCOMPANIMENT   
      (opt accompaniment 
	   ((accompaniment given)
	    (accompaniment
	     ((cat np) (realize-function-as qualifier)
	      (alt (((polarity +) 
		     (functional-prep ((alt ("wm" given)))))
		    ((polarity -)
		     (functional-prep "bly"))))
	      ;; (:! realize-accompaniment) ;; UNDEFINED!!!
              ))))
	 
      (opt manner 
	   ((manner given) ;; i49f, i77f
	    (manner ((opt ((cat ap)
			   (realize-function-as describer)))
		     ;; (:! realize-manner) ;; UNDEFINED!!!
                     ))))
      
      ;; MEASURE
      (opt measure
	   ((measure given)
	    (measure ((realize-function-as qualifier)
		      (functional-prep "Sel")
		      ;; (:! realize-measure) ;; UNDEFINED!!!
                      ))))

      ;; DIRECTION
      (opt direction 
	   ((direction given) ;;i23f HaZer pnimit
	    (direction ((alt (((realize-function-as classifier)
			       (cat ap))
			      ((realize-function-as qualifier))))))
	    (direction (
                        ;; (:! realize-direction) ;; UNDEFINED!!!
                        ))))
      ;; IN RDP
      (opt time 
	   ((time given)
	    (time ((alt temporal-relation
			(((temporal-relation during)
			  (alt (((realize-function-as qualifier))
				((realize-function-as classifier))))
			  (functional-prep ((alt (given "b")))))
			 ((temporal-relation before)
			  (realize-function-as qualifier)
			  (functional-prep "lfney"))
			 ((temporal-relation after)
			  (realize-function-as qualifier)
			  (functional-prep ((alt ("AHarey" given)))))
			 ((temporal-relation beginning)
			  (realize-function-as qualifier)
			  (functional-prep "betHilat"))))
		   ;; (:! realize-time) ;; UNDEFINED!!!
                   ))))

      ;; ABOUT RDP
      (opt matter 
	      ((matter given)
	       (matter ((functional-prep ((alt ("wal" given))))
			;; (:! realize-matter) ;; UNDEFINED!!!
                        ))))
      ;; COMPARISON
      (opt comparison 
	   ((comparison given)
	    (comparison ((realize-function-as qualifier)
                         (punctuation ((before ",")))
			 (functional-prep ((alt ("c" "cmo" given))))
			 ;; (:! realize-comparison) ;; UNDEFINED!!!
                         ))))
      ;; ORIGIN
      (opt origin 
	   ((origin given) ;; i90f
	    (origin ((realize-function-as ((alt (describer qualifier))))))
	    (origin ((functional-prep "m")
		     ;; (:! realize-origin) ;; UNDEFINED!!!
                     ))))
      ;; REPRESENTEED (BE-MAKE RDP)
      (opt represented ((represented given)
                        (represented ((functional-prep "Sel")
                                      ;; (:! realize-represented) ;; UNDEFINED!!!
                                      ))))
      ;; NAME -BE RDP
      (opt name
           ((name given)
            (name ((realize-function-as ((alt (classifier qualifier))))
                       (functional-prep "wal Sem")
                       ;; (:! realize-name) ;; UNDEFINED!!!
                       ))))
      ;; DESTINATION
      (opt destination
           ((destination given)
            (destination ((realize-function-as qualifier)
                          ;(functional-prep "l")
                          ;; (:! realize-destination) ;; UNDEFINED!!!
                          ))))
      ;; RANGE
      (opt range
           ((range given)
            (range ((alt (((cat ap)
                           (realize-function-as describer))
                          ((realize-function-as qualifier))))
                    ;; (:! realize-range) ;; UNDEFINED!!!
                    ))))
      (opt exeption
           ((exeption given)
            (exeption ((realize-function-as qualifier)
                       ;; (:! realize-exeption) ;; UNDEFINED!!!
                       ))))

      ;; SOME USEFUL ATTRIBUTIVE FEATURES
      ;; yd 21/12
      (opt color
	   ((color given)
	    (color ((realize-function-as describer)
		       ;; (:! realize-color) ;; UNDEFINED!!!
                       ))))
      (opt size
	   ((size given)
	    (size ((realize-function-as describer)
		   ;; (:! realize-size) ;; UNDEFINED!!!
                   ))))
      (opt shape
	   ((shape given)
	    (shape ((realize-function-as describer)
		   ;; (:! realize-shape) ;; UNDEFINED!!!
                   ))))
	    
  )))))

;;

(make-modifier-alternation inclusion)
(make-modifier-alternation shape)

(make-modifier-alternation color)
(make-modifier-alternation size)
(make-modifier-alternation range)
(make-modifier-alternation has-part)
(make-modifier-alternation cause)
(make-modifier-alternation caused-by)
(make-modifier-alternation origin :dprep "m")
(make-modifier-alternation whole)
(make-modifier-alternation human-relator)
(make-modifier-alternation exeption :dprep "milvad")
(make-modifier-alternation destination :dprep "l")
(make-modifier-alternation represented)
(make-modifier-alternation name :dprep "wal Sem")
(make-modifier-alternation purpose :dprep "l")
(make-modifier-alternation material :dprep "m")
(make-modifier-alternation owner)	;; this should be with possessor
(make-modifier-alternation content :dprep "wm")
(make-modifier-alternation part-of :dprep "wm")
(make-modifier-alternation config-units)
(make-modifier-alternation type)
(make-modifier-alternation author)
(make-modifier-alternation manufacturer)
(make-modifier-alternation location  :dprep "in")
(make-modifier-alternation instrument)
(make-modifier-alternation accompaniment)
(make-modifier-alternation manner :dprep "b")
(make-modifier-alternation time)
(make-modifier-alternation comparison)
(make-modifier-alternation matter)
(make-modifier-alternation producer)
(make-modifier-alternation accompaniment :dprep "wm") ;; qualifier only
(make-modifier-alternation attribute)
(make-modifier-alternation measure :dprep "Sel")
(make-modifier-alternation direction :dprep "l")
;On Nominal Non-Predicating Adjectives in English (European
;                             University Studies, Series Xiv, Anglo-Saxon Language and
;                             Literature, Vol 168) 
;                                   Steffi George / Paperback / Published 1987 
;                                   Our Price: $30.65 (Special Order)
;The grammar of nominal compounding with special reference to
;                             Danish, English, and French 
;                                   Laurie Bauer
;                                   (Hard to Find)
