;;; -----------------------------------------------------------------------
;;; File:         hgr-modular.l
;;; Description:  
;;; Author:       Yael Dahan Netzer
;;; Created:      5 Jun based on hgr1.l
;;; Modified:     
;;; Package:      FUF
;;; -----------------------------------------------------------------------
;; cochavit 6231312

;; This cat for lexical-compounds in Hebrew
(def-conj noun-compound
  (cat #(under noun-compound))
  (nismax no)
  (pattern (head somex))
  (head ((cat noun)
         (nismax yes)))
  (classifier ((cat np-head)))
  (alt (((classifier ((cat #(under noun-compound))))
         (somex {^ classifier})
         (somex ((definite {^2 definite})
                 (mark-definite {^2 mark-definite}))))
        ((definite no)
         (somex {^ classifier}))
        ((mark-definite given)
         (mark-definite no)
         (somex {^ classifier}))
        ((somex ((cat phrase)
                 (pattern (det {^2 classifier}))
                 (det ((cat article)
                       (definite yes))))))))
  (head ((number {^2 number})
         (gender {^ gender}))))

(define *u-grammar*
  (refresh-fug
   '((alt top (:index cat)
          (((cat clause)
            (complex none)
            (process {^ proc})
            (process-type {^ process type})
            (partic {^ participants})
            (:! transitivity)
            
            (verb {^ proc})
            
            (proc ((cat vp)
                   ;; here its not good enought, existential - noun is treated as object and
                   ;; it will not work.
                   (polarity {^2 polarity})
                   (v ((lex {^2 lex})
                       (cat verb)
                       (person {^3 synt-roles subject person})
                       (number {^3 synt-roles subject number})
                       (gender {^3 synt-roles subject gender})))))
            
            (alt circum
                 (((circum none))
                  ((circum given)
                   (circum
                    ((cat phrase)
                     (opt manner 
                          ((manner given)
                           (manner ((cat pp)
                                    (prep ((lex ((alt ("b" given))))))
                                    (np ((cat np)))))))
                     (opt location 
                          ((location given)
                           (location ((cat pp)
                                      (np ((cat np)))))))
                     (opt time 
                          ((time given)
                           (time ((cat pp)
                                  (np ((cat np)))))))
                     (pattern (manner time location)))))))
            
            
            (ralt clause-order ;(:index clause-order)
                  (:demo "What is the order of the clause?")
                  (
                   ((clause-order subj-verb)
                    (pattern ({^ synt-roles subject} {^ proc}
                              {^ synt-roles object} {^ circum})))
                   ((clause-order verb-subj)
                    (pattern ({^ proc} {^ synt-roles
                                          subject} {^ synt-roles
                                                      object} {^ circum})))
                   ((clause-order #(under existential))
                    (pattern ({^ circum location}
                              {^ proc}
                              {^ synt-roles subject}
                              {^ circum manner}
                              {^ circum time})))
                   ((clause-order time-vso)
                    (pattern ({^ circum time} 
                              {^ proc}
                              {^ synt-roles subject}
                              {^ synt-roles object}
                              {^ circum manner}
                              {^ circum location}
                              )))
                   ((clause-order location-svo)
                    (pattern ({^ circum location}
                              {^ synt-roles subject}
                              {^ proc}
                              {^ synt-roles object}
                              {^ circum manner}
                              {^ circum time}))))))
           
           
           ;; ================================================
           ((cat #(under np))
            (alt np-gap 
                 (((gap none)
                   (alt type-of-np (:index cat)
                        (;((:& cat-partitive))
                                        ;((:& cat-measure))
                         ((:& simple-np)))))
                  ((gap #(under yes))
                   (cat #(under np)) 
                   (:& np-propagate)
                   (:! np-type)
                   (:! np-number)))))
           ;; np-head
           ((cat np-head)
            (complex none)
            (alt np-head (((cat noun) (generic-cat noun))
                          ((:& noun-compound))
                          ((:& person-name)))))
           
           
           ;; =================================================		
           ((:& complex))
           ;; ===============================================	
           ((cat adj))
           ;; ====================================	
           ((cat pp)
            (pattern (prep np))
            (prep ((cat prep) (lex given)))
            (np ((cat np)
                 (gap none)
                 (definite {^2 definite})
                 (case objective))))
           ;; =========================================
           ((cat simple-ap)
            (complex none)
            (generic-cat ap)
            (pattern (dots head dots))
            ;; caHol, caHol bahir, cHol einayim, caHol shel shamayim.
            (head ((cat adj)
                   (lex {^2 lex})
                   (number {^2 number})
                   (gender {^2 gender})
                   (adj-type {^2 adj-type})))
            
            (alt (((classifier none))
                  ((classifier given)
                   ;; yeled caHol ceheh
                   (alt (((classifier ((cat adj)))
                          ;; (realize-as none)
                          (pattern (dots head classifier dots)))
                         ;; yeled cHol weinayim
                         ;; somex - classifier cannot be a pronoun.
                         ((classifier ((cat ((alt (common proper))))
                                       (mark-definite {^2 mark-definite})
                                       (definite {^2 definite}))) ;;np?
                          ;; (realize-as construct)
                          (pattern (dots head classifier))
                          (head ((nismax yes)
                                 (mark-definite no)))))))))
            (alt (((qualifier none)) ;; gavoha me ha memutza
                  ((qualifier given)
                   (qualifier 
                    ((alt (((cat pp))
                           ((cat clause))))))
                   ;; take care of such: past participles - 
                   ;; niyarot tzmudim la dolar.
                   (pattern (dots head dots qualifier)))))
            
            (alt superlative-comparative
                 (((superlative no)
                   (comparative no))
                  ((superlative #(under yes))
                   (comparative no)
                   (definite yes)
                   (head ((mark-definite no)))
                   (modifier ((lex "haxy"))))
                  ;; (ralt (((head ((mark-definite no)))
                          ;; (modifier ((lex "haxy"))))
                         ;; ((qualifier ((cat pp)
                                      ;; (prep ((lex "b")))
                                      ;; (np ((cat common) (lex "yoter")))))))))
                               
                                   
                  ((comparative #(under yes))
                   (modifier ((lex "yoter"))))))
            
            ;; what about polarity????? lo-yafe...
            
            (alt modifier
                 (((modifier none))
                  ((modifier given)
                   (modifier ((cat adv)
                              (definite {^2 definite})))
                   (pattern (dots modifier head dots))))) ;;
            ;; so adverb can get definiteness.
            ;; I do not know about the next feature - check.
            ;; (head ((mark-definite no)))))) ;;?? is it true?
            ;; not all adverbs are possible: tinok tamid atzbani
            
            (alt definite-ap (:wait definite)
                 (((definite no))
                  ((definite yes)
                   (alt (((mark-definite yes)
                          (head ((mark-definite yes)))
                          (pattern (det dots head dots)))
                         ((head ((mark-definite no)))))))))			     
            
            (det ((cat article)
                  (definite {^2 definite}))))
           
           
           ;; I'm not sure if this is an AP or NP: cHol-einayim.
		 
		 
           ;; ============================
           ((cat vp)
            (v ((cat verb)))
            (alt (((polarity positive)
                   (pattern (v)))
                  ((polarity #(under negative))
                   (pattern (notf v)))))
            (notf ((cat phrase)
                   (lex "lO"))))
           ((cat verb))
           ;; ============================
           
           ((cat adv))
           ;; ===============================
           ;; CAT CARDINAL
           ;; ===============================
           ((cat simple-cardinal)
            (complex none)
            (alt cardinal-value 
                 (((value given)
                   (control (and (integer? @{^ value})
                                 (> @{^ value} 0) 
                                 (< @{^ value} 11)))
                   (digit no))
                  ((digit yes))
                  ((digit no)))))
           
           ;;===============================
           ;; CAT ORDINAL
           ;;===============================		
           ((:& cat-ordinal))
           ;;===============================		
           ;; CAT ARTICLE
           ;;===============================		
           
           ((cat article)
            (alt (:wait definite)
                 (((definite no))
                  ((definite yes)
                   (lex "ha")))))
           ;;===============================		
           ;; CAT PHRASE
           ;;===============================		
           ((cat phrase))
           
           ;; this should be corrected because if word is in smixut then it does not
           ;; get a definite 'ha'...
           ;;===============================		
           ;; CAT QUANTIFIER
           ;;===============================		
           ((cat quantifier)
            (alt quantifier-type
                 (((type amount)
                   (pattern (dots head))
                   (head ((lex {^2 lex})))
                   (alt (((definite no)
                          (head ((cat phrase)))
                          (pattern (head)))
                         ((definite yes)
                          (alt (((morph-as quantifier)
                                 (head ((cat phrase))))
                                ;; this is not good, quantifiers should get morpholgy, BUT the reason now
                                ;; I'm doing it is the possibility for: rav rabim... TO CHANGE
                                ((morph-as adj)
                                 (head ((cat adj)
                                        (nismax {^2 nismax})
                                        (number {^2 number})
                                        (gender {^2 gender}))))))
                          (alt (((nismax no)
                                 (det ((cat article)
                                       (definite yes)))
                                 (pattern (det head)))
                                ((nismax yes)
                                 (pattern (head)))))))))
                  ((type total)))))
           ((cat quant-partitive)
            (alt (((quantity given)
                   (object given)
                   (pattern (quantity object)))
                  ((quantity none)
                   (pattern (head))
                   (head ((lex {^2 lex})
                          (cat phrase)))))))
           ;;==============================================================
           ;; CAT FRACTION
           ;;==============================================================
           ((cat fraction)
            (control (number? @{^ num}))
            (control (number? @{^ den}))
            (digit ((alt (no yes)))))
           
           ;; ==============================================================
           ;; 10 CAT LIST : agglutinated heterogeneous constituents --------
           ;; ==============================================================
           ;; List is for an agglutination of heterogeneous constituents all
           ;; playing together the same syntactic function in a larger
           ;; constituent.
           ;; For example, a list of describers or qualifiers in an NP.
           ;; Each element of the list can be of a different cat (unlike
           ;; conjunction). 
           ;; Lists have one main feature:
           ;; elements: a list of features in car/cdr form (~ macro is useful).
           ;; Just recurse on all elements of the list with no additional 
           ;; processing 
           ((cat list)
            (elements {^ distinct})  ;; for compatibility with complex
            (cset ((- semr kind)))
            (alt list 
                 (:demo "How many elements are there in the list?")
                 (((elements none))                           ;; 0 elements
                  ((elements ((car given) (cdr none)))        ;; 1 elements 
                   (first {^  elements car})
                   (cset (first))                 ;; to eliminate any others
                   (pattern (first)))
                  ((first {^  elements car})      ;; more 
                   (first given)
                   (rest ((cat list)
                          (elements {^ ^ elements cdr})
                          (elements given)))
                   (cset (first rest))  
                   (pattern (first rest))))))
           
           ;; ==============================================================
           )))))

