;;; -----------------------------------------------------------------------
;;; File:         htransitivity.l
;;; Description:  transitivity system form Hebrew verbs.
;;; Author:       Yael Netzer
;;; Created:      24 Jul 1996
;;; Modified:     
;;; Package:      FUG5
;;; -----------------------------------------------------------------------

;;; types needed here
(cfuf:define-feature-type 'simple-process '(event relation))
(cfuf:define-feature-type 'event '(material mental verbal))
(cfuf:define-feature-type 'mental '(perception thinking))
(cfuf:define-feature-type 'relation '(ascriptive possessive locative))
(cfuf:define-feature-type 'locative '(spatial temporal accompaniment existential natural-phenom))
;; Take structure of SURGE

(def-alt transitivity
  (((process-type simple-process)
    (proc ((type ((alt (material existential ascriptive))))))
    (:! simple-process))
   ((process-type composite))))

(def-alt simple-process
  (((process-type material)
   (participants ((fset (agent affected))))
   (:! material-simple))
   ((process-type relation)
    (proc ((lex ((alt ("hyh" given))))))
    (alt mode (((process ((type ascriptive)
                          (mode equative)))
                (partic ((fset (identified identifier)))))
               ((process ((type ascriptive)
                          (mode attributive)))
                (partic ((fset (carrier attribute)))))
               ((process ((type existential)))
                (partic ((located given)))
                (partic ((fset (located location)))))))
    (:! relational-simple))))

(def-alt material-simple
  (((process ((agentive yes)))
    (partic ((agent given)
             (agent ((cat np)))))
;                     (synt-function subject)))))
    
    (alt (((proc ((effective yes)))
           (partic ((affected ((cat np))))))
          ((proc ((effective no)))
           (partic ((affected none))))))
    (alt voice-material 
	 (((voice active)
           (partic ((agent ((synt-function subject)))))
	   (synt-roles ((subject {^2 partic agent})))
	   (opt (({^3 proc effective} yes)
		 (synt-roles ((object {^2 partic affected})))
		 (partic ((affected ((synt-function object))))))))
	  ((voice passive)))))
   ((process ((agentive no) (effective yes)))
    (partic ((agent none))))))


    

(def-alt relational-simple
  (((process-type ascriptive)
    (:! ascriptive-mode)
    (:! ascriptive-equative-process))
   ((process-type locative) ;; existential
    (:! locative-mode))))

(def-alt ascriptive-mode
  (((proc ((mode attributive)))
    (partic ((carrier given)
             (attribute given)))
    (synt-roles ((subject {^2 partic carrier})
		 (object {^2 partic attribute})))
    (synt-roles ((object ((gender {^2 subject gender})
			  (number {^2 subject number})))))
    (partic ((carrier ((cat np)  (synt-function subject)))
             (attribute ((cat ap) (definite no))))))
   ((proc ((mode equative)))
    (partic ((identifier given)
              (identified given)))
    (synt-roles ((subject {^2 partic identified})
		 (object {^2 partic identifier})))
    (partic ((identifier ((cat np)))
                             ;(synt-function object)))
                                     ;;; here I should watch out because it will add an 'et' prep.
	     (identified ((synt-function subject)
			  (cat np))))))))

(def-alt locative-mode
  (((proc ((type existential)))
    (partic ((located given)))
    (synt-roles ((subject {^2 partic located})))
    (partic ((located ((synt-function subject))))))
;             ({^2 proc v gender} {^ gender})
;             ({^2 proc v number} {^ number}))))
   ((proc ((type none))))))


(def-alt ascriptive-equative-process
   ;;; definiteness of the  process depends on the definiteness of the identifier.
   ;;; Usualy, if the process is definite, then there must be a copula.
   
   ;(proc ((definte {^2 partic identifier definite})))
   ;;; 
   ;(((partic ((identified ((definite no)))
   ;           (identifier ((definite yes)))))
   ;  ;(proc ((copula yes))))
   (((proc ((type ascriptive) (mode equative)))
     (partic ((identified
                ((alt (((cat pronoun)
                   ;;; here there is a distinction between definite and indefinite pronouns.
                   ;;; I did not take care of such yet
                   ;;; (indefinite: mishehu...)
                   ;;; Also: ze, ele, elu.. is also a copula, should take care.
                   ;;; also when predicate is more definite than the subject.
                     (alt (((cat personal-pronoun)
                            ({^2 proc copula} no))
                           ((cat demonstrative-pronoun)
                            ({^2 proc copula} yes)))))
                    ;; at ktsina -- check further
                    ((cat proper)
                     (alt ((({^ identifier definite} yes)
                            ({^2 proc copula} yes))
                         ;;; copula is possible also when predicate is much longer than subject
                           (({^3 proc copula} no)))))
                                        ; ({^3 proc copula} no) ;;orit ktsina
                    ((cat common)
                     (alt ((({^ identifier definite} yes)
                            ({^2 proc copula} no))

                           (({^ identifier definite} no)
                            (alt (((definite yes)        ;;;there should be a possibility here for (copula no)
                                   ({^2 proc copula} yes))
                                  ((definite no)         ;; I know its the same here but the second is obligatory
                                   ({^2 proc copula} yes))))) ;; and the former isn't.
                           
                           ((possessor given)
                            (possessor-style inflected)
                            ({^ identifier definite} no)
                            ({^2 proc copula} no)) ;; here its not definite but most common. so must be alternative.
                           ))))))))))
    ((proc ((mode attributive)))
     (partic ((carrier ((alt (((definite no)
                               ({^3 proc copula} yes)) ;;; not definite  |
                              (({^3 tense} past)
			       ({^3 proc copula} yes))
			      ((definite yes)          ;;;               |but most common
                               
			       ({^3 proc copula} no))  ;;; same          |
            ;;; NOW - there is a matter here of 'heaviness' of subject/predicate.
            ;;; when predicate is 'heavier' than subject, copula is rendered even if subject is indefinite
            ;;; and if definite subject is 'heavier' than predicate, copula is favoured.
            ;;  That is:  Hatsilim hem teimim vs. Hatsilim teimim yoter.
            ;;;           ha Hatsilim teimim vs. ha Hatsilim she bishalti etmol hem teimim. (?)
                              
            )))))))
    ((proc ((mode none))))))
    







