(require 'alist)
(require 'macro)

(define aget-assoc (predicate->asso equal?))
(define aget-value (alist-inquirer equal?))
(define aput-assoc (alist-associator equal?))

;; If you add a predicate modifier to the grammar, add its name here too so
;; that the binding module will know how to look for it in the total fd.
;; Assumption: all pred-modifs appear under the feature (pred-modif ...)
;; within clauses.
(define (get-pred-modifiers)
  '(score manner means instrument comparison matter direction distance
	  location path origin destination duration))

(define (_make-map-pred-modif sem-role)
  (let ((path (make-path '^ 'pred-modif sem-role)))
    (eval `(def-alt ,(string->symbol (string-append "map-" (symbol->string sem-role) "-pred-adjunct"))
             (((end-adverbial-4 given) (end-adverbial-5 ,path))
              ((end-adverbial-3 given) (end-adverbial-4 ,path))
              ((end-adverbial-2 given) (end-adverbial-3 ,path))
              ((end-adverbial-1 given) (end-adverbial-2 ,path))
              ((end-adverbial-1 ,path)))))))

(define (_make-pred-modif sem-role)
  (let* ((str-sem-role (symbol->string sem-role))
         (qrel-conj (string->symbol (string-append str-sem-role "-qrel")))
         (map-alt (string->symbol (string-append "map-" str-sem-role "-pred-adjunct")))
         (cat-alt (string->symbol (string-append "cat-" str-sem-role "-pred-adjunct"))))
    (eval `(def-alt ,(string->symbol (string-append str-sem-role "-pred-modif"))
             (:demo ,(string-append "Is there a " str-sem-role " predicate-modifier?"))
             (((pred-modif ((,sem-role none))))
              ((pred-modif ((,sem-role given)
                            (,sem-role ((:& ,qrel-conj)
                                        (synt-funct pred-adjunct)
                                        (:! ,cat-alt)))))
               (:! ,map-alt)))))))

(define (_make-map-circum sem-role synt-funct)
  (let* ((path-elts `(^ circum ,sem-role))
         (path (apply make-path path-elts)))
    (eval `(def-alt ,(string->symbol (string-append "map-" (symbol->string sem-role) "-" (symbol->string synt-funct)))
             (((circum ((,sem-role ((position given)))))
               (alt circum-position-given (:index (circum ,sem-role position))
                    (((circum ((,sem-role ((position front)
                                           (opt ((punctuation ((after ",")))))))))
                      (alt front-circum (((front-adverbial-1 given) (front-adverbial-2 ,path))
                                         ((front-adverbial-1 ,path)))))
                     ((circum ((,sem-role ((position end)
                                           (opt ((synt-funct #(under disjunct))
                                                 (punctuation ((before ",")))))))))
                      (alt end-circum (((end-adverbial-4 given) (end-adverbial-5 ,path))
                                       ((end-adverbial-3 given) (end-adverbial-4 ,path))	 
                                       ((end-adverbial-2 given) (end-adverbial-3 ,path))
                                       ((end-adverbial-1 given) (end-adverbial-2 ,path))
                                       ((end-adverbial-1 ,path)))))
                     ((circum ((,sem-role ((position header)
                                           (cat ((alt header-cat (address date))))))))
                      (alt header-circum (((headers ((1 given))) 
                                           (headers ((2 ,(apply make-path (cons '^ path-elts)))
                                                     (2 ((punctuation ((before ","))))))))
                                          ((headers ((1 ,(apply make-path (cons '^ path-elts))))))))))))
              ((alt circum-position-unspecified
                    (((end-adverbial-5 given)
                      (front-adverbial-2 ,path)
                      (opt ((circum ((,sem-role ((punctuation ((after ","))))))))))
                     ((end-adverbial-4 given) 
                      (front-adverbial-1 given)
                      (end-adverbial-5 ,path)
                      (opt ((circum ((,sem-role ((synt-funct #(under disjunct))
                                                 (punctuation ((before ","))))))))))
                     ((end-adverbial-4 given) 
                      (front-adverbial-1 ,path)
                      (opt ((circum ((,sem-role ((punctuation ((after ","))))))))))
                     ((end-adverbial-3 given) 
                      (front-adverbial-1 given)	     
                      (opt ((circum ((,sem-role ((synt-funct #(under disjunct))
                                                 (punctuation ((before ",")))))))))
                      (end-adverbial-4 ,path))
                     ((end-adverbial-3 given) 
                      (front-adverbial-1 ,path)
                      (opt ((circum ((,sem-role ((punctuation ((after ","))))))))))
                     ((end-adverbial-2 given) 
                      (front-adverbial-1 given)	     
                      (opt ((circum ((,sem-role ((synt-funct #(under disjunct))
                                                 (punctuation ((before ",")))))))))
                      (end-adverbial-3 ,path))
                     ((end-adverbial-2 given) 
                      (front-adverbial-1 ,path)
                      (opt ((circum ((,sem-role ((punctuation ((after ","))))))))))
                     ((end-adverbial-1 given) 
                      (front-adverbial-1 given) 
                      (opt ((circum ((,sem-role ((synt-funct #(under disjunct))
                                                 (punctuation ((before ",")))))))))
                      (end-adverbial-2 ,path))
                     ((end-adverbial-1 given) 
                      (front-adverbial-1 ,path)
                      (opt ((circum ((,sem-role ((punctuation ((after ","))))))))))
                     ((end-adverbial-1 ,path)
                      (opt ((circum ((,sem-role ((synt-funct #(under disjunct))
                                                 (punctuation ((before ","))))))))))))))))))


(define (_make-sent-adjunct sem-role)
  (let* ((str-sem-role (symbol->string sem-role))
         (qrel-conj (string->symbol (string-append str-sem-role "-qrel")))
         (map-alt (string->symbol (string-append "map-" str-sem-role "-sent-adjunct")))
         (cat-alt (string->symbol (string-append "cat-" str-sem-role "-sent-adjunct"))))
    (eval `(def-alt ,(string->symbol (string-append str-sem-role "-circum"))
             (:demo ,(string-append "Is there a " str-sem-role " circumstantial?"))
             (((circum ((,sem-role none))))
              ((circum ((,sem-role given)
                        (,sem-role ((:& ,qrel-conj)
                                    (synt-funct sent-adjunct)
                                    (:! ,cat-alt)))))
               (:! ,map-alt)))))))

(define (_make-disjunct sem-role)
  (let* ((str-sem-role (symbol->string sem-role))
         (qrel-conj (string->symbol (string-append str-sem-role "-qrel")))
         (map-alt (string->symbol (string-append "map-" str-sem-role "-disjunct")))
         (cat-alt (string->symbol (string-append "cat-" str-sem-role "-disjunct"))))
    (eval `(def-alt ,(string->symbol (string-append str-sem-role "-circum"))
             (:demo ,(string-append "Is there a " str-sem-role " circumstantial?"))
             (((circum ((,sem-role none))))
              ((circum ((,sem-role given)
                        (,sem-role ((synt-funct disjunct)
                                    (:! ,cat-alt)))))
               (:! ,map-alt)))))))

(define (_make-final-circum sem-role)
  (let* ((str-sem-role (symbol->string sem-role))
         (path (make-path '^ 'circum sem-role))
         (qrel-conj (string->symbol (string-append str-sem-role "-qrel")))
         (map-alt (string->symbol (string-append "map-" str-sem-role "-disjunct")))
         (cat-alt (string->symbol (string-append "cat-" str-sem-role "-disjunct"))))
    (eval `(def-alt ,(string->symbol (string-append str-sem-role "-circum"))
             (:demo ,(string-append "Is there a " str-sem-role " circumstantial?"))
             (((circum ((,sem-role none))))
              ((circum ((,sem-role given)
                        (,sem-role ((synt-funct disjunct)
                                    (opt ((punctuation ((before ",")))))
                                    (:! ,cat-alt)))))
               (final-adverbial ,path)))))))

;; Macro to define stereotypical conjs grouping the features concerning
;; questions and relatives
(define (_make-qrel sem-role keys)
  (let ((key-alist (list (cons ':q-prep 'none)
                         (cons ':q-pron 'none)
                         (cons ':rel-mark 'none)
                         (cons ':q-embedded 'yes)
                         (cons ':rel-embedded 'yes))))
    (letrec ((eval-keys (lambda (keys)
                          (if (null? keys)
                              'done
                              (begin
                               (aput-assoc key-alist (car keys) (cadr keys))
                               (eval-keys (cddr keys)))))))
        (eval-keys keys))
    (define q-prep (aget-value key-alist ':q-prep))
    (define q-pron (aget-value key-alist ':q-pron))
    (define rel-mark (aget-value key-alist ':rel-mark))
    (define q-embedded (aget-value key-alist ':q-embedded))
    (define rel-embedded (aget-value key-alist ':rel-embedded))
    (eval `(def-conj ,(string->symbol (string-append (symbol->string sem-role) "-qrel"))
             (question-prep ((lex ((alt (given ,q-prep))))))
             (question-pronoun ((lex ((alt (given ,q-pron))))))
             (relative-marker ((lex ((alt (given ,rel-mark))))))
             (question-embedded ((alt (given ,q-embedded))))
             (relative-embedded ((alt (given ,rel-embedded))))))))

(defmacro make-map-pred-modif (sem-role) `(_make-map-pred-modif ',sem-role))
(defmacro make-pred-modif (sem-role) `(_make-pred-modif ',sem-role))
(defmacro make-map-circum (sem-role synt-funct) `(_make-map-circum ',sem-role ',synt-funct))
(defmacro make-sent-adjunct (sem-role) `(_make-sent-adjunct ',sem-role))
(defmacro make-disjunct (sem-role) `(_make-disjunct ',sem-role))
(defmacro make-final-circum (sem-role) `(_make-final-circum ',sem-role))
(defmacro make-qrel (sem-role . keys) `(_make-qrel ',sem-role ',keys))
