;; yael 1/2/99
;; ========================================================

(define (dispatch-morph fd)
 ;; first must check availability of features:
 ;; - binding
 ;; - cat
 ;; - lex
  (let ((binding (get-binding (cfuf:find-child fd 'binding)))
	(cat  (get-cat (cfuf:find-child fd 'cat)))
	(lex  (get-lex (cfuf:find-child fd 'lex))))
    (cond (binding
	   (morph-binding (get-attributes fd 'binding)))
	  ((morphed-cat? cat) 
	   (morph-cat (get-morph-function cat) (get-attributes fd cat)))
	  ((unmorphed-cat? cat)
	    lex)
	  (else 
	   (format nil "<unknown cat ~s: ~a>" cat lex)))))


;;========================================================
;; Either returns a cat or #f if not normal.

(define *normal-cats* '(adj adv conj modal prep relpro punctuation
				       phrase noun verb pronoun article ordinal
				       cardinal fraction))

;;========================================================
;; selectors
;;========================================================
(define (get-cat cat)
  (normalize-cat (get-type cat)))

(define (normalize-cat cat)
  (let ((normal (member-if (lambda (c) (subsume c cat)) *normal-cats*)))
    (unless (null? normal) (car normal))))

(define (get-lex lex)
  (if (empty-fd? lex) #f
      (let ((slex (get-type lex)))
	(if (string? slex) slex
	    (string-downcase (format nil "~s" slex))))))

(define (get-binding binding)
  (and (not (empty-fd? binding))
      (subsume (get-type binding) 'pro)))

;;========================================================
(define *morphed-cats* '(noun verb adj pronoun article det ordinal cardinal fraction))

(define (morphed-cat?  cat)
  (member cat *morphed-cats*))

(define (unmorphed-cat? cat) cat)

;;==========================================================
;; this can be done like the lexicon.
;; 
(define (morph-cat function attributes)
  (apply function attributes))

;; which function to each cat
(define (get-morph-function cat)
  (case cat
    ((noun) morph-noun)
    ((verb) morph-verb)
    ((adj) morph-adj)
    ((pronoun) morph-pronoun)
    ((article det) morph-determiner)
    ((ordinal cardinal) morph-numeric)
    ((fraction) morph-fraction)
    (else dont-morph)))

;; which args for which morph
(define (get-attributes fd cat)
  (case cat
    ((noun)
     (get-types-of '(lex number a-an feature) fd))
    ((verb)
     (get-types-of '(lex ending number person tense) fd))
    ((adj) 
     (get-types-of '(lex superlative comparative inflected) fd))
    ((pronoun)
     (get-types-of '(lex pronoun-type case gender number distance animate
			 person restrictive) fd))
    ((det article)
     (get-types-of '(lex number) fd))
    ((ordinal cardinal)
     (get-types-of '(lex cat value digit) fd))
    ((fraction)
     (get-types-of '(lex num den digit) fd))))
    
;; this is needed for default checking.
(define (get-types-of feature-list in-fd)
       (map (lambda (type) (get-default type (cfuf:get-type (cfuf:find-child in-fd type))))
	    feature-list))

;; use new dispatch method for default (don't use check-... procedures)
(define (get-types-of feature-list in-fd)
       (map (lambda (type) (check-or-get-default type (cfuf:get-type (cfuf:find-child in-fd type))))
	    feature-list))

;;***check it from here!
;; 
;; make a macro that gets a list of morph-functions and arguments+defualts, and builds the following procedures.
;; then make a procedure like - init linearizer.

;;=======================================================================



