;;; -*- Mode:Lisp; Syntax:Common-Lisp; Package:FUG5 -*-
;;; -----------------------------------------------------------------------
;;; File:         LEXICON.SCM
;;; Description:  Lexicon for FUG5
;;; Author:       Jay Meyer 
;;; Created:      16-Mar-87
;;; Modified:     01-Jul-88 (Michael Elhadad)
;;;               30 Apr 90 - moved exports from here to fug5
;;; Package:      FUG5
;;; Status:       Experimental
;;; -----------------------------------------------------------------------
;;;
;;; FUF - a functional unification-based text generation system. (Ver. 5.2)
;;;  
;;; Copyright (c) 19{87-93} by Michael Elhadad. all rights reserved.
;;;  
;;; Permission to use, copy, and/or distribute for any purpose and
;;; without fee is hereby granted, provided that both the above copyright
;;; notice and this permission notice appear in all copies and derived works.
;;; Fees for distribution or use of this software or derived works may only
;;; be charged with express written permission of the copyright holder.
;;; THIS SOFTWARE IS PROVIDED ``AS IS'' WITHOUT EXPRESS OR IMPLIED WARRANTY.
;;; -----------------------------------------------------------------------
(require 'format)
(require 'common-list-functions)
;(in-package "FUG5")
(format t "Lexicon...~%")

;; -----------------------------------------------------------------------
;; Lexicon - JMyers comments as of 3/16/87
;; The lexicon contains only irregular plurals and irregular verb conjugations
;; [and pronouns - experimental]. It is accessed during the process of 
;; linearization.  Entries are represented as strings to control capitalization
;; and to allow the use of phrases.  There is no particular rationale for the
;; entries listed here and the lexicon should be extended as needed. 
;; -----------------------------------------------------------------------

;; ------------------------
;; hash table procedures yd
;; ------------
(require 'hash-table)


(define hget-assoc (predicate->hash-asso equal?))
(define hget-value (hash-inquirer equal?))
(define hrem-assoc (hash-remover equal?))
(define hput-assoc (hash-associator equal?))



(define (gethash key table) (or (hget-value table key) '()))

(define (first list) (list-ref list 0))
(define (second list) (list-ref list 1))
(define (third list)  (list-ref list 2))
(define (fourth list) (list-ref list 3))
(define (fifth list)  (list-ref list 4))
(define (sixth list) (list-ref list 5))




;; ------------------------------------------------------------
;; GLOBAL VARIABLES
;; ------------------------------------------------------------

(define *dictionary* nil) ;"The lexicon")
(define *irreg-plurals* nil) 
;  "List of singular-form paired with plural-form") 
(define *irreg-verbs* nil) 
;  "List of root verb paired with list of present-third-person-singular,
;  past, present-participle and past-participle forms")
(define *pronouns-person* nil)
(define *pronouns-number* nil)
(define *pronouns-gender* nil)
(define *pronouns-case* nil)
(define *irreg-super-comparatives* nil)

(set! *irreg-plurals*
      ;; (key plural)
      ;; some pronouns are included to prevent pluralization
      '(("calf" "calves")
        ("child" "children")
        ("clothes" "clothes")
        ("data" "data")
        ("deer" "deer")
        ("die" "dice")
        ("elf" "elves") 
        ("glasses" "glasses")
        ("goods" "goods")
        ("goose" "geese")
        ("half" "halves") 
        ("knife" "knives")
        ("leaf" "leaves")
        ("life" "lives")
        ("loaf" "loaves")
        ("man" "men")
        ("many" "many") 
        ("mouse" "mice")
        ("oats" "oats")
        ("ox" "oxen")
        ("pants" "pants")
        ("person" "people")
        ("pliers" "pliers")
        ("scissors" "scissors")
        ("self" "selves")
        ("sheaf" "sheaves")
        ("sheep" "sheep")
        ("some" "some")
        ("thanks" "thanks")
        ("that" "that")
        ("them" "them")
        ("there" "there")
        ("these" "these")
        ("they" "they")
        ("thief" "thieves")
        ("those" "those")
        ("tongs" "tongs")
        ("trousers" "trousers")
        ("trout" "trout") 
        ("which" "which")
        ("wife" "wives")
        ("woman" "women")))

;; (sort *irreg-plurals* #'string-lessp :key #'car) 

;; The irregular conjugation of "to be" is handled by a function 
;; in the linearizer (morph-be).

(set! *irreg-verbs*
;; (root present-third-person-singular past present-participle past-participle)
      '(("become" "becomes" "became" "becoming" "become")
        ("buy" "buys" "bought" "buying" "bought")
        ("come" "comes" "came" "coming" "come")
        ("do" "does" "did" "doing" "done")
        ("eat" "eats" "ate" "eating" "eaten")
        ("fall" "falls" "fell" "falling" "fallen")
        ("get" "gets" "got" "getting" "gotten")
        ("give" "gives" "gave" "giving" "given")
        ("go" "goes" "went" "going" "gone")
        ("grow" "grows" "grew" "growing" "grown")
        ("harbor" "harbors" "harbored" "harboring" "harbored")
        ("have" "has" "had" "having" "had")
        ("know" "knows" "knew" "knowing" "known")
        ("leave" "leaves" "left" "leaving" "left")
        ("make" "makes" "made" "making" "made")
        ("see" "sees" "saw" "seeing" "seen")
        ("take" "takes" "took" "taking" "taken")
        ("throw" "throws" "threw" "throwing" "thrown")
        ("undertake" "undertakes" "undertook" "undertaking" "undertaken")
        ("write" "writes" "wrote" "writing" "written")))

(set! *pronouns-person*
      ;;(key(third) first second)
      '(("he" "I" "you")))
(set! *pronouns-gender*
      ;;(key(masculine) feminine neuter)
      '(("he" "she" "it")))

(set! *pronouns-number*
      ;;(key(sing) plural)
      '(("I" "we")
        ("he" "they")
        ("she" "they")
        ("it" "they")))

(set! *pronouns-case*
;;(key(subjective) objective possessive reflexive)
      '(("I" "me" "mine" "myself")
        ("you" "you" "yours" "yourself")
        ("he" "him" "his" "himself")
        ("she" "her" "hers" "herself")
        ("it" "it" "its" "itself")
        ("we" "us" "ours" "ourselves")
        ("they" "them" "theirs" "themselves")))

(set! *irreg-super-comparatives*
;;(base comparative superlative)
      '(("good" "better" "best")
        ("bad" "worse" "worst")
        ("far" "further" "furthest")
        ("many" "more" "most")
        ("little" "less" "least")))


;; Lexfetch is the function called during linearization 
;; to retrieve information from the lexicon 
(define (lexfetch key property)
  ;"Retrieve information from the lexicon.
  ;Called during linearization.
  ;Properties can be: 
  ;verb : present-third-person-singular past present-participle past-participle
  ;noun : plural
  ;pronoun : subjective objective possessive reflexive."
  (let ((value (gethash key *dictionary*)))
    (if (not (null? value))
      (let ((kval (safe-assoc property value)))
        (if (null? kval) kval (cdr kval)))
      #f)))


(define (lexstore key property value)
  "Stores irregular lexical information into lexicon.
  For the possible properties cf lexfetch."
  (let ((oldvalue (lexfetch key property)))
       (cond ((and oldvalue (not (null? oldvalue))) (set-cdr! (safe-assoc property (gethash key *dictionary*)) value))
             (else (hput-assoc *dictionary* key
                               (acons property value (gethash key *dictionary*)))))))
 
(define (store-plurals list)
  (map (lambda (x) (lexstore (car x) 'plural (cadr x)))
        list))

(define (store-verbs list)
  (map (lambda (x) 
            (let ((key (car x)))
              (lexstore key 'present-third-person-singular (cadr x))
              (lexstore key 'past (caddr x))
              (lexstore key 'present-participle (cadddr x))
              (lexstore key 'past-participle (car (last x 1)))))
        list))


(define (store-pronouns)
  (map (lambda (x)
            (let ((key (car x)))
              (lexstore key 'first (cadr x))
             (lexstore key 'second (car (last x 1)))))
        *pronouns-person*)
  (store-plurals *pronouns-number*)
  (map (lambda (x)
            (let ((key (car x)))
              (lexstore key 'feminine (cadr x))
              (lexstore key 'neuter (car (last x 1)))))
        *pronouns-gender*)
  (map (lambda (x) 
            (let ((key (car x)))
              (lexstore key 'objective (cadr x))
              (lexstore key 'possessive (caddr x))
              (lexstore key 'reflexive (car (last x 1)))))
        *pronouns-case*))


;; YD
(define (store-adjectives list)
  (map (lambda (x)
            (let ((key (car x)))
              (lexstore key 'comparative (cadr x))
              (lexstore key 'superlative (caddr x))))
        list))



(define (initialize-lexicon size)
  (set! *dictionary* (make-hash-table (cond (size) (else '200))))
  (store-plurals *irreg-plurals*)
  (store-verbs *irreg-verbs*)
  (store-adjectives *irreg-super-comparatives*)
  (store-pronouns))




(initialize-lexicon 200)

  ;; Add an irregular verb to the lexicon with store-verbs
(store-verbs '( ("happen" "happens" "happened" "happening" "happened")) )
(store-verbs '( ("hurt" "hurts" "hurt" "hurting" "hurt")) )
(store-verbs '( ("line up" "lines up" "lined up" "lining up" "lined up")) )
(store-verbs '( ("win" "wins" "won" "winning" "won")) )
(store-verbs '(("build" "builds" "built" "building" "built")))
(store-verbs '(("come from" "comes from" "came from" "coming from" "came from")))
(store-verbs '(("frighten" "frightens" "frightened" "frightening" "frightened")))
(store-verbs '(("hold" "holds" "held" "holding" "held")))
(store-verbs '(("lose" "loses" "lost" "losing" "lost")))
(store-verbs '(("panic" "panics" "panicked" "panicking" "panicked")))
(store-verbs '(("run" "runs" "ran" "running" "ran")))
(store-verbs '(("slow down" "slows down" "slowed down" "slowing down" "slowed down")))
(store-verbs '(("suffer from" "suffers from" "suffered from" "suffering from" "suffered from")))
(store-verbs '(("trade for" "trades for" "traded for" "trading for" "traded for")))
(store-verbs '(("win" "wins" "won" "winning" "won")))
(store-verbs '(("work out" "works out" "worked out" "working out" "worked out")))
(store-plurals '(("percent" "percent")))



;; -----------------------------------------------------------------------
;(provide "$fug5/lexicon")
;; -----------------------