;; yael 1/2/99
;; =====================================================================
;; Linearizer
;; =====================================================================

(define (linearize fd)
  (let ((pattern (get-pattern fd))
        (punctuation (get-punctuation fd)))
    (cond (
	   ;; filter

           (filtered? fd)
;(format t "filtered? ~A~%" fd)
	   ;; do nothing
	   #f
	   )
	  ;; recurse - if pattern exists.
	  ((pattern? pattern)
;(format t "pattern? ~A~%" pattern)
	   (list-punctuation punctuation (recurse-on pattern)))
	  ;; act - morphology
	  (else
;(format t "else? ~A~%" fd)
	   (list-punctuation punctuation (act-on fd))))))

;; =====================================================================
;; selectors
;; =====================================================================
;; if FD, return #F if now pattern or the fd-list part of the pattern.
(define (get-pattern fd)
  (and (cfuf:fd? fd)
       (let ((pattern (cfuf:find-child fd 'pattern)))
	 (and (special? pattern)
	      (cfuf:get-type pattern)))))

(define (get-punctuation fd)
  (and (cfuf:fd? fd)
       (cfuf:find-child fd 'punctuation)))
    
;;=====================================================================
;; FILTERS:
;;=====================================================================
;; make a list *filters* of filter functions.

(define (filtered? fd)
  (some-pred *filters* fd))

(define (some-pred pred-list item)
  (cond ((null? pred-list) #f)
        (((car pred-list) item) #T)
        (else (some-pred (cdr pred-list) item))))

(define empty-fd? cfuf:empty-fd?)

(define (gapped-fd? fd)
  (eq? (cfuf:get-type (cfuf:find-child fd 'gap)) 'yes))

(define (not-fd? fd)
  (not (cfuf:fd? fd)))
 
(define (leaf-fd? fd)
  (and (not (special? fd))
       (not (internal? fd))
       (atom? (get-type fd))))

;;=====================================================================

(define (internal? fd)
  (eq? (get-type fd) '*internal*))

(define (pattern? pattern) pattern)


(define *filters* (list not-fd? empty-fd? gapped-fd? leaf-fd?))

;;=====================================================================
;; call for actions
;;=====================================================================

(define (recurse-on pattern)
  (map linearize (clean-pattern pattern)))

;; 11/2/99 add clean pattern to unmark mergable constituents.
;; 
(define (clean-pattern pattern)
  "Removes DOTS and POUND from pattern. (leaves only names of constituents)
   Then remove mergeable marks (* c) -> c"
  (remove-if (lambda (item) (member item '(dots pound))) 
	     (unmark-mergeable pattern)))

(define (unmark-mergeable pattern)
  (map (lambda (node)
	 (if (marked-mergeable? node) (car node) node))
          pattern)) 

(define (marked-mergeable? node) (list? node))

;; in file morphology.scm

(define (act-on fd)
  (dispatch-morph fd))

;;====================================================================
;; general CFUF utilities
;;====================================================================

(define (subsume t1 t2)
  (eq? (cfuf:get-glb t1 t2) t2))

;;====================================================================
;;call-linearizer
;;====================================================================
;; Scheme Lineraizer for CFUF                                               ;;
;;============================================================================
;; print a list of strings.

(define (print-sentence list)
  (do ((list list (cdr list))
       (string (make-string 0) (string-append string  (car list))))
      ((null? list) string)))

;;
(define (call-linearizer list)
  (if (leaf? list)
    (list "fail")
    (capitalize
     (punctuate (linearize list)
                (subsume 'interrogative (get-value list 'mood)))
     (get-value list 'capitalize))))

;;============================================================================
