(require 'cfuf)
(require 'object->string)
(require 'format)

(verbose 2)

(define *u-compiled-grammar* #f)

(define (eval-in-dir dir thunk)
  (let ((cwd (getcwd)))
    (chdir dir)
    (thunk)
    (chdir cwd)))

(define (get-env-var var-name)
  (let ((var-value (getenv var-name)))
    (if (not var-value)
        (error (string-append "The " var-value " environment variable must be set!")))
    var-value))

(defmacro assert (caller exp)
  `(if (not ,exp)
     (error (format #f "~s: assertion failed ~s" ,caller ',exp))))

(defmacro wrap (caller exp)
  `(let ((res '*))
     (format t "+++ ~s~%" ,caller)
     (set! res ,exp)
     (format t "--- ~s~%" ,caller)
     res))

(define make-path
  (lambda args
    (cond
     ((null? args) '{})
     ((not (pair? args)) (error "Invalid path specification -- " args))
     ((null? (cdr args)) (string->symbol (string-append "{" (object->string (car args)) "}")))
     (else (letrec ((loop (lambda (args acc)
                            (cond
                             ((null? args) (string->symbol acc))
                             ((null? (cdr args)) (string->symbol (string-append acc " " (object->string (car args)) "}")))
                             (else (loop (cdr args) (string-append acc " " (object->string (car args)))))))))
             (loop (cdr args) (string-append "{" (object->string (car args)))))))))

(define myprint (lambda args
                  (if (output-port? (car args))
                    (for-each (lambda (x)
                                (display x (car args))
                                (display #\space (car args))) (cdr args))
                    (for-each (lambda (x)
                                (display x)
                                (display #\space)) args))))

(define myprintln (lambda args (apply myprint args) (newline)))

(define (write-to-file cfd file-name how-to-print)
  (if (not (eq? how-to-print '*NONE*))
    (begin
      (case how-to-print
        ('*PPRINT-FD* (cfuf:file-pprint-fd cfd 'fd file-name))
        ('*FD-TO-LIST-PRINT* (call-with-output-file file-name
                               (lambda (port)
                                 (print port (list (string->symbol "define fd '") (cfuf:fd->list cfd))))))
        ('*FD-TO-LIST* (cfuf:fd->list cfd)))
      ;; (let ((copy (cfuf:copy-fd cfd ())))
      ;; (cfuf:file-pprint-fd copy 'fd (string-append file-name ".C"))
      ;; (let ((copy (cfuf:compile-fd ())))
      ;; (cfuf:paste-fd! copy '() cfd '())
      ;; (cfuf:file-pprint-fd copy 'fd (string-append file-name ".C")))
      'done)))

;; (define control-demo print)
(define control-demo (lambda args #t))
(define *input* #t)
