(require 'common-list-functions)
(require 'i/o-extensions)
(require 'macro)
(require 'hash-table)
(define hget-assoc (predicate->hash-asso equal?))
(define hget-value (hash-inquirer equal?))
(define hput-assoc (hash-associator equal?))
(require 'alist)
(define aget-assoc (predicate->asso equal?))
(define aget-value (alist-inquirer equal?))
(define aput-assoc (alist-associator equal?))

(define (get-env-var var-name)
  (let ((var-value (getenv var-name)))
    (if (not var-value)
        (error (string-append "The " var-value " environment variable must be set!")))
    var-value))

;; Grammar information--------------------------------------------------------
(require 'struct)
(define-record grinfo (cpath    ;; the grammar code path
                       cload    ;; the name of the code file to be loaded (will be run from cpath)
                       ipath    ;; the grammar inputs path
                       ilist    ;; the list of input files in ipath
                       lpath    ;; the grammar linearizer path
                       lload))  ;; the name of the file to be loaded in lpath
(define (make-grammar-info cpath cload
                           ipath ilist
                           lpath lload)
  (make-grinfo (get-env-var cpath) cload
               (get-env-var ipath) ilist
               (get-env-var lpath) lload))
;; ---------------------------------------------------------------------------

(define *cfuf-toplevel-path* (get-env-var "CFUF_TOPLEVEL_PATH"))
(define *surge-path* (get-env-var "SURGE_PATH"))
(define *cfuf-testsuit-path* (get-env-var "CFUF_TESTSUIT_PATH"))
(define *cfuf-common-path* (get-env-var "CFUF_COMMON_PATH"))

(define (load-cfuf)
  (let ((cwd (getcwd)))
    (chdir *cfuf-toplevel-path*)
    (load "base.scm")
    (load "refresh-fug.scm")
    (load "test.scm")
    (chdir cwd)))

(define (load-code grammar-info)
  (ASSERT 'load-code (grinfo->cpath grammar-info))
  (chdir (grinfo->cpath grammar-info))
  (load (grinfo->cload grammar-info)))

(define (load-types grammar-info)
  (ASSERT 'load-types (grinfo->cpath grammar-info))
  (chdir (grinfo->cpath grammar-info))
  (load "types.scm"))

(define (load-inputs grammar-info)
  (ASSERT 'load-inputs (grinfo->ipath grammar-info))
  (chdir (grinfo->ipath grammar-info))
  (for-each load (grinfo->ilist grammar-info)))

(define (load-linearizer grammar-info)
  (ASSERT 'load-linearizer (grinfo->lpath grammar-info))
  (chdir (grinfo->lpath grammar-info))
  (load (grinfo->lload grammar-info)))

(define (load-grammar grammar-info . flags)
  (let ((cwd (getcwd)))
    (if (member 'linearizer flags) (load-linearizer grammar-info))
    (if (member 'code flags) (load-code grammar-info))
    (if (member 'inputs flags) (load-inputs grammar-info))
    (if (member 'types flags) (load-types grammar-info))
    (chdir cwd)
    (if (member 'compile flags)
      (begin
        (format t ";Compiling *u-compiled-grammar* from *u-grammar* ...~%")
        (set! *u-compiled-grammar* (cfuf:compile-grammar *u-grammar*))
        (format t ";Done compiling *u-compiled-grammar*.~%")))
    'done))

(load-cfuf)

(define (load-surge-linearizer)
  (let ((cwd (getcwd)))
    (chdir *surge-path*)
    (load (string-append "linearizer" *path-separator* "root.scm"))
    (chdir cwd)))

(load-surge-linearizer)

