(define *hash-tests* (make-hash-table 1000))
(define *ordered-tests* ())
(define *test-count* 0)

(define *test-mgr*
  (let* ((test-wrapper (lambda (fd)
                         (let ((cfd fd))
                           (lambda ()
                             (if (not (cfuf:fd? cfd))
                                 (let ((tmp (cfuf:compile-fd cfd)))
                                   (if tmp (set! cfd tmp))))
                             cfd))))
         (methods (list (cons 'def-test
                              (lambda (name expected fd)
                                (let ((name+found (hget-assoc name *hash-tests*)))
                                  (if name+found
                                      (begin
                                       (format t "Redefining test ~s~%" name)
                                       (set-cdr! name+found (cons expected (test-wrapper fd))))
                                      (begin
                                       (set! *ordered-tests* (cons name *ordered-tests*))
                                       (hput-assoc *hash-tests* name (cons expected (test-wrapper fd)))
                                       (set! *test-count* (1+ *test-count*)))))))
                        (cons 'get-test
                              (lambda (name)
                                (let ((found (hget-value *hash-tests* name)))
                                  (if found (cfuf:copy-fd ((cdr found))) #f))))
                        (cons 'clear-tests
                              (lambda ()
                                (set! *hash-tests* (make-hash-table 1000))
                                (set! *ordered-tests* ())
                                (set! *test-count* 0))))))
    
    (lambda (key . args) (apply (aget-value methods key) args))))
       
(define (def-test-func name expected fd) (*test-mgr* 'def-test name expected fd))
(defmacro def-test (name expected fd) `(*test-mgr* 'def-test ',name ',expected ',fd))
(defmacro get-test (name) `(*test-mgr* 'get-test ,name))
(defmacro clear-tests () `(*test-mgr* 'clear-tests))

(define (extract-tests from to)
  (let ((l (member to *ordered-tests*)))
    (if (not l)
      (error "Invalid :to argument" to)
      (letrec ((loop (lambda (l acc)
                       (cond
                        ((null? l) (error "Invalid :from argument" from))
                        ((equal? (car l) from) (cons from acc))
                        (else (loop (cdr l) (cons (car l) acc)))))))
        (loop l ())))))

(define :test ':test)
(define :from ':from)
(define :to ':to)
(define :timed ':timed)
(define :item ':item)
(define :limit ':limit)
(define :fun ':fun)
(define :linearize ':linearize)

(define *u-compiled-grammar* #f)

(define test
  (let ((u-timer (cfuf:make-timer))
        (l-timer (cfuf:make-timer))
        (first-test (lambda () (if (null? *ordered-tests*) 0 (car (last *ordered-tests* 1)))))
        (last-test (lambda () (if (null? *ordered-tests*) 0 (car *ordered-tests*)))))
    (lambda args
      (cfuf:reset-timer u-timer)
      (cfuf:reset-timer l-timer)

      (let ((equal? equal?)
            (from (first-test))
            (to (last-test))
            (timed #f)
            (item ())
            (limit 2500)
            (fun (lambda (cfd name)
                   (let ((copy (cfuf:copy-fd cfd)))
                     (if (cfuf:unify-fd! copy *u-compiled-grammar*)
                         copy
                         #f))))
            (linearize #t)
            (total 0)
            (correct 0)
            (problems ())
            (oldlimit (cfuf:ulimit)))
        (let ((key-alist (list (cons :test (lambda (val) (set! equal? val)))
                               (cons :from (lambda (val) (set! from val)))
                               (cons :to (lambda (val) (set! to val)))
                               (cons :timed (lambda (val) (set! timed val)))
                               (cons :item (lambda (val) (set! item val)))
                               (cons :limit (lambda (val) (set! limit val)))
                               (cons :fun (lambda (val) (set! fun val)))
                               (cons :linearize (lambda (val) (set! linearize val)))
                             )))
          (letrec ((eval-keys (lambda (keys)
                                (if (null? keys)
                                    'done
                                    (begin
                                     ((aget-value key-alist (car keys)) (cadr keys))
                                     (eval-keys (cddr keys)))))))
                  (eval-keys args)))

        (cfuf:ulimit limit)

        (format t "~%====================~%~%")
        (for-each (lambda (name)
                    (let ((found (hget-value *hash-tests* name)))
                      (if (not found)
                        (format t "Test ~s does not exist.~%" name)
                        (let ((input ((cdr found)))
                              (expected (car found))
                              (result #f)
                              (printed "<fail>"))

                          (if linearize
                            (format t "~s --> ~s~%~%" name expected)
                            (format t "~s --> ~%~%" name))
                          
                          (if timed
                            (let ((u-elapsed #f)
                                  (l-elapsed '(0 . 0)))
                              (cfuf:go-timer u-timer)
                              (set! result (fun input name))
                              (cfuf:stop-timer u-timer)
                              (set! u-elapsed (cfuf:last-elapsed u-timer))
                              (if (and linearize result)
                                (begin
                                  (cfuf:go-timer l-timer)
                                  (set! printed (print-sentence (call-linearizer result)))
                                  (cfuf:stop-timer l-timer)
                                  (set! l-elapsed (cfuf:last-elapsed l-timer))))
                              (format t " real time ~,2f + ~,2f seconds~%" (car u-elapsed) (car l-elapsed))
                              (format t "  run time ~,2f + ~,2f seconds~%" (cdr u-elapsed) (cdr l-elapsed)))
                            (begin
                              (set! result (fun input name))
                              (if (and linearize result)
                                (set! printed (print-sentence (call-linearizer result))))))

                          (set! total (1+ total))

                          (format t "~d branches.~%" (cfuf:ucount))

                          (cond
                           ((not linearize)
                            (if result
                              (begin
                                (format t "OK~%")
                                (set! correct (1+ correct)))
                              (begin
                                (format t "Failed~%")
                                (set! problems (cons name problems))
                                (cfuf:forget-last-elapsed u-timer))))
                           ((and (not (pair? expected)) (equal? printed expected))
                            (format t "OK~%")
                            (set! correct (1+ correct)))
                           ((and (pair? expected)
                                 (pair? (member-if (lambda (elt) (equal? printed elt)) expected)))
                            (format t "~s~%OK~%" printed)
                            (set! correct (1+ correct)))
                           (else (set! problems (cons name problems))
                                 (cfuf:forget-last-elapsed u-timer)
                                 (if (pair? expected)
                                   (format t "Expected one of:~%~{         ~s~%~}" expected)
                                   (format t "Expected ~s~%" expected))
                                 (format t "Instead  ~s~%" printed)))
                          )))
                    (format t "====================~%~%")
                    (force-output))
                  (cond ((pair? item) item)
                        ((null? item) (extract-tests from to))
                        ((symbol? item) (list item))
                        (else (error "Invalid :item argument" item))))
        
        (cfuf:ulimit oldlimit)

        (if timed
          (let ((u-elapsed (cfuf:total-elapsed u-timer))
                (l-elapsed (cfuf:total-elapsed l-timer)))
            (format t "Time spent in unification of ~d \"good\" fds:\n" correct)
            (format t " real time ~,2f + ~,2f seconds, (~,2f + ~,2f in average)\n"
                    (car u-elapsed) (car l-elapsed) (/ (car u-elapsed) correct) (/ (car l-elapsed) correct))
            (format t "  run time ~,2f + ~,2f seconds, (~,2f + ~,2f in average)\n"
                    (cdr u-elapsed) (cdr l-elapsed) (/ (cdr u-elapsed) correct) (/ (cdr l-elapsed) correct))))
            
        (format t "~%~%~d test~:P run - ~d correct.~%" total correct)
            
        (if (pair? problems)
          (format t "The following tests are incorrect: ~s~%"  (nreverse problems)))
        
        'done))))
