#ifndef _COMPILE_HELPER_
#define _COMPILE_HELPER_

#include <fstream>

#ifdef WIN32
using std::ifstream;
using std::ofstream;
#endif

class CFDNode;
class CGrNode;

struct Helper
{
	struct CmdLineParam
	{
		enum {ADJACENT = 1,MUST = 2,INT = 4,DOUBLE = 8,STRING = 12,BOOL = 16};
		// Must be set prior the call
		char *szKey;
		int type;
		int nKeyLen;        // ParseCmdLine() will calculate if 0
		// Set by the ParseCmdLine()
		union Value
		{
			int *pInt;
			double *pDouble;
			char **pString;
			bool *pBool;
			Value()					{}
			Value(int *_pInt)       { pInt = _pInt; }
			Value(double *_pDouble) { pDouble = _pDouble; }
			Value(char **_pString)  { pString = _pString; }
			Value(bool *_pBool)     { pBool = _pBool; }
		};
		Value val;
		CmdLineParam(char *_szKey,int _type,int _nKeyLen,Value _val)
		{
			szKey = _szKey;
			type = _type;
			nKeyLen = _nKeyLen;
			val = _val;
		}
	};
	static int ParseCmdLine(int argc,char *argv[],CmdLineParam *pParams,int nParams,int nMinArgsLeft,int nMaxArgsLeft = 0xFFFF);

	static void SetSurgeTypes();
	static void SetTypes(char *types[]);

	enum {SILENT = 1,POSITION = 2};
	static token_t SkipDefine(uint count,uint flags = 0);

	// These are for dumping FDs
	static void DumpScm(const string& strFileName,CFDNode *pNode);
	static void DumpTcl(const string& strFileName,CFDNode *pNode);
	static void DumpDot(const string& strFileName,CFDNode *pNode);

	// These are for dumping Grammars
	static void DumpPlain(const string& strFileName,CGrNode *pNode);
	static void DumpTcl(const string& strFileName,CGrNode *pNode);
	static void DumpDot(const string& strFileName,CGrNode *pNode);
};

class CInputFileStream:public CInputStream,private ifstream
{
	int c;
public:
	explicit CInputFileStream(char *szSrcFileName):ifstream(szSrcFileName),c(EOF){}
	int Get()   { return c = ifstream::get(); }
	void Unget(){ if (c!=EOF){ putback(c);c=EOF; } }
	bool IsOK() { return good(); }
};

#endif
