#include "cfuf.hxx"
#include "common.hxx"

#include "p_parser.hxx"
#include "p_special.hxx"
#include "p_pattern.hxx"
#include "p_cset.hxx"
#include "p_fset.hxx"
#include "p_control.hxx"
#include <iomanip>

#ifdef WIN32
using std::cout;
using std::setw;
#endif

class CDummyGraph:public CFDBuilder
{
	int m_indent;
public:
	CDummyGraph():m_indent(0){}
	Fail *Initialize();
	Fail *Realize();
	Fail *EnterSubgraph(token_t edge);
	void EnterSubgraph(int upcount);
	void LeaveSubgraph();
	Fail *AddLeaf(token_t edge,const hCType& type)        { return AddFeature("AddLeaf",edge,type); }
	Fail *AddPath(token_t edge,const hCPath& path)        { return AddFeature("AddPath",edge,path); }
	Fail *AddLeaf(const hCType& type)                     { return AddFeature("AddLeaf","*NULL*",type); }
	Fail *AddPath(const hCPath& path)                     { return AddFeature("AddPath","*NULL*",path); }
	Fail *AddSpecial(token_t attr,const hCObject& obj)    { return AddFeature("AddSpecial",attr,obj); }
private:
	void indent();
	Fail *AddFeature(char *szCmdName,token_t attr,const hCObject& feat);
};

struct CmdLine
{
	static bool bSingle;
	static int nCount;
	static int nFrom;
};
bool CmdLine::bSingle = true;
int CmdLine::nCount = 0;
int CmdLine::nFrom = 1;

bool Run(int i = 0);

Helper::CmdLineParam param1("-single",Helper::CmdLineParam::BOOL,7,&CmdLine::bSingle);
Helper::CmdLineParam param2("-n",Helper::CmdLineParam::INT | Helper::CmdLineParam::ADJACENT,2,&CmdLine::nCount);
Helper::CmdLineParam param3("-from",Helper::CmdLineParam::INT | Helper::CmdLineParam::ADJACENT,5,&CmdLine::nFrom);

main(int argc,char *argv[])
{
	Helper::CmdLineParam params[3] = {param1,param2,param3};

	int nArgsDone = Helper::ParseCmdLine(argc,argv,params,sizeof(params)/sizeof(Helper::CmdLineParam),1,1);
	if (nArgsDone < 0)
	{
		cerr << "Usage: " << *argv << " [-single] [-n#] [-from#] filename\n";
		return 1;
	}

	if (CmdLine::nFrom > 0 && CmdLine::nCount > 0)
	{
		CmdLine::bSingle = false;
	}
    
	CFUF::Initialize();
	g_pTheInput = new CInputFileStream(argv[++nArgsDone]);
	if (!g_pTheInput->IsOK())
	{
		cerr << "File not found: " << argv[nArgsDone] << endl;
		return 1;
	}

	CDummyGraph fd;
	ParseFD::set_fd(&fd);

	SpecialFeatureParser<FsetParser,CFset,CDummyGraph> fset_parser;
	SpecialFeatureParser<CsetParser,CCset,CDummyGraph> cset_parser;
	SpecialFeatureParser<PatternParser,CPattern,CDummyGraph> pattern_parser;
	SpecialFeatureParser<ControlParser,CControl,CDummyGraph> control_parser;
	ParseFD::register_feature(Predefined::fset,&fset_parser);
	ParseFD::register_feature(Predefined::cset,&cset_parser);
	ParseFD::register_feature(Predefined::pattern,&pattern_parser);
	ParseFD::register_feature(Predefined::control,&control_parser);
  
	int exitCode = 0;
	try
	{
		if (CmdLine::bSingle)
		{
			Run();
		}
		else
		{
			Helper::SkipDefine(CmdLine::nFrom-1,Helper::SILENT);
			for (int i=CmdLine::nFrom;i<CmdLine::nCount+CmdLine::nFrom && Run(i);++i);
		}
	}
	catch(Fail *e)
	{
		cout << e << endl;
		delete e;
		exitCode = 1;
	}

	delete g_pTheInput;
	CFUF::Shutdown();

	return exitCode;
}
bool Run(int i)
{
	token_t tokFDName = Helper::SkipDefine(1,Helper::POSITION);
	if (!tokFDName)
	{
		return false;
	}

	if (i)
	{
		cout << "----[" << i << "] " << tokFDName << endl;
	}

	if (Fail *f = ParseFD::parse())
	{
		throw f;
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// CDummyGraph methods
Fail *CDummyGraph::Initialize()
{
	cout << "Initialize\n";
	m_indent+=2;
	return 0; 
}
Fail *CDummyGraph::Realize()
{ 
	m_indent-=2;
	cout << "Realize\n";
	return 0; 
}
Fail *CDummyGraph::EnterSubgraph(token_t edge)
{ 
	indent();
	cout << "EnterSubgraph " << edge << endl;
	m_indent+=2;
	return 0; 
}
void CDummyGraph::EnterSubgraph(int upcount)  
{
	indent();
	cout << "EnterSubgraph " << CPath(upcount) << endl;
	m_indent+=2; 
}
void CDummyGraph::LeaveSubgraph()  
{
	m_indent-=2;
	indent();
	cout << "LeaveSubgraph\n"; 
}
void CDummyGraph::indent()
{ 
	cout << setw(m_indent) << ""; 
}
Fail *CDummyGraph::AddFeature(char *szCmdName,token_t attr,const hCObject& feat)
{
	indent();
	cout << szCmdName << ' ' << attr << ' ' << feat << endl;
	return 0;
}
