#include "cfuf.hxx"
#include "common.hxx"

#include "b_debug_tracer.hxx"
#include "b_bkclass.hxx"
#include "b_altparams.hxx"
#include "p_altparams.hxx"

#ifdef WIN32
#include <iostream>
using std::cerr;
using std::cout;
#endif

class COStream:public COutputStream
{
	ostream& m_os;
public:
	explicit COStream(ostream& os):m_os(os){}
	COutputStream& operator<<(const string& str)    { m_os << str;return *this; }
	COutputStream& operator<<(const char *str)      { m_os << str;return *this; }
	COutputStream& operator<<(char c)               { m_os << c;return *this; }
	COutputStream& operator<<(long l)               { m_os << l;return *this; }
	COutputStream& operator<<(const CMyObject& obj) { m_os << obj.toString();return *this; }
	COutputStream& operator<<(const CMyObject *obj) { m_os << (obj ? obj->toString() : "NULL");return *this; }
	void flush()                                    { m_os.flush(); }
};

void register_default_params();

main(int argc,char *argv[])
{
	int nArgsDone = Helper::ParseCmdLine(argc,argv,0,0,1,1);
	if (nArgsDone < 0)
	{
		cerr << "Usage: " << *argv << " filename\n";
		return 1;
	}

	CFUF::Initialize();
	g_pTheInput = new CInputFileStream(argv[++nArgsDone]);
	if (!g_pTheInput->IsOK())
	{
		cerr << "File not found: " << argv[nArgsDone] << endl;
		return 1;
	}

	auto_ptr<COStream> erroutp(new COStream(cerr));
	g_pTheOutput = erroutp.get();

	register_default_params();
	token_t token;
	int exitCode = 0;
	try
	{
		CAltParams *alt_params = AltHeaderParser::parse_alt_header();
		cout << "----" << alt_params << "----" << endl;
		delete alt_params;
		while (g_pTheLexer->get(token)) cout << token << endl;
	}
	catch(Fail *e)
	{
		cerr << e << endl;
		delete e;
		exitCode = 1;
	}

	delete g_pTheInput;
	CFUF::Shutdown();

	return exitCode;
}
void register_default_params()
{
	BKClass::insert(Symbols::insert("a"),Symbols::insert("a"));
	BKClass::insert(Symbols::insert("b"),Symbols::insert("b"));
	BKClass::insert(Symbols::insert("c"),Symbols::insert("c"));
	static CIndexParser parse_index;
	static CBKClassParser parse_bkclass;
	static CDemoParser parse_demo;
	static CWaitParser parse_wait;
	AltHeaderParser::register_param(Predefined::index,&parse_index);
	AltHeaderParser::register_param(Predefined::bkclass,&parse_bkclass);
	AltHeaderParser::register_param(Predefined::demo,&parse_demo);
	AltHeaderParser::register_param(Predefined::wait,&parse_wait);
}
