#include "cfuf.hxx"
#include "common.hxx"

#include "p_cset.hxx"
#include "p_fset.hxx"
#include "p_pattern.hxx"
#include "p_control.hxx"

#ifdef WIN32
using std::cout;
#endif

struct CmdLine
{
	static bool bCset;
	static bool bFset;
	static bool bPattern;
	static bool bControl;
};
bool CmdLine::bCset;
bool CmdLine::bFset;
bool CmdLine::bPattern;
bool CmdLine::bControl;

Helper::CmdLineParam param1("-cset",Helper::CmdLineParam::BOOL,0,&CmdLine::bCset);
Helper::CmdLineParam param2("-fset",Helper::CmdLineParam::BOOL,0,&CmdLine::bFset);
Helper::CmdLineParam param3("-pattern",Helper::CmdLineParam::BOOL,0,&CmdLine::bPattern);
Helper::CmdLineParam param4("-control",Helper::CmdLineParam::BOOL,0,&CmdLine::bControl);

main(int argc,char *argv[])
{
	Helper::CmdLineParam params[4] = {param1,param2,param3,param4};
	
	int nArgsDone = Helper::ParseCmdLine(argc,argv,params,sizeof(params)/sizeof(Helper::CmdLineParam),1,1);
	if (nArgsDone < 0 || (CmdLine::bCset + CmdLine::bFset + CmdLine::bPattern + CmdLine::bControl) != 1)
	{
		cerr << "Usage: " << *argv << " {-cset | -fset | -pattern | -control} filename\n";
		return 1;
	}

	CFUF::Initialize();
	g_pTheInput = new CInputFileStream(argv[++nArgsDone]);
	if (!g_pTheInput->IsOK())
	{
		cerr << "File not found: " << argv[nArgsDone] << endl;
		return 1;
	}

	CsetParser cset_parser;
	FsetParser fset_parser;
	PatternParser pattern_parser;
	ControlParser control_parser;

	try
	{
		CMyObject *special = 0;
		if (CmdLine::bCset) special = cset_parser.parse_special();
		else if (CmdLine::bFset) special = fset_parser.parse_special();
		else if (CmdLine::bPattern) special = pattern_parser.parse_special();
		else if (CmdLine::bControl) special = control_parser.parse_special();
		cout << "----" << special << "----" << endl;
		delete special;
	}
	catch(Fail *e)
	{
		cout << e << endl;
		delete e;
	}

	delete g_pTheInput;
	CFUF::Shutdown();

	return 0;
}
