#ifndef _BASE_BITS_
#define _BASE_BITS_

#include "b_defs.hxx"
#include <memory.h>

class CBitMatrix{
  int *m_matrix;          // the matrix itself
  int m_nMatrixSize;      // its size (== height == width)
  int m_nIntRowSize;      // the size of a row in the sizeof(int) units
public:
  explicit CBitMatrix(int size):m_nMatrixSize(size),
    m_nIntRowSize(INTDIV(size)+(INTREM(size) ? 1 : 0)){
      m_matrix=new int[m_nMatrixSize*m_nIntRowSize];
      memset(m_matrix,0,m_nMatrixSize*m_nIntRowSize*sizeof(int));
  }
  ~CBitMatrix()                 { delete[] m_matrix; }
  int *begin(int row)           { return m_matrix+row*m_nIntRowSize; }
  int *end(int row)             { return begin(row)+m_nIntRowSize; }
  void set_bit(int x, int y)    { m_matrix[y*m_nIntRowSize+INTDIV(x)] |= (1 << INTREM(x)); }
  void clear_bit(int x, int y)  { m_matrix[y*m_nIntRowSize+INTDIV(x)] &= ~(1 << INTREM(x)); }
  void toggle_bit(int x, int y) { m_matrix[y*m_nIntRowSize+INTDIV(x)] ^= (1 << INTREM(x)); }
  int is_set(int x, int y)      { return m_matrix[y*m_nIntRowSize+INTDIV(x)] & (1 << INTREM(x)); }
  void transpose(int n=-1);
  int size_int()                { return m_nIntRowSize; }
  int size()                    { return m_nMatrixSize; }
  void print(int n=-1);
  void print_row(int row,int n=-1)      { print_row(begin(row),n); }
  void print_row(int *it,int n=-1);
};

#endif
