#include "stdcfuf.h"
#include "b_bkclass.hxx"

const int BKClass::capacity = sizeof(uint64) << 3;
uint BKClass::s_nclasses = 0;
BKClass::token2uint64_t BKClass::s_class2mask;
BKClass::token2uint64_t BKClass::s_feature2bkcode;
uint64 BKClass::s_fail_bk = 0;
CTokArray64 BKClass::s_bkclasses;

////////////////////////////////////////////////////////////////////////////////
// BKClass methods
//
void BKClass::reset()
{
  s_nclasses = 0;
  s_class2mask.erase(s_class2mask.begin(),s_class2mask.end());
  s_feature2bkcode.erase(s_feature2bkcode.begin(),s_feature2bkcode.end());
  s_fail_bk = 0;
  s_bkclasses.init();
}
bool BKClass::insert(token_t feature,token_t bkclass)
{
  pair<token2uint64_t::iterator,bool> res; // res.second is set true when a new key is inserted in token2uint64_t
  uint64 class_mask = 1 << s_nclasses;
  
  res = s_class2mask.insert(token2uint64_t::value_type(bkclass,class_mask));
  if (res.second)                       // true if bkclass has never been seen before
    if (class_mask){                    // if did not overflow the capacity (class_mask != 0) -
      s_bkclasses[s_nclasses] = bkclass;// save the bkclass in the reverse map (log2(class_mask) is the key, bkclass is the value)
      ++s_nclasses;                     // and update the class count
    }else{                              // else - there is no more space (s_nclasses == 64, ==> class_mask == 0)
      s_class2mask.erase(res.first);    // undo the insert operation
      return false;                     // return false to indicate failure
    }
  assert((res.first->second == class_mask) == res.second);
  class_mask = res.first->second;       // get the mask for the given bkclass
  res = s_feature2bkcode.insert(token2uint64_t::value_type(feature,class_mask));
  if (!res.second) res.first->second |= class_mask;     // the feature already exists - update its bkclass list manually
  return true;
}
uint64 BKClass::bkclass2mask(token_t bkclass)
{
  token2uint64_t::iterator found = s_class2mask.find(bkclass);
  return found == s_class2mask.end() ? 0 : found->second;
}
uint64 BKClass::feature2bkcode(token_t feature)
{
  token2uint64_t::iterator found = s_feature2bkcode.find(feature);
  return found == s_feature2bkcode.end() ? 0 : found->second;
}
void BKClass::bkcode2toklist(uint64 alt_bk,TokenList& toklist)
{
  for (uint log2_alt_bk = 0;alt_bk;alt_bk >>= 1,++log2_alt_bk)
    if (alt_bk & 1){
      assert(s_bkclasses[log2_alt_bk]);
      toklist.push_back(s_bkclasses[log2_alt_bk]);
    }
}
