#ifndef _BASE_CSET_
#define _BASE_CSET_

#include "b_objects.hxx"
#include <vector.h>

////////////////////////////////////////////////////////////////////////////////
// The following generic interface is both minimal and sufficient to implement
// the require toString() method.I did not want to replicate the code twice (it
// is not trivial) - once for CCset and the second time for CCsetType.
class GenCsetComponent
{
public:
	virtual ~GenCsetComponent(){}
	virtual uint size() const =0;
	virtual string str() const =0;
};

class GenCset
{
public:
	virtual ~GenCset(){}
	virtual const GenCsetComponent& abs() const =0;
	virtual const GenCsetComponent& base() const =0;
	virtual const GenCsetComponent& inc() const =0;
	virtual const GenCsetComponent& dec() const =0;
	string str() const;
private:
	bool unparse_part(const GenCsetComponent&,string&,char *,bool) const;
};
////////////////////////////////////////////////////////////////////////////////

class CCsetComponent:public GenCsetComponent,public vector<hCAddr>
{
public:
	uint size() const 
	{
		return vector<hCAddr>::size(); 
	}
	string str() const;
};

class CCset:public CMyObject,public GenCset
{
	friend class CCsetType;
	friend class CsetParser;
protected:
	CCsetComponent m_abs,m_base,m_inc,m_dec;
public:
	const GenCsetComponent& abs() const     { return m_abs; }
	const GenCsetComponent& base() const    { return m_base; }
	const GenCsetComponent& inc() const     { return m_inc; }
	const GenCsetComponent& dec() const		{ return m_dec; }
	string toString() const					{ return GenCset::str(); }
	static token_t feature()				{ return Predefined::cset; }
};

#ifndef _BASE_EXTFWD_
class CCsetType;
typedef Handle<CCset> hCCset;
typedef Handle<CCsetType> hCCsetType;
#endif

#endif
