#ifndef _BASE_CYQUE_
#define _BASE_CYQUE_

template<class T,class Ptr,class Ref>
struct __cyque_iterator{
  typedef __cyque_iterator self;
  
  __cyque_iterator(T *buf, unsigned int nCapacity, unsigned int iCur):m_buf(buf),m_nCapacity(nCapacity),m_iCur(iCur){}
  
  typedef T value_type;
  typedef Ptr pointer;
  typedef Ref reference;
  
  __cyque_iterator():m_buf(0),m_nCapacity(0),m_iCur(0){}
  
  self& operator++(){
    ++m_iCur;
    return *this;
  }
  
  operator bool() const { return m_buf; }
  bool operator!() const { return !m_buf; }
  
  bool operator==(const self& r) const { return m_buf == r.m_buf && m_iCur == r.m_iCur; }
  bool operator!=(const self& r) const { return !operator==(r); }
  
  reference operator*() const { return m_buf[m_iCur%m_nCapacity]; }
  pointer operator->() const  { return &(operator*()); }
  operator pointer() const { return &(operator*()); }
  
private:
  T *m_buf;
  unsigned int m_nCapacity;
  unsigned int m_iCur;
};

template<class T>
class cyque{
public:
  friend struct __cyque_iterator<T,T*,T&>;
  friend struct __cyque_iterator<T,const T*,const T&>;
  typedef __cyque_iterator<T,T*,T&> iterator;
  typedef __cyque_iterator<T,const T*,const T&> const_iterator;
public:
  cyque(unsigned int nCapacity):m_iStart(0),m_nCount(0),m_nCapacity(nCapacity),m_nBufSize(nCapacity){
    m_buf = new T[m_nBufSize];
  }
  ~cyque(){ delete[] m_buf; }
  iterator begin()              { return iterator(m_buf,m_nCapacity,m_iStart); }
  iterator end()                { return iterator(m_buf,m_nCapacity,m_iStart + m_nCount); }
  const_iterator begin() const  { return const_iterator(m_buf,m_nCapacity,m_iStart); }
  const_iterator end() const    { return const_iterator(m_buf,m_nCapacity,m_iStart + m_nCount); }
  T& front()                    { return m_buf[m_iStart]; }
  T& rear()                     { return m_buf[(m_iStart+m_nCount-1)%m_nCapacity]; }
  void pop_back(){
    --m_nCount;                                 // decrement the count
    (m_buf+(m_iStart+m_nCount)%m_nCapacity)->~T(); // delete the element (cyque owns it)
  }
  void pop_front(){
    (m_buf+m_iStart)->~T();                     // delete the element (cyque owns it)
    ++m_iStart;m_iStart%=m_nCapacity;           // advance the head pointer
    --m_nCount;                                 // decrement the count
  }
  void push_back(const T& x){
    if (full()) pop_front();                    // pop the front element if the cyclic queue is full
    m_buf[(m_iStart+m_nCount)%m_nCapacity]=x;   // store the new element
    ++m_nCount;                                 // increment the count
  }
  void push_front(const T& x){
    if (full()) pop_back();                     // pop the rear element if the cyclic queue is full
    m_iStart+=m_nCapacity-1;m_iStart%=m_nCapacity;      // update the first element pointer
    m_buf[m_iStart]=x;                          // store the new element
    ++m_nCount;                                 // increment the count
  }
  unsigned int capacity() const { return m_nCapacity; }
  unsigned int size() const     { return m_nCount; }
  bool full() const             { return m_nCount == m_nCapacity; }
  // The resize is quite primitive. It deletes all the elements and then changes the capacity of the cyclic queue
  void resize(unsigned int nCapacity){
    if (nCapacity > m_nBufSize){        // the new capacity exceeds the physical length of the buffer
      delete[] m_buf;                   // delete the old buffer (and all the elements)
      m_buf = new T[m_nBufSize = nCapacity];   // allocate a new buffer
    }else while (size()) pop_front();   // delete all the elements
    
    m_iStart = m_nCount = 0;            // the queue is empty
    m_nCapacity = nCapacity;
  }
  T& operator[](int i)          { i+=m_iStart;return m_buf[i%m_nCapacity]; }
private:
  T *m_buf;
  unsigned int m_iStart;
  unsigned int m_nCount;
  unsigned int m_nCapacity;
  unsigned int m_nBufSize;
};

#endif
