#include "stdcfuf.h"
#include <limits.h>
#include <assert.h>
#include "b_env.hxx"

#ifdef STATS_1
extern int clock_hit,clock_miss;
#endif

// The requested capacity of the clock is incremented to account for the difference
// between the first grammar branch utick and the first grammar branch count.
// The former starts with INPUT_TICK+1 (i.e. 2), while the latter with 1.
inline int INTCAP(tick_t n){ ++n;return INTDIV(ISIZE - INTREM(n) + n); }

const tick_t CClock::INVALID_TICK = 0;
const tick_t CClock::INPUT_TICK = 1;
const tick_t CClock::MAX_TICK = USHRT_MAX;

/////////////////////////////////////////////////////////////////////////////////
// CClock methods
//
CClock::CClock(tick_t capacity)
{
  m_bitmap = 0;
  if (!capacity) capacity = 2500;
  Reset(capacity);
}
void CClock::Reset(tick_t capacity)
{
  if (capacity){
    delete[] m_bitmap;
    m_intcapacity = INTCAP(capacity);
    m_bitmap=new int[m_intcapacity];
  }
  m_tick_count = m_itop = INPUT_TICK;
  memset(m_bitmap,0,sizeof(int)*m_intcapacity);
  SetBit(m_itop);
}
void CClock::Proceed()
{
  g_pTheEnv->m_tickCurrent = m_itop = ++m_tick_count;

  assert(INTDIV(m_itop) < m_intcapacity);

  SetBit(m_itop);
}
void CClock::Retreat()
{
  ClearBit(m_itop);
  do --m_itop;
  while (m_itop && !GetBit(m_itop));

  assert(m_itop);       // should never be INVALID_TICK (0)

  g_pTheEnv->m_tickCurrent = m_itop;
}
int CClock::IsCurrent(tick_t n) const
{
  assert(INTDIV(n) < m_intcapacity);
  
#ifndef STATS_1
  return GetBit(n);
#else
  int hit = GetBit(n);
  hit ? ++clock_hit : ++clock_miss;
  return hit;
#endif
}

/////////////////////////////////////////////////////////////////////////////////
// Env methods
//
Env::Env():m_none(new CNoneType),m_nil(new CNilType),m_any(new CAnyType),m_given(new CGivenType)
{
  m_internal[0] = new CInternalType;
  m_internal[1] = new CInternalType(Predefined::any);
  Init();
}
void Env::Init()
{
  m_nMaxTokenLen = 500;
  m_nErrLogLen = 100;
  m_nInitTypesCap = 4;
  m_nInitLinksCap = 2;
  m_bSplitPaths = true;
  m_bCorrectGiven = false;
  m_tickCurrent = CClock::INPUT_TICK;
  m_bInterrupted = false;
  m_nTraceLevel = 0;
  m_nMaxUCount = 2500;
  m_bInput = false;
  m_bUnifying = false;
}
void Env::SetUnifying(bool bUnifying)
{
  m_bUnifying=bUnifying;
  m_tickCurrent=CClock::INPUT_TICK;
}
int Env::SetMaxUCount(uint n)
{
  if (n > CClock::MAX_TICK - 32) n = CClock::MAX_TICK - 32;
  m_clock.Reset(n);
  return m_nMaxUCount = n;
}
