#ifndef _BASE_ENV_
#define _BASE_ENV_

typedef ushort tick_t;

class CClock{
  tick_t m_tick_count;
  int m_itop;
  int m_intcapacity;
  int *m_bitmap;
public:
  static const tick_t MAX_TICK;
  static const tick_t INVALID_TICK;
  static const tick_t INPUT_TICK;
  
  explicit CClock(tick_t capacity = 0);
  ~CClock(){ delete[] m_bitmap; }
  void Reset(tick_t capacity = 0);
  void Proceed();
  void Retreat();
  int IsCurrent(tick_t n) const;
private:
  inline int GetBit(tick_t n) const { return m_bitmap[INTDIV(n)] & (1 << INTREM(n)); }
  inline void SetBit(tick_t n)      { m_bitmap[INTDIV(n)] |= 1 << INTREM(n); }
  inline void ClearBit(tick_t n)    { m_bitmap[INTDIV(n)] &= ~(1 << INTREM(n)); }
};

////////////////////////////////////////////////////////////////////////////////
// Object of this class serves as the reference of nodes being on the current path.
// Only CFDWalk has read-write access to the class.All the others  - read-only.
// Should never be consulted during operations which might change the structure of  
// the current fd (as only node ids and not the nodes itselves are stored here).
//
class CFDWalk;
class CCurLocRef:private vector<int>{
public:
  typedef const_reverse_iterator LocIter;
  
  uint GetSize() const  { return vector<int>::size(); }
  LocIter Begin() const { return rbegin(); }
  
  void Push(int id)     { push_back(id); }
  void Pop()            { pop_back(); }
  void Reset()          { erase(vector<int>::begin(),vector<int>::end()); }
};

class CType;
typedef Handle<CType> hCType;

struct Env{
  int m_nMaxTokenLen;   // for lexer - maximum length of a token
  int m_nErrLogLen;     // for lexer - length of the error log
  int m_nInitTypesCap;  // initial type capacity of an fd node
  int m_nInitLinksCap;  // initial link capacity of an fd node
  bool m_bSplitPaths;   // for dump - controls splitting of paths ending with a final node
  bool m_bCorrectGiven; // controls GIVEN attribute interpretation
  tick_t m_tickCurrent; // current unification step id (clock tick)
  CClock m_clock;       // the unification clock
  CCurLocRef m_curLocRef;
  bool m_bInterrupted;  // may be set by Ctrl+C handler when Ctrl+C is pressed
  int m_nTraceLevel;    // current trace level (defaults to 0, i.e. none)
  
  Env();
  void Init();
  void SetUnifying(bool bUnifying);
  bool IsInput() const                          { return m_bInput && !m_bUnifying; }
  const hCType& none() const                    { return m_none; }
  const hCType& nil() const                     { return m_nil; }
  const hCType& any() const                     { return m_any; }
  const hCType& given() const                   { return m_given; }
  const hCType& internal(uint i) const          { return m_internal[i]; }
  const hCType& internal(token_t title) const   { return m_internal[title == Predefined::any]; }

  int GetMaxUCount() const { return m_nMaxUCount; }
  int SetMaxUCount(uint n);
private:
  friend struct ParseFD;        // must have write access to m_bInput
  
  int m_nMaxUCount;             // maximal count of branches to visit during unification - the failure threshold
  bool m_bInput;
  bool m_bUnifying;
  hCType m_none;
  hCType m_nil;
  hCType m_any;
  hCType m_given;
  hCType m_internal[2];
};

#endif
