#include "stdcfuf.h"
#include "b_lexer.hxx"
#include <assert.h>
#include <cctype>
#include <string.h>

class TokenFail:public Fail{
  string m_str;
public:
  explicit TokenFail(const char *buf):m_str(buf){}     // save the guilty string
  string GetDesc() const;
};

token_t Delim::single_quote;
token_t Delim::double_quote;
token_t Delim::ws;
token_t Delim::open_paren;
token_t Delim::close_paren;
token_t Delim::open_curly;
token_t Delim::close_curly;
token_t Delim::tilde;
token_t Delim::colon;
token_t Delim::bar;
token_t Delim::at;
token_t Delim::percent;
token_t Delim::roof;
token_t Delim::star;
int Delim::s_is_delim[INTDIV(256)];
token_t Delim::s_tokens[256];
/////////////////////////////////////////////////////////////////////////////////
// Delim methods
//
void Delim::init()
{
  char delim_str[2]={0,0};
  char *ptr="\'\" (){}~#@%^*:;\t\n";

  memset(s_is_delim,0,sizeof(s_is_delim));
  memset(s_tokens,0,sizeof(s_tokens));
  for (int i=0;*ptr;ptr++,i++){
    delim_str[0]=*ptr;
    s_is_delim[INTDIV(*ptr)] |= (1<<INTREM(*ptr));
    s_tokens[*ptr]=Symbols::insert(delim_str);
  }
  single_quote  =s_tokens['\''];
  double_quote  =s_tokens['"'];
  ws            =s_tokens[' '];
  open_paren    =s_tokens['('];
  close_paren   =s_tokens[')'];
  open_curly    =s_tokens['{'];
  close_curly   =s_tokens['}'];
  tilde         =s_tokens['~'];
  bar           =s_tokens['#'];
  at            =s_tokens['@'];
  percent       =s_tokens['%'];
  roof          =s_tokens['^'];
  star          =s_tokens['*'];
  colon         =s_tokens[':'];
}
bool Delim::delim2token(int c,token_t& token)
{
  assert(c<256);
  token=s_tokens[c];
  return is_delim(c);
}
bool Delim::is_delim(token_t token)
{
  char *symbol=TOK2STR(token);
  return *symbol && symbol[1]=='\0' && is_delim(*symbol);
}
bool Delim::is_delim(int c)
{
  return s_is_delim[INTDIV(c)] & (1 << INTREM(c));
}

/////////////////////////////////////////////////////////////////////////////////
// CLexer::CErrLog methods
//
string CLexer::CErrLog::get() const
{
  string str;
  str.reserve(m_chars.size()+1);
  cyque<char>::const_iterator it = m_chars.begin();
  for (uint i = m_chars.size();i;--i,++it) str += *it;
  return str;
}

/////////////////////////////////////////////////////////////////////////////////
// CLexer::CBuffer methods
//
void CLexer::CBuffer::reset()
{
  if (m_len < g_pTheEnv->m_nMaxTokenLen+2){
    m_len = g_pTheEnv->m_nMaxTokenLen+2;
    delete m_buf;
    m_buf=new char[m_len];
  }
}

bool CLexer::s_bFlushTokenPutbackQueue = true;
/////////////////////////////////////////////////////////////////////////////////
// CLexer methods
//
CLexer::CLexer()  { construct(); }
CLexer::~CLexer() { destroy(); }
void CLexer::destroy()
{
  if (s_bFlushTokenPutbackQueue)
    while (m_putback.size()) m_putback.pop_back();
}
void CLexer::construct()
{
  m_delim_next=false;
  m_is_string=false;
  m_buf.reset();
  m_errlog.reset();
}
bool CLexer::get(int& c)
{
  if (m_putback.size()){
    if (!m_putback_str){
      m_putback_str = TOK2STR(m_putback.back());
      m_putback_len = strlen(m_putback_str);
      m_putback_i=0;
    }
    c=m_putback_str[m_putback_i++];
    if (m_putback_i==m_putback_len){
      m_putback.pop_back();
      m_putback_str=NULL;
    }
  }else c = getc();
  if (c==EOF) return false;
  m_errlog.put(c);
  return true;
}
void CLexer::putback(token_t token)
{
  m_putback.push_back(token);
  m_putback_str=NULL;
}
bool CLexer::get(token_t& token) throw (Fail *)
{
  bool res = true;
  if (m_putback.size()){
    token=m_putback.back();
    m_putback.pop_back();
  }else res = m_delim_next ? get_delim(token) : get_token(token);
  return res;
}
bool CLexer::get_delim(token_t& token)
{
  if (!ok()) return false;

  int c,count=-1;
  do{
    ++count;
    if ((c = getc()) == ';') while ((c = getc()) != EOF && c!='\n');
  }while (isspace(c));
  if (count && c!=')' && c!='}'){     // one or more consecutive white spaces encountered
    ungetc();
    token=Delim::ws;
    if (c!=EOF) c = ' ';
  }else if (c!=EOF){
    Delim::delim2token(c,token);
    if (token == Delim::double_quote) m_is_string=!m_is_string;
  }
  if (c!=EOF) m_errlog.put(c);
  m_delim_next=false;
  return count || c!=EOF;
}
bool CLexer::get_token(token_t& token) throw (Fail *)
{
  if (!ok()) return false;
  
  int len = read();                             // read from file until a delimiter is met
  if (len > g_pTheEnv->m_nMaxTokenLen) throw new TokenFail(m_buf.c_str());
  if ((m_delim_next=(len > 0))){                // buf contains a non empty string
    token=Symbols::insert(m_buf.c_str());     // insert the string to the hash to get the token
    return true;
  }
  return get_delim(token);
}
int CLexer::read()
{
  int c,i;
  bool bEscaped;
  for (i=0;i<g_pTheEnv->m_nMaxTokenLen+1;i++){
    c = getc();
    if (bEscaped = m_is_string && (c=='\\')) c = getc();
    if (c==EOF) break;
    if (!bEscaped){
      if ((m_is_string && c=='"') || (!m_is_string && Delim::is_delim(c))){
        ungetc();
        break;
      }
      if (m_is_string && c=='~') while (isspace(c = getc()));
    }
    m_buf[i] = m_is_string ? c : tolower(c);
    m_errlog.put(m_buf[i]);
  }
  m_buf[i]='\0';
  return i;
}

/////////////////////////////////////////////////////////////////////////////////
// TokenFail methods
//
string TokenFail::GetDesc() const
{
  string str("String <");
  return str << m_str << "> exceeds the maximum token length (" << g_pTheEnv->m_nMaxTokenLen << ')';
}
