#include "stdcfuf.h"
#include "b_tokhier.hxx"
#include <function.h>

int bitwise_or(int x,int y)  { return x | y; }
int bitwise_and(int x,int y) { return x & y; }

class bits_inserter{
  CBitrow2token *m_phmBits;
  CBitMatrix *m_pBits;
public:
  bits_inserter(CBitrow2token *phmBits,CBitMatrix *pBits):m_phmBits(phmBits),m_pBits(pBits){}
  void operator()(const CToken2index::value_type& value){
    pair<CBitrow2token::iterator,bool> res=
      m_phmBits->insert(CBitrow2token::value_type(m_pBits->begin(value.second),value.first));
    assert(res.second);
  }
};

////////////////////////////////////////////////////////////////////////////////
// CTokenHier methods
//
CTokenHier::CTokenHier(int max) { construct(max); }
CTokenHier::~CTokenHier()       { destroy(); }
void CTokenHier::destroy()
{
  delete m_pBits;
  delete m_phmTokens;
  delete m_phmBits;
  delete[] m_result;
}
void CTokenHier::construct(int max)
{
  m_nMaxTokens=max;
  m_nTokensCount=0;
  m_bRealized=false;
  m_phmTokens=new CToken2index(max);
  m_pBits=new CBitMatrix(max);
  m_result=new int[m_pBits->size_int()];
  m_eqrows.m_pBits=m_pBits;
  m_hashrow.m_pBits=m_pBits;
  m_phmBits=new CBitrow2token(max,m_hashrow,m_eqrows);
}
Fail *CTokenHier::insert_pair(token_t parent,token_t child)
{
	// cout << "Inserting a pair (" << parent << ',' << child << ')';

	if (m_bRealized) return new TokenRealizedFail;

	ret::uint_t i = add_token(parent);
	if (i.fail) return i.fail;
	int par_irow=i.val;

	i = add_token(child);
	if (i.fail) return i.fail;
	int chi_irow=i.val;

	// cout << ",at rows (" << par_irow << ',' << chi_irow << ')' << endl;

	// update all the child's children (including itself)
	for (int j=0;j<m_nTokensCount;j++)
	{
		if (m_pBits->is_set(chi_irow,j))
			transform(m_pBits->begin(j),m_pBits->end(j),m_pBits->begin(par_irow),
					m_pBits->begin(j),ptr_fun(bitwise_or));
	}

	// cout << "----" << endl;
	// m_pBits->print(m_nTokensCount);

	// check for a cycle
	return equal(m_pBits->begin(par_irow),
			m_pBits->end(par_irow),
			m_pBits->begin(chi_irow)) ? new TokenCycleFail : 0;
}
ret::uint_t CTokenHier::add_token(token_t token)
{
	pair<CToken2index::iterator,bool> res;

	// Insert the parent token into the tokens hash map
	res=m_phmTokens->insert(CToken2index::value_type(token,-1));
	if (!res.second)                      // if it is already present (i.e. insertion failed)
	{
		return res.first->second;    // return the row index
	}
	else if (m_nTokensCount==m_nMaxTokens){// else, parent is a new token - check the load
		m_phmTokens->erase(res.first);    // remove the just added entry if overflown
		return new TokenTooManyFail;      // return an appropriate failure
	}
	int irow=m_nTokensCount;              // ok - can insert the parent
	m_pBits->set_bit(irow,irow);          // update the bit matrix
	res.first->second=irow;          // update the tokens hash map
	++m_nTokensCount;                     // advance the token count
	return irow;                          // return the row index
}
void CTokenHier::Realize()
{
  if (m_bRealized) return;
  // cout << "before transpose:" << endl;
  // m_pBits->print(m_nTokensCount);

  m_pBits->transpose(m_nTokensCount);

  // cout << "-----------------------" << endl <<"after transpose:" << endl;
  // m_pBits->print(m_nTokensCount);
  
  for_each(m_phmTokens->begin(),m_phmTokens->end(),bits_inserter(m_phmBits,m_pBits));
  m_bRealized=true;
}
ret::utoken_t CTokenHier::get_glb(token_t t1,token_t t2)
{
  if (t1 == t2) return t1;
  
  if (!m_bRealized) return new TokenNotYetFail;

  CToken2index::iterator it1,it2;
  it1=m_phmTokens->find(t1);
  it2=m_phmTokens->find(t2);
  if (it1==m_phmTokens->end() || it2==m_phmTokens->end()) return (token_t)0;

  // printf("%5s (%-3d): ",t1,it1->second);
  // m_pBits->print_row(it1->second);
  // printf("%5s (%-3d): ",t2,it2->second);
  // m_pBits->print_row(it2->second);

  transform(m_pBits->begin(it1->second),m_pBits->end(it1->second),
            m_pBits->begin(it2->second),m_result,ptr_fun(bitwise_and));

  // cout << "Result: ";
  // m_pBits->print_row(m_result);
  
  CBitrow2token::iterator glb_it = m_phmBits->find(m_result);       // find the result
  return (glb_it == m_phmBits->end()) ? token_t(0) : glb_it->second;
}
