#ifndef _COMPILER_ALTNODE_
#define _COMPILER_ALTNODE_

#include "c_grnode.hxx"
#include "c_curloc.hxx"
#include "b_altparams.hxx"

class CBranchNode;
class CBareGr;

class CAltNode:public CGrNode{
public:
  typedef EdgeList::const_iterator loc_iterator;
protected:
  EdgeList m_loc;     // the location of this alt relative to the current constituent (i.e. not an absolute address)
  CAltParams *m_params;
public:
  CAltNode(const CCurLoc& loc,CAltParams *params,CBareGr& gr):CGrNode(gr),m_loc(loc.value()),m_params(params){}
  ~CAltNode(){ delete m_params; }
  virtual void add_branch(CGrNode *branch){
    add_branch(branch,this);
  }
  CAltParams *params() const    { return m_params; }
  const EdgeList *location() const { return &m_loc; }
  string type() const           { return "alt"; }
  string edge_color() const     { return "red"; }
protected:
  void add_branch(CGrNode *branch,CAltNode *parent_alt);
};

class COptNode:public CAltNode{
public:
  COptNode(const CCurLoc& loc,CAltParams *params,CBareGr& gr):CAltNode(loc,params,gr){}
  void add_branch(CGrNode *branch){
    CAltNode::add_branch(branch,this);
  }
  string type() const           { return "opt"; }
};

class CRaltNode:public CAltNode{
public:
  CRaltNode(const CCurLoc& loc,CAltParams *params,CBareGr& gr):CAltNode(loc,params,gr){}
  void add_branch(CGrNode *branch){
    CAltNode::add_branch(branch,this);
  }
  CGrNode *first_child();
  string type() const           { return "ralt"; }
private:
  void shuffle_children();
};

inline CAltNode *node2alt(CGrNode *node)     { return cast<CAltNode>(node); }
inline COptNode *node2opt(CGrNode *node)     { return cast<COptNode>(node); }
inline CRaltNode *node2ralt(CGrNode *node)   { return cast<CRaltNode>(node); }

#endif
