#include "stdcfuf.h"

// #define F_TRACE

#include "b_debug_tracer.hxx"
#include "c_barefd.hxx"
#include "b_bkclass.hxx"

#ifdef WIN32
#pragma warning(disable : 4355)
#endif

class GivenFail2:public Fail{
public:
  GivenFail2(token_t _feature):feature(_feature){}
  string GetDesc() const {
    string str("Feature ");
    return str << feature << " must be given";
  }
private:
  token_t feature;
};

////////////////////////////////////////////////////////////////////////////////
// CBareFD::CInfoStack methods
//
CBareFD::CInfoStack::CInfoStack(CBareFD *owner):
  m_owner(owner),m_loc(new InfoStack),m_balance(0){}
CBareFD::CInfoStack::~CInfoStack()
{
  F_TRACE0();
  
  delete m_loc;
  while (m_undo_stack.size()){
    delete m_undo_stack.top().second;
    m_undo_stack.pop();
  }
}
void CBareFD::CInfoStack::reset()
{
  F_TRACE0();
  
  while (m_undo_stack.size()){
    delete m_undo_stack.top().second;
    m_undo_stack.pop();
  }
  pop_many(m_loc->size());
  m_balance=0;
}
struct pair_second:public unary_function<CBareFD::Info,token_t>{
  result_type operator()(const argument_type& x){ return x.second; }
};
void CBareFD::CInfoStack::get_elist(EdgeList& elist) const
{
  F_TRACE0();
  
  static pair_second op;
  transform(m_loc->begin()+1,m_loc->end(),back_inserter(elist),op);
}
Fail *CBareFD::CInfoStack::update(token_t feat)
{
  F_TRACE1(edge);
  
  m_undo_stack.push(Undo());
  ret::unode_t n = m_owner->EnterSubgraphProt(feat);
  if (n.fail) return n.fail;
  ++m_balance;
  return 0;
}
void CBareFD::CInfoStack::update(int upcount)
{
  F_TRACE1(path);
  
  if (upcount==CPath::ABS || upcount >= int(m_loc->size())){
    m_undo_stack.push(Undo(CPath::ABS,m_loc));
    CFDNode *base = m_owner->root();
    m_loc=new InfoStack;
    push(base,Predefined::null);
  }else{
    InfoStack *stack=new InfoStack(m_loc->end()-upcount,m_loc->end());
    m_undo_stack.push(Undo(CPath::CUR,stack));
    m_loc->erase(m_loc->end()-upcount,m_loc->end());
  }
  ++m_balance;
}
void CBareFD::CInfoStack::undo()
{
  F_TRACE0();
  
  Undo undo=m_undo_stack.top();
  m_undo_stack.pop();
  if (!undo.second) pop();
  else if (undo.first==CPath::ABS){
    pop_many(m_loc->size());
    delete m_loc;
    m_loc=undo.second;
  }else{
    assert(undo.first==CPath::CUR);
    copy(undo.second->begin(),undo.second->end(),back_inserter(*m_loc));
    delete undo.second;
  }
  --m_balance;
}
void CBareFD::CInfoStack::push(CFDNode *node,token_t title)
{
  F_TRACE2(node,title);
  
  m_loc->push_back(Info(node,title));
}
void CBareFD::CInfoStack::pop()
{
  F_TRACE0();
  
  m_loc->pop_back();
}

////////////////////////////////////////////////////////////////////////////////
// CBareFD::CAnyStack methods
//
CBareFD::CAnyStack::CAnyStack(CBareFD *fd):m_fd(fd){}
void CBareFD::CAnyStack::reset()
{
	F_TRACE0();

	while (size()) pop();
}
void CBareFD::CAnyStack::push(CFDNode *node)
{
	F_TRACE1(node);

	while (size() && !g_pTheEnv->m_clock.IsCurrent(BaseClass::top().first))
	{
		pop();
	}
	BaseClass::push(make_pair(g_pTheEnv->m_tickCurrent,node));
}
CFDNode *CBareFD::CAnyStack::top()
{
	F_TRACE0();

	while (size())
	{
		CFDNode *node = BaseClass::top().second;
		if (node->any())
		{
			assert(BKClass::fail_bk() == 0);              // assert that the failure log is clean
			for (CFDParentIterator p_it(node);!p_it.is_past_the_end();p_it.advance())   // iterate on the parents of the failed node
			{
				BKClass::add_fail_bk(p_it->first->title()); // acknowledge the failed parent edges
			}
			return node;
		}
		BaseClass::pop();                 // proceed to the next one
	}
	return NULL;
}

////////////////////////////////////////////////////////////////////////////////
// CBareFD methods
//
CBareFD::CBareFD():m_stack(this),m_root(NULL),m_count(0),m_any_stack(this){}
CBareFD::~CBareFD()
{
  F_TRACE0();

  delete m_root;
}
////////////////////////////////////////////////////////////////////////////////
// A call to reset must be followed by either Initialize() or destruction of the fd.
void CBareFD::reset()
{
  F_TRACE0();
  
  m_curType=hCType();  // delete the cached current type
  m_any_stack.reset();  // reset the stack of any nodes
  m_stack.reset();      // reset the node stack
  delete m_root;        // delete the graph nodes decrementing the node count along the way
  assert(m_count==0);
  m_root=NULL;
}
////////////////////////////////////////////////////////////////////////////////
Fail *CBareFD::Initialize()
{
  F_TRACE0();

  assert(m_stack.empty() && m_count==0);
  g_pTheEnv->SetUnifying(false);
  g_pTheEnv->m_clock.Reset();        // reset the clock
  m_root = new CFDNode(this,g_pTheEnv->internal(uint(0)));
  m_stack.push(m_root,Predefined::null);
  return 0;
}
Fail *CBareFD::Realize()
{
  F_TRACE0();

  // cout << m_stack.size() << ','
       // << (m_stack.topnode() == m_root) << ','
       // << m_stack.no_undo() << ','
       // << m_root->group_id() << endl;
  assert(m_stack.size() == 1 &&
         m_stack.topnode() == m_root &&
         m_stack.no_undo() &&
         m_root->group_id() == 0);
  return 0;
}
const EdgeList *CBareFD::cur_loc() const
{
  F_TRACE0();
  
  static EdgeList elist;
  elist.erase(elist.begin(),elist.end());
  m_stack.get_elist(elist);
  return &elist;
}
void CBareFD::GetCurNodeAndType() const
{
  F_TRACE0();

  m_curNode = m_stack.topnode();
  m_curType = m_curNode->data().GetType();
  assert(m_curType.pointee());
}
/////////////////////////////////////////////////////////////////////////////////
// Handle EnterSubgraph command
ret::ubool_t CBareFD::BeginEnterSubgraph(token_t edge)
{
  F_TRACE1(edge);
  
  if (g_pTheEnv->m_bInterrupted) throw new InterruptFail;
  
  GetCurNodeAndType();                        // get the current top node and type
  if (!m_curType->none()){             // if the current node is not none
    if (m_curType->internal())         // if the current node is internal then - 
      return false;                     // to be continued
    return new INFail(m_curType);      // else report internal-normal error
  }
  m_stack.push(m_curNode,edge);        // else - enter the same none node again
  return true;                          // no need to continue - the work is done
}
Fail *CBareFD::EndEnterSubgraph(token_t edge,token_t intitle)
{
  F_TRACE2(edge,intitle);
  
  CFDNode *dst_node = m_curNode->FindChild(edge);     // lookup a child connected to the current node by the <edge>
  if (dst_node){                                        // if found - must check whether it can be entered
    hCType dst_type = dst_node->data().GetType();      // get the child type
    assert(dst_type.pointee());                         // assert its validity
    if (!dst_type->final()){ // dst_type is allowed to be final, for the following fd to be ok - ((a 1) (a ((alt (1 2)))))
      // I have to pass the representative because only the representative has the correct input value of the node group
      ret::utype_t u = dst_type->unify(g_pTheEnv->internal(intitle),dst_node->representative());
      if (u.fail) return u.fail;
      dst_node->data().SetType(u.val);
    }
  }else dst_node = m_curNode->AddChild(edge,g_pTheEnv->internal(intitle));   // not found - add a new child
  m_stack.push(dst_node,edge);  // update the node stack
  return 0;
}
Fail *CBareFD::EnterSubgraphProt(token_t edge,token_t intitle)
{
  F_TRACE2(edge,intitle);

  Fail *f = 0;
  ret::ubool_t b = BeginEnterSubgraph(edge);
  if (b.fail) f = b.fail;
  else{
    if (b.val) return 0;
    f = EndEnterSubgraph(edge,intitle);
  }
  return f ? new FDBuildFail(f,"[enter subgraph]") : 0;
}
// End of EnterSubgraph section
/////////////////////////////////////////////////////////////////////////////////
// Handle AddLeaf command
// The slave type is guaranteed to be not internal.
Fail *CBareFD::UnifyTypes(const hCType& master,const hCType& slave,CFDNode *master_node)
{
  ret::utype_t u = master->unify(slave,master_node->representative());
  if (u.fail){
    int retry = (u.fail->IsInternalNormal() ? 2 : (u.fail->IsInternalGiven() ? 1 : 0));
    if (!retry) return u.fail;
    delete u.fail;
    if (retry == 2) u = StaticUnify::unify_i_f(master_node,slave);
    else u = StaticUnify::unify_i_g(master_node);
    if (u.fail) return u.fail;
  }
  master_node->data().SetType(u.val);           // update the type of the found child
  return 0;
}
ret::ubool_t CBareFD::BeginAddLeaf(bool has_edge,const hCType& leaf)
{
  F_TRACE1(leaf);

  if (g_pTheEnv->m_bInterrupted) throw new InterruptFail;
  
  GetCurNodeAndType();                        // get the current top node and type
  if (m_curType->none()){              // if the current node is none -
    if (leaf->nil() || leaf->none())    // and the leaf is either <nil> or <none>
      return true;                      // done
    return new NNFail(m_curType,leaf); // else report normal-normal error
  }else if (m_curType->final() && has_edge) // if the current node is final and there is an edge to be added
    return new INFail(m_curType);      // report internal-normal error
  return false;                         // else - to be continued
}
ret::unode_t CBareFD::EndAddLeaf(token_t edge,const hCType& leaf)
{
	F_TRACE2(edge,leaf);

	CFDNode *dst_node = m_curNode->FindChild(edge);     // lookup a child connected to the current node by the <edge>
	if (dst_node)
	{													// if found - must check whether it can be entered
		hCType dst_type = dst_node->data().GetType();   // get the child type
		assert(dst_type.pointee());								// assert its validity
		if (Fail *f = UnifyTypes(dst_type,leaf,dst_node))	// unify the leaf with the child type and update the child
		{
			return f;                                       // return failure if failed
		}
		return dst_node;                                    // return the child node if ok
	}
	if (leaf->given())
	{
		return new GivenFail2(edge);// report the GIVEN error if the leaf is either <given> or #(under ...)
	}
	return m_curNode->AddChild(edge,leaf);      // add a new child node
}
ret::unode_t CBareFD::AddLeafProt(token_t edge,const hCType& leaf)
{
  F_TRACE2(edge,leaf);

  ret::unode_t n;
  ret::ubool_t b = BeginAddLeaf(true,leaf);
  
  if (b.fail) n.fail = b.fail;
  else{
    if (b.val) return m_curNode;
    assert(m_curType->internal());
    n = EndAddLeaf(edge,leaf);
  }
  if (n.fail) return new FDBuildFail(n.fail,"[add leaf]");
  return n;
}
// This special case happens in those two cases only:
// (({^ ... ^} leaf)) --> (({^ ... ^} ((NULL leaf))))
// ((x ((alt (y ...))))) --> ((x ((NULL y))) (x ...))
// In either case, current top node is never of an intermediate type.
ret::unode_t CBareFD::AddLeafProt(const hCType& leaf)
{
  F_TRACE1(leaf);

  ret::ubool_t b = BeginAddLeaf(false,leaf);
  ret::unode_t n(m_curNode);
  
  if (b.fail) n.fail = b.fail;
  else{
    if (b.val) return n;
    assert(!is_intermediate(m_curType));
    n.fail = UnifyTypes(m_curType,leaf,m_curNode);
  }
  if (n.fail){
    assert(BKClass::fail_bk() == 0);              // assert that the failure log is clean
    for (CFDParentIterator p_it(m_curNode);!p_it.is_past_the_end();p_it.advance()) // iterate on the parents of the current node
      BKClass::add_fail_bk(p_it->first->title()); // acknowledge the failed parent edges
    return new FDBuildFail(n.fail,"[add leaf]");
  }
  return n;
}
// End of AddLeaf section
/////////////////////////////////////////////////////////////////////////////////
// Handle AddPath command
ret::unode_t CBareFD::AddPathProt(token_t edge,const hCPath& path)
{
  F_TRACE2(edge,path);

  Fail *f = 0;
  ret::unode_t n1 = AddLeafProt(edge,g_pTheEnv->nil());
  if (!(f = n1.fail)){
    ret::unode_t n2 = HardPeekPath(path);
    if (!(f = n2.fail)){
      ret::unode_t u = StaticUnify::unify_nodes(n1.val,n2.val);
      if (!(f = u.fail)) return u;
    }
  }
  return new FDBuildFail(f,"[add path]");
}
ret::unode_t CBareFD::AddPathProt(const hCPath& path)
{
  F_TRACE1(path);
  
  Fail *f = 0;
  ret::unode_t n = HardPeekPath(path,true);              // ascend exactly the upcount specified
  if (!(f = n.fail)){
    ret::unode_t u = StaticUnify::unify_nodes(m_stack.topnode(),n.val);
    if (!(f = u.fail)) return u;
  }
  return new FDBuildFail(f,"[add path]");
}
// End of AddPath section
/////////////////////////////////////////////////////////////////////////////////
// Returns the node addressed by the path located at the right side of an attr-value pair
// of the kind (feature path).Will never be called if the path is on the left, like in
// ({^ a b} c), since this is translated by the parser into ({^ a} ((b c))).The EnterSubgraph()
// method deals with paths in its own way, it does not use this function (the call ({^4} a) is
// translated to ({^4} ((a)))).
// If so the path upcount in a relative path must be decremented by one.In (a ((b {^2 ...})))
// the fd is positioned at {a} when adding (b {^2 ...}), so we have to ascent only 1 up, instead of
// 2 specified in the path (the specification count b in).
// <is_exact> parameter controlls whether to treat the path specification exactly or to modify it
// in the above fashion.
// By default <is_exact> is false (there is almost always a non NULL feature on the left side of the
// attribute-value pair).
// (The implementation of the above idea is not straight forward, as one might think.One reason is
// that m_stack.end() points PAST the end and not at the end)
ret::unode_t CBareFD::HardPeekPath(CPath *path,bool is_exact)
{
  F_TRACE1(path);
  
  int upcount = (path->upcount() > 0 && is_exact) ? path->upcount() + 1 : path->upcount();
    
  assert(upcount);
    
  Info start = (upcount == CPath::ABS || upcount >= m_stack.size()) ?
    m_stack.front() : *(m_stack.end() - upcount);
  
  if (path->size()){
    m_stack.push(start.first,start.second);
    CPath::const_iterator it=path->begin();
    uint j = 0;
    ret::unode_t n;
    for (uint i=path->size()-1;i && !n.fail;++it,--i,++j) n.fail = EnterSubgraphProt(*it,Predefined::nil);
    if (!n.fail) n = AddLeafProt(*it,g_pTheEnv->nil());
    while (j--) LeaveSubgraphProt();
    m_stack.pop();
    return n;
  }
  return start.first;
}
ret::unode_t CBareFD::HardPeekAddr(CAddr *addr)
{
  F_TRACE1(addr);
  
  CArc *arc = cast<CArc>(addr);
  if (arc) return AddLeafProt(arc->title(),g_pTheEnv->nil());
  else{
    CPath *path = cast<CPath>(addr);
    assert(path);
    return HardPeekPath(path);
  }
}

////////////////////////////////////////////////////////////////////////////////
// GetDesc() methods of the Fail family of classes
//
string FDBuildFail::GetDesc() const
{
  string str(prefix);
  return str << ": " << m_pReason->GetDesc();
}
