#ifndef _COMPILER_FD_
#define _COMPILER_FD_

#include "c_fdnode.hxx"
#include "c_fdunify.hxx"
#include "b_graphbase.hxx"
#include <stack.h>

class FDBuildFail:public ComplexFail
{
public:
	FDBuildFail(Fail *e,char *_prefix):ComplexFail(e),prefix(_prefix){}
	string GetDesc() const;
private:
	char *prefix;
};
  
class CBareFD:public CFDBuilder
{
public:
	typedef pair<CFDNode *,token_t> Info;
private:
	////////////////////////////////////////////////////////////
	// CInfoStack is responsible for managing the stack of nodes
	// representing the current fd location.
	class CInfoStack
	{
	public:
		typedef vector<Info> InfoStack;
		typedef InfoStack::iterator iterator;
		struct Undo:public pair<int,InfoStack *>
		{
			Undo():pair<int,InfoStack *>(0,NULL){}
			Undo(int n,InfoStack *istack):pair<int,InfoStack *>(n,istack){}
		};
		typedef stack<Undo> UndoStack;
	public:
		explicit CInfoStack(CBareFD *owner);
		~CInfoStack();
		void reset();
		void get_elist(EdgeList& elist) const;
		Fail *update(token_t feat);
		void update(int upcount);
		void undo();
		CFDNode *topnode() const    { return m_loc->back().first; }
		token_t& topedge() const    { return m_loc->back().second; }
		void push(CFDNode *node,token_t title);
		void pop();
		void pop_many(int n)        { while (n--) pop(); }
		bool no_undo() const        { return m_undo_stack.empty(); }
		int balance() const         { return m_balance; }
		int size() const            { return m_loc->size(); }
		int empty() const           { return m_loc->empty(); }
		const Info& front() const   { return m_loc->front(); }
		iterator begin()            { return m_loc->begin(); }
		iterator end()              { return m_loc->end(); }
	private:
		CBareFD *m_owner;
		InfoStack *m_loc;
		UndoStack m_undo_stack;
		int m_balance;
	};
	//
	////////////////////////////////////////////////////////////
	// CAnyStack is the stack of nodes suspected of being of ANY
	// type.It is checked at the end of unification.
	typedef stack<pair<tick_t,CFDNode *> > CAnyStack_BaseClass;
	class CAnyStack:public CAnyStack_BaseClass
	{
		typedef CAnyStack_BaseClass BaseClass;
	public:
		explicit CAnyStack(CBareFD *fd);
		void reset();
		void push(CFDNode *node);
		CFDNode *top();
	private:
		CBareFD *m_fd;
	};
	//
	////////////////////////////////////////////////////////////
public:
	CBareFD();
	~CBareFD();
	CAnyStack& any_stack(){ return m_any_stack; }
	void reset();
	CFDNode *root() const { return m_root; }
	int count() const     { return m_count; }
	const EdgeList *cur_loc() const;
	// Graph creation commands----------------------------
	Fail *Initialize();
	Fail *Realize();
	Fail *EnterSubgraph(token_t edge)              { return m_stack.update(edge); }
	void EnterSubgraph(int upcount)                { m_stack.update(upcount); }
	void LeaveSubgraph()                           { m_stack.undo(); }
	Fail *AddLeaf(token_t edge,const hCType& type) { return AddLeafProt(edge,type).fail; }
	Fail *AddPath(token_t edge,const hCPath& path) { return AddPathProt(edge,path).fail; }
	Fail *AddLeaf(const hCType& type)              { return AddLeafProt(type).fail; }
	Fail *AddPath(const hCPath& path)              { return AddPathProt(path).fail; }
	// End of graph creation commands---------------------
	ret::unode_t HardPeekAddr(CAddr *addr);         // Peeks the address, creating the missing nodes.Can fail.
	ret::unode_t HardPeekPath(CPath *path,bool is_exact=false);
	friend class CInfoStack;
	friend class CAnyStack;
	friend class CFDNode;
protected:
	CInfoStack m_stack;
	CFDNode *m_root;
	// common for all the fd nodes
	int m_count;
	CAnyStack m_any_stack;
	// frequently used in graph construction methods
	mutable CFDNode *m_curNode;
	mutable hCType m_curType;
protected:
	void LeaveSubgraphProt() { m_stack.pop(); }
	virtual Fail *EnterSubgraphProt(token_t edge,token_t intitle = Predefined::nil);
	virtual ret::unode_t AddLeafProt(token_t,const hCType&);
	ret::unode_t AddPathProt(token_t,const hCPath&);
	ret::unode_t AddLeafProt(const hCType&);
	ret::unode_t AddPathProt(const hCPath&);
	void GetCurNodeAndType() const;
	ret::ubool_t BeginEnterSubgraph(token_t edge);
	Fail *EndEnterSubgraph(token_t edge,token_t intitle);
	ret::ubool_t BeginAddLeaf(bool has_edge,const hCType& leaf);
	ret::unode_t EndAddLeaf(token_t edge,const hCType& leaf);
private:
	Fail *UnifyTypes(const hCType& master,const hCType& slave,CFDNode *master_node);
};

#endif
