#include "stdcfuf.h"

// #define F_TRACE
// #define OP_TRACE

#include "b_debug_tracer.hxx"
#include "c_barefd.hxx"
#include "c_baregr.hxx"
#include "c_barecmds.hxx"

/////////////////////////////////////////////////////////////////////////////////
// CBareGr::CIndexTracker methods
//
void CBareGr::CIndexTracker::EnterBranch(CBranchNode *branch)
{
  F_TRACE1(branch->str());
  
  m_index = const_cast<TokenList *>(node2alt(branch->parent())->params()->index());
  push(IndexInfo(m_index->begin(),0,g_pTheEnv->nil()));
}
hCType CBareGr::CIndexTracker::LeaveBranch()
{
  F_TRACE0();

  hCType t = top().type;
  pop();
  if (t->final()) return t;
  return g_pTheEnv->nil();
}
void CBareGr::CIndexTracker::restore_branch(CBranchNode *branch)
{
  F_TRACE1(branch->str());
  
  if (branch->id())   // if the branch is not the top level (i.e. there is the parent alt)
    m_index = const_cast<TokenList *>(node2alt(branch->parent())->params()->index());  // retrieve the respective index
  else branch->set_index_type(g_pTheEnv->nil()); // else set the index for the top level
}
void CBareGr::CIndexTracker::EnterSubgraph(token_t edge)
{
  F_TRACE1(edge);

  if (top().balance == 0 && top().curpos != m_index->end() && edge == *(top().curpos))
    ++(top().curpos);
  else ++(top().balance);
}
void CBareGr::CIndexTracker::EnterSubgraph(int upcount)
{
  F_TRACE1(upcount);
  
  ++(top().balance);
}
void CBareGr::CIndexTracker::LeaveSubgraph()
{
  F_TRACE0();

  if (top().balance) --(top().balance);
  else{
    assert(top().curpos != m_index->begin());
    --(top().curpos);
  }
}
void CBareGr::CIndexTracker::AddLeaf(token_t edge,const hCType& type)
{
  F_TRACE2(edge,type);

  if (top().balance == 0 && top().curpos != m_index->end() && edge == *(top().curpos)){
    if (type->none() || top().curpos + 1 == m_index->end()) top().type = type;
  }
}

/////////////////////////////////////////////////////////////////////////////////
// CBareGr::GrStack methods
//
CGrNode *CBareGr::GrStack::pop_and_top()
{
  F_TRACE0();
  
  CGrNode *top_elt = top();
  top_elt->connect_children();
  stack<CGrNode *>::pop();
  CGrNode *top_elt2 = top();
  top_elt2->set_range_end(top_elt->range().second);
  return top_elt2;
}
CGrNode *CBareGr::GrStack::pop()
{
  F_TRACE0();
  
  CGrNode *top_elt = top();
  top_elt->connect_children();
  stack<CGrNode *>::pop();
  return top_elt;
}

/////////////////////////////////////////////////////////////////////////////////
// CBareGr methods
//
Fail *CBareGr::Initialize()
{
  F_TRACE0();
  
  m_root = m_cur_branch = new CBranchNode(*this);
  m_grstack.push(m_root);
  return 0;
}
Fail *CBareGr::Realize()
{
  F_TRACE0();
  
  assert(m_grstack.size()==1 && m_grstack.pop() == m_root);
  m_root->set_range_end(m_cur_branch->range().second);
  m_cur_branch=NULL;
  return 0;
}
const EdgeList *CBareGr::cur_loc() const
{
  F_TRACE0();
  
  return &m_curloc.value();
}
Fail *CBareGr::EnterAlt(CAltParams *params)
{
  F_TRACE1(params);
  OP_TRACE1(params);

  topmost_index(params);
  m_grstack.push(m_cur_branch->add_alt(new CAltNode(m_curloc,params,*this)));
  return 0;
}
Fail *CBareGr::EnterRalt(CAltParams *params)
{
  F_TRACE1(params);
  OP_TRACE1(params);
  
  topmost_index(params);
  m_grstack.push(m_cur_branch->add_alt(new CRaltNode(m_curloc,params,*this)));
  return 0;
}
Fail *CBareGr::EnterOpt(CAltParams *params)
{
  F_TRACE1(params);
  OP_TRACE1(params);
  
  m_grstack.push(m_cur_branch->add_alt(new COptNode(m_curloc,params,*this)));
  return 0;
}
/*
  Augment the topmost alt with (:index cat) if it is not already
  indexed.
*/
void CBareGr::topmost_index(CAltParams *params)
{
  F_TRACE1(params);

  if (m_cur_branch->id() == 0)  // add (:index cat) to the topmost alt if
    params->augment_root();     // it lacks the index at all
}
Fail *CBareGr::LeaveAlt()
{
  OP_TRACE0();
  
  m_cur_branch = node2branch(m_grstack.pop_and_top());
  m_itracker.restore_branch(m_cur_branch);
  return 0;
}
Fail *CBareGr::EnterBranch()
{
  OP_TRACE0();
  
  if (g_pTheEnv->m_bInterrupted) throw new InterruptFail;
  CAltNode *alt = node2alt(m_grstack.top());
  assert(alt);
  m_cur_branch = new CBranchNode(*this);
  alt->add_branch(m_cur_branch);
  m_grstack.push(m_cur_branch);
  m_itracker.EnterBranch(m_cur_branch);
  return 0;
}
Fail *CBareGr::LeaveBranch()
{
  OP_TRACE0();
  
  m_grstack.pop_and_top();
  m_cur_branch->set_index_type(m_itracker.LeaveBranch());
  return 0;
}
Fail *CBareGr::EnterSubgraph(token_t edge)
{
  OP_TRACE1(edge);
  
  m_curloc.update(edge);
  m_cur_branch->store_cmd(new CEnterSubgraphCmd_edge(edge));
  m_itracker.EnterSubgraph(edge);
  return 0;
}
void CBareGr::EnterSubgraph(int upcount)
{
  OP_TRACE1(upcount);
  
  m_curloc.update(upcount);
  m_cur_branch->store_cmd(new CEnterSubgraphCmd_path(upcount));
  m_itracker.EnterSubgraph(upcount);
}
void CBareGr::LeaveSubgraph()
{
  OP_TRACE0();
  
  m_curloc.undo();
  m_cur_branch->store_cmd(new CLeaveSubgraphCmd);
  m_itracker.LeaveSubgraph();
}
Fail *CBareGr::AddLeaf(token_t edge,const hCType& type)
{
  OP_TRACE2(edge,type);

  m_cur_branch->store_cmd(new CAddLeafCmd_2arg(edge,type));
  m_itracker.AddLeaf(edge,type);
  return 0;
}
Fail *CBareGr::AddLeaf(const hCType& type)
{
  OP_TRACE1(type);
  
  m_cur_branch->store_cmd(new CAddLeafCmd_1arg(type));
  return 0;
}
Fail *CBareGr::AddPath(token_t edge,const hCPath& path)
{
  OP_TRACE2(edge,path);
  
  m_cur_branch->store_cmd(new CAddPathCmd_2arg(edge,path));
  return 0;
}
Fail *CBareGr::AddPath(const hCPath& path)
{
  OP_TRACE1(path);
  
  m_cur_branch->store_cmd(new CAddPathCmd_1arg(path));
  return 0;
}

/////////////////////////////////////////////////////////////////////////////////
// C[Enter/Leave]subgraph group methods
//
Fail *CEnterSubgraphCmd_edge::exec(CFDBuilder *fd){ return fd->EnterSubgraph(m_edge); }
string CEnterSubgraphCmd_edge::str() const
{
  string str("(");
  return str << m_edge << " (";
}
Fail *CEnterSubgraphCmd_path::exec(CFDBuilder *fd){ fd->EnterSubgraph(m_upcount);return 0; }
string CEnterSubgraphCmd_path::str() const
{
  string str("(");
  return str << CPath(m_upcount) << " (";
}
Fail *CLeaveSubgraphCmd::exec(CFDBuilder *fd)   { fd->LeaveSubgraph();return 0; }
string CLeaveSubgraphCmd::str() const           { return "))"; }

/////////////////////////////////////////////////////////////////////////////////
// CAddLeaf group methods
//
Fail *CAddLeafCmd_2arg::exec(CFDBuilder *fd){ return fd->AddLeaf(m_edge,m_type); }
string CAddLeafCmd_2arg::str() const
{
  string str("(");
  return str << m_edge << " " << m_type << ")";
}
Fail *CAddLeafCmd_1arg::exec(CFDBuilder *fd){ return fd->AddLeaf(m_type); }
string CAddLeafCmd_1arg::str() const { return m_type->toString(); }

/////////////////////////////////////////////////////////////////////////////////
// CAddPath group methods
//
Fail *CAddPathCmd_2arg::exec(CFDBuilder *fd){ return fd->AddPath(m_edge,m_path); }
string CAddPathCmd_2arg::str() const
{
  string str("(");
  return str << m_edge << " " << m_path << ")";
}
Fail *CAddPathCmd_1arg::exec(CFDBuilder *fd){ return fd->AddPath(m_path); }
string CAddPathCmd_1arg::str() const { return m_path->toString(); }
