#ifndef _COMPILER_GRAMMAR_
#define _COMPILER_GRAMMAR_

#include "c_curloc.hxx"
#include "c_branchnode.hxx"
#include "c_altnode.hxx"
#include "b_altparams.hxx"
#include "b_graphbase.hxx"

class CTokenHier;

class CBareGr:public CGrammarBuilder
{
	friend class CGrNode;
public:
	struct IndexInfo
	{
		TokenList::const_iterator curpos;
		int balance;
		hCType type;
		IndexInfo():balance(1),type(g_pTheEnv->nil()){}
		IndexInfo(const TokenList::const_iterator& _curpos,int _balance,const hCType& _type):
			curpos(_curpos),balance(_balance),type(_type){}
	};
	class CIndexTracker:private stack<IndexInfo>
	{
		TokenList *m_index;         // the index of the current branch
	public:
		CIndexTracker():m_index(0)
		{ 
			push(IndexInfo()); 
		}
		void EnterBranch(CBranchNode *branch);
		hCType LeaveBranch();
		void restore_branch(CBranchNode *branch);
		void EnterSubgraph(token_t edge);
		void EnterSubgraph(int upcount);
		void LeaveSubgraph();
		void AddLeaf(token_t edge,const hCType& type);
	};
	struct GrStack:public stack<CGrNode *>
	{
		CGrNode *pop_and_top();
		CGrNode *pop();
	};
protected:
	CCurLoc m_curloc;
	GrStack m_grstack;
	CGrNode *m_root;
	CBranchNode *m_cur_branch;
	CIndexTracker m_itracker;
	// These are common for all the grammar nodes
	int m_count;
public:
	CBareGr():m_root(NULL),m_cur_branch(NULL),m_count(0){}
	~CBareGr()
	{ 
		delete m_root; 
	}
	int count() const 
	{ 
		return m_count; 
	}
	Fail *Initialize();
	Fail *Realize();
	const EdgeList *cur_loc() const;
	Fail *EnterAlt(CAltParams *);
	Fail *EnterRalt(CAltParams *);
	Fail *EnterOpt(CAltParams *);
	Fail *LeaveAlt();
	Fail *LeaveRalt()     
	{
		return LeaveAlt(); 
	}
	Fail *LeaveOpt()      
	{
		return LeaveAlt(); 
	}
	Fail *EnterBranch();
	Fail *LeaveBranch();
	Fail *EnterSubgraph(token_t edge);
	void EnterSubgraph(int upcount);
	void LeaveSubgraph();
	Fail *AddLeaf(token_t edge,const hCType& type);
	Fail *AddPath(token_t edge,const hCPath& path);
	Fail *AddLeaf(const hCType& type);
	Fail *AddPath(const hCPath& path);
	CGrNode *root() const         
	{
		return m_root; 
	}
private:
	void EnterAlt_node(CGrNode *alt_node);
	void LeaveAlt_node();
	void topmost_index(CAltParams *);
};

#endif
