#include "stdcfuf.h"

// #define OP_TRACE
// #define F_TRACE

#include "b_debug_tracer.hxx"
#include "c_extfd.hxx"
#include "b_bkclass.hxx"
#include "c_pattern.hxx"
#include "c_cset.hxx"
#include "c_fset.hxx"
#include "b_control.hxx"

class FDCheckFsetFail:public Fail{
public:
  FDCheckFsetFail(const hCFsetType& _ftype,token_t _feature):ftype(_ftype),feature(_feature){}
  string GetDesc() const;
private:
  hCFsetType ftype;
  token_t feature;
};

////////////////////////////////////////////////////////////////////////////////
// CExtFD methods
//
CFsetType *CExtFD::GetCurFset(CFDNode *curNode)
{
  if (!curNode) curNode = m_curNode;
  assert(curNode);
  CFDNode *fset_node = curNode->FindChild(Predefined::fset);
  if (fset_node){
    CType *fSetType = fset_node->data().GetType();
    CFsetType *ftype = cast<CFsetType>(fSetType);
    if (ftype) return ftype;
    else assert(fSetType->nil());
  }
  return 0;
}
Fail *CExtFD::EnterSubgraphProt(token_t edge,token_t intitle)
{
  F_TRACE2(edge,intitle);

  Fail *f = 0;
  ret::ubool_t b = BeginEnterSubgraph(edge);
  if (!(f = b.fail)){
    if (b.val) return 0;
    CFsetType *ftype = GetCurFset();
    if (ftype && !ftype->IsFeatureAllowed(edge)){
      if (intitle==Predefined::any) f = new FDCheckFsetFail(ftype,edge);
      else if (!m_curNode->FindChild(edge)){                // if this is a new feature
        m_stack.push(m_curNode->AddChild(edge,g_pTheEnv->none()),edge);  // force the its value to be none
        return 0;
      } // else if the feature already exists - ok.The ways fsets are added ensures it.
    }
    if (!f) f = CBareFD::EndEnterSubgraph(edge,intitle);
  }
  if (f){
    assert(BKClass::fail_bk() == 0);    // assert that the failure log is clean
    BKClass::add_fail_bk(edge);         // acknowledge the failed edge
    return new FDBuildFail(f,"[enter subgraph]");
  }
  return 0;
}
ret::unode_t CExtFD::AddLeafProt(token_t edge,const hCType& _leaf)
{
  F_TRACE2(edge,_leaf);

  hCType leaf(_leaf);
  ret::unode_t n;
  ret::ubool_t b = CBareFD::BeginAddLeaf(true,leaf);
  
  if (!(n.fail = b.fail)){
    if (b.val) return m_curNode;
    if (!leaf->none()){
      CFsetType *ftype = GetCurFset();
      if (ftype && !ftype->IsFeatureAllowed(edge)){
        if (leaf->nil()) leaf = g_pTheEnv->none();
        else n.fail = new FDCheckFsetFail(ftype,edge);
      }
    }
    if (!n.fail) n = CBareFD::EndAddLeaf(edge,leaf);
  }
  if (n.fail){
    assert(BKClass::fail_bk() == 0);    // assert that the failure log is clean
    BKClass::add_fail_bk(edge);         // acknowledge the failed edge
    return new FDBuildFail(n.fail,"[add leaf]");
  }
  return n;
}
/*
  Pairwise unification of nodes is unacceptable in general, because of the special
  types.They have to be unified the very last, but the very last in pairwise
  unification might not be such in the group unification as whole.
  ( U{a b c d} might not be the same as U{U{U{a,b},c},d} )
*/
Fail *CExtFD::satisfy_constraint(const Constraint& c)
{
  F_TRACE0();

  CFDNode *first=c.front();
  Constraint::const_iterator it=c.begin();
  ret::unode_t n;
  for (++it;it!=c.end() && !n.fail;++it) n = StaticUnify::unify_nodes(first,*it);
  return n.fail ? new FDBuildFail(n.fail,"[add pattern]") : 0;
}
////////////////////////////////////////////////////////////////////////////////
// VERY IMPORTANT:
// Functions using m_curNode and/or m_curType must save them in a local variable,
// since creation/unification of special types can modify them indirectly through
// SoftPeekAddr()
pair<CFDNode *,CType *> CExtFD::AddSpecialProt(token_t tknSpecialFeature)
{
	if (g_pTheEnv->m_bInterrupted)
	{
		throw new InterruptFail;
	}

	GetCurNodeAndType();
	if (m_curType->none()) 
	{
		return pair<CFDNode *,CType *>(m_curNode,0);
	}

	assert(m_curType->internal());

	CFDNode *special_node = m_curNode->FindChild(tknSpecialFeature);
	CType *special_type;
	if (special_node) 
	{
		special_type = special_node->data().GetType();
	}
	else
	{
		special_node = m_curNode->AddChild(tknSpecialFeature,g_pTheEnv->nil());
		special_type = g_pTheEnv->nil();
	}

	assert(special_type);

	return make_pair(special_node,special_type);
}
template <class CSpecialType,class hSpecial>
ret::unode_t SetSpecialNode(CFDNode *pSpecialNode,CType *pSpecialType,const hSpecial& special)
{
	if (!pSpecialType)
	{
		return pSpecialNode;
	}

	ret::utype_t u;
	CSpecialType *stype = cast<CSpecialType>(pSpecialType);
	if (stype)
	{
		u = stype->special_unify(special);	// might change the value of m_curNode
	}
	else
	{
		assert(pSpecialType && pSpecialType->nil());
		u = CSpecialType::create(pSpecialNode,special);
	}

	if (u.fail) 
	{
		return u.fail;
	}

	pSpecialNode->data().SetType(u.val);
	return pSpecialNode;
}
ret::unode_t CExtFD::AddPatternProt(const hCPattern& pattern)
{
	pair<CFDNode *,CType *> found = AddSpecialProt(pattern->feature());
	return SetSpecialNode<CPatternType,hCPattern>(found.first,found.second,pattern);
}
ret::unode_t CExtFD::AddCsetProt(const hCCset& cset)
{
	pair<CFDNode *,CType *> found = AddSpecialProt(cset->feature());
	return SetSpecialNode<CCsetType,hCCset>(found.first,found.second,cset);
}
ret::unode_t CExtFD::AddFsetProt(const hCFset& fset)
{
	pair<CFDNode *,CType *> found = AddSpecialProt(fset->feature());
	return SetSpecialNode<CFsetType,hCFset>(found.first,found.second,fset);
}
Fail *CExtFD::AddControlProt(const hCControl& control)
{
	F_TRACE1(control);

	if (g_pTheEnv->m_bInterrupted)
	{
		throw new InterruptFail;
	}

	GetCurNodeAndType();

	string expr;
	vector<StrOrAddr>::const_iterator stmt_it = control->stmt().begin();
	vector<bool>::const_iterator chooser_it = control->chooser().begin();
	for (uint i=0;i<control->stmt().size();++i,++stmt_it,++chooser_it)
	{
		// cout << "---" << expr << "---\n";
		if (*chooser_it)
		{
			expr << *(stmt_it->str);
		}
		else
		{
			CFDNode *node = SoftPeekAddr(stmt_it->addr);
			hCType type = node->data().GetType();
			// cout << "===" << stmt_it->addr << "===" << type << "===\n";
			if (bool(type) && type->final())
			{
				assert(!type->under());
				if (type->quoted()) 
				{
					expr << type;
				}
				else
				{
					char *str = TOK2STR(cast<CNormalType>(type)->title());
					char *endptr;
					strtod(str,&endptr);     // try converting the type to a number
					if (*endptr) 
					{
						expr << '\'' << str; // if *endptr != '\0' then str is not a number - output double quotes
					}
					else 
					{
						expr << str;         // otherwise str is a number and does not need any quotes
					}
				}
			}
			else 
			{
				return new FDControlFail(expr);
			}
		}
	}
	return g_pTheInterp->EvalControl(expr.c_str()) ? 0 : new FDControlFail(expr);
}
/////////////////////////////////////////////////////////////////////////////////
// The SoftPeekAddr and SoftPeekPath return the node at the specified generic address
// or path. They never fail, a node on the path is added even if its parent nodes is
// not an internal node. They are used with special types cset and pattern, which
// might hold references to such (ordinarily illegal) locations.
CFDNode *CExtFD::SoftPeekAddr(CAddr *addr)
{
  F_TRACE0();
  
  GetCurNodeAndType();

  CArc *arc = cast<CArc>(addr);
  if (arc){
    bool dummy;
    return SoftPeekAdvance(m_curNode,arc->title(),dummy);
  }else{
    CPath *path = cast<CPath>(addr);
    assert(path);
    return SoftPeekPath(path);
  }
}
CFDNode *CExtFD::SoftPeekPath(CPath *path)
{
  F_TRACE1(path);
  
  int upcount = path->upcount();
  assert(upcount);

  CFDNode *node = (upcount == CPath::ABS || upcount >= m_stack.size()) ?
    m_root : (m_stack.end() - upcount)->first;
  
  bool cont = true;
  CPath::const_iterator it=path->begin();
  for (uint i=path->size();cont && i;--i,++it) node = SoftPeekAdvance(node,*it,cont);
  return node;
}
CFDNode *CExtFD::SoftPeekAdvance(CFDNode *from,token_t edge,bool& cont)
{
  F_TRACE2(from,edge);
  
  if (from->data().GetType()->none()){  // if the <from> node is NONE, then
    cont = false;                       // no point to continue beyond
    return from;                        // we've got the winner
  }
  cont = true;
  CFDNode *found = from->FindChild(edge,true);  // search for a child even if <from> is not internal (insist is true)
  if (!found){                                  // if not found - add the new child to the node
    hCType leaf(g_pTheEnv->nil());              // normally a nil node is added
    CFsetType *ftype = GetCurFset(from);        // but if the <from> node has an explicit fset
    if (ftype && !ftype->IsFeatureAllowed(edge))// which does not allow for this particular feature
      leaf = g_pTheEnv->none();                 // a none node is added instead
    found = from->AddChild(edge,leaf); 
  }
  return found;
}

////////////////////////////////////////////////////////////////////////////////
// FDFsetFail methods
//
FDControlFail::FDControlFail(string _expr):expr(_expr)
{
  assert(BKClass::fail_bk() == 0);              // assert that the failure log is clean
  BKClass::add_fail_bk(Predefined::control);  // acknowledge the failed edge
}

////////////////////////////////////////////////////////////////////////////////
// FDFsetFail methods
//
string FDCheckFsetFail::GetDesc() const
{
  string str("Feature ");
  return str << feature << " and fset " << ftype << " are inconsistent!";
}

