#include "stdcfuf.h"

// #define F_TRACE
// #define OP_TRACE

#include "b_debug_tracer.hxx"
#include "c_extgr.hxx"
#include "c_patnode.hxx"
#include "c_extfd.hxx"
#include "c_pattern.hxx"
#include "b_cset.hxx"
#include "b_fset.hxx"
#include "b_control.hxx"
#include "c_extcmds.hxx"

/////////////////////////////////////////////////////////////////////////////////
// CAddCsetCmd methods
//
CAddCsetCmd::CAddCsetCmd(const hCCset& cset):m_cset(cset){}
Fail *CAddCsetCmd::exec(CFDBuilder *_fd)
{
  CExtFD *fd=static_cast<CExtFD *>(_fd);
  assert(fd);
  return fd->AddSpecial(Predefined::cset,m_cset);
}
string CAddCsetCmd::str() const
{
  string str("(");
  return str << Predefined::cset << " " << m_cset << ")"; 
}

/////////////////////////////////////////////////////////////////////////////////
// CAddFsetCmd methods
//
CAddFsetCmd::CAddFsetCmd(const hCFset& fset):m_fset(fset){}
Fail *CAddFsetCmd::exec(CFDBuilder *_fd)
{
  CExtFD *fd=static_cast<CExtFD *>(_fd);
  assert(fd);
  return fd->AddSpecial(Predefined::fset,m_fset);
}
string CAddFsetCmd::str() const
{
  string str("(");
  return str << Predefined::fset << " " << m_fset << ")"; 
}

/////////////////////////////////////////////////////////////////////////////////
// CAddControlCmd methods
//
CAddControlCmd::CAddControlCmd(const hCControl& control):m_control(control){}
Fail *CAddControlCmd::exec(CFDBuilder *_fd)
{
  CExtFD *fd=static_cast<CExtFD *>(_fd);
  assert(fd);
  return fd->AddSpecial(Predefined::control,m_control);
}
string CAddControlCmd::str() const
{
  string str("(");
  return str << Predefined::control << " " << m_control << ")"; 
}

/////////////////////////////////////////////////////////////////////////////////
// CAddPatternCmd methods
//
CAddPatternCmd::CAddPatternCmd(const hCPatternType& pattern):m_pattern(pattern){}
Fail *CAddPatternCmd::exec(CFDBuilder *_fd)
{
  CExtFD *fd=static_cast<CExtFD *>(_fd);
  assert(bool(m_pattern) && fd);
  
  CConstraintIterator cur_c(m_pattern->constraint_iterator());
  Fail *f = 0;
  for (;!cur_c.is_past_the_end() && !f;cur_c.advance())
    f = fd->satisfy_constraint(*cur_c);
  return f;
}
string CAddPatternCmd::str() const
{
  string str("(");
  return str << Predefined::pattern << " " << m_pattern << ")"; 
}

/////////////////////////////////////////////////////////////////////////////////
// CExtGr methods
//
Fail *CExtGr::AddSpecial(token_t,const hCPattern& pattern)
{
  OP_TRACE1(pattern);

  if (g_pTheEnv->m_bInterrupted) throw new InterruptFail;
  CPatAltNode *pat_alt = new CPatAltNode(pattern,m_curloc,*this);
  m_cur_branch->add_alt(pat_alt);
  CPatBranchNode *pat_branch=new CPatBranchNode(*this);
  pat_alt->add_branch(pat_branch);
  pat_alt->set_range_end(pat_branch->range().second);
  m_cur_branch->set_range_end(pat_branch->range().second);
  return 0;
}
Fail *CExtGr::AddSpecial(token_t,const hCCset& cset)
{
  m_cur_branch->store_cmd(new CAddCsetCmd(cset));
  return 0;
}
Fail *CExtGr::AddSpecial(token_t,const hCFset& fset)
{
  m_cur_branch->store_cmd(new CAddFsetCmd(fset));
  return 0;
}
Fail *CExtGr::AddSpecial(token_t,const hCControl& control)
{
  m_cur_branch->store_cmd(new CAddControlCmd(control));
  return 0;
}
