#include "stdcfuf.h"

// #define F_TRACE

#include "b_debug_tracer.hxx"
#include "c_fditerator.hxx"
#include "c_barefd.hxx"
#include "c_fdpprint.hxx"
#include <algo.h>
#include <iomanip.h>
#ifdef WIN32
#include <ostream.h>
#else
#include <ostream.h>
#endif

#ifdef WIN32
using std::setw;
#endif

class CDoPrint:public CUnparseFD
{
public:
	CDoPrint(bool bDoingAll,ostream& os,int indent,bool newline);
	Fail *StartInternal(CFDNode *root);
	Fail *EndInternal();
	Fail *StartEndLeaf(CFDNode *root);
	Fail *EnterSubgraph(token_t,CFDNode *,EdgeList *);
	Fail *LeaveSubgraph();
	Fail *AddLeaf(token_t,CFDNode *,EdgeList *);
	Fail *AddPath(token_t,CFDNode *,EdgeList *);
	bool IsDoingAll() const { return m_bDoingAll; }
	int GetOrder() const    { return ASCENDING; }
private:
	ostream& m_os;
	bool m_bDoingAll;
	int m_indent;
	bool m_newline;
	stack<int> save_indent;
private:
	void do_indent()
	{
		if (m_newline)
		{
			m_os << endl << setw(m_indent) << ""; 
		}
	}
};

/////////////////////////////////////////////////////////////////////////////////
// CDoPrint methods
//
CDoPrint::CDoPrint(bool bDoingAll,ostream& os,int indent,bool newline):m_os(os)
{
	m_bDoingAll = bDoingAll;
	m_indent = indent + 1;
	m_newline = newline;
	if (newline)
	{
		m_os << setw(indent) << ""; // indent the first line if so requested
	}
}
Fail *CDoPrint::StartInternal(CFDNode *root)
{
	if (root->any())
	{
		m_os << "(({} " << g_pTheEnv->any() << ')';  // enter the fd and print ANY immediately
		m_newline = true;           // the actual fd starts on the next line
	}
	else
	{
		m_os << '(';                // enter the fd
		m_newline = false;          // the very first pair is not to be indented
	}
	return 0;
}
Fail *CDoPrint::EndInternal()
{
	m_os << ')';                  // leave the fd
	return 0;
}
Fail *CDoPrint::StartEndLeaf(CFDNode *root)
{
	m_os << root->data().GetType();  // the fd is just a single leaf
	return 0;
}
Fail *CDoPrint::EnterSubgraph(token_t edge,CFDNode *node,EdgeList *)
{
	F_TRACE1(edge);

	do_indent();
	if (node->any())   // special case when internal node is reported as ANY
	{
		m_os << '(' << edge << ' ' << g_pTheEnv->any() << ')';  // output the ANY type
		m_newline = true;                   // the first attr-value pair of the subfd is not to be indented
		do_indent();
	}
	m_newline = false;                    // the first attr-value pair of the subfd is not to be indented
	m_os << "(" << edge << " (";          // output the edge
	save_indent.push(m_indent);           // save the current indentation
	m_indent+=strlen(TOK2STR(edge))+3;    // update the indentation for the new subfd
	return 0;
}
Fail *CDoPrint::LeaveSubgraph()
{
	F_TRACE0();

	m_newline = true;                     // continue indented
	m_indent = save_indent.top();         // get the previous indentation
	m_os << "))";                         // finish the current subfd
	save_indent.pop();                    // return one level up
	return 0;
}
Fail *CDoPrint::AddLeaf(token_t edge,CFDNode *node,EdgeList *)
{
	F_TRACE1(edge);

	CType *type = node->data().GetType();
	if (type->nil()) 
	{
		return 0;
	}

	do_indent();                          
	m_os << '(' << edge << ' ' << type << ')'; // output the attr-value pair
	m_newline = true;              // assure that the next pairs will be indented
	return 0;
}
Fail *CDoPrint::AddPath(token_t edge,CFDNode *node,EdgeList *path)
{
	F_TRACE2(edge,path);

	do_indent();
	m_os << "(" << edge << " {" << path->GetString() << "})";  // output the attr-value pair
	m_newline = true;              // assure that the next pairs will be indented
	return 0;
}

/////////////////////////////////////////////////////////////////////////////////
// CPrettyPrintFD methods
//
CPrettyPrintFD::CPrettyPrintFD(CFDNode *root):m_walker(root){}
void CPrettyPrintFD::display(bool bDoingAll,ostream& os,int indent,bool newline)
{
	CDoPrint printer(bDoingAll,os,indent,newline);
	m_walker.walk(&printer);
}
