#ifndef _COMPILER_FDWALK_
#define _COMPILER_FDWALK_

#include <algo.h>
#include <deque.h>

class CUnparseFD;
class CFDNode;

class CFDWalk
{
public:
	typedef pair<token_t,CFDNode *> child_type;

private:
	typedef deque<child_type> ch_queue;
	typedef ch_queue::iterator ch_iterator;
  
	typedef bool (*order_t)(const child_type& x,const child_type& y);
	static const order_t NONE;
	static const order_t ASCENDING;
	static const order_t DESCENDING;
	order_t m_order;              // comparison function
  
private:
	typedef bool (CFDChildIterator::*iter_halter_t)() const;
	static const iter_halter_t DO_ALL;
	static const iter_halter_t DO_NORMAL;
	iter_halter_t m_halter;
  
public:
	CFDWalk(CFDNode *root);
	void ignore(token_t feature);         // add a feature (normally special) to be ignored
	Fail *walk(CUnparseFD *pUnparser);

private:
	vector<EdgeList *> m_visited;
	EdgeList m_curloc;
	EdgeList m_ignored;           // features to be ignored during the walk, intended for specials only
	CUnparseFD *m_pUnparser;
	CFDNode *m_pNode;
  
private:
	Fail *WalkFD(CFDNode *);
	Fail *WalkAttrValue(token_t,CFDNode *);
};

class CUnparseFD{
public:
  enum {DESCENDING = -1,ARBITRARY = 0,ASCENDING = 1};
  virtual ~CUnparseFD(){}
  virtual Fail *StartInternal(CFDNode *root)   { return 0; }  //-+-> These start/end walk on an fd which
  virtual Fail *EndInternal()                  { return 0; }  //-+   root is an internal node
  virtual Fail *StartEndLeaf(CFDNode *root)    { return 0; }  // This starts/ends walk on an fd which root is a leaf node
  virtual Fail *StartEndSpecial(CFDNode *root) { return StartEndLeaf(root); }  // and this is for an fd which root is a special node
  virtual Fail *EnterSubgraph(token_t edge,CFDNode *node,EdgeList *path) =0;
  virtual Fail *LeaveSubgraph() =0;
  virtual Fail *AddLeaf(token_t edge,CFDNode *node,EdgeList *path) =0;
  virtual Fail *AddPath(token_t edge,CFDNode *node,EdgeList *path) =0;
  virtual bool IsDoingAll() const { return true; }      // by default visit all the nodes including special
  virtual int GetOrder() const    { return ARBITRARY; } // by default visit the nodes in an arbitrary order
};

#endif
