#include "stdcfuf.h"
#include "c_extfd.hxx"
#include "c_fset.hxx"
#include "b_bkclass.hxx"
#include "t_tracer_l3.hxx"

class FDAddFsetFail_1:public FDFsetFail{
public:
  FDAddFsetFail_1(Fail *e,const hCFset& _fset):FDFsetFail(e),fset(_fset){}
  string GetDesc() const {
    string str("Failed to add fset ");
    return str << fset << ": " << m_pReason->GetDesc();
  }
private:
  hCFset fset;
};

class FDAddFsetFail_2:public FDFsetFail{
public:
  FDAddFsetFail_2(Fail *e,const hCFsetType& _ftype,const hCFset& _fset):
    FDFsetFail(e),ftype(_ftype),fset(_fset){}
  string GetDesc() const {
    string str("Fsets ");
    return str << ftype << " and " << fset << " are inconsistent: " << m_pReason->GetDesc();
  }
private:
  hCFsetType ftype;
  hCFset fset;
};

class FDUnifyFsetsFail:public FDFsetFail{
public:
  FDUnifyFsetsFail(Fail *e,const hCFsetType& _ftype1,const hCFsetType& _ftype2):
    FDFsetFail(e),ftype1(_ftype1),ftype2(_ftype2){}
  string GetDesc() const {
    string str("Fsets ");
    return str << ftype1 << " and " << ftype2 << " are inconsistent: " << m_pReason->GetDesc();
  }
private:
  hCFsetType ftype1,ftype2;
};

static Fail *fail;
////////////////////////////////////////////////////////////////////////////////
// CFsetType methods
//
CFsetType::CFsetType(CFDNode *owner,const CFsetType& r):m_fset(r.m_fset),m_owner(owner){}
CFsetType::CFsetType(CFDNode *owner,const hCFset& fset):m_fset(fset),m_owner(owner)
{
  if ((fail = check_parents(m_owner)))
    fail = new FDAddFsetFail_1(fail,fset);
}
CFsetType::CFsetType(const hCFsetType& ftype,const hCFset& fset):
  m_fset(new CFset(5)),m_owner(ftype->m_owner)
{
  unify_fsets(ftype->m_fset,fset);
  if ((fail = check_parents(m_owner)))
    fail = new FDAddFsetFail_2(fail,ftype,fset);
}
// The actual nodes are not linked together at that point, so I have to handle to different sets of parents
// If the unification succeeds then the two owner nodes will be linked together and m_owner will be the
// resulting composite node.
CFsetType::CFsetType(const hCFsetType& ftype1,const hCFsetType& ftype2,CFDNode *owner1,CFDNode *owner2):
  m_fset(new CFset(5)),m_owner(ftype1->m_owner)
{
  unify_fsets(ftype1->m_fset,ftype2->m_fset);
  fail = check_parents(owner1);
  if (!fail) fail = check_parents(owner2);
  if (fail) fail = new FDUnifyFsetsFail(fail,ftype1,ftype2);
}
ret::utype_t CFsetType::create(CFDNode *owner,const hCFset& fset)
{
  hCType res_type(new CFsetType(owner,fset));
  if (fail) return fail;
  return res_type;
}
ret::utype_t CFsetType::special_unify(const hCFset& fset)
{
  DBG_TRACE3(Tracer_l3::special_unify(this,fset.pointee()));
  hCType res_type(new CFsetType(this,fset));
  if (fail) return fail;
  return res_type;
}
ret::utype_t CFsetType::unify_with_special(const hCType& type,CFDNodeBase *arg1,CFDNodeBase *arg2)
{
  CFsetType *ftype = cast<CFsetType>(type);
  if (!ftype && !type->nil()) return new SSFail(this,type);
  CFDNode *fnode1 = static_cast<CFDNode *>(arg1);
  CFDNode *fnode2 = static_cast<CFDNode *>(arg2);
  hCType res_type;
  if (ftype){
    DBG_TRACE3(Tracer_l3::special_unify(this,ftype));
    res_type = new CFsetType(this,ftype,fnode1,fnode2);
  }else{ // two nodes are merged, must recheck the fset 
    fail = check_parents(fnode1);
    if (!fail && fnode2) fail = check_parents(fnode2);
    if (fail) fail = new FDAddFsetFail_1(fail,m_fset);
    else res_type = this;
  }
  if (fail) return fail;
  return res_type;
}
Fail *CFsetType::check_parents(CFDNode *owner)
{
  assert(owner);
  hash_set<int> ids;
  for (CFDParentIterator p_it(owner);!p_it.is_past_the_end();p_it.advance()){
    int id = p_it->second->group_id();
    if (ids.find(id) == ids.end()){
      if (Fail *f = validate_fset(p_it->second)) return f;
      ids.insert(id);
    }
  }
  return 0;
}
void CFsetType::unify_fsets(const hCFset& fset1,const hCFset& fset2)
{
  if (fset1->size() < fset2->size()) intersect(fset1,fset2);
  else intersect(fset2,fset1);
}
void CFsetType::intersect(const hCFset& fset1,const hCFset& fset2)
{
  CFset::const_iterator it1=fset1->begin();
  for (uint i=fset1->size();i;--i,++it1)
    if (fset2->find(*it1)!=fset2->end()) m_fset->insert(*it1);
}
// Given the node which is the owner of the fset feature check if the attributes
// of the node are consistent with the fset.
Fail *CFsetType::validate_fset(CFDNode *node)
{
  for (CFDChildIterator it(node);!it.is_special();it.advance()) // iterate on normal features (ignore fsets,csets and so on)
    if (it->first->title() != Predefined::fset &&               // if the feature is not fset (happens when an fset is created)
        m_fset->find(it->first->title())==m_fset->end()){       // and it is absent from the appropriate set
      ret::utype_t u = StaticUnify::unify_i_f(it->second,g_pTheEnv->none()); // unify the feature value with none (in effect cancelling it)
      if (u.fail) return u.fail;
    }
  return 0;
}
bool CFsetType::IsFeatureAllowed(token_t f) const
{
  return (f == Predefined::fset ||
          f == Predefined::cset ||
          f == Predefined::pattern ||
          m_fset->find(f)!=m_fset->end());
}
bool CFsetType::is_equal(CType *type) const
{
  CFsetType *ftype = cast<CFsetType>(type);
  if (!ftype || m_fset->size() != ftype->m_fset->size()) return false;
  CFset::const_iterator feat_it = ftype->m_fset->begin();
  for (uint i=m_fset->size();i;--i,++feat_it)
    if (m_fset->find(*feat_it) == m_fset->end()) return false;
  return true;
}
Fail *CFsetType::add_self(CExtFD *fd) const
{
  return fd->AddSpecial(Predefined::fset,m_fset);
}

////////////////////////////////////////////////////////////////////////////////
// FDFsetFail methods
//
FDFsetFail::FDFsetFail(Fail *e):ComplexFail(e)
{
  assert(BKClass::fail_bk() == 0);            // assert that the failure log is clean
  BKClass::add_fail_bk(Predefined::fset);     // acknowledge the failed edge
}
