#ifndef _COMPILER_GENERICSET_
#define _COMPILER_GENERICSET_

////////////////////////////////////////////////////////////////////////////////
// The following generic interface is both minimal and sufficient to implement
// cset unification.
class GenUnifyCsetIterator
{
public:
	virtual ~GenUnifyCsetIterator(){}
	virtual void advance() =0;
	virtual CFDNode *node() const =0;
};
class GenUnifyCsetComponent
{
public:
	virtual ~GenUnifyCsetComponent(){}
	virtual uint size() const =0;
	virtual GenUnifyCsetIterator& begin() const =0;
};
class GenUnifyCset
{
public:
	virtual ~GenUnifyCset(){}
	virtual const GenUnifyCsetComponent& abs() const =0;
	virtual const GenUnifyCsetComponent& base() const =0;
	virtual const GenUnifyCsetComponent& inc() const =0;
	virtual const GenUnifyCsetComponent& dec() const =0;
};

//////////////////////////////////////////////////////////////////
// Wrap CCset to comply to the above interface.
// Rationale for the name: CCset objects come from the parser, i.e.
// from the original fd (the input).
//
class COrigIterator:public GenUnifyCsetIterator
{
	CCsetComponent::const_iterator m_it;  // actual iterator of the CCset component
	mutable CFDNode *m_node;
	CExtFD *m_fd;
public:
	void init(CExtFD *fd,const CCsetComponent::const_iterator& begin)
	{
		m_fd = fd;
		m_it = begin;
		m_node = 0;
	}
	void advance()
	{
		++m_it;
		m_node=0;
	}
	CFDNode *node() const 
	{
		if (!m_node) 
		{
			m_node = m_fd->SoftPeekAddr(*m_it);
		}
		return m_node;
	}
};
class COrigComponent:public GenUnifyCsetComponent
{
	const CCsetComponent& m_component;    // actual CCset component
	mutable COrigIterator m_it;
	CExtFD *m_fd;
public:
	COrigComponent(CExtFD *fd,const GenCsetComponent& component):
		m_component(dynamic_cast<const CCsetComponent&>(component))
	{
		m_fd = fd;
	}
	uint size() const                             { return m_component.size(); }
	GenUnifyCsetIterator& begin() const           { m_it.init(m_fd,m_component.begin());return m_it; }
};
class COrigCset:public GenUnifyCset
{
	COrigComponent m_abs,m_base,m_inc,m_dec;
public:
	COrigCset(CExtFD *fd,const CCset *cset):
		m_abs(fd,cset->abs()),m_base(fd,cset->base()),m_inc(fd,cset->inc()),m_dec(fd,cset->dec()){}
	const GenUnifyCsetComponent& abs() const      { return m_abs; }
	const GenUnifyCsetComponent& base() const     { return m_base; }
	const GenUnifyCsetComponent& inc() const      { return m_inc; }
	const GenUnifyCsetComponent& dec() const      { return m_dec; }
};

//////////////////////////////////////////////////////////////////
// Wrap CCsetType to comply to the above interface.
// Rationale for the name: CCsetType objects are true type objects
// unlike CCset.
//
class CTypeIterator:public GenUnifyCsetIterator
{
	CCsetTypeComponent::const_iterator m_it;      // actual iterator of the CCsetType component
public:
	void init(const CCsetTypeComponent::const_iterator& begin)
	{ 
		m_it = begin; 
	}
	void advance()        { ++m_it; }
	CFDNode *node() const { return *m_it; }
};
class CTypeComponent:public GenUnifyCsetComponent
{
	const CCsetTypeComponent& m_component;        // actual CCsetType component
	mutable CTypeIterator m_it;
public:
	CTypeComponent(const GenCsetComponent& component):
		m_component(dynamic_cast<const CCsetTypeComponent&>(component))
	{
	}
	uint size() const
	{ 
		return m_component.size(); 
	}
	GenUnifyCsetIterator& begin() const           
	{
		m_it.init(m_component.begin());
		return m_it; 
	}
};
class CTypeCset:public GenUnifyCset
{
	CTypeComponent m_abs,m_base,m_inc,m_dec;
public:
	CTypeCset(const CCsetType *ctype):
		m_abs(ctype->abs()),m_base(ctype->base()),m_inc(ctype->inc()),m_dec(ctype->dec()){}
	const GenUnifyCsetComponent& abs() const      { return m_abs; }
	const GenUnifyCsetComponent& base() const     { return m_base; }
	const GenUnifyCsetComponent& inc() const      { return m_inc; }
	const GenUnifyCsetComponent& dec() const      { return m_dec; }
};

#endif
