#include "stdcfuf.h"
#include "c_grnode.hxx"
#include "c_griterator.hxx"
#include "c_branchnode.hxx"
#include "c_altnode.hxx"
#include "c_patnode.hxx"
#include "c_tkdot.hxx"
#include "c_grdump.hxx"
#include <algo.h>
#include <iomanip.h>
#ifdef WIN32
#include <ostream.h>
#else
#include <ostream.h>
#endif

static ostream *g_pOS;
static void DoGrammarDump(CGrNode *root,int indent);

#ifdef WIN32
using std::setw;
#endif

void GrammarDump::grdump(ostream& os,CGrNode *root)
{
  g_pOS = &os;
  DoGrammarDump(root,0);
}
void DoGrammarDump(CGrNode *root,int indent)
{
  if (node2pat_branch(root)) return;
  string str(root->str());
  if (str.size()){
    (*g_pOS) << setw(indent) << "" << str;
    CAltNode *alt = node2alt(root);
    if (alt){
      (*g_pOS) << " (" << alt->range().first << '/' << alt->ch_count() << ')';
      if (alt->params()) (*g_pOS) << ' ' << alt->params();
      (*g_pOS) << " at {" << alt->location()->GetString() << "}";
    }
    (*g_pOS) << endl;
  }
  CGrNode::ch_iterator child_i;
  for (child_i=root->ch_begin();child_i!=root->ch_end();++child_i)
    DoGrammarDump(*child_i,indent+2);
}

void GrammarDump::gr2tkdot(ostream& os,CGrNode *root)
{
  OutputTKDot::Prefix(os);                       // start the tkdot script
  ///////////////////////////////////////////////////////////////////////////////
  // List all the nodes first
  os << "$g setnodeattributes shape plaintext\n\n";
  os << "#Total nodes: " << root->count() << "\n\n";  // the number of the nodes in the grammar tree
  
  ///////////////////////////////////////////////////////////////////////////////
  // List all the nodes first
  os << "set list_of_nodes {\n";

  // the extra brackets are for VC++, which otherwise thinks that node_i is declared twice
  {
    for (CGrTreeIterator node_i(root);!node_i.is_past_the_end();node_i.advance())       // iterate the grammar tree
      os << '\t' << node_i->id() << "\t\"" << node_i->dotlabel(false) << "\" \\\n";   // output the node label and id
  }
  os << "}\n\n";
  // End of the node list
  ///////////////////////////////////////////////////////////////////////////////
  // Create the tkdot node objects from the node list
  os << "foreach {i label} $list_of_nodes {\n";
  os << "    set nodes($i) [$g addnode label $label]\n";
  os << "}\n\n";
  // End of tkdot node objects creation
  ///////////////////////////////////////////////////////////////////////////////
  // List all the edges next
  os << "set list_of_edges {\n";
  
  CGrNode::ch_iterator child_i;
  {
    for (CGrTreeIterator node_i(root);!node_i.is_past_the_end();node_i.advance())       // visit the grammar nodes again
      for (child_i=node_i->ch_begin();child_i!=node_i->ch_end();++child_i)              // iterate the children of each node
        os << '\t' << node_i->id() << '\t' << (*child_i)->id() << "\t\""              // output the edge with label and color
             << (*child_i)->edge_label() << "\"\t" << (*child_i)->edge_color() << " \\\n";
  }
  os << "}\n\n";
  // End of the edge list
  ///////////////////////////////////////////////////////////////////////////////
  // Create the tkdot edge objects from the edge list
  os << "foreach {src dst label color} $list_of_edges {\n";
  os << "    $nodes($src) addedge $nodes($dst) label $label color $color\n";
  os << "}\n\n";
  // End of tkdot edge objects creation
  ///////////////////////////////////////////////////////////////////////////////
  
  OutputTKDot::Suffix(os);                       // end the tkdot script
}

void GrammarDump::gr2dot(ostream& os,CGrNode *root)
{
  os << "digraph G\n";
  os << "{\n";
  os << "  ranksep=2;\n";
  os << "  node [shape=plaintext,label=\"\"];\n";
  
  ///////////////////////////////////////////////////////////////////////////////
  // List all the nodes with labels first
  {
    // the extra brackets are for VC++, which otherwise thinks that node_i is declared twice
    for (CGrTreeIterator node_i(root);!node_i.is_past_the_end();node_i.advance())       // iterate the grammar tree
    {  
      string label = node_i->dotlabel(false);
      if (!label.empty())
      {
        os << "  n" << node_i->id() << " [label=\"" << label << "\"];\n";   // output the node label and id
      }
    }
  }

  ///////////////////////////////////////////////////////////////////////////////
  // List all the edges next
  CGrNode::ch_iterator child_i;
  {
    for (CGrTreeIterator node_i(root);!node_i.is_past_the_end();node_i.advance())       // visit the grammar nodes again
      for (child_i=node_i->ch_begin();child_i!=node_i->ch_end();++child_i)              // iterate the children of each node
        os << "  n" << node_i->id() << " -> n" << (*child_i)->id() << " [label=\""      // output the edge with label and color
          << (*child_i)->edge_label() << "\",color=" << (*child_i)->edge_color() << "];\n";
  }
  os << "}\n";
  // End of the edge list
  ///////////////////////////////////////////////////////////////////////////////
}
