#ifndef _COMPILER_GRNODE_
#define _COMPILER_GRNODE_

#include "b_objects.hxx"

class CBareGr;

class CGrNode{
public:
  typedef vector<CGrNode *> ChildrenList;
  typedef ChildrenList::iterator ch_iterator;
  typedef pair<int,int> Range;
  friend class CBranchNode;
  friend class CAltNode;
  friend class CRaltNode;
protected:
  int m_depth;
  CGrNode *m_parent;
  CGrNode *m_grandparent;
  Range m_range;
  int m_rank;
  int m_id;
  CGrNode *m_right_sibling;
  ChildrenList m_children;
  CBareGr& m_gr;
public:
  CGrNode(int depth,CGrNode *parent,CGrNode *grandparent,int range_start,CBareGr& gr);
  virtual ~CGrNode();
  virtual CGrNode *right_sibling()      { return m_right_sibling; }
  virtual CGrNode *grandparent()        { return m_grandparent; }
  virtual CGrNode *parent()             { return m_parent; }
  virtual CGrNode *first_child()        { return m_children.empty() ? NULL : m_children[0]; }
  void connect_children();
  int depth() const                     { return m_depth; }
  int rank() const                      { return m_rank; }
  Range range() const                   { return m_range; }
  void set_range_end(int end)           { m_range.second=end; }
  ch_iterator ch_begin()                { return m_children.begin(); }
  ch_iterator ch_end()                  { return m_children.end(); }
  int ch_count() const                  { return m_children.size(); }
  bool is_leaf() const                  { return m_children.empty(); }
  int id() const                        { return m_id; }
  int count() const;
  virtual string type() const           { return "*ANONYMOUS*"; }
  virtual string edge_label() const     { return ""; }
  virtual string edge_color() const     { return "black"; }
  virtual string str() const            { return type(); }
  virtual string dotlabel(bool is_short) const;
protected:
  explicit CGrNode(CBareGr& gr);
  void construct(int depth,CGrNode *parent,CGrNode *grandparent,int range_start);
};

template <class T>
inline T *cast(CGrNode *obj)            { return dynamic_cast<T *>(obj); }

#endif
