#ifndef __CFUF_FD__
#define __CFUF_FD__

#include "u_fd.hxx"
#include "c_fdcmp.hxx"
#include "c_fdwalk.hxx"

class CFUFFD{
  
  class CExtra:public CFDExtra{
  public:
    CExtra(int i = 0)   { m_nRef = i; }
    void AddRef()       { ++m_nRef; }
    void RemRef()       { --m_nRef; }
    int GetRef() const  { return m_nRef; }
  private:
    int m_nRef;
  };
  
public:
  static ufd_t Parse(CInputStream& inputStream);
  static fd_t MakeEmpty();
  void Destroy();

  bool IsEmpty();
  token_t GetSpecialAttr();
  
  Fail *Unparse(CUnparseFD& unparser);
  Fail *Unparse(CUnparseFD& unparser,const EdgeList& lstIgnored);
  ret::ubool_t Unify(grammar_t gr);
  void PrettyPrint(char *szFDName,char *szFileName);
  
  bool IsMergeable() const   { return m_bMergeable; }
  
  CType *GetType();
  fd_t FindDescendant(const EdgeList& path);
  fd_t FindChild(token_t edge);
  pair<token_t,fd_t> GetFirstChild();
  pair<token_t,fd_t> GetNextChild();
  pair<token_t,fd_t> GetFirstParent();
  pair<token_t,fd_t> GetNextParent();

  fd_t Copy();
  void Delete();
  fd_t Cut();
  Fail *Paste(fd_t srcFD);
  bool IsEqual(fd_t fd);

  static uint GetUCount(){ return s_nUCount; }

protected:
  bool InitEmpty(CFDNode *pNode,bool bMergeable = false);

  typedef void (CFUFFD::*GetRelativeFunc)(pair<CArc_c *,CFDNode *>&);
  pair<token_t,fd_t> GetRelative(CFUFFD::GetRelativeFunc f,bool bChild);
  
  void ProtGetFirstChild(pair<CArc_c *,CFDNode *>& ch);
  void ProtGetNextChild(pair<CArc_c *,CFDNode *>& ch);
  void ProtGetFirstParent(pair<CArc_c *,CFDNode *>& par);
  void ProtGetNextParent(pair<CArc_c *,CFDNode *>& par);
  
public:
  CFDNode *GetRoot() const        { return m_pNode; }
  const EdgeList& GetPath() const { return m_path; }
  CFD *GetFD() const              { return dynamic_cast<CFD *>(m_pNode->fd()); }
  CExtra *GetFDExtra() const      { return dynamic_cast<CExtra *>(GetFD()->extra); }
  
  explicit CFUFFD(CFDNode *pRoot,CPath *pNodePath = NULL,bool bMergeable = false);
  CFUFFD(CFDNode *pNode,const EdgeList& masterPath,token_t edge,bool bChild);
  CFUFFD(CFDNode *pNodeChild,const EdgeList& masterPath,const EdgeList& path);

protected:
  ~CFUFFD();
  
  CFDNode *m_pNode;             // the root node of this FD reference
  EdgeList m_path;              // the path to the root node (needed, because there are may be more than one)
  bool m_bMergeable;            // true if this FD reference is mergeable (can be true only if referenced from a pattern)
  CFDChildIterator m_itChild;
  CFDParentIterator m_itParent;
  
  static uint s_nUCount;
};

#endif
