#include "stdcfuf.h"
#include "cfuf.hxx"
#include "cfuf_grammar.hxx"
#include "ctrlc.hxx"

#include "p_alt.hxx"
#include "p_altparams.hxx"
#include "p_special.hxx"
#include "p_pattern.hxx"
#include "p_cset.hxx"
#include "p_fset.hxx"
#include "p_control.hxx"

bool CFUFGrammar::m_bDoWait = false;
bool CFUFGrammar::m_bDoIndex = true;
bool CFUFGrammar::m_bDoDemo = true;
bool CFUFGrammar::m_bDoBKClass = true;
////////////////////////////////////////////////////////////////////////////////
// CFUFGrammar methods
//
ugrammar_t CFUFGrammar::Parse(CInputStream& inputStream)
{
  g_pTheLexer->reset();
  g_pTheInput = &inputStream;

  CGrammar *pGrammar = new CGrammar;
  try{
    CShieldCtrlC shield_ctrlc;
    ParseFD::set_grammar(pGrammar);
    ///////////////////////////////////////////////////////////////////////////////
    // Register the fset,cset and pattern features.
    // Ignore the tpattern and control features silently (just skip them).
    SpecialFeatureParser<FsetParser,CFset,CGrammar> fset_parser;         // create fset parser
    SpecialFeatureParser<CsetParser,CCset,CGrammar> cset_parser;         // create cset parser
    SpecialFeatureParser<PatternParser,CPattern,CGrammar> pattern_parser;// create pattern parser 
    SpecialFeatureParser<ControlParser,CControl,CGrammar> control_parser;// create control parser 
    ParseFD::register_feature(Predefined::fset,&fset_parser);           // register the fset feature
    ParseFD::register_feature(Predefined::cset,&cset_parser);           // register the cset feature
    ParseFD::register_feature(Predefined::pattern,&pattern_parser);     // register the pattern feature
    ParseFD::register_feature(Predefined::control,&control_parser);     // register the control feature
    
    FeatureSkipper skip;                                                // create feature skipper
    ParseFD::register_feature(Predefined::tpattern,&skip);              // register the tpattern feature (skip)
    ///////////////////////////////////////////////////////////////////////////////
    // Register alt,ralt and opt features.
    // They are the features making difference between an fd and a grammar.
    AltParser alt_parser;         // create alt parser
    RaltParser ralt_parser;       // create ralt parser
    OptParser opt_parser;         // create opt parser
    ParseFD::register_feature(Predefined::alt,&alt_parser);       // register the alt feature
    ParseFD::register_feature(Predefined::ralt,&ralt_parser);     // register the ralt feature
    ParseFD::register_feature(Predefined::opt,&opt_parser);       // register the opt feature
    ///////////////////////////////////////////////////////////////////////////////
    // Register alt parameters index and demo.
    // Ignore bk-class and wait issuing a warning (I could have done it silent too).
    // These are created static since they are independant of the client grammar and parser.
    CIndexParser parse_index;           // create index parser
    CBKClassParser parse_bkclass;       // create bk-class parser
    CDemoParser parse_demo;             // create demo parser
    CWaitParser parse_wait;             // create wait parser
    if (m_bDoIndex)   AltHeaderParser::register_param(Predefined::index,&parse_index);     // register the index parameter
    if (m_bDoWait)    AltHeaderParser::register_param(Predefined::wait,&parse_wait);       // register the wait parameter
    if (m_bDoBKClass) AltHeaderParser::register_param(Predefined::bkclass,&parse_bkclass); // register the bkclass parameter
    if (m_bDoDemo)    AltHeaderParser::register_param(Predefined::demo,&parse_demo);       // register the demo parameter
    ///////////////////////////////////////////////////////////////////////////////
    return ugrammar_t(new CFUFGrammar(pGrammar),ParseFD::parse());
  }catch(Fail *e){      // an exceptional condition (like Ctrl+C or invalid input)
    delete pGrammar;
    return e;
  }
}
void CFUFGrammar::Destroy()
{
  CDisableCtrlC disable_ctrlc;
  delete this;
}
////////////////////////////////////////////////////////////////////////////////
CFUFGrammar::CFUFGrammar(CGrammar *pGrammar)
{
  m_pGrammar = pGrammar;
}
CFUFGrammar::~CFUFGrammar()
{
  delete m_pGrammar;
}
