#include "stdcfuf.h"
#include "u_ctrv.hxx"
#include "cfuf.hxx"
#include "cfuf_misc.hxx"
#include <iostream.h>

#ifdef WIN32
using std::cout;
#endif

class COStream:public COutputStream
{
	ostream& m_os;
public:
	explicit COStream(ostream& os):m_os(os){}
	COutputStream& operator<<(const string& str)    { m_os << str;return *this; }
	COutputStream& operator<<(const char *str)      { m_os << str;return *this; }
	COutputStream& operator<<(char c)               { m_os << c;return *this; }
	COutputStream& operator<<(long l)               { m_os << l;return *this; }
	COutputStream& operator<<(const CMyObject& obj) { m_os << obj.toString();return *this; }
	COutputStream& operator<<(const CMyObject *obj) { m_os << (obj ? obj->toString() : "NULL");return *this; }
	void flush()                                    { m_os.flush(); }
};

////////////////////////////////////////////////////////////////////////////////
// CFUFMisc methods
//
void CFUFMisc::EnablePathSplit(bool bEnable = true)
{
  g_pTheEnv->m_bSplitPaths = bEnable;
}
void CFUFMisc::EnableCorrectGiven(bool bEnable = true)
{
  g_pTheEnv->m_bCorrectGiven = bEnable;
}
bool CFUFMisc::IsPathSplitEnabled()
{
  return g_pTheEnv->m_bSplitPaths;
}
bool CFUFMisc::IsCorrectGivenEnabled()
{
  return g_pTheEnv->m_bCorrectGiven;
}
void CFUFMisc::SetSilent(bool bOn)
{
  if (bOn) g_pTheOutput = &g_nullOutput;     // redirect all the error output to the null device
  else{
    static COStream coutStream(cout);
    g_pTheOutput = &coutStream;
  }
}
bool CFUFMisc::IsSilent()
{
  return g_pTheOutput == &g_nullOutput;
}
void CFUFMisc::SetTraceLevel(uint level)
{
  g_pTheEnv->m_nTraceLevel = level;
}
uint CFUFMisc::GetTraceLevel()
{
  return g_pTheEnv->m_nTraceLevel;
}
void CFUFMisc::SetULimit(ushort limit)
{
  g_pTheEnv->SetMaxUCount(limit);
}
uint CFUFMisc::GetULimit()
{
  return g_pTheEnv->GetMaxUCount();
}
void CFUFMisc::RegisterCategoriesNotUnified(const TokenList& lstCategories)
{
  UnregisterCategoriesNotUnified();
  CConstituentTraversor::s_not_unified.insert(lstCategories.begin(),lstCategories.end());
}
void CFUFMisc::UnregisterCategoriesNotUnified()
{
  CConstituentTraversor::s_not_unified.erase(CConstituentTraversor::s_not_unified.begin(),
                                             CConstituentTraversor::s_not_unified.end());
}
void CFUFMisc::UnregisterCategoriesNotUnified(const TokenList& lstCategories)
{
  TokenList::const_iterator it = lstCategories.begin();
  for (uint i=lstCategories.size();i;--i,++it)
    CConstituentTraversor::s_not_unified.erase(*it);
}
void CFUFMisc::GetCategoriesNotUnified(TokenList& lstCategories)
{
#ifdef WIN32
  lstCategories.reserve(lstCategories.size() + CConstituentTraversor::s_not_unified.size());

  TokenHash::const_iterator itNotUnified = CConstituentTraversor::s_not_unified.begin();
  for (uint i = CConstituentTraversor::s_not_unified.size();i;--i,++itNotUnified)
  {
	  lstCategories.push_back(*itNotUnified);
  }
#else
	lstCategories.insert(lstCategories.end(),
							CConstituentTraversor::s_not_unified.begin(),
							CConstituentTraversor::s_not_unified.end());
#endif
}
