#include "stdcfuf.h"
#include "cfuf.hxx"
#include "cfuf_fd.hxx"
#include "cfuf_special.hxx"

#include "p_objects.hxx"
#include "c_pattern.hxx"
#include "c_cset.hxx"
#include "c_fset.hxx"
#include "c_fdnode2path.hxx"
#include "ctrlc.hxx"

////////////////////////////////////////////////////////////////////////////////
// CFUFSpecial::Pattern methods
//
CFUFSpecial::Pattern::Pattern()
{
  m_arrValues = 0;
  m_nCount = 0;
}
void CFUFSpecial::Pattern::Delete()
{
  delete m_arrValues;
  m_arrValues = 0;
  m_nCount = 0;
}
CFUFSpecial::Pattern *CFUFSpecial::Pattern::Init(CType *type)
{
  CPatternType *ptype = dynamic_cast<CPatternType *>(type);
  if (!ptype) return 0;
  
  Delete();
  
  m_nCount = ptype->values().size();
  m_arrValues = new fd_t[m_nCount];
  PatternValues::const_iterator p_it = ptype->values().begin();
  
  for (uint i=0;i<m_nCount;++i,++p_it){
    CFDNode *pNode = (*p_it)->node();   // returns NULL on dots
    m_arrValues[i] = pNode ?
      new CFUFFD(pNode,Node2Path::get_abs_path(pNode),(*p_it)->is_mergeable()) : 0;
  }
  
  return this;
}

////////////////////////////////////////////////////////////////////////////////
// CFUFSpecial::Cset methods
//
CFUFSpecial::Cset::Cset()
{
  for (uint i = 0;i<4;++i){
    m_arrValues[i] = 0;
    m_nCount[i] = 0;
  }
}
void CFUFSpecial::Cset::Delete()
{
	for (uint i = 0;i<4;++i)
	{
		delete m_arrValues[i];
		m_arrValues[i] = 0;
		m_nCount[i] = 0;
	}
}
CFUFSpecial::Cset *CFUFSpecial::Cset::Init(CType *type)
{
	CCsetType *ctype = dynamic_cast<CCsetType *>(type);
	if (!ctype) return 0;

	Delete();

	GetComponent(ctype->abs(),Cset::ABS);
	GetComponent(ctype->base(),Cset::BASE);
	GetComponent(ctype->inc(),Cset::INC);
	GetComponent(ctype->dec(),Cset::DEC);
	return this;
}
void CFUFSpecial::Cset::GetComponent(const GenCsetComponent& _c,uint what)
{
	m_nCount[what] = _c.size();
	if (!m_nCount[what]) return;

	const CCsetTypeComponent& c = dynamic_cast<const CCsetTypeComponent&>(_c);

	m_arrValues[what] = new fd_t[m_nCount[what]];
	CCsetTypeComponent::const_iterator c_it = c.begin();
	for (uint i=0;i<m_nCount[what];++i,++c_it) 
	{
		m_arrValues[what][i] = new CFUFFD(*c_it,Node2Path::get_abs_path(*c_it));
	}
}

////////////////////////////////////////////////////////////////////////////////
// CFUFSpecial::Fset methods
//
CFUFSpecial::Fset::Fset()
{
  m_arrFeatures = 0;
  m_nCount = 0;
}
void CFUFSpecial::Fset::Delete()
{
  delete m_arrFeatures;
  m_arrFeatures = 0;
  m_nCount = 0;
}
CFUFSpecial::Fset *CFUFSpecial::Fset::Init(CType *type)
{
  CFsetType *ftype = dynamic_cast<CFsetType *>(type);
  if (!ftype) return 0;
  
  Delete();
  
  CFset *const fset = ftype->GetFset();
  m_nCount = fset->size();
  m_arrFeatures = new token_t[m_nCount];
  CFset::const_iterator f_it = fset->begin();
  
  for (uint i=0;i<m_nCount;++i,++f_it) m_arrFeatures[i] = *f_it;
  return this;
}

////////////////////////////////////////////////////////////////////////////////
// CFUFSpecial methods
//
CFUFSpecial::Pattern *CFUFSpecial::GetPattern(CType *type)
{
  CDisableCtrlC disable_ctrlc;
  static Pattern s_pattern;
  return s_pattern.Init(type);
}
CFUFSpecial::Cset *CFUFSpecial::GetCset(CType *type)
{
  CDisableCtrlC disable_ctrlc;
  static Cset s_cset;
  return s_cset.Init(type);
}
CFUFSpecial::Fset *CFUFSpecial::GetFset(CType *type)
{
  CDisableCtrlC disable_ctrlc;
  static Fset s_fset;
  return s_fset.Init(type);
}
