ifndef CFUF_CONFIG
CFUF_CONFIG=debug
endif
CFG_FILE=../config.$(shell uname)/$(CFUF_CONFIG)

ifneq ($(shell test -f $(CFG_FILE) ; echo $$?),0)

.DEFAULT:
	@echo "The config file $(CFG_FILE) is not found!"
	@exit 1

else

include $(CFG_FILE)

SCMDIR=../scm5b3/
INITDIR=../../scm/toplevel/
C=gcc
OBJDIR=$(OBJ_ROOT)scm5b3/$(CFG)/
BINDIR=$(PRODUCT_ROOT)$(CFG)/
SRC=scm.c continue.c crs.c ecrt0.c edline.c eval.c findexec.c gsubr.c ioext.c \
	posix.c ramap.c record.c repl.c rope.c sc2.c scl.c script.c socket.c \
	subr.c sys.c time.c unif.c unix.c
OBJ=$(SRC:%.c=$(OBJDIR)%.o)
INITS=-DINITS='init_ramap();init_gsubr();init_crs();init_socket();init_posix();init_unix();init_sc2();init_record();init_ioext();init_cfuf()'
COMPILED_INITS=-DCOMPILED_INITS='init_edline()'
COMPILE_C=$(C) $(CFLAGS) -c $(INC) $(DEFINES)
TARGET=$(BINDIR)libscm.a
DEP_FILE=$(OBJDIR)/Dependencies

.PHONY: clean dirs

ifeq ($(shell test -f $(DEP_FILE) ; echo $$?),0)

VPATH=$(shell cd $(SCMDIR);pwd)/

all: dirs $(TARGET)

NEWFLAGS=$(OBJDIR)newflags.h
$(SCMDIR)scmflags.h: FORCE
	@echo Making $@
	@echo '#define IMPLINIT "$(shell cd $(INITDIR);pwd)/Init.scm"' > $(NEWFLAGS)
	@echo '#define CAUTIOUS' >> $(NEWFLAGS)
	@echo '#define BIGNUMS' >> $(NEWFLAGS)
	@echo '#define ARRAYS' >> $(NEWFLAGS)
	@echo '#define FLOATS' >> $(NEWFLAGS)
	@echo '#define ENGNOT' >> $(NEWFLAGS)
	@echo '#define CCLO' >> $(NEWFLAGS)
	@echo '#define ENGNOT' >> $(NEWFLAGS)
	@echo '#define MACRO' >> $(NEWFLAGS)
	@echo '#define TICKS' >> $(NEWFLAGS)
	@-if (diff $(NEWFLAGS) $(SCMDIR)scmflags.h) then rm $(NEWFLAGS); \
	else mv $(NEWFLAGS) $(SCMDIR)scmflags.h; fi

$(TARGET): $(SCMDIR)scmflags.h $(OBJ)
	rm -f $@
	ar crv $@ $^

_%: %.c
	$(COMPILE_C) $< -o /dev/null

$(OBJDIR)%.o: %.c
	$(COMPILE_C) $< -o $@

$(OBJDIR)scm.o: scm.c
	$(COMPILE_C) $(INITS) $(COMPILED_INITS) $< -o $@

FORCE:

include $(DEP_FILE)

else

.DEFAULT:
	@echo 'Run "make dep"'
	@exit 1

endif

dep: dirs
	(cd $(SCMDIR);makedepend -w128 -p'$$(OBJDIR)' -Y $(INC) $(DEFINES) $(SRC) -f- 1> $(DEP_FILE) 2> /dev/null)

dirs:
	mkdir -p $(OBJDIR) $(BINDIR)

clean:
	rm -rf $(OBJDIR) $(TARGET)

endif
