#include "stdcfuf.h"
#include "b_debug_tracer.hxx"
#include "b_altparams.hxx"
#include "p_altparams.hxx"
#include <hash_set.h>
#include <algo.h>
#include <function.h>
#include "b_bkclass.hxx"

AltHeaderParser::ParamRegistry AltHeaderParser::s_reg;
/////////////////////////////////////////////////////////////////////////////////
// AltHeaderParser methods
//
void AltHeaderParser::register_param(token_t key,AltParamParser *parser)
{
  s_reg[key] = parser;  // the previous parser is overriden with the new one
}
CAltParams *AltHeaderParser::parse_alt_header() throw (Fail *)
{
  enum {START_,COLON_,AFTER_NAME_,AFTER_PARAM_,PARAM_VALUE_,PARAM_KEY_,CLOSE_PARAM_,END_,ERROR_};
  token_t token;
  int state=START_;
  AltParamParser *param_parser=NULL;
  auto_ptr<CAltParams> the_params(new CAltParams(Predefined::anonymous));
  
  while (state<END_ && (g_pTheLexer->get(token))){
    if (token==Delim::percent){
      while ((g_pTheLexer->get(token)) && token!=Delim::percent);
      state=AFTER_PARAM_;
      continue;
    }
    switch(state){
    case START_:
      if (token==Delim::open_paren) state=COLON_;
      else if (Delim::is_delim(token)) state=ERROR_;
      else{             // assume that the name is given
        state=AFTER_NAME_;
        the_params->m_name = token;
      }
      break;
    case AFTER_NAME_:
      if (token==Delim::ws);
      else if (token==Delim::open_paren) state=COLON_;  // expect for the next parameter
      else if (token==Delim::close_paren){    // the assumption that the name is given is wrong
        g_pTheLexer->putback(token);                // put back the last two tokens
        g_pTheLexer->putback(the_params->m_name);
        the_params->m_name = Predefined::anonymous; // reset the name to anonymous
        state = END_;                           // and finish (no more parameters are given)
      }else{                                   // the only parameter given was the name of the alt
        g_pTheLexer->putback(token);                // put back the lookahead token and finish
        state=END_;
      }
      break;
    case COLON_:
      if (token==Delim::ws);
      else if (token==Delim::colon) state=PARAM_KEY_;
      else{
        g_pTheLexer->putback(token);
        g_pTheLexer->putback(Delim::open_paren);
        state=END_;
      }
      break;
    case PARAM_KEY_:                                    // the current token represents the key
      {
        ParamRegistry::iterator found = s_reg.find(token);// get the param entry in the parse map
        if (found==s_reg.end()){                        // if such a parameter is not registered
          (*g_pTheOutput) << "Warning: an unregistered parameter " << token << " near \n<\n" << g_pTheLexer->errlog() << "\n>\n";
          BaseParser::skip_sexpr(true);                 // skip the value of the unregistered parameter
          state=AFTER_PARAM_;
        }else{                                          // the parameter is registered
          param_parser=found->second;              // get its parser 
          state=PARAM_VALUE_;
        }
      }
      break;
    case PARAM_VALUE_:
      if (token==Delim::ws){
        param_parser->parse_alt_param(the_params.get());
        state=CLOSE_PARAM_;
      }else state=ERROR_;
      break;
    case CLOSE_PARAM_:
      state = (token==Delim::close_paren) ? AFTER_PARAM_ : ERROR_;
      break;
    case AFTER_PARAM_:
      if (token==Delim::ws);
      else if (token==Delim::open_paren) state=COLON_;
      else{
        g_pTheLexer->putback(token);
        state=END_;
      }
      break;
    }
  }
  if (state!=END_) throw new ParserFail;
  return the_params.release();
}

/////////////////////////////////////////////////////////////////////////////////
// AltParamParser methods
//
void AltParamParser::already_registered(token_t key)
{
  (*g_pTheOutput) << "Warning: multiple occurrences of the registered parameter " << key
              << " near \n<\n" << g_pTheLexer->errlog() << "\n>\n";
  BaseParser::skip_sexpr(false);
}

/////////////////////////////////////////////////////////////////////////////////
// CIndexParser methods
//
void CIndexParser::parse_alt_param(CAltParams *params) throw (Fail *)
{
  if (params->m_index.empty()){
    m_params = params;
    BaseParser::parse_list(this,false,true);
  }else already_registered(Predefined::index);
}
Fail *CIndexParser::parse_list_elem(token_t first)
{
  if (Delim::is_delim(first)) throw new ParserFail;
  m_params->m_index.push_back(first);
  return 0;
}

/////////////////////////////////////////////////////////////////////////////////
// CBKClassParser methods
//
void CBKClassParser::parse_alt_param(CAltParams *params) throw (Fail *)
{
  if (params->m_bkclasses) already_registered(Predefined::bkclass);
  else{
    m_params = params;
    BaseParser::parse_list(this,false,true);
  }
}
Fail *CBKClassParser::parse_list_elem(token_t first)
{
  if (Delim::is_delim(first)) throw new ParserFail;
  uint64 mask = BKClass::bkclass2mask(first);
  if (mask) m_params->m_bkclasses |= mask;
  else (*g_pTheOutput) << "Warning: unknown bk class " << first
                   << " near \n<\n" << g_pTheLexer->errlog() << "\n>\n";
  return 0;
}

/////////////////////////////////////////////////////////////////////////////////
// CDemoParser methods
//
void CDemoParser::parse_alt_param(CAltParams *params) throw (Fail *)
{
  if (params->m_demo) already_registered(Predefined::demo);
  else{
    token_t token;
    if (!g_pTheLexer->get(token) || token!=Delim::double_quote ||
        !g_pTheLexer->get(token)) throw new ParserFail();
    if (token!=Delim::double_quote){
      params->m_demo=token;
      if (!g_pTheLexer->get(token) || token!=Delim::double_quote) throw new ParserFail();
    }else params->m_demo=Predefined::null;
  }
}
