#include "stdcfuf.h"
#include "p_external.hxx"
#include "p_typeparsers.hxx"

CAddr *BaseParser::parse_addr() throw (Fail *)
{
  token_t token;

  if (!g_pTheLexer->get(token)) throw new ParserFail;
  if (token==Delim::open_curly){
    CPathParser path_parser;
    return path_parser.parse();
  }
  if (Delim::is_delim(token)) throw new ParserFail;
  return new CArc(token);
}
CType *BaseParser::parse_type() throw (Fail *)
{
  static CAtomTypeParser atom_parser;
  static CUnderTypeParser under_parser;
  static CQuotedTypeParser quoted_parser;
  token_t token;
  CNormalTypeParser *normal_parser;

  if (!g_pTheLexer->get(token)) throw new ParserFail;
  if (token==Delim::bar) normal_parser = &under_parser;
  else if (token==Delim::double_quote) normal_parser = &quoted_parser;
  else if (!Delim::is_delim(token)){
    atom_parser.set_token(token);
    normal_parser = &atom_parser;
  }else throw new ParserFail;
  return normal_parser->parse();
}
CMyObject *BaseParser::parse_value() throw (Fail *)
{
  token_t token;

  if (!g_pTheLexer->get(token)) throw new ParserFail;
  if (token==Delim::open_curly){
    CPathParser path_parser;
    return path_parser.parse();
  }
  g_pTheLexer->putback(token);
  return parse_type();
}
void BaseParser::skip_sexpr(bool close_expr) throw (Fail *)
{
  int c,balance=1;      // I assume that one opening parenthesis has already been read

  while (g_pTheLexer->get(c)){
    if (c=='(') ++balance;
    else if (c==')') --balance;
    else if (c==EOF) break;
    if (!balance){      // reached the end of the expression
      if (!close_expr)          // if not asked to close the expression
        g_pTheLexer->unget();       // return the closing parenthesis back to the stream
      return;
    }
  }
  throw new ParserFail;
}
Fail *BaseParser::parse_list(ListElemParser *parser,bool allow_empty,bool allow_one,token_t sol,token_t eol)
{
  enum {FIRST_,SPACE_,NEXT_,ERROR_};
  token_t token;
  int state=FIRST_;

  if (!g_pTheLexer->get(token)) throw new ParserFail;

  if (allow_one && token != sol)
    return parser->parse_list_elem(token);

  if (token!=sol) throw new ParserFail;
  
  while (state < ERROR_ && g_pTheLexer->get(token)){
    if (token == eol){
      if (allow_empty) return 0;
      break;
    }
    switch(state){
    case SPACE_:
      state = (token==Delim::ws) ? NEXT_ : ERROR_;
      break;
    case NEXT_:
      if (Fail *f = parser->parse_list_elem(token)) return f;
      allow_empty=true;
      state=SPACE_;
      break;
    case FIRST_:
      if (token==Delim::ws);
      else{
        if (Fail *f = parser->parse_list_elem(token)) return f;
        allow_empty=true;
        state=SPACE_;
      }
      break;
    }
  }
  throw new ParserFail;
}
