#include "stdcfuf.h"
#include "p_objects.hxx"
#include "p_external.hxx"
#include "p_typeparsers.hxx"

/////////////////////////////////////////////////////////////////////////////////
// CPathParser methods
//
CPath *CPathParser::parse() throw (Fail *)
{
  enum {START_,AFTER_ROOF_,AFTER_ROOF2_,AFTER_TITLE_,TITLE_,END_,ERROR_};
  token_t token;
  int state=START_;
  CPath *path=new CPath;
  int upcount = 0;

  while (state<END_ && g_pTheLexer->get(token))
    switch(state){
    case START_:
      if (token==Delim::roof) state=AFTER_ROOF_;
      else if (token==Delim::close_curly) state=END_;
      else if (token==Delim::ws);
      else if (Delim::is_delim(token)) state=ERROR_;
      else{
        path->push_back(token);
        state=AFTER_TITLE_;
      }
      break;
    case AFTER_ROOF_:
      if (token==Delim::ws){
        ++upcount;
        state=START_;
      }else if (token==Delim::close_curly){
        ++upcount;
        state=END_;
      }else{
        char *str=TOK2STR(token);
        char *endptr;
        int n=strtol(str,&endptr,10);
        if (*endptr) state=ERROR_;
        else{
          upcount+=n;
          state=AFTER_ROOF2_;
        }
      }
      break;
    case AFTER_ROOF2_:
      if (token==Delim::ws) state=START_;
      else if (token==Delim::close_curly) state=END_;
      else state=ERROR_;
      break;
    case AFTER_TITLE_:
      if (token==Delim::ws) state=TITLE_;
      else if (token==Delim::close_curly) state=END_;
      else state=ERROR_;
      break;
    case TITLE_:
      if (Delim::is_delim(token)) state=ERROR_;
      else{
        path->push_back(token);
        state=AFTER_TITLE_;
      }
      break;
    }
  if (state!=END_){
    delete path;
    throw new ParserFail;
  }
  if (upcount) path->set_upcount(upcount);
  return path;
}

/////////////////////////////////////////////////////////////////////////////////
// CAtomTypeParser methods
//
CType *CAtomTypeParser::parse() throw (Fail *)
{
  if (m_token == Predefined::none) return g_pTheEnv->none();
  if (m_token == Predefined::nil) return g_pTheEnv->nil();
  if (m_token == Predefined::any) return g_pTheEnv->any();
  if (m_token == Predefined::given) return g_pTheEnv->given();
  return new CAtomType(m_token);
}

/////////////////////////////////////////////////////////////////////////////////
// CUnderTypeParser methods
//
CType *CUnderTypeParser::parse() throw (Fail *)
{
  enum {START_,BAR_,SPACE_,TITLE_,CLOSE_,END_,ERROR_};
  enum {UNDER_,SUNDER_,EXTERNAL_};
  token_t token;
  int state=START_;
  int nTypeKind;
  token_t tknTitle;
  
  while (state<END_ && g_pTheLexer->get(token))
    switch(state){
    case START_:
      state = token==Delim::open_paren ? BAR_ : ERROR_;
      break;
    case BAR_:
      if (token==Delim::ws);
      else{
        state=SPACE_;
        if (token == Predefined::external) nTypeKind=EXTERNAL_;
        else if (token == Predefined::under || token == Predefined::le) nTypeKind=UNDER_;
        else if (token == Predefined::sunder || token == Predefined::lt) nTypeKind=SUNDER_;
        else state=ERROR_;
      }
      break;
    case SPACE_:
      state = (token==Delim::ws) ? TITLE_ :  ERROR_;
      break;
    case TITLE_:
      if (Delim::is_delim(token)) state=ERROR_;
      else{
        tknTitle=token;
        state=CLOSE_;
      }
      break;
    case CLOSE_:
      state = (token==Delim::close_paren) ? END_ :  ERROR_;
      break;
    }
  if (state!=END_) throw new ParserFail;
  if (nTypeKind == EXTERNAL_) return new CExternalType(tknTitle);
  // Guard against such types as #(under nil),#(under any), etc...
  if (tknTitle == Predefined::none) return g_pTheEnv->none();
  if (tknTitle == Predefined::nil) return g_pTheEnv->nil();
  if (tknTitle == Predefined::any) return g_pTheEnv->any();
  if (tknTitle == Predefined::given) return g_pTheEnv->given();
  return (nTypeKind == UNDER_) ? new CUnderType(tknTitle) : new CStrictUnderType(tknTitle);
}

/////////////////////////////////////////////////////////////////////////////////
// CQuotedTypeParser methods
//
CType *CQuotedTypeParser::parse() throw (Fail *)
{
  token_t token;
  token_t title;
  
  if (!g_pTheLexer->get(token)) throw new ParserFail;
  if (token!=Delim::double_quote){
    title=token;
    if (!g_pTheLexer->get(token) || token!=Delim::double_quote) throw new ParserFail;
  }else title=Predefined::null;
  return new CQuotedType(title);
}

/////////////////////////////////////////////////////////////////////////////////
// ParserFail methods
//
ParserFail::ParserFail():m_errlog(g_pTheLexer->errlog()){}
string ParserFail::GetDesc() const
{
  string str;
  return str << "Parser exception near: \n<\n" << m_errlog << "\n>\n";
}
