#ifndef _PARSER_OBJECTS_
#define _PARSER_OBJECTS_

#include "b_objects.hxx"
#include "b_lexer.hxx"
#include <hash_map.h>

class ParserFail:public Fail{
  string m_errlog;
public:
  ParserFail();
  bool IsAbort() const { return true; }
  string GetDesc() const;
};

class CPathParser{
public:
  CPath *parse() throw (Fail *);
};

// Emulate BaseParser namespace (gcc 2.8.1 does not support the keyword namespace)
struct BaseParser{
  static CAddr *parse_addr() throw (Fail *);      // parses an address (arc or path)
  static CType *parse_type() throw (Fail *);      // parses a normal type
  static CMyObject *parse_value() throw (Fail *); // parses a value (path or type)
  static void skip_sexpr(bool close_expr) throw (Fail *);

  struct ListElemParser{                        // parses a list element
    virtual ~ListElemParser(){}
    virtual Fail *parse_list_elem(token_t first) =0;
  };

  static Fail *
  parse_list(ListElemParser *parser,// list element parser
             bool allow_empty,      // if true an empty list is not an error
             bool allow_one,        // if true a list of one element w/o the {end,start}-of-list tokens is not an error
             token_t =Delim::open_paren,      // start-of-list token
             token_t =Delim::close_paren);    // end-of-list token
};
/////////////////////////////////////////////////////////////////////////////////

#endif
