#include "stdcfuf.h"
#include "b_debug_tracer.hxx"
#include "p_external.hxx"
#include "p_parser.hxx"
#include <assert.h>

inline void skip_opt_space(token_t& token)
{
  if (g_pTheLexer->get(token) && token==Delim::ws) g_pTheLexer->get(token);
}

ParseFD::Registry ParseFD::s_reg;
ParseFD::Graph ParseFD::s_graph;
// in LISP-FUF (n === m) is translated into (n ((lex "m")))
// but in (n ((lex m))) m is not quoted
bool ParseFD::s_quotify = false;

/////////////////////////////////////////////////////////////////////////////////
// ParseFD methods
//
void ParseFD::register_feature(token_t feature,FeatureParser *parser)
{
  s_reg[feature] = parser;
}
void ParseFD::unregister_feature(token_t feature)
{
  s_reg.erase(feature);
}
Fail *ParseFD::parse()
{
  Fail *f = 0;
  
  g_pTheEnv->m_bInput = (dynamic_cast<CGrammarBuilder *>(s_graph.fd) == 0);
  f = s_graph.fd->Initialize();
  if (!f) f = parse_value();
  if (!f) f = s_graph.fd->Realize();
  return f;
}
Fail *ParseFD::parse_no_init()
{
  return parse_value();
}
Fail *ParseFD::parse_fd()
{
  enum {ATTR_VALUE_,END_,ERROR_};
  token_t token;
  int state=ATTR_VALUE_;
  
  while (state<END_ && (g_pTheLexer->get(token))){
    if (token==Delim::percent){
      while ((g_pTheLexer->get(token)) && token!=Delim::percent);
      continue;
    }
    Fail *f = 0;
    if (token==Delim::ws);
    else if (token==Delim::open_paren) f = parse_attr_value();
    else state = (token==Delim::close_paren) ? END_ : ERROR_;
    if (f) return f;
  }
  if (state!=END_) throw new ParserFail;
  return 0;
}
Fail *ParseFD::parse_attr_value()
{
  enum {ATTR_,VALUE_,CLOSE_,END_,ERROR_};
  token_t token;
  int state=ATTR_;
  hCAddr addr;

  while (state<END_ && (g_pTheLexer->get(token)))
    switch(state){
    case ATTR_:
      if (token!=Delim::ws) g_pTheLexer->putback(token);
      addr.reset(BaseParser::parse_addr());
      state = VALUE_;
      break;
    case VALUE_:
      if (token==Delim::ws){
        Fail *f = 0;
        if (obj2path(addr)){
          hCPath path(cast_h2h<CPath>(addr));
          s_graph.fd->EnterSubgraph(path->upcount());
          if (path->empty()) f = parse_value();
          else{
            CPath::const_iterator it = path->begin();
            uint j = 0;
            for (uint i = path->size() - 1;i && !f;--i,++it,++j) f = s_graph.fd->EnterSubgraph(*it);
            if (!f) f = parse_value(*it);
            if (!f) for (;j;--j) s_graph.fd->LeaveSubgraph();
          }
          if (!f) s_graph.fd->LeaveSubgraph();
        }else{
          CArc *arc = obj2arc(addr);
          assert(arc);
          f = parse_value(arc->title());
        }
        if (f) return f;
        state=CLOSE_;
      }else state=ERROR_;
      break;
    case CLOSE_:
      state = (token==Delim::close_paren) ? END_ : ERROR_;
      break;
    }
  if (state!=END_) throw new ParserFail;
  return 0;
}
Fail *ParseFD::parse_value(token_t edge)
{
  Fail *f = 0;
  token_t token;
  
  Registry::iterator found = s_reg.find(edge);          // get the feature entry in the parse map
  if (found!=s_reg.end()){                              // if the feature is registered as special
    FeatureParser *parser=found->second;           // get the registered parser
    return parser->parse_feature(edge);                 // and parse the feature value
  }
  if (!g_pTheLexer->get(token)) throw new ParserFail;       // no special feature found, continue as usual
  if (token==Delim::open_paren){                        // expecting a subfd
    if (!g_pTheLexer->get(token)) throw new ParserFail;     // fetch the next token
    if (token != Delim::close_paren){                   // if this is not ')'
      g_pTheLexer->putback(token);                          // return it back to the token stream
      f = s_graph.fd->EnterSubgraph(edge);             // enter the subgraph
      if (!f) f = parse_fd();                           // parse it
      if (!f) s_graph.fd->LeaveSubgraph();             // leave it
    }
  }else if (token == Predefined::lex_shortcut){         // a lex shortcut is met - translate (a === b) into (a ((lex b)))
    if ((f = s_graph.fd->EnterSubgraph(edge))) return f;
    if (!g_pTheLexer->get(token) || token!=Delim::ws) throw new ParserFail;
    s_quotify = true;
    f = parse_value(Predefined::lex);
    s_quotify = false;
    if (!f) s_graph.fd->LeaveSubgraph();
  }else if (token==Delim::tilde){                       // the feature is actually a list
    skip_opt_space(token);
    if (!g_pTheLexer->ok() || token!=Delim::open_paren) throw new ParserFail;
    f = parse_list(edge);                               // parse the list
  }else{                                                // otherwise a normal feature with a normal value is encountered
    g_pTheLexer->putback(token);
    f = add_value(edge,BaseParser::parse_value());      // parse the value (either path, type or external) and add to the graph
  }
  return f;
}
Fail *ParseFD::parse_list(token_t edge)
{
  Fail *f = 0;
  token_t token;

  if (!g_pTheLexer->get(token)) throw new ParserFail;
  if (token==Delim::close_paren)
    f = s_graph.fd->AddLeaf(edge,g_pTheEnv->none());
  else{
    if (token!=Delim::ws) g_pTheLexer->putback(token);
    f = s_graph.fd->EnterSubgraph(edge);
    if (!f) f = parse_value(Predefined::car);
    if (!f) f = parse_list(Predefined::cdr);
    if (!f) s_graph.fd->LeaveSubgraph();
  }
  return f;
}
Fail *ParseFD::parse_value()
{
  Fail *f = 0;
  token_t token;
  
  if (!g_pTheLexer->get(token)) throw new ParserFail;   // no special feature found, continue as usual
  if (token==Delim::open_paren) f = parse_fd();     // expecting a subfd - parse it
  else if (token==Delim::tilde){                    // the feature is actually a list
    skip_opt_space(token);
    if (!g_pTheLexer->ok() || token!=Delim::open_paren) throw new ParserFail;
    f = parse_list();                               // parse the list
  }else{                                            // otherwise a normal feature with a normal value is encountered
    g_pTheLexer->putback(token);
    f = add_value(BaseParser::parse_value());       // parse the value (either path, type or external) and add to the graph
  }
  return f;
}
Fail *ParseFD::parse_list()
{
  Fail *f = 0;
  token_t token;

  if (!g_pTheLexer->get(token)) throw new ParserFail;
  if (token==Delim::close_paren)
    f = s_graph.fd->AddLeaf(g_pTheEnv->none());
  else{
    if (token!=Delim::ws) g_pTheLexer->putback(token);
    f = parse_value(Predefined::car);
    if (!f) f = parse_list(Predefined::cdr);
  }
  return f;
}
Fail *ParseFD::eval_external(CExternalType *ext)
{
  Fail *f = 0;
  const EdgeList *elist = s_graph.fd->cur_loc();   // get the edge list representing the current location
  assert(elist);
  string expr;
  expr << '(' << ext->GetExternalName() << " '(" << elist->GetString() << "))";
  CInputStream *pExtInputStream = g_pTheInterp->EvalExtern(expr.c_str());      // evaluate the external function
  if (pExtInputStream){
    CSwitchInputStream switchStream(pExtInputStream);
    f = parse_value();
    delete pExtInputStream;
  }
  delete ext;                                   // delete the external object
  return f;
}
Fail *ParseFD::add_value(token_t edge,CMyObject *value)
{
  Fail *f = 0;
  CPath *_path = obj2path(value);
  if (_path){
    hCPath path(_path);
    f = s_graph.fd->AddPath(edge,path);
  }else if (CExternalType *ext = obj2external(value)){  // the value is actually an external reference
    f = s_graph.fd->EnterSubgraph(edge);                // enter the external subfd
    if (!f) f = eval_external(ext);                     // evaluate the external function and parse the return value
    if (!f) s_graph.fd->LeaveSubgraph();                // leave the external subfd
  }else{
    hCType type(cast<CType>(value));
    assert(type.pointee());
    if (type->nil()) return 0;
    if (!type->quoted() && s_quotify)
      type.reset(new CQuotedType(cast<CNormalType>(value)->title()));
    f = s_graph.fd->AddLeaf(edge,type);
  }
  return f;
}
Fail *ParseFD::add_value(CMyObject *value)
{
  Fail *f = 0;
  CPath *_path = obj2path(value);
  if (_path){
    hCPath path(_path);
    f = s_graph.fd->AddPath(path);
  }else if (CExternalType *ext = obj2external(value))   // the value is actually an external reference
    f = eval_external(ext);                             // evaluate the external function and parse the return value
  else{
    hCType type(cast<CType>(value));
    assert(type.pointee());
    if (type->nil()) return 0;
    f = s_graph.fd->AddLeaf(type);
  }
  return f;
}
