#ifndef _PARSER_FD_
#define _PARSER_FD_

#include "p_objects.hxx"
#include "b_graphbase.hxx"
#include <hash_map.h>

class FeatureParser{
public:
  virtual ~FeatureParser(){}
  virtual Fail *parse_feature(token_t) =0;
};

class FeatureSkipper:public FeatureParser{
public:
  Fail *parse_feature(token_t){
    BaseParser::skip_sexpr(false);
    return 0;
  }
};

class CExternalType;
class CGrammarBuilder;

struct ParseFD{

  static void set_fd(CFDBuilder *fd)            { s_graph.fd = fd; }
  static void set_grammar(CGrammarBuilder *gr)  { s_graph.gr = gr; }

  static CFDBuilder *fd()                       { return s_graph.fd; }
  static CGrammarBuilder *grammar()             { return s_graph.gr; }
  
  static Fail *parse();
  static Fail *parse_no_init();
  static void register_feature(token_t,FeatureParser *);
  static void unregister_feature(token_t);
  
private:
  typedef hash_map<token_t,FeatureParser *,hashtoken,eqtokens> Registry;
  union Graph{
    CFDBuilder *fd;
    CGrammarBuilder *gr;
    Graph():fd(0){}
  };
  static Registry s_reg;
  static Graph s_graph;
  static bool s_quotify;
  enum {OK_,ERROR_,FATAL_};
private:
  static Fail *parse_fd();
  static Fail *parse_attr_value();
  static Fail *parse_value(token_t);
  static Fail *parse_list(token_t);
  static Fail *parse_value();
  static Fail *parse_list();
  static Fail *add_value(token_t,CMyObject *);
  static Fail *add_value(CMyObject *);
  static Fail *eval_external(CExternalType *ext);
};

#endif
