#include "stdcfuf.h"
#include "c_patnode.hxx"
#include "t_base.hxx"
#include "t_branch.hxx"
#include "b_graphbase.hxx"

auto_ptr<BranchInfo> Tracer_l2::m_binfo;

/////////////////////////////////////////////////////////////////////////////////
// Tracer_l2 methods
//
void Tracer_l2::init(BranchInfo *binfo)
{
  m_binfo.reset(binfo);
}
void Tracer_l2::accept_b()
{
  CGrNode *alt = m_binfo->m_cur_branch->parent();
  if (!alt) return;
  if (node2pat_alt(alt)){
    CPatBranchNode *pat_branch = node2pat_branch(m_binfo->m_cur_branch);
    msg_pat_b("Trying",node2pat_alt(alt),pat_branch,"->");
    CConstraintIterator cur_c(pat_branch->pattern()->constraint_iterator());
    for (;!cur_c.is_past_the_end();cur_c.advance()){
      Formatter::prompt(' ',"",5);
      (*g_pTheOutput) << "Adding constraint: " << cur_c->str() << ENDL;
    }
  }else{
    token_t demo = node2alt(alt)->params()->demo();
    if (demo){
      (*g_pTheOutput) << "\n    \"" << demo << "\"\n    ";
      for (char *ptr = TOK2STR(demo);*ptr;++ptr) (*g_pTheOutput) << '-';
      (*g_pTheOutput) << ENDL;
    }
    msg_b("Entering",node2alt(alt),m_binfo->m_cur_branch->rank(),"->");
  }
}
void Tracer_l2::fail_b()
{
  CGrNode *alt = m_binfo->m_cur_branch->parent();
  if (!alt) return;
  if (node2pat_alt(alt)) msg_pat_b("Failed",node2pat_alt(alt),node2pat_branch(m_binfo->m_cur_branch),"->");
  else msg_b("Failed",node2alt(alt),m_binfo->m_cur_branch->rank(),"->");
}
void Tracer_l2::success_b(CGrNode *branch)
{
  CGrNode *alt = branch->parent();
  if (!alt) return;
  if (node2pat_alt(alt)) msg_pat_b("Success with",node2pat_alt(alt),node2pat_branch(branch),"<-");
  else msg_b("Success at",node2alt(alt),branch->rank(),"<-");
}
void Tracer_l2::bkprune_b()
{
  CGrNode *alt = m_binfo->m_cur_branch->parent();
  if (!alt) return;
  if (node2pat_alt(alt)) msg_pat_b("Pruned",node2pat_alt(alt),node2pat_branch(m_binfo->m_cur_branch),"BK");
  else msg_b("Pruned",node2alt(alt),m_binfo->m_cur_branch->rank(),"BK");
}
void Tracer_l2::msg_b(char *status,CAltNode *alt,int brank,char *prompt)
{
  Formatter::indent(alt->depth());
  Formatter::prompt('-',prompt);
  (*g_pTheOutput) << status << ' ' << alt->str() << ' ' << alt->params()->name()
              << " (" << long(alt->range().first) << '/' << long(alt->ch_count())
              << ") branch #" << long(brank) << ENDL;
}
void Tracer_l2::msg_pat_b(char *status,CPatAltNode *alt,CPatBranchNode *branch,char *prompt)
{
  CPatternType *ptype = branch->pattern();
  assert(ptype);
  Formatter::prompt('-',prompt);
  (*g_pTheOutput) << status << " pattern " << ptype << " (" << long(alt->range().first) << ", #" << long(branch->rank()) << ")\n";
}
void Tracer_l2::report_failure(Fail *e)
{
  Formatter::prompt(' ',"",5);
  (*g_pTheOutput) << e->GetDesc() << ENDL;
}
void Tracer_l2::report_wait(const CWaitElem *welem,CFDBuilder *fd)
{
  Formatter::prompt(' ',"WT");
  (*g_pTheOutput) << '<' << welem->str() << "> wait condition is not satisfied at {" << fd->cur_loc()->GetString() << "}\n";
}
void Tracer_l2::freeze_b(CAltNode *alt)
{
  wt_msg_b("Freezing",alt);
}
void Tracer_l2::melt_b(EdgeList *elist,CAltNode *alt)
{
  const char *delim = (elist->empty() || alt->location()->empty()) ? "" : " ";
  Formatter::prompt(' ',"WT");
  (*g_pTheOutput) << "Melting " << alt->str() << ' ' << alt->params()->name()
              << " (" << long(alt->range().first) << '/' << long(alt->ch_count()) << ") at {"
              << elist->GetString() << delim << alt->location()->GetString() << "}\n";
}
void Tracer_l2::unmelt_b(CAltNode *alt)
{
  wt_msg_b("Unmelting",alt);
}
void Tracer_l2::unfreeze_b(CAltNode *alt)
{
  wt_msg_b("Unfreezing",alt);
}
void Tracer_l2::wt_msg_b(char *status,CAltNode *alt)
{
  Formatter::prompt(' ',"WT");
  (*g_pTheOutput) << status << ' ' << alt->str() << ' ' << alt->params()->name()
              << " (" << long(alt->range().first) << '/' << long(alt->ch_count()) << ")\n";
}
void Tracer_l2::scanfreezer_b(bool bForceUnmelt)
{
  Formatter::prompt(' ',"WT");
  (*g_pTheOutput) << (bForceUnmelt ? "Forcing any frozen alts to melt:\n" : "Looking for frozen alts to melt:\n");
}
