#include "stdcfuf.h"
#include "u_fd.hxx"
#include "u_gr.hxx"
#include "c_fdnode2path.hxx"
#include "t_constituent.hxx"
#include "t_base.hxx"

auto_ptr<ConstituentInfo> Tracer_l1::m_cinfo;
uint Tracer_l1::m_standby_count0;
uint Tracer_l1::m_standby_count1;
uint Tracer_l1::m_standby_count2;

/////////////////////////////////////////////////////////////////////////////////
// Tracer_l1 methods
//
void Tracer_l1::init(ConstituentInfo *cinfo)
{
  m_cinfo.reset(cinfo);
}
void Tracer_l1::start_acquire_subs()
{
  m_standby_count0=m_cinfo->m_standby_queue.size();
  m_standby_count1=m_standby_count2=0;
}
void Tracer_l1::cset_acquired()
{
  m_standby_count1=m_cinfo->m_standby_queue.size();
}
void Tracer_l1::cat_acquired()
{
  m_standby_count2=m_cinfo->m_standby_queue.size();
}
void Tracer_l1::end_acquire_subs()
{
  Formatter::rewind();
  Formatter::prompt(' ',"EXPAND");
  if (m_cinfo->m_standby_queue.size() == m_standby_count0)
    (*g_pTheOutput) << "Constituent " << '{' << m_cinfo->m_cur_elist->GetString() << '}' << " is a leaf.\n";
  else{
    (*g_pTheOutput) << "Expanded " << '{' << m_cinfo->m_cur_elist->GetString() << '}' << " into:\n";
    StandByQueue::const_iterator it = m_cinfo->m_standby_queue.begin();
    it+=m_standby_count0;
    uint i;
    for (i=m_standby_count0;i<m_cinfo->m_standby_queue.size();++i,++it){
      Formatter::prompt(' ',"EXPAND",9);
      (*g_pTheOutput) << '{' << it->elist->GetString() << '}' << ENDL;
    }
    Formatter::rewind();
    Formatter::prompt(' ',"EXPAND");
    (*g_pTheOutput) << "Cset/Cat/Pattern contribution: [";
    if (m_standby_count2==0) (*g_pTheOutput) << long(i-m_standby_count0) << "/0/0]\n";
    else (*g_pTheOutput) << long(m_standby_count1-m_standby_count0) << '/'
                     << long(m_standby_count2-m_standby_count1) << '/'
                     << long(i-m_standby_count2) << "]\n";
  }
}
void Tracer_l1::any_failed(CFDNode *any_node)
{
  (*g_pTheOutput) << "Any count is non zero (" << Node2Path::get_abs_path(any_node) << ").Retreating...\n";
}
void Tracer_l1::msg_c(char *status)
{
  CFDNode *cat_node = m_cinfo->m_cur_cnode->FindChild(Predefined::cat);
  string cat_type = cat_node ? cat_node->data().GetType()->toString() : "NIL";
  Formatter::rewind();
  (*g_pTheOutput) << ">\n>*** " << status << " cat " << cat_type << " at level "
              << '{' << m_cinfo->m_cur_elist->GetString() << '}' << " ***<\n>\n";
}
void Tracer_l1::skip_c()
{
  CFDNode *cat_node = m_cinfo->m_cur_cnode->FindChild(Predefined::cat);
  string cat_type = cat_node ? cat_node->data().GetType()->toString() : "NIL";
  string cur_elist(m_cinfo->m_cur_elist->GetString());
  Formatter::rewind();
  (*g_pTheOutput) << ">\n>*** Cat " << cat_type << " at level {"
              << cur_elist << "} is not unified. ***<\n>\n\n[EXPAND]:Constituent {"
              << cur_elist << "} is a leaf.\n";
}
