#ifndef _TRACE_CONSTITUENT_
#define _TRACE_CONSTITUENT_

#include "u_ctrv.hxx"

typedef CConstituentTraversor::DoneStack DoneStack;
typedef CConstituentTraversor::StandByQueue StandByQueue;
typedef EdgeList *EdgeListPtr;
typedef CFDNode *CFDNodePtr;

struct ConstituentInfo{
  const DoneStack& m_done_stack;
  const StandByQueue& m_standby_queue;
  const EdgeListPtr& m_cur_elist;
  CFDNodePtr& m_cur_cnode;
  ConstituentInfo(const DoneStack& done_stack,
                  const StandByQueue& standby_queue,
                  const EdgeListPtr& cur_elist,
                  CFDNodePtr& cur_cnode):
    m_done_stack(done_stack),
    m_standby_queue(standby_queue),
    m_cur_elist(cur_elist),
    m_cur_cnode(cur_cnode){}
};

////////////////////////////////////////////////////////////////////////
// Rsponsible for tracing of constituent traversal
struct Tracer_l1{
public:
  static void init(ConstituentInfo *cinfo);
  static void skip_c();
  static void accept_c()       { msg_c("Trying"); }
  static void fail_c()         { msg_c("Failed"); }
  static void retry_c()        { msg_c("Retrying"); }
  static void start_acquire_subs();             // start of the subconstituent acquisition
  static void cset_acquired();                  // done processing cset contribution
  static void cat_acquired();                   // done processing cat feature contribution
  static void end_acquire_subs();               // done processing pattern contribution - finished
  static void any_failed(CFDNode *any_node);    // report the ANY failure
private:
  static void msg_c(char *status);
private:
  static auto_ptr<ConstituentInfo> m_cinfo;
  static uint m_standby_count0;
  static uint m_standby_count1;
  static uint m_standby_count2;
};

#endif
