#ifndef __TIMER__
#define __TIMER__

#ifdef WIN32

#include <sys/timeb.h>

#ifndef WIN32_LEAN_AND_MEAN
struct FILETIME 
{
	unsigned int dwLowDateTime;
	unsigned int dwHighDateTime;
};
#endif

#define FACTOR 1000

#else

#include <sys/times.h>
#include <sys/param.h>
#include <unistd.h>

#ifndef HZ
#define FACTOR 60
#else
#define FACTOR HZ
#endif

#endif

class CTimer
{
	struct Time
	{
		long last;
		long total;
		Time()
		{
			Reset();
		}
		void Reset()
		{
			last = total = 0;
		}
		void CancelLast()
		{
			total -= last;
			last = 0;
		}
	};
	Time m_real;
	Time m_run;
#ifdef WIN32
	struct _timeb m_start;
	union UFileTime
	{
		FILETIME fileTime;
		unsigned __int64 quadPart;
		FILETIME *operator&(){ return &fileTime; }
		UFileTime& operator+(const UFileTime& rhs)
		{
			quadPart += rhs.quadPart;
			return *this;
		}	
		UFileTime& operator-(const UFileTime& rhs)
		{
			quadPart -= rhs.quadPart;
			return *this;
		}
	};
	UFileTime m_kernelTimeStart,m_userTimeStart;
#else
	struct tms m_start;
#endif
public:
	void Reset()
	{
		m_real.Reset();
		m_run.Reset();
	}
	void CancelLast()
	{
		m_real.CancelLast();
		m_run.CancelLast();
	}
	void Go();
	void Stop();
public:
	double GetTotalElapsedReal() const { return double(m_real.total)/FACTOR; }
	double GetLastElapsedReal() const  { return double(m_real.last)/FACTOR; }
	double GetTotalElapsedRun() const  { return double(m_run.total)/FACTOR; }
	double GetLastElapsedRun() const   { return double(m_run.last)/FACTOR; }
};

#endif
