#include "stdcfuf.h"

// #define F_TRACE
// #define MANUAL_C_SORT
#define AUTO_C_SORT
// #define COMPLEX_SORT
// #define NO_C_SORT
// #define NO_WAIT

#include "c_debug.hxx"
#include "b_debug_tracer.hxx"
#include "u_fd.hxx"
#include "c_baregr.hxx"
#include "c_pattern.hxx"
#include "u_btrv.hxx"
#include "u_ctrv.hxx"
#include "c_cset.hxx"
#include "c_fdnode2path.hxx"
#include "t_tracer_l1.hxx"
#ifndef NO_WAIT
#include "u_freezer.hxx"
#endif

#ifdef WIN32
using std::cout;
#endif

inline bool IsCategory(CFDNode *node)
{
  if (!node) return false;
  const hCType& type = node->data().GetType();
  return type->final() && !type->none();
}
inline bool HasGroundChildren(CFDNode *node)
{
  hCType type = node->data().GetType();
  if (!type->internal()) return false;          // the node must be internal
  ret::utype_t u=StaticUnify::unify_i_g(node);  // unify the node with GIVEN
  if (!u.fail) return true;                     // return true if succeeded
  delete u.fail;                                // else - the node has no ground children
  return false;                                 // return false
}
#ifdef AUTO_C_SORT
inline bool comp(const CConstituentTraversor::ConstituentStandBy& c1,
                 const CConstituentTraversor::ConstituentStandBy& c2)
{
  CFDNode *catNode1 = c1.node->FindChild(Predefined::cat);
  CFDNode *catNode2 = c2.node->FindChild(Predefined::cat);
  bool bCat1 = IsCategory(catNode1);
  bool bCat2 = IsCategory(catNode2);
#ifdef COMPLEX_SORT
  if (bCat1 && bCat2){
    return catNode1->group_tstamp() < catNode2->group_tstamp();
  }else if (bCat1 == bCat2){
    return c1.node->group_tstamp() < c2.node->group_tstamp();
  }else return bCat1 && !bCat2;
#else
  return bCat1 == bCat2 && c1.node->group_tstamp() < c2.node->group_tstamp() || bCat1 && !bCat2;
#endif
}
#endif

TokenHash CConstituentTraversor::s_not_unified;
/////////////////////////////////////////////////////////////////////////////////
// CConstituentTraversor::ConstituentStandBy methods
//
void CConstituentTraversor::ConstituentStandBy::destroy()
{
  F_TRACE0();
  
  assert(elist);
  delete elist;
  elist = 0;
  assert(node->data().GetMark());
  node->data().Unmark();
}

/////////////////////////////////////////////////////////////////////////////////
// CConstituentTraversor::StandByQueue methods
//
CConstituentTraversor::StandByQueue::~StandByQueue()
{
  F_TRACE0();
  
  while (size()){
    back().destroy();
    pop_back();
  }
}

/////////////////////////////////////////////////////////////////////////////////
// CConstituentTraversor::ConstituentDone methods
//
void CConstituentTraversor::ConstituentDone::destroy()
{
  F_TRACE0();

#ifdef NO_REGISTER_NOT_UNIFIED
  assert(branch_trv);
#endif
  assert(elist);
  assert(node->data().GetMark());
  // cout << "@@@ " << node->representative() << ',' << Node2Path::get_abs_path(node) << endl;
  node->data().Unmark();
  delete elist;
  delete branch_trv;
  elist = 0;
  branch_trv = 0;
}

/////////////////////////////////////////////////////////////////////////////////
// CConstituentTraversor::DoneStack methods
//
CConstituentTraversor::DoneStack::~DoneStack()
{
  F_TRACE0();
  
  while (size()){
    top().destroy();
    pop();
  }
}

/////////////////////////////////////////////////////////////////////////////////
// CConstituentTraversor::CAcquireSubs methods
//
#ifdef NO_C_SORT
static deque<CConstituentTraversor::ConstituentStandBy> cats,rest;
static bool put_cats = false;
void print_subc(deque<CConstituentTraversor::ConstituentStandBy>& subs,int n)
{
  F_TRACE1(nsubs);
  
  deque<CConstituentTraversor::ConstituentStandBy>::const_iterator it = subs.end()-n;
  for (int i=n;i;--i,++it) cerr << " - {" << it->elist->str() << "}\n";
}
#elif defined(AUTO_C_SORT)
void CConstituentTraversor::CAcquireSubs::print_subc(int nsubs)
{
	F_TRACE1(nsubs);

	StandByQueue::const_iterator it = m_standby_queue.end()-nsubs;
	for (int i=0;i<nsubs;++i,++it)
	{
		const EdgeList *pElist = it->elist;
		if (pElist->begin() == pElist->end()) 
		{
			cout << " - {}\n";
		}
		else
		{
			cout << " - {";
			copy(pElist->begin(),pElist->end()-1,ostream_iterator<token_t>(cout," "));
			cout << pElist->back() << "}\n";
		}
	}
}
#else
void CConstituentTraversor::CAcquireSubs::print_subc(int){}
#endif
int CConstituentTraversor::CAcquireSubs::operator()()
{
  F_TRACE0();
  
  DBG_TRACE1(Tracer_l1::start_acquire_subs());
  int nsubs = acquire();
#ifdef MANUAL_C_SORT
  if (nsubs > 1){
    vector<ConstituentStandBy> recent(nsubs);
    int i;
    for (i=nsubs-1;i>=0;--i){
      recent[i] = m_standby_queue.back();
      m_standby_queue.pop_back();
    }
    char order[10];
    for (i=0;i<nsubs;++i){
      order[i]='1'+i;
      cout << order[i] << ". {";
      copy(recent[i].elist->begin(),recent[i].elist->end(),ostream_iterator<token_t>(cout," "));
      cout << "\b}\n";
    }
    order[i]='\0';
    cout << "Choose the order of the subconstituents [" << order << "]: " << flush;
    cin.get(order,10,'\n');cin.ignore(100,'\n');
    if (!*order){
      for (i = 0;i<nsubs;++i) order[i]='1'+i;
      order[i]='\0';
    }
    for (i=0;i<nsubs;++i) m_standby_queue.push_back(recent[order[i]-'1']);
  }
#elif defined(AUTO_C_SORT)
  if (nsubs > 1){
    // cout << "Before reordering:\n";
    // print_subc(nsubs);
    sort(m_standby_queue.end()-nsubs,m_standby_queue.end(),comp);
    // cout << "After reordering:\n";
    // print_subc(nsubs);
  }
#elif defined(NO_C_SORT)
  while (cats.size()){
    m_standby_queue.push_back(cats.front());
    cats.pop_front();
  }
  while (rest.size()){
    m_standby_queue.push_back(rest.front());
    rest.pop_front();
  }
#endif
  DBG_TRACE1(Tracer_l1::end_acquire_subs());
  return nsubs;
}
int CConstituentTraversor::CAcquireSubs::acquire()
{
  F_TRACE0();
  
  CGroupData *gdata;
  int count=0;
  vector<bool> visited(m_cur_cnode->fd()->count(),false);       // Initialize the map of visited nodes

  CFDNode *cset_node=m_cur_cnode->FindChild(Predefined::cset);// get the cset node
  if (cset_node){                                       // and proceed if it exists at all
    CCsetType *ctype=cast<CCsetType>(cset_node->data().GetType());
    if (ctype){
      if (ctype->empty()) return 0;
      count=cset_take(ctype->abs(),visited);            // acquire the absolute component of the cset
      if (ctype->abs().size()) return count;          // if exists - done.
      cset_ignore(ctype->dec(),visited);                // rule out the decrement component
      count+=cset_take(ctype->inc(),visited);           // acquire the increment component
      count+=cset_take(ctype->base(),visited);          // add the base component
      if (ctype->base().size()) return count;         // if it exists - done, else continue to implicit cset
    }
  }

  DBG_TRACE1(Tracer_l1::cset_acquired());
  
  ///////////////////////////////////////////////////////////////
  // Get implicit cset which is all the children having the cat feature
  // plus all the internal nodes mentioned in the pattern.Order is crucial.
  // First we must look for the former and only afterwards for the latter.
#ifdef NO_C_SORT
  put_cats = true;
#endif
  // Find all the children having the cat feature.
  vector<int> save_i;
  save_i.reserve(100);
  for (CFDChildIterator it(m_cur_cnode);!it.is_past_the_end();it.advance()){
    CFDNode *node = it->second;
    gdata  = &(node->data());
    if (!visited[gdata->id()]){                 // if the node is encountered the first time
      visited[gdata->id()]=true;                // remember the node is visited (we might see it again in the same loop)
      if (!gdata->GetMark()){                  // if the node is not already selected
        if (IsCategory(node->FindChild(Predefined::cat))){ // if the node has a category feature
          put_on_standby(gdata,node,it->first); // select the node and save on the standby queue
          ++count;
        }else save_i.push_back(gdata->id());    // save the index (will give it another try in the pattern section)
      }
    }
  }
  while (save_i.size()){                        // Forget the visited entries for the nodes ruled out because they
    visited[save_i.back()]=false;               // lacked the cat feature.They might get lucky in the pattern section.
    save_i.pop_back();                       
  }
#ifdef NO_C_SORT
  put_cats = false;
#endif
  
  DBG_TRACE1(Tracer_l1::cat_acquired());
  
  // Find all the internal nodes mentioned in the pattern.
  CFDNode *pat_node=m_cur_cnode->FindChild(Predefined::pattern);  // get the pattern node
  if (pat_node){                                                // and proceed if it exists at all
    CPatternType *ptype=cast<CPatternType>(pat_node->data().GetType());
    if (ptype){
      for (CPatternIterator cl_it(ptype);!cl_it.is_past_the_end();cl_it.advance()){
        CFDNode *node = cl_it->node();
        gdata  = &(node->data());
        if (!visited[gdata->id()]){         // if the node is encountered the first time
          visited[gdata->id()]=true;        // remember the node is visited and
          if (!gdata->GetMark() &&          // if the node is not already selected and
              HasGroundChildren(node)){     // it has ground children
            put_on_standby(gdata,node,Node2Path::get_abs_path(node));  // select it and save on the standby queue
            ++count;
          }
        }
      }
    }
  }
  return count;         // return the number of found subconstituents
}
int CConstituentTraversor::CAcquireSubs::cset_take(const GenCsetComponent& _set,vector<bool>& visited)
{
  F_TRACE0();

  const CCsetTypeComponent& set = dynamic_cast<const CCsetTypeComponent&>(_set);

  CGroupData *gdata;
  int count=0;
  
  for (CCsetTypeComponent::const_iterator it=set.begin();it!=set.end();++it){
    CFDNode *node = *it;
    gdata  = &(node->data());           // get the group data of the node
    if (!visited[gdata->id()]){         // if the node is encountered the first time
      visited[gdata->id()]=true;        // remember the node is visited and
      if (!gdata->GetMark() &&          // if the node is not already selected and
          HasGroundChildren(node)){     // it has ground children
        put_on_standby(gdata,node,Node2Path::get_abs_path(node)); // select it and save on the standby queue
        ++count;
      }
    }
  }
  return count;
}
void CConstituentTraversor::CAcquireSubs::cset_ignore(const GenCsetComponent& _set,vector<bool>& visited)
{
  F_TRACE0();
  
  const CCsetTypeComponent& set = dynamic_cast<const CCsetTypeComponent&>(_set);

  CGroupData *gdata;
  for (CCsetTypeComponent::const_iterator it=set.begin();it!=set.end();++it){
    gdata  = &((*it)->data());         // get the group data of the node
    if (!visited[gdata->id()])         // if the node is encountered the first time
      visited[gdata->id()]=true;       // remember the node is visited and
  }
}
void CConstituentTraversor::CAcquireSubs::put_on_standby(CGroupData *gdata,CFDNode *node,CArc *arc)
{
  F_TRACE3(gdata,node,arc);
  
  gdata->Mark();  // mark the node as a future constituent
  EdgeList *elist=get_new_elist(arc,m_cur_elist);             // get the absolute path to the constituent
#ifdef NO_C_SORT
  if (put_cats) cats.push_back(ConstituentStandBy(node,elist));  // put it to standby queue
  else rest.push_back(ConstituentStandBy(node,elist));  // put it to standby queue
#else
  m_standby_queue.push_back(ConstituentStandBy(node,elist));  // put it to standby queue
#endif
}
void CConstituentTraversor::CAcquireSubs::put_on_standby(CGroupData *gdata,CFDNode *node,CPath *path)
{
  F_TRACE3(gdata,node,path);
  
  gdata->Mark();  // mark the node as a future constituent
  EdgeList *elist=get_new_elist(path,m_cur_elist);            // get the absolute path to the constituent
#ifdef NO_C_SORT
  if (put_cats) cats.push_back(ConstituentStandBy(node,elist));  // put it to standby queue
  else rest.push_back(ConstituentStandBy(node,elist));  // put it to standby queue
#else
  m_standby_queue.push_back(ConstituentStandBy(node,elist));  // put it to standby queue
#endif
}
EdgeList *CConstituentTraversor::CAcquireSubs::get_new_elist(CArc *arc,EdgeList *elist)
{
  F_TRACE1(arc);
  
  EdgeList *new_elist=new EdgeList;
  new_elist->reserve(elist->size()+1);
  copy(elist->begin(),elist->end(),back_inserter(*new_elist));
  new_elist->push_back(arc->title());
  return new_elist;
}
EdgeList *CConstituentTraversor::CAcquireSubs::get_new_elist(CPath *path,EdgeList *elist)
{
  F_TRACE1(path);

  assert(path->upcount() == CPath::ABS);
  return new EdgeList(*path);
}

/////////////////////////////////////////////////////////////////////////////////
// CConstituentTraversor methods
//
CConstituentTraversor::CConstituentTraversor(CFDNode *node,CBareGr *gr):
  m_fd(dynamic_cast<CFD *>(node->fd())),
  m_groot(gr->root()),
  m_acquire_subs(m_cur_cnode,m_cur_elist,m_standby_queue)
{
  F_TRACE0();
  
  node->data().Mark();        // mark it as a constituent node (the first one)
  DBG_TRACE1(Tracer_l1::init(new ConstituentInfo(m_done_stack,m_standby_queue,m_cur_elist,m_cur_cnode)));
  accept(node,new EdgeList,0);
}
/*
  The done stack and the standby queue will see to the proper destruction of their contents.
  The only constituent which requires manual deletion is the current one.
*/ 
CConstituentTraversor::~CConstituentTraversor()
{
  F_TRACE0();
  
  m_cur_cnode->data().Unmark();
  delete m_cur_elist;
  delete m_cur_branch_trv;
#ifndef NO_WAIT
  Freezer::Init();
#endif
}
ustatus_t CConstituentTraversor::accept(CFDNode *node,EdgeList *elist,CGrNode *branch)
{
  F_TRACE0();
  
  m_cur_cnode=node;
  m_cur_elist=elist;
  m_fd->go_constituent_loc(m_cur_elist);
#ifndef NO_REGISTER_NOT_UNIFIED
  if (!is_unified(m_cur_cnode)){
    m_cur_branch_trv = 0;
    DBG_TRACE1(Tracer_l1::skip_c());
    return finish_cur_constituent();
  }
#endif
  m_cur_branch_trv = new CBranchTraversor(m_fd,branch ? branch : m_groot);
  DBG_TRACE1(Tracer_l1::accept_c());
  return OK;
}
ustatus_t CConstituentTraversor::proceed()
{
  F_TRACE0();
  DUMP_FD(m_fd);
  ustatus_t status = m_cur_branch_trv->proceed();
#ifndef NO_WAIT
  Freezer::Cell *melting;
#endif
  
  for (;;)
    switch (status){
    case OK:
      return OK;
    case FINISHED:
#ifndef NO_WAIT
      if (m_cur_branch_trv->top_alt())              // true when the constituent was build from a melted alt
        Freezer::CommitLastMelted();                // transit from ALT_MELTING to ALT_MELTED
      melting = Freezer::GetReady(m_fd,false);
      if (melting)      // if an alt was melted, create for it a subconstituent and insert it in the off order fashion
        m_standby_queue.push_front(ConstituentStandBy(melting->c_node,      // the constituent base of the melted alt
                                                      melting->c_elist,     // the path to the constituent
                                                      melting->alt->first_child()));    // the first grammar branch
      status = finish_cur_constituent();
      if (status == FORCE_MELT) break;
      return status;
#else
      return finish_cur_constituent();
#endif
    case FAILED:
      return fail_cur_constituent();
#ifndef NO_WAIT
    case FROZEN:
      Freezer::Freeze(m_cur_cnode,m_cur_elist);
      status = m_cur_branch_trv->freeze();
      break;
    case FORCE_MELT:
      melting = Freezer::GetReady(m_fd,true);
      if (melting){
          m_done_stack.push(ConstituentDone(0,m_cur_branch_trv,m_cur_cnode,m_cur_elist));
          rewind();                                     // return from the current constituent location to the base
          return accept(melting->c_node,                // the constituent base of the melted alt
                        melting->c_elist,               // the path to the constituent
                        melting->alt->first_child());   // the first grammar branch
      }
      return finish_ctrv();
#endif
    default:
      assert(false);
      return DUMMY;
    }
}
/*
  If the unification has successfully ended m_done_stack contains all the
  successful constituents but the current one (i.e. m_cur_cnode, m_cur_elist, etc ...
  are not in the the stack).
*/
ustatus_t CConstituentTraversor::finish_cur_constituent()
{
  F_TRACE0();
  
  assert(m_cur_cnode->data().GetMark());

  // Push all the subconstituents onto the standby queue.
  // With the exception when m_cur_branch_trv is 0. In this case the constituent was skipped
  // (registered as not unified).
  int nsubs = m_cur_branch_trv ? m_acquire_subs() : 0;
  ConstituentStandBy next;
  if (!m_standby_queue.empty()){             // there are constituents in standby
    next=m_standby_queue.front();            // pick the first constituent from standby
    m_standby_queue.pop_front();             // remove it from the standby queue
    m_done_stack.push(ConstituentDone(nsubs,m_cur_branch_trv,m_cur_cnode,m_cur_elist));
    rewind();                                // return from the current constituent location to the base
    return accept(next.node,next.elist,next.branch); // make it the current constituent
  }
#ifndef NO_WAIT
  return FORCE_MELT;
#else
  return finish_ctrv();
#endif
}
/*
  The unification has finished, but there may be ANY nodes left behind.
  If there are - fail and retreat.
*/
ustatus_t CConstituentTraversor::finish_ctrv()
{
  CFDNode *any_node=m_fd->any_stack().top();  // get the last ANY node (if any)
  if (any_node){                              // if exists
    DBG_TRACE1(Tracer_l1::any_failed(any_node));// report the failure to the tracer
    return retreat();                         // force failure of the current constituent
  }
  rewind();                                   // return from the current constituent location to the base
  return FINISHED;                            // indicate success if there are no any nodes
}
/*
  If the unification has failed the done stack and the standby queue are empty.
  The current constituent is the root constituent.
*/
ustatus_t CConstituentTraversor::fail_cur_constituent()
{
  F_TRACE0();
  
  DUMP_FD(m_fd);
  DBG_TRACE1(Tracer_l1::fail_c());

#ifndef NO_WAIT
  // m_cur_branch_trv->top_alt() is not NULL when the constituent was build from a melted alt
  bool bMelted = m_cur_branch_trv ? bool(m_cur_branch_trv->top_alt()) : false;
  if (bMelted)                  // if a constituent was build from a melted alt (i.e. not a normal subconstituent)
    Freezer::UnmeltLast();      // undo the melting (back to freezer)
#endif
    
  rewind();                                     // return from the current constituent location to the base
  if (m_done_stack.empty()) return FAILED;      // exit if no more constituents to retry
  ConstituentDone prev = m_done_stack.top();    // else - pick the last one visited from the done stack
  m_done_stack.pop();                           // remove it from the done stack
  
  assert(m_cur_cnode->data().GetMark());
  
#ifndef NO_WAIT
  if (!bMelted)                                 // if the current is a normal subconstituent return it to the
#endif
    m_standby_queue.push_front(ConstituentStandBy(m_cur_cnode,m_cur_elist));  // standby queue (where it belongs)
  for (int i=prev.sub_count;i;--i,m_standby_queue.pop_back()){  // The now being retried constituent has added some
    ConstituentStandBy subc = m_standby_queue.back();           // subconstituents when it was finished.Now that it
    subc.destroy();                                             // from the standby queue
  }
  // must be careful not to use m_cur_elist, since subc.destroy()
  // might have destroyed it
  
  prev2cur(prev);               // make the retried constituent to be the current one
  return retreat();
}
void CConstituentTraversor::prev2cur(const ConstituentDone& prev)
{
  F_TRACE0();
  
  m_cur_cnode=prev.node;                // restore the previous constituent
  m_cur_elist=prev.elist;               // and its location
  delete m_cur_branch_trv;              // delete the failed branch traversor
  m_cur_branch_trv=prev.branch_trv;     // restore the previous one
  
  assert(m_cur_cnode->data().GetMark());
  DBG_TRACE1(Tracer_l1::retry_c());
  
  m_fd->go_constituent_loc(m_cur_elist);// position the FD at the constituent base

#ifndef NO_REGISTER_NOT_UNIFIED
  if (!m_cur_branch_trv) return;        // retreating a skipped constituent
#endif
  
  m_cur_branch_trv->wake_up();          // notify the BTM
  
#ifndef NO_WAIT
  if (m_cur_branch_trv->top_alt() != 0) // true when the constituent was build from a melted alt
    Freezer::UncommitLastMelted();      // undo the melting (back to freezer)
#endif
}
ustatus_t CConstituentTraversor::retreat()
{
  F_TRACE0();

  if (m_cur_branch_trv && m_cur_branch_trv->retreat() == OK) return OK;
  return fail_cur_constituent();
}
bool CConstituentTraversor::is_unified(CFDNode *cnode)
{
  CFDNode *catnode = cnode->FindChild(Predefined::cat); // get the cat child of the node
  if (!catnode) return true;                    // ok if not found
  CType *type = catnode->data().GetType();     // get the value of the cat feature
  if (!type->final()) return true;
  CNormalType *normal = cast<CNormalType>(type);
  assert(normal);
  return s_not_unified.find(normal->title()) == s_not_unified.end();
}
void CConstituentTraversor::rewind() const
{
  m_fd->ungo_constituent_loc(m_cur_elist);
  assert(m_fd->root() == m_fd->top_node());     // TEMP
}
void CConstituentTraversor::ungo_loc() const
{
  m_cur_branch_trv->ungo_loc();
  rewind();
  assert(m_fd->root() == m_fd->top_node());     // TEMP
}
