#ifndef _UNIFY_CTRV_
#define _UNIFY_CTRV_

#include "u_base.hxx"
#include "u_btrv.hxx"

class EdgeList;
class CFDNode;
class CFD;
class CGrNode;
class CBareGr;
class CBranchTraversor;
class GenCsetComponent;
class CGroupData;

class CConstituentTraversor{
public:
  struct ConstituentStandBy{
    CFDNode *node;              // the constituent base
    EdgeList *elist;            // the path from the FD root to the constituent base
    CGrNode *branch;            // the top branch (0 if the Grammar root branch, else - the first branch of the melted alt)
    ConstituentStandBy():node(0),elist(0),branch(0){}
    ConstituentStandBy(CFDNode *_node,EdgeList *_elist,CGrNode *_branch = 0){
      node = _node;
      elist = _elist;
      branch = _branch;
    }
    void destroy();
  };
  struct StandByQueue:public deque<ConstituentStandBy>{
    ~StandByQueue();
  };
  struct ConstituentDone{
    CFDNode *node;              // the constituent base
    EdgeList *elist;            // the path from the FD root to the constituent base
    int sub_count;              // the number of subconstituents
    CBranchTraversor *branch_trv;       // the BTM
    ConstituentDone(int _sub_count,CBranchTraversor *_branch_trv,CFDNode *_node,EdgeList *_elist){
      node = _node;
      elist = _elist;
      sub_count = _sub_count;
      branch_trv = _branch_trv;
    }
    void destroy();
  };
  struct DoneStack:public stack<ConstituentDone>{
    ~DoneStack();
  };
  class CAcquireSubs{
    CFDNode*& m_cur_cnode;
    EdgeList*& m_cur_elist;
    StandByQueue& m_standby_queue;
  public:
    CAcquireSubs(CFDNode*& cur_cnode,EdgeList*& cur_elist,StandByQueue& standby_queue):
      m_cur_cnode(cur_cnode),m_cur_elist(cur_elist),m_standby_queue(standby_queue){}
    int operator()();
  private:
    int acquire();
	int cset_take(const GenCsetComponent&,vector<bool>&);
    void cset_ignore(const GenCsetComponent&,vector<bool>&);
    void put_on_standby(CGroupData *,CFDNode *,CArc *);
    void put_on_standby(CGroupData *,CFDNode *,CPath *);
    EdgeList *get_new_elist(CArc *arc,EdgeList *elist);
    EdgeList *get_new_elist(CPath *path,EdgeList *elist);
    void print_subc(int);  // for debugging purposes
  };
public:
#ifndef NO_REGISTER_NOT_UNIFIED
  static TokenHash s_not_unified;
#endif
private:
  CFD *m_fd;
  CGrNode *m_groot;
  DoneStack m_done_stack;
  StandByQueue m_standby_queue;
  CFDNode *m_cur_cnode;
  EdgeList *m_cur_elist;
  CBranchTraversor *m_cur_branch_trv;
  CAcquireSubs m_acquire_subs;
public:
  CConstituentTraversor(CFDNode *node,CBareGr *gr);
  ~CConstituentTraversor();
  ustatus_t proceed();
  ustatus_t retreat();
  CGrNode *cur_branch() const { return m_cur_branch_trv->cur_branch(); }
  void ungo_loc() const;
private:
  ustatus_t accept(CFDNode *node,EdgeList *elist,CGrNode *branch);
  ustatus_t finish_cur_constituent();
  ustatus_t fail_cur_constituent();
  ustatus_t finish_ctrv();
  void prev2cur(const ConstituentDone& prev);
#ifndef NO_REGISTER_NOT_UNIFIED
  bool is_unified(CFDNode *cnode);
#endif
  void rewind() const;
};

#endif
