#include "stdcfuf.h"

// #define F_TRACE

#include "b_debug_tracer.hxx"
#include "c_altnode.hxx"
#include "c_branchnode.hxx"
#include "u_fd.hxx"
#include "c_pattern.hxx"
#include "t_tracer_l2.hxx"
#include <assert.h>

////////////////////////////////////////////////////////////////////////////////
// CFD methods
//
void CFD::start_unify()
{
  F_TRACE0();
  
  g_pTheEnv->SetUnifying(true);
  g_pTheEnv->m_clock.Reset();        // reset the clock
}
void CFD::end_unify(bool ok)
{
  F_TRACE1(ok);
  
  g_pTheEnv->SetUnifying(false);    // (the clock is NOT reset)
  m_stack.pop();
  if (ok){
    m_root->cleanup();          // cleanup the root node recursively if unification succeeded
    g_pTheEnv->m_clock.Reset();     // reset the clock
  }else{                        // else - unification failed
    g_pTheEnv->m_clock.Reset();     // reset the clock FIRST
    m_root->data().SetType(g_pTheEnv->none());
    m_root->cleanup();
  }
  m_stack.push(m_root,Predefined::null);        // reInitialize the stack of nodes
  g_pTheEnv->m_clock.Reset();                        // reset the clock
}
ret::utype_t CFD::AddPattern(const hCPattern& pattern)
{
  F_TRACE1(pattern);

  ret::unode_t n = AddPatternProt(pattern);
  if (n.fail) return n.fail;
  return n.val->data().GetType();
}
void CFD::set_base(CFDNode *node)
{
  F_TRACE0();

  assert(m_stack.size() == 1);
  m_stack.pop();
  m_stack.push(node,Predefined::null);
}
void CFD::go_constituent_loc(const EdgeList *elist)
{
  F_TRACE0();
  descent(elist);
}
void CFD::ungo_constituent_loc(const EdgeList *elist)
{
  F_TRACE0();
  ascent(elist);
}
CFDNode *CFD::go_alt_loc(CGrNode *_alt)
{
  F_TRACE0();
  
  CAltNode *alt=node2alt(_alt);
  assert(alt);
  descent(alt->location());
  return m_stack.topnode();
}
void CFD::ungo_alt_loc(CGrNode *_alt)
{
  F_TRACE0();
  
  if (_alt){
    CAltNode *alt=node2alt(_alt);
    assert(alt);
    ascent(alt->location());
  }
}
bool CFD::index_matches(CBranchNode *branch,const hCType& inode_type)
{
  F_TRACE2(branch,inode_type);
  
  assert(branch->index_type().pointee());
  ret::utype_t u = branch->index_type()->unify(inode_type);
  if (!u.fail) return true;
  bool ok = u.fail->IsInternalNormal();
  DBG_TRACE2(if (!ok) Tracer_l2::report_failure(u.fail));
  delete u.fail;
  return ok;
}
bool CFD::is_wait_satisfied(CAltNode *alt,CFDNode *src)
{
  assert(src == top_node());
  CWait *wait = alt->params()->wait();  // get the alt's wait list
  if (!wait) return true;               // the alt is OK if the wait list is absent
  CWait::const_iterator it = wait->begin();
  for (uint i = wait->size();i;--i,++it){
    CAddr *addr = it->addr();
    CFDNode *dst;

    if (CArc *arc = obj2arc(addr)) dst = src->FindChild(arc->title());
    else{
      CPath *path = obj2path(addr);
      assert(path);
      int upcount = (path->upcount() == 0) ? 1 : path->upcount();
      CFDNode *node = (upcount == CPath::ABS || upcount >= m_stack.size()) ?
        m_root : (m_stack.end() - upcount)->first;
      dst = node->FindDescendant(path->begin(),path->end());
    }

    hCType dst_type = dst ? dst->data().GetType() : g_pTheEnv->nil();

    if (!dst_type->final()){    // the expected node type must be final
      DBG_TRACE2(Tracer_l2::report_wait(&*it,this));
      return false;       
    }
    
    ret::utype_t u = it->type()->unify(dst_type);
    if (u.fail){
      DBG_TRACE2(Tracer_l2::report_wait(&*it,this));
      delete u.fail;
      return false;
    }
  }
  return true;
}
void CFD::descent(const EdgeList *elist)
{
  EdgeList::const_iterator it = elist->begin();
  for (uint i=elist->size();i;--i,++it){
    Fail *f = CBareFD::EnterSubgraphProt(*it);   // silent operation (no undo and balance information),skip fset checks
    assert(!f);
  }
}
void CFD::ascent(const EdgeList *elist)
{
  for (uint i=elist->size();i;--i) LeaveSubgraphProt();
}
