#ifndef _UNIFY_FREEZER_
#define _UNIFY_FREEZER_

#include <vector.h>
#include <stack.h>

class CAltNode;
class CFDNode;
class EdgeList;
class CFD;

struct Freezer
{
	enum state_t {ALT_FROZEN,ALT_MELTING,ALT_MELTED};
	struct Cell
	{
		CAltNode *alt;        // the frozen alt
		CFDNode *c_node;      // the contemporary constituent node
		EdgeList *c_elist;    // the absolute location of the constituent node
		state_t state;
		Cell(CAltNode *_alt,CFDNode *node,EdgeList *elist);
		~Cell();
	};
	static CAltNode *toBeFrozen;
	static Cell *GetReady(CFD *fd,bool bForceMelt);
	static void Freeze(CFDNode *cnode,EdgeList *elist);
	static void Init();
	static void UnmeltLast();
	static void UnfreezeLast();
	static void CommitLastMelted();
	static void UncommitLastMelted();
private:
	typedef vector<Cell *> FrozenAlts;
	typedef stack<Cell *> MeltedAlts;
	static FrozenAlts frozenAlts;
	static MeltedAlts meltedAlts;
};

#endif
