#include "stdcfuf.h"

// #define F_TRACE

#include "c_debug.hxx"
#include "b_debug_tracer.hxx"
#include "u_fd.hxx"
#include "u_gr.hxx"
#include "u_unify.hxx"
#include "u_btrv.hxx"
#include "u_ctrv.hxx"
#include "c_extcmds.hxx"
#include "t_tracer_l2.hxx"
#include "t_tracer_l3.hxx"

class SetStackFail:public Fail{
public:
  string GetDesc() const { return "Invalid path"; }
};

class CPrepareExec{
public:
  CPrepareExec(CFD *fd,CGrammar *gr,const ustatus_t& res):m_fd(fd),m_gr(gr),m_res(res){
    m_fd->start_unify();
  }
  ~CPrepareExec(){
    m_fd->end_unify(m_res == FINISHED);
    m_gr->cleanup();                // cleanup the grammar (remove patterns stored in pattern branches)
  }
private:
  CFD *m_fd;
  CGrammar *m_gr;
  const ustatus_t& m_res;
};

/////////////////////////////////////////////////////////////////////////////////
// CUnifyEngine methods
//
ret::ubool_t CUnifyEngine::exec(const EdgeList& elist,CFDNode *node,CGrammar *gr)
{
  F_TRACE0();

  m_fd = dynamic_cast<CFD *>(node->fd());       // get the host fd
  CMarkFDPos cur_pos(m_fd);                     // mark the current fd position
  if (Fail *f = set_stack(elist,node)) return f;
  ustatus_t res=OK;
  CPrepareExec prepare(m_fd,gr,res);
  m_ucount = 0;
  {     // I need ctrv to be destroyed before the call to end_unify() in ~CPrepareExec
    CConstituentTraversor ctrv(node,gr);
    // do while the unification is not done and the branch count is below maximum
    while (res==OK && m_ucount < g_pTheEnv->GetMaxUCount()) 
      res = unify_branch(node2branch(ctrv.cur_branch()))// unify the current branch with the fd
        ? ctrv.proceed()                                // if ok - proceed to the next branch
        : ctrv.retreat();                               // else - retreat to the previous and retry it the next iteration
    // the FD must be positioned at the root, even if the unification was forced to stop in the middle
    if (res==OK) ctrv.ungo_loc();  
  }
  return res == FINISHED;
}
Fail *CUnifyEngine::set_stack(const EdgeList& elist,CFDNode *node)
{
  EdgeList::const_iterator it = elist.begin();  // descent down the edge list
  Fail *f = 0;
  for (uint i=elist.size();i && !f;--i,++it)    // this may fail, but CMarkFDPos
    f = m_fd->EnterSubgraph(*it);               // destructor will restore the original state
  if (f) return f;
  if (!node->is_equiv(m_fd->top_node())) return new SetStackFail;
  return 0;
}
bool CUnifyEngine::unify_branch(CBranchNode *branch)
{
  F_TRACE1(branch);
  assert(branch);
  
  ++m_ucount;
  
  bool ok=true;
  {
    CMarkFDPos cur_pos(m_fd);   // I want it destroyed just before ungo_alt_loc() (but not before report_failure() if failed)
    Fail *f = 0;
    for (CCmdIterator it(branch);!it.is_past_the_end() && !f;it.advance()){
      f = it->exec(m_fd);
      DUMP_FD(m_fd);
    }
    if (f){
      DBG_TRACE2(Tracer_l2::report_failure(f));
      delete f;
      ok=false;
    }
  }
  m_fd->ungo_alt_loc(branch->parent());  // return to the constituent base
  return ok;
}
