#include <stdio.h>
#include <iostream.h>
#include <string>
#include <std/bastring.cc>
#include "test.inl"
#include "parser.hxx"

typedef void (*DeleteObjectFunc)(void *pObj);
const int FUF_ENTITY_TYPE = 111;
struct LispFUFEntity
{
  unsigned char code;                   // FUF_ENTITY_TYPE
  unsigned int size:24;                 // 3
  unsigned int unused;                  // 0
  void *pObject;                        // pointer to the object
  DeleteObjectFunc DeleteObject;        // function that deletes the objects
  static LispFUFEntity *GetValue(LispVal lsp_fufent){ return reinterpret_cast<LispFUFEntity *>(lsp_fufent+sv_size_adj); }
  bool IsValid(){ return code == FUF_ENTITY_TYPE && size == 3 && unused == 0; }
};

#define DECLARE_LCB(Name,Args) \
typedef LispVal (*Name##Func)Args; \
static Name##Func Name
#define DEFINE_LCB(Name) LCB::Name##Func LCB::Name
#define SET_LCB(Name) LCB::Name = reinterpret_cast<LCB::Name##Func>(lisp_call_address(idx##Name))

struct LCB
{
  DECLARE_LCB(ConstructFUFEntity,());
  DECLARE_LCB(Error,(const char *szMsg));
};
DEFINE_LCB(ConstructFUFEntity);
DEFINE_LCB(Error);

extern "C" void RegisterLCB(int idxConstructFUFEntity,int idxError)
{
  SET_LCB(ConstructFUFEntity);
  SET_LCB(Error);
}
extern "C" void DeleteFUFEntity(LispVal lsp_fufent)
{
  LispFUFEntity *pFUFEntity = LispFUFEntity::GetValue(lsp_fufent);
  assert(pFUFEntity->IsValid());
  pFUFEntity->DeleteObject(pFUFEntity->pObject);
  pFUFEntity->pObject = 0;
}

extern "C" void ParseLispExp(LispVal lsp_val)
{
  CLispInputStream inpStream(lsp_val);
  int c;
  try{
    while (inpStream.IsOK()){
      putchar(inpStream.Get());
    }
    fflush(stdout);
  }catch(LispLexerFail *e){
    cout << e->GetDesc() << endl;
    delete e;
  }catch(...){
    cout << "Caught something\n";
  }
}

/////////////////////////////////////////////////////////////////////////////////
// LispLexerFail methods
//
string LispLexerFail::GetDesc() const
{
  return "Failed!";
}

/////////////////////////////////////////////////////////////////////////////////
// CLispInputStream methods
//
void CLispInputStream::Reset(LispVal input)
{
  m_ds.erase(m_ds.begin(),m_ds.end());
  m_ret.erase(m_ret.begin(),m_ret.end());
  Init(input);
}
void CLispInputStream::Init(LispVal input)
{
  m_input = input;
  m_buf = &m_dummy;
  m_pos = -1;
  m_str = "";
  m_ds.push(m_input,true,0);
  m_ret.push(0);
  m_ungot = false;
  m_dummy = '\0';
}
int CLispInputStream::Get()
{
  m_ungot = false;
  if (m_buf[++m_pos]){
    return m_buf[m_pos];
  }
  Dismember();
  m_pos = 0;
  if (m_ret.empty()){
    m_ungot = true;
    return EOF;
  }
  m_buf = m_str.c_str();
  return *m_buf;
}
void CLispInputStream::Unget()
{
  if (m_ungot) return;
  --m_pos;
  m_ungot = true;
}

// When making a recursive call there is no application to the enter_code field
// of the stack Frame.It is set to -2 then.
#define REC_CALL(arg_fd,arg_start,ret_code) \
m_ds.push(fd,start,-2,i); \
m_ret.push(ret_code); \
fd = arg_fd;start = arg_start; \
goto __dismember_start; \
__dismember_ret_##ret_code: \
m_ds.pop(fd,start,i)

#define RETURN(enter_code) \
m_ds.push(fd,start,enter_code,i); \
return; \
__dismember_enter_##enter_code:

#define END() \
m_ds.push(fd,start,-1,i); \
return

void CLispInputStream::Dismember() throw (LispLexerFail *)
{
  static char buf[100];
    
  LispVal fd;
  bool start;
  ushort i;
  
  switch (m_ds.pop(fd,start,i)){
  case -1:goto __dismember_end;
  case 1:goto __dismember_enter_1;
  case 2:goto __dismember_enter_2;
  case 3:goto __dismember_enter_3;
  case 4:goto __dismember_enter_4;
  case -2:assert(false);
  }
  
__dismember_start:
  if (LispValue::IsList(fd)){
    if (start){
      m_str = '(';
      RETURN(1);
    }
    REC_CALL(LispList::GetCar(fd),true,1);
    if (!LispValue::IsNil(LispList::GetCdr(fd))){
      m_str = ' ';
      RETURN(2);
      REC_CALL(LispList::GetCdr(fd),false,2);
    }
    if (start){
      m_str = ')';
      END();
    }
  }else if (LispValue::IsFixnum(fd)){
    m_str = MakeStringFromInt(LispFixnum::GetValue(fd));
    END();
  }else if (LispValue::IsChar(fd)){
    m_str = LispChar::GetValue(fd);
    END();
  }else if (LispValue::IsNil(fd)){
    m_str = "nil";
    END();
  }else if (LispValue::IsSymbol(fd)){
    m_str = LispSymbol::IsKeyword(fd) ? ":" : "";
    m_str += LispSymbol::GetValue(fd);
    END();
  }else if (LispValue::IsString(fd)){
    ((m_str = '"') += LispString::GetValue(fd)) += '"';
    END();
  }else if (LispValue::IsVector(fd)){
    m_str = "#(";
    RETURN(3);
    LispVector *pVec = LispVector::GetValue(fd);
    if (pVec->size){
      for (i=0;i<pVec->size-1;++i){
        REC_CALL(pVec->data[i],true,3);
        m_str = ' ';
        RETURN(4);
        pVec = LispVector::GetValue(fd);
      }
      REC_CALL(pVec->data[i],true,4);
    }
    m_str = ')';
    END();
  }else if (LispValue::IsBignum(fd)){
    m_str = MakeStringFromInt(LispBignum::GetValue(fd)->integer);
    END();
  }else if (LispValue::IsFloat(fd)){
    sprintf(buf,"%f",LispFloat::GetValue(fd));
    m_str = buf;
    END();
  }else if (LispValue::IsDouble(fd)){
    sprintf(buf,"%f",LispDouble::GetValue(fd));
    m_str = buf;
    END();
  }else throw new LispLexerFail(fd);
  
__dismember_end:
  switch (m_ret.pop()){
  case 1:goto __dismember_ret_1;
  case 2:goto __dismember_ret_2;
  case 3:goto __dismember_ret_3;
  case 4:goto __dismember_ret_4;
  }
  return;
}

/////////////////////////////////////////////////////////////////////
// CLispInputStream::DataStack methods
//
void CLispInputStream::DataStack::push(LispVal fd,bool start,byte enter_code,ushort i)
{
  push_back(Frame(fd,start,enter_code,i));
}
CLispInputStream::byte CLispInputStream::DataStack::pop(LispVal& fd,bool& start,ushort& i)
{
  Frame f(back());
  pop_back();
  fd = f.fd;
  start = f.start;
  i = f.i;
  return f.enter_code;
}

/////////////////////////////////////////////////////////////////////
// CLispInputStream::RetAddrStack methods
//
CLispInputStream::uchar CLispInputStream::RetAddrStack::pop()
{
  uchar tmp = back();
  pop_back();
  return tmp;
}

/////////////////////////////////////////////////////////////////////
// misc functions
//
typedef long long int64;
typedef unsigned long uint;
string MakeStringFromInt(int _n)
{
  const int BUFSIZE = 20;
  static char szBuf[BUFSIZE];
  szBuf[BUFSIZE-1] = '\0';
  
  int64 n = _n;
  if (_n < 0) n = -n;

  uint i = BUFSIZE-1;
  do{
    --i;
    szBuf[i] = '0'+n%10;
    n/=10;
  }while (i && n);
  assert(i);

  if (_n < 0) szBuf[--i] = '-';

  return string(szBuf+i);
}
