#include <assert.h>
#include <iostream.h>
#include "test.inl"

typedef void (*DeleteObjectFunc)(void *pObj);
const int FUF_ENTITY_TYPE = 111;
struct LispFUFEntity
{
  unsigned char code;                   // FUF_ENTITY_TYPE
  unsigned int size:24;                 // 3
  unsigned int unused;                  // 0
  void *pObject;                        // pointer to the object
  DeleteObjectFunc DeleteObject;        // function that deletes the objects
  static LispFUFEntity *GetValue(LispVal lsp_fufent){ return reinterpret_cast<LispFUFEntity *>(lsp_fufent+sv_size_adj); }
  bool IsValid(){ return code == FUF_ENTITY_TYPE && size == 3 && unused == 0; }
};

#define DECLARE_LCB(Name,Args) \
typedef LispVal (*Name##Func)Args; \
static Name##Func Name
#define DEFINE_LCB(Name) LCB::Name##Func LCB::Name
#define SET_LCB(Name) LCB::Name = reinterpret_cast<LCB::Name##Func>(lisp_call_address(idx##Name))

struct LCB
{
  DECLARE_LCB(ConstructFUFEntity,());
  DECLARE_LCB(Error,(const char *szMsg));
  DECLARE_LCB(DefPrimitive,(const char *szLispName,const char *szCName,int nArgs));
};
DEFINE_LCB(ConstructFUFEntity);
DEFINE_LCB(Error);
DEFINE_LCB(DefPrimitive);

void Initialize();

extern "C" void RegisterLCB(int idxConstructFUFEntity,int idxError,int idxDefPrimitive)
{
  SET_LCB(ConstructFUFEntity);
  SET_LCB(Error);
  SET_LCB(DefPrimitive);
  Initialize();
}
extern "C" void DeleteFUFEntity(LispVal lsp_fufent)
{
  LispFUFEntity *pFUFEntity = LispFUFEntity::GetValue(lsp_fufent);
  assert(pFUFEntity->IsValid());
  pFUFEntity->DeleteObject(pFUFEntity->pObject);
  pFUFEntity->pObject = 0;
}

#define DECLARE_CONVERT(Class) static Class *Convert(LispVal lsp_val)
#define DEFINE_CONVERT(Class) \
Class *Class::Convert(LispVal lsp_val) \
{ \
  if (GetType(lsp_val) != FUF_ENTITY_TYPE) return 0; \
  LispFUFEntity *pFUFEntity = LispFUFEntity::GetValue(lsp_val); \
  return (pFUFEntity->IsValid() && \
          pFUFEntity->pObject && \
          int(pFUFEntity->DeleteObject) == int(Delete)) ? reinterpret_cast<Class *>(pFUFEntity->pObject) : 0; \
}
#define DEFINE_CONSTRUCT(Class) \
extern "C" LispVal Construct##Class(int x,double y) \
{ \
  LispVal lsp_val = LCB::ConstructFUFEntity(); \
  LispFUFEntity *pFUFEntity = LispFUFEntity::GetValue(lsp_val); \
  assert(pFUFEntity->IsValid()); \
  pFUFEntity->pObject = new Class(x,y); \
  pFUFEntity->DeleteObject = reinterpret_cast<DeleteObjectFunc>(Class::Delete); \
  return lsp_val; \
}
  
class A
{
  int m_x;
  double m_y;
public:
  A(int x,double y){
    printf("A(%d,%.3f)\n",x,y);
    m_x = x;
    m_y = y;
  }
  ~A(){
    printf("~A(%d,%.3f)\n",m_x,m_y);
  }
  void Dump(){
    printf("A: %d %.3f\n",m_x,m_y);
  }
  static void Delete(A *pObj)
  {
    delete pObj;
  }
  DECLARE_CONVERT(A);
};
DEFINE_CONVERT(A);
DEFINE_CONSTRUCT(A);

extern "C" void PrintA(LispVal lsp_val)
{
  A *pObj = A::Convert(lsp_val);
  if (!pObj) LCB::Error("Incorrect A object");
  else pObj->Dump();
}

extern "C" void PrintInfo(LispVal lsp_val)
{
  CLispType lispType;
  lispType.PrintLispTypeInfo(lsp_val);
}

void Initialize()
{
  LCB::DefPrimitive("'print-info","PrintInfo",1);
  LCB::DefPrimitive("'print-a","PrintA",1);
}
