#include "auxiliary.hxx"
#include "runfuf.hxx"
#include "cfuf.hxx"
#include "cfuf_fd.hxx"
#include "cfuf_grammar.hxx"
#include "cfuf_special.hxx"

DECLARE_PRIMITIVE(FD,GetUCount,());
DECLARE_PRIMITIVE(FD,Compile,(LispVal));
DECLARE_PRIMITIVE(FD,Copy,(LispVal));
DECLARE_PRIMITIVE(FD,Cut,(LispVal));
DECLARE_PRIMITIVE(FD,Delete,(LispVal));
DECLARE_PRIMITIVE(FD,GetFirstChild,(LispVal));
DECLARE_PRIMITIVE(FD,GetFirstParent,(LispVal));
DECLARE_PRIMITIVE(FD,GetNextChild,(LispVal));
DECLARE_PRIMITIVE(FD,GetNextParent,(LispVal));
DECLARE_PRIMITIVE(FD,GetSpecialAttr,(LispVal));
DECLARE_PRIMITIVE(FD,GetType,(LispVal));
DECLARE_PRIMITIVE(FD,IsEmptyFD,(LispVal));
DECLARE_PRIMITIVE(FD,IsFD,(LispVal));
DECLARE_PRIMITIVE(FD,FindChild,(LispVal,LispVal));
DECLARE_PRIMITIVE(FD,IsEqual,(LispVal,LispVal));
DECLARE_PRIMITIVE(FD,Paste,(LispVal,LispVal));
DECLARE_PRIMITIVE(FD,ToList,(LispVal,LispVal));
DECLARE_PRIMITIVE(FD,Unify,(LispVal,LispVal));
DECLARE_PRIMITIVE(FD,PrettyPrint,(LispVal,LispVal,LispVal));

struct LispFD
{
  typedef fd_t Entity;
  
  typedef pair<token_t,fd_t> (CFUFFD::*GetRelativeFunc)();
  static LispVal GetRelative(LispVal lisp_fd,GetRelativeFunc f,char *szCaller);
  
  static LispVal pattern2lisp(CFUFSpecial::Pattern *p);
  static LispVal cset2lisp(CFUFSpecial::Cset *c);
  static LispVal component2lisp(CFUFSpecial::Cset *c,uint nComponent);
  static LispVal fset2lisp(CFUFSpecial::Fset *f);

  static char *LispFD::GetSzFDName(LispVal lsp_fdName)
  {
    static char szFDNameBuf[100];
    return LispAux::GetSzSymbolOrString(szFDNameBuf,lsp_fdName,szFDPrettyPrint);
  }
  static char *LispFD::GetSzFileName(LispVal lsp_fileName)
  {
    static char szFileNameBuf[1000];
    return LispAux::GetSzSymbolOrString(szFileNameBuf,lsp_fileName,szFDPrettyPrint);
  }

  static void Delete(fd_t fd)
  {
    fd->Destroy();
  }
};

void PlugFD()
{
  DEFINE_PRIMITIVE(0,FD,GetUCount);
  DEFINE_PRIMITIVE(1,FD,Compile);
  DEFINE_PRIMITIVE(1,FD,Copy);
  DEFINE_PRIMITIVE(1,FD,Cut);
  DEFINE_PRIMITIVE(1,FD,Delete);
  DEFINE_PRIMITIVE(1,FD,GetFirstChild);
  DEFINE_PRIMITIVE(1,FD,GetFirstParent);
  DEFINE_PRIMITIVE(1,FD,GetNextChild);
  DEFINE_PRIMITIVE(1,FD,GetNextParent);
  DEFINE_PRIMITIVE(1,FD,GetSpecialAttr);
  DEFINE_PRIMITIVE(1,FD,GetType);
  DEFINE_PRIMITIVE(1,FD,IsEmptyFD);
  DEFINE_PRIMITIVE(1,FD,IsFD);
  DEFINE_PRIMITIVE(2,FD,FindChild);
  DEFINE_PRIMITIVE(2,FD,IsEqual);
  DEFINE_PRIMITIVE(2,FD,Paste);
  DEFINE_PRIMITIVE(2,FD,ToList);
  DEFINE_PRIMITIVE(2,FD,Unify);
  DEFINE_PRIMITIVE(3,FD,PrettyPrint);
}
/////////////////////////////////////////////////////////////////////////////////
// LispFD methods
//
LispVal LispFD::GetRelative(LispVal lsp_fd,GetRelativeFunc f,char *szCaller)
{
  fd_t fd;
  LispAux::Convert<LispFD>(lsp_fd,fd,szCaller);
  pair<token_t,fd_t> relative = (fd->*f)();
  assert(relative.second);
  return LCB::Cons(LispAux::token2lisp(relative.first),LispAux::Create<LispFD>(relative.second));
}
LispVal LispFD_GetFirstChild(LispVal lsp_fd)
{
  return LispFD::GetRelative(lsp_fd,&CFUFFD::GetFirstChild,szFDGetFirstChild);
}
LispVal LispFD_GetNextChild(LispVal lsp_fd)
{
  return LispFD::GetRelative(lsp_fd,&CFUFFD::GetNextChild,szFDGetNextChild);
}
LispVal LispFD_GetFirstParent(LispVal lsp_fd)
{
  return LispFD::GetRelative(lsp_fd,&CFUFFD::GetFirstParent,szFDGetFirstParent);
}
/////////////////////////////////////////////////////////////////////////////////
struct FDCleanup:public LispAux::Cleanup{
  FDCleanup(CFUFFD *pCFUFFD){ m_pCFUFFD = pCFUFFD; }
  void Perform(){ m_pCFUFFD->Destroy(); }
private:
  CFUFFD *m_pCFUFFD;
};
LispVal LispFD_Compile(LispVal lsp_fd)
{
  CLispInputStream istr(lsp_fd);
  ufd_t fd = CFUFFD::Parse(istr);
  if (fd.fail){
    FDCleanup cleanup(fd.val);
    return LispAux::Failed(szFDCompile,fd.fail,cleanup);
  }
  return LispAux::Create<LispFD>(fd.val);
}
LispVal LispFD_IsFD(LispVal lsp_obj)
{
  fd_t dummy;
  return LispAux::Convert<LispFD>(lsp_obj,dummy) ? LispValue::TRUE : LispValue::FALSE;
}
LispVal LispFD_IsEmptyFD(LispVal lsp_obj)
{
  fd_t fd;
  return LispAux::Convert<LispFD>(lsp_obj,fd) && fd->IsEmpty() ? LispValue::TRUE : LispValue::FALSE;
}
LispVal LispFD_Unify(LispVal lsp_fd,LispVal lsp_gr)
{
  fd_t fd;
  grammar_t gr;
  LispAux::Convert<LispFD>(lsp_fd,fd,szFDUnify);
  LispAux::Convert<LispGrammar>(lsp_gr,gr,szFDUnify);
  
  ret::ubool_t u = fd->Unify(gr);
  if (u.fail) return LispAux::Failed(szFDUnify,u.fail);
  return u.val ? LispValue::TRUE : LispValue::FALSE;
}
LispVal LispFD_Copy(LispVal lsp_fd)
{
  fd_t fd;
  LispAux::Convert<LispFD>(lsp_fd,fd,szFDCopy);
  return LispAux::Create<LispFD>(fd->Copy());
}
LispVal LispFD_Cut(LispVal lsp_fd)
{
  fd_t fd;
  LispAux::Convert<LispFD>(lsp_fd,fd,szFDCut);
  return LispAux::Create<LispFD>(fd->Cut());
}
LispVal LispFD_Delete(LispVal lsp_fd)
{
  fd_t fd;
  LispAux::Convert<LispFD>(lsp_fd,fd,szFDDelete);
  fd->Delete();
  return LispValue::TRUE;
}
LispVal LispFD_Paste(LispVal lsp_fdDst,LispVal lsp_fdSrc)
{
  fd_t fdDst,fdSrc;
  LispAux::Convert<LispFD>(lsp_fdDst,fdDst,szFDPaste);
  LispAux::Convert<LispFD>(lsp_fdSrc,fdSrc,szFDPaste);
  Fail *f = fdDst->Paste(fdSrc);
  return f ? LispAux::Failed(szFDPaste,f) : LispValue::TRUE;
}
LispVal LispFD_IsEqual(LispVal lsp_fd1,LispVal lsp_fd2)
{
  fd_t fd1,fd2;
  LispAux::Convert<LispFD>(lsp_fd1,fd1,szFDIsEqual);
  LispAux::Convert<LispFD>(lsp_fd2,fd2,szFDIsEqual);
  return fd1->IsEqual(fd2) ? LispValue::TRUE : LispValue::FALSE;
}
LispVal LispFD_GetType(LispVal lsp_fd)
{
  fd_t fd;
  LispAux::Convert<LispFD>(lsp_fd,fd,szFDGetType);
  CType *type = fd->GetType();
  if (type->special()){
    token_t attr = fd->GetSpecialAttr();
    if (attr == Predefined::pattern) return LispFD::pattern2lisp(CFUFSpecial::GetPattern(type));
    if (attr == Predefined::cset)    return LispFD::cset2lisp(CFUFSpecial::GetCset(type));
    if (attr == Predefined::fset)    return LispFD::fset2lisp(CFUFSpecial::GetFset(type));
    assert(false);
    return LispValue::FALSE;
  }
  return LispAux::type2lisp(type);
}
LispVal LispFD_GetUCount()
{
  return IntToFixnum(CFUFFD::GetUCount());
}
LispVal LispFD_GetSpecialAttr(LispVal lsp_fd)
{
  fd_t fd;
  LispAux::Convert<LispFD>(lsp_fd,fd,szFDGetSpecialAttr);
  return LispAux::token2lisp(fd->GetSpecialAttr());
}
LispVal LispFD_FindChild(LispVal lsp_fd,LispVal lsp_lstEdges)
{
  fd_t fd;
  LispAux::Convert<LispFD>(lsp_fd,fd,szFDFindChild);
  EdgeList lstEdges;
  LispAux::lisp2elist(lstEdges,lsp_lstEdges,szFDFindChild);
  return LispAux::Create<LispFD>(fd->FindDescendant(lstEdges));
}
LispVal LispFD_GetNextParent(LispVal lsp_fd)
{
  return LispFD::GetRelative(lsp_fd,&CFUFFD::GetNextParent,szFDGetNextParent);
}
LispVal LispFD_PrettyPrint(LispVal lsp_fd,LispVal lsp_fdName,LispVal lsp_fileName)
{
  fd_t fd;
  LispAux::Convert<LispFD>(lsp_fd,fd,szFDPrettyPrint);

  char *szFDName = LispFD::GetSzFDName(lsp_fdName);
  char *szFileName = LispFD::GetSzFileName(lsp_fileName);
    
  fd->PrettyPrint(szFDName,szFileName);
  return LispValue::TRUE;
}

class CFD2list:public CUnparseFD
{
public:
  CFD2list(LispVal lsp_attr)
  {
    m_lsp_list = LispValue::NIL;
    m_bDoingAll = (lsp_attr != LispValue::FALSE);
  }
  Fail *StartInternal(CFDNode *root);
  Fail *StartEndLeaf(CFDNode *root);
  Fail *EnterSubgraph(token_t edge,CFDNode *node,EdgeList *path);
  Fail *LeaveSubgraph();
  Fail *AddLeaf(token_t edge,CFDNode *node,EdgeList *path);
  Fail *AddPath(token_t edge,CFDNode *node,EdgeList *path);
  bool IsDoingAll() const { return m_bDoingAll; }
  int GetOrder() const    { return ASCENDING; }
  LispVal GetLispList() const  { return m_lsp_list; }
private:
  stack<pair<token_t,LispVal> > m_nest;
  LispVal m_lsp_list;
  bool m_bDoingAll;
};

LispVal LispFD_ToList(LispVal lsp_fd,LispVal lsp_attr)
{
  fd_t fd;
  LispAux::Convert<LispFD>(lsp_fd,fd,szFDToList);

  EdgeList lstIgnored;
  if (lsp_attr != LispValue::TRUE && lsp_attr != LispValue::FALSE)
    LispAux::lisp2elist(lstIgnored,lsp_attr,szFDToList);
  
  CFD2list fd2list(lsp_attr);
  CFDWalk walker(fd->GetRoot());
  
  EdgeList::const_iterator it = lstIgnored.begin();
  for (uint i = lstIgnored.size();i;--i,++it)
    walker.ignore(*it);
  
  Fail *f = walker.walk(&fd2list);
  assert(!f);
  return fd2list.GetLispList();
}

/////////////////////////////////////////////////////////////////////////////////
// CFD2list methods
//
Fail *CFD2list::StartInternal(CFDNode *root)
{
  if (root->any())             // if the root is ANY, then output the pair ({} any)
    m_lsp_list = LCB::ConsPairToList(LispAux::str2lisp("{}"),LispAux::token2lisp(Predefined::any),m_lsp_list);
  return 0;
}
Fail *CFD2list::StartEndLeaf(CFDNode *root)        // the fd is just a single node
{
  m_lsp_list = LispAux::type2lisp(root->data().get_type());
  return 0;
}
Fail *CFD2list::EnterSubgraph(token_t edge,CFDNode *node,EdgeList *)
{
  if (node->any())             // special case when internal node is reported as ANY
    m_lsp_list = LCB::ConsPairToList(LispAux::token2lisp(edge),LispAux::token2lisp(Predefined::any),m_lsp_list);
  m_nest.push(make_pair(edge,m_lsp_list));
  m_lsp_list = LispValue::NIL;
  return 0;
}
Fail *CFD2list::LeaveSubgraph()
{
  pair<token_t,LISP> saved = m_nest.top();                         // get the parent feature and list
  LispVal lsp_edge = LispAux::token2lisp(saved.first);                  // convert the feature to a LispVal object
  m_lsp_list = LCB::ConsPairToList(lsp_edge,m_lsp_list,saved.second); // cons the (lsp_edge lsp_value) pair to the result
  m_nest.pop();                 // return one level up
  return 0;
}
Fail *CFD2list::AddLeaf(token_t edge,CFDNode *node,EdgeList *)
{
  CType *type = node->data().get_type();        // get the type
  if (type->nil()) return 0;
  LispVal lsp_edge = LispAux::token2lisp(edge);       // convert the edge to a scheme attribute
  LispVal lsp_value = LispAux::type2lisp(type);       // convert the type to a scheme value
  m_lsp_list = LCB::ConsPairToList(lsp_edge,lsp_value,m_lsp_list);  // cons the (lsp_value lsp_edge) pair to the result
  return 0;
}
Fail *CFD2list::AddPath(token_t edge,CFDNode *,EdgeList *path)
{
  LispVal lsp_edge = LispAux::token2lisp(edge);       // convert the edge to a scheme attribute
  static string strPath;
  ((strPath = '{') += path->GetString()) += '}';
  LispVal lsp_value = LispAux::str2lisp(strPath);     // convert the path to a scheme value
  m_lsp_list = LCB::ConsPairToList(lsp_edge,lsp_value,m_lsp_list); // cons the (lsp_value lsp_edge) pair to the result
  return 0;
}

/////////////////////////////////////////////////////////////////////////////////
LispVal LispFD::pattern2lisp(CFUFSpecial::Pattern *p)
{
  static LispVal lsp_dots = LispAux::token2lisp(Predefined::dots);
  LispVal lsp_acc=LispValue::NIL;
  LispVal lsp_val;

  for (int i = p->m_nCount - 1; i >= 0; --i){
    if (fd_t fd = p->m_arrValues[i]){
      lsp_val = LispAux::Create<LispFD>(fd);
      if (fd->IsMergeable()) lsp_val = LCB::Cons(lsp_val,LispValue::NIL);
    }else lsp_val = lsp_dots;
    lsp_acc = LCB::Cons(lsp_val,lsp_acc);
  }
  return lsp_acc;
}
LispVal LispFD::cset2lisp(CFUFSpecial::Cset *c)
{
  LispVal lsp_acc = LispValue::NIL;
  for (int i = CFUFSpecial::Cset::DEC; i >= CFUFSpecial::Cset::ABS; --i)
    lsp_acc = LCB::Cons(component2lisp(c,i),lsp_acc);
  return lsp_acc;
}
LispVal LispFD::component2lisp(CFUFSpecial::Cset *c,uint nComponent)
{
  LispVal lsp_acc = LispValue::NIL;
  for (int i = c->m_nCount[nComponent] - 1; i >= 0; --i)
    lsp_acc = LCB::Cons(LispAux::Create<LispFD>(c->m_arrValues[nComponent][i]),lsp_acc);
  return lsp_acc;
}
LispVal LispFD::fset2lisp(CFUFSpecial::Fset *f)
{
  LispVal lsp_acc=LispValue::NIL;

  for (int i = f->m_nCount - 1; i >= 0; --i)
    lsp_acc = LCB::Cons(LispAux::token2lisp(f->m_arrFeatures[i]),lsp_acc);
    
  return lsp_acc;
}
