#include "auxiliary.hxx"
#include "runfuf.hxx"
#include "cfuf.hxx"
#include "cfuf_grammar.hxx"

DECLARE_PRIMITIVE(Grammar,Compile,(LispVal lsp_gr));
DECLARE_PRIMITIVE(Grammar,IsGrammar,(LispVal lsp_val));
DECLARE_PRIMITIVE(Grammar,DoingDemo,(LispVal lsp_bYes));
DECLARE_PRIMITIVE(Grammar,DoingWait,(LispVal lsp_bYes));
DECLARE_PRIMITIVE(Grammar,DoingIndex,(LispVal lsp_bYes));
DECLARE_PRIMITIVE(Grammar,DoingBKClass,(LispVal lsp_bYes));

typedef bool (*IsParamEnabledFunc)();
typedef void (*EnableParamFunc)(bool bEnable);

struct LispGrammar
{
  typedef grammar_t Entity;
  
  static LispVal DoingParam(LispVal lsp_bYes,IsParamEnabledFunc IsParamEnabled,EnableParamFunc EnableParam);
  
  static void Delete(grammar_t gr)
  {
    gr->Destroy();
  }
};

void PlugGrammar()
{
  DEFINE_PRIMITIVE(1,Grammar,Compile);
  DEFINE_PRIMITIVE(1,Grammar,IsGrammar);
  DEFINE_PRIMITIVE(1,Grammar,DoingDemo);
  DEFINE_PRIMITIVE(1,Grammar,DoingWait);
  DEFINE_PRIMITIVE(1,Grammar,DoingIndex);
  DEFINE_PRIMITIVE(1,Grammar,DoingBKClass);
}
/////////////////////////////////////////////////////////////////////////////////
// LispGrammar methods
//
LispVal LispGrammar::DoingParam(LispVal lsp_bYes,IsParamEnabledFunc IsParamEnabled,EnableParamFunc EnableParam)
{
  if (lsp_bYes==IntToFixnum(0)) return IsParamEnabled() ? LispValue::TRUE : LispValue::FALSE;
  EnableParam(lsp_bYes != LispValue::FALSE);
  return IntToFixnum(0);
}
/////////////////////////////////////////////////////////////////////////////////
struct GrammarCleanup:public ScmAux::Cleanup
{
  GrammarCleanup(CFUFGrammar *pCFUFGrammar){ m_pCFUFGrammar = pCFUFGrammar; }
  void Perform(){ m_pCFUFGrammar->Destroy(); }
private:
  CFUFGrammar *m_pCFUFGrammar;
};
LispVal LispGrammar_Compile(LispVal lsp_gr)
{
  CScmInputStream istr(lsp_gr);
  ugrammar_t gr = CFUFGrammar::Parse(istr);
  if (gr.fail){
    GrammarCleanup cleanup(gr.val);
    return ScmAux::Failed(szGrammarCompile,gr.fail,cleanup);
  }
  return ScmAux::Create<LispGrammar>(gr.val);
}
LispVal LispGrammar_DoingDemo(LispVal lsp_bYes)
{
  return DoingParam(lsp_bYes,CFUFGrammar::IsDemoEnabled,CFUFGrammar::EnableDemo);
}
LispVal LispGrammar_DoingWait(LispVal lsp_bYes)
{
  return DoingParam(lsp_bYes,CFUFGrammar::IsWaitEnabled,CFUFGrammar::EnableWait);
}
LispVal LispGrammar_DoingIndex(LispVal lsp_bYes)
{
  return DoingParam(lsp_bYes,CFUFGrammar::IsIndexEnabled,CFUFGrammar::EnableIndex);
}
LispVal LispGrammar_DoingBKClass(LispVal lsp_bYes)
{
  return DoingParam(lsp_bYes,CFUFGrammar::IsBKClassEnabled,CFUFGrammar::EnableBKClass);
}
LispVal LispGrammar_IsGrammar(LispVal lsp_obj)
{
  grammar_t dummy;
  return LispAux::Convert<LispGrammar>(lsp_obj,dummy) ? LispValue::TRUE : LispValue::FALSE;
}
