#include "auxiliary.hxx"
#include "lcb.hxx"
#include <stdio.h>

extern "C" void DeleteFUFEntity(LispVal lsp_fufent)
{
  LispFUFEntity *pFUFEntity = LispFUFEntity::GetValue(lsp_fufent);
  assert(pFUFEntity->IsValid());
  pFUFEntity->DeleteObject(pFUFEntity->pObject);
  pFUFEntity->pObject = 0;
}

LispAux::Cleanup LispAux::s_defClean;
////////////////////////////////////////////////////////////////////////////////
// LispAux methods
//
token_t LispAux::lisp2token(LispVal lsp_obj,char *szCaller)
{
  static char szBuf[1000];
  if (LispValue::IsSymbol(lsp_obj)){
    sprintf(szBuf,"%s%s",
            LispSymbol::IsKeyword(lsp_obj) ? ":" : "",
            LispSymbol::GetValue(lsp_obj));
  }else if (LispValue::IsFixnum(lsp_obj)){
    sprintf(szBuf,"%d",LispFixnum::GetValue(lsp_obj));
  }else if (LispValue::IsFloat(lsp_obj)){
    sprintf(szBuf,"%f",LispFloat::GetValue(lsp_obj));
  }else if (LispValue::IsDouble(lsp_obj)){
    sprintf(szBuf,"%f",LispDouble::GetValue(lsp_obj));
  }else if (LispValue::IsString(lsp_obj)){
    sprintf(szBuf,"\"%s\"",LispString::GetValue(lsp_obj));
  }else if (LispValue::IsNil(lsp_obj)){
    return Predefined::nil;
  }else{
    if (szCaller) LCB::Error(szCaller,lsp_obj);
    return 0;
  }
  return Symbols::insert(szBuf);  // retrieve the token
}
void LispAux::lisp2elist(EdgeList& rLstTokens,LispVal lsp_lstTokens,char *szCaller)
{
  if (LispValue::IsNil(lsp_lstTokens)) return;    // the list is empty - return
  
  LispVal lsp_lstSave = lsp_lstTokens;
  token_t token;
  if (LispValue::IsList(lsp_lstTokens)){
    while (LispValue::IsList(lsp_lstTokens)){
      token = lisp2token(LispList::GetCar(lsp_lstTokens));
      if (!token) break;
      rLstTokens.push_back(token);
      lsp_lstTokens = LispList::GetCdr(lsp_lstTokens);
    }
    if (LispValue::IsNil(lsp_lstTokens)) return;
  }else{                        // the syntax is such, that a single element is allowed to appear not as a list ('a instead of '(a))
    token = lisp2token(lsp_lstTokens);
    if (token){
      rLstTokens.push_back(token);
      return;
    }
  }
  (&rLstTokens)->~EdgeList();
  LCB::Error(szCaller,lsp_lstSave);
}
LispVal LispAux::token2lisp(token_t token)
{
  if (!token) return LispValue::FALSE;
  
  if (TOK2STR(token)[0] == '"'){ // token is a quoted string
    return LCB::EvalStr(token);
  }
  static string strToken;
  (strToken = "'") += token;
  return LCB::EvalStr(strToken.c_str());
}
LispVal LispAux::type2lisp(CType *type)
{
  assert(type);

  static string strType;
  if (type->quoted()) ((strType = '"') += type->toString()) += '"';
  else (strType = "'") += type->toString();
  return LCB::EvalStr(strType.c_str());
}
LispVal LispAux::str2lisp(const string& str)
{
  return LCB::EvalStr(str.c_str());
}
/*
LispVal LispAux::elist2lisp(const EdgeList& elist)
{
  LispVal res = EOL;
  EdgeList::const_reverse_iterator it = elist.rbegin();
  for (uint i=elist.size();i;--i,++it) res = cons(token2lisp(*it),res);
  return res;
}
*/
char *LispAux::GetSzSymbolOrString(char szBuf[],LispVal lsp_val,char *szCaller)
{
  char *szVal = szBuf;
  
  if (LispValue::IsSymbol(lsp_val)){
    sprintf(szBuf,"%s%s",
            LispSymbol::IsKeyword(lsp_val) ? ":" : "",
            LispSymbol::GetValue(lsp_val));
  }else if (LispValue::IsString(lsp_val)){
    szVal = LispString::GetValue(lsp_val);
  }else{
    if (szCaller) LCB::Error(szCaller,lsp_val);
    return 0;
  }
  return szVal;
}
LispVal LispAux::Failed(const char *who,Fail *e,Cleanup& cleanup)
{
  bool bAbort;
  { // strFailDesc must be destroyed before ASSERT
    string strFailDesc(e->GetDesc()); 
    if (bAbort = e->IsAbort()) cerr << who << ": " << strFailDesc << ENDL;
    else (*g_pTheOutput) << who << ": " << strFailDesc << ENDL;
    delete e;
  }
  cleanup.Perform();
  if (bAbort) LCB::Error(who,LispValue::FALSE);
  
  return LispValue::FALSE;
}
