#ifndef __LISP_AUX__
#define __LISP_AUX__

#include "lisp.h"
#include "objects_b.hxx"
#include "lcb.hxx"
#include "lispval.hxx"

class CFUFGrammar;
class CFUFFD;

typedef void (*DeleteObjectFunc)(void *pObj);
const int FUF_ENTITY_TYPE = 111;

struct LispFUFEntity
{
  unsigned char code;                   // FUF_ENTITY_TYPE
  unsigned int size:24;                 // 3
  unsigned int unused;                  // 0
  void *pObject;                        // pointer to the object
  DeleteObjectFunc DeleteObject;        // function that deletes the objects
  static LispFUFEntity *GetValue(LispVal lsp_fufent){ return reinterpret_cast<LispFUFEntity *>(lsp_fufent+sv_size_adj); }
  bool IsValid(){ return code == FUF_ENTITY_TYPE && size == 3 && unused == 0; }
};

struct LispAux{
  template<class T>
  static LispVal Create(typename T::Entity ent){
    LispVal lsp_fufent = LCB::ConstructFUFEntity();
    LispFUFEntity *pFUFEntity = LispFUFEntity::GetValue(lsp_fufent);
    assert(pFUFEntity->IsValid());
    pFUFEntity->DeleteObject = reinterpret_cast<DeleteObjectFunc>(T::Delete);
    pFUFEntity->pObject = ent;
    return lsp_fufent;
  }
  template<class T>
  static bool Convert(LispVal lsp_val,typename T::Entity& rEnt,char *szCaller = 0)
  {
    if (GetType(lsp_val) == FUF_ENTITY_TYPE){
      LispFUFEntity *pFUFEntity = LispFUFEntity::GetValue(lsp_val);
      if (pFUFEntity->IsValid() &&
          pFUFEntity->pObject &&
          int(pFUFEntity->DeleteObject) == int(T::Delete)){
        rEnt = reinterpret_cast<typename T::Entity>(pFUFEntity->pObject);
        return true;
      }
    }
    if (szCaller) LCB::Error(szCaller,lsp_val);
    return false;
  }

  // static CFUFGrammar *lisp2gr(LISP lisp_gr,int nArg,char *szCaller);
  // static LISP gr2lisp(CFUFGrammar *pGrammar);
  
  // static CFUFFD *lisp2fd(LISP lisp_fd,int nArg,char *szCaller);
  // static LISP fd2lisp(CFUFFD *pFD);
  
  static token_t lisp2token(LispVal lisp_obj,char *szCaller = 0);
  static LispVal token2lisp(token_t);
  static LispVal type2lisp(CType *);
  static char *GetSzSymbolOrString(char szBuf[],LispVal lsp_val,char *szCaller = 0);
  static LispVal str2lisp(const string& str);

  static void lisp2elist(EdgeList& rLstTokens,LispVal lisp_lstTokens,char *szCaller);
  // static LISP elist2lisp(const EdgeList& elist);

  struct Cleanup{
    virtual ~Cleanup(){}
    virtual void Perform(){}
  };
  static LispVal Failed(const char *who,Fail *e,Cleanup& cleanup = s_defClean);

  // static void gc(double weight);
private:
  static Cleanup s_defClean;
};

#define DECLARE_PRIMITIVE(Module,Name,Args) \
extern char *sz##Module##Name; \
extern "C" LispVal Lisp##Module##_##Name##Args

#define ADD_QUOTES(String) #String
#define DEFINE_PRIMITIVE(Arity,Module,Name) \
LCB::DefPrimitive(sz##Module##Name,ADD_QUOTES(Lisp##Module##_##Name),Arity)

#endif
