#ifndef __LISP_VALUE__
#define __LISP_VALUE__

struct LispValue
{
  static LispVal FALSE,TRUE,NIL;
  static bool IsList(LispVal lsp_val)   { return GetType(lsp_val) == ListType; }
  static bool IsNil(LispVal lsp_val)    { return GetType(lsp_val) == NilType; }
  static bool IsFixnum(LispVal lsp_val) { return GetType(lsp_val) == FixnumType || GetType(lsp_val) == OddFixnumType; }
  static bool IsBignum(LispVal lsp_val) { return GetType(lsp_val) == BignumType; }
  static bool IsChar(LispVal lsp_val)   { return GetType(lsp_val) == CharType; }
  static bool IsSymbol(LispVal lsp_val) { return GetType(lsp_val) == SymbolType; }
  static bool IsString(LispVal lsp_val) { return GetType(lsp_val) == V_charType; }
  static bool IsVector(LispVal lsp_val) { return GetType(lsp_val) == V_svecType; }
  static bool IsFloat(LispVal lsp_val)  { return GetType(lsp_val) == S_FloatType; }
  static bool IsDouble(LispVal lsp_val) { return GetType(lsp_val) == D_FloatType; }
};
struct LispList
{
  static LispVal GetCar(LispVal lsp_list){ return Car(lsp_list); }
  static LispVal GetCdr(LispVal lsp_list){ return Cdr(lsp_list); }
};
struct LispFixnum
{
  static int GetValue(LispVal lsp_fixnum){ return FixnumToInt(int(lsp_fixnum)); }
};
struct LispBignum
{
  unsigned char code;
  unsigned char unused;
  unsigned short nwords;
  union
  {
    unsigned int integer;
    unsigned short words[1];
  };
  static LispBignum *GetValue(LispVal lsp_bignum){ return (LispBignum *)(lsp_bignum-OtherTypeOfs); }
};
struct LispChar
{
  static char GetValue(LispVal lsp_char){ return ScharToChar(int(lsp_char)); }
};
struct LispString
{
  static char *GetValue(LispVal lsp_string){ return (char *)Vecdata(lsp_string); }
  static int GetSize(LispVal lsp_string)   { return VecSize(lsp_string); }
};
struct LispSymbol
{
  static char *GetValue(LispVal lsp_symbol){ return LispString::GetValue(SymbolName(lsp_symbol)); }
  static bool IsKeyword(LispVal lsp_symbol){
    LispVal lsp_packageName = (LispVal)*((int *)(SymbolPackage(lsp_symbol)-OtherTypeOfs)+3);
    return strcmp(LispString::GetValue(lsp_packageName),"KEYWORD") == 0;
  }
};
struct LispVector
{
  unsigned int code:8;
  unsigned int size:24;
  LispVal data[1];
  static LispVector *GetValue(LispVal lsp_vector){ return (LispVector *)(lsp_vector+sv_size_adj); }
};
struct LispFloat
{
  static double GetValue(LispVal lsp_float){ return S_FtoFloat(lsp_float); }
};
struct LispDouble
{
  static double GetValue(LispVal lsp_double){ return D_FtoFloat(lsp_double); }
};

#endif
