#ifndef __LISP_CFUF__
#define __LISP_CFUF__

#include "lisp.h"
#include "objects_b.hxx"

class CLispInterp:public CInterp,public RCObject{
public:
  bool EvalControl(const char *const exp);
  CInputStream *EvalExtern(const char *const exp);
};

class LispLexerFail:public Fail{
  LispVal m_exp;
public:
  explicit LispLexerFail(LispVal exp):m_exp(exp){}
  string GetDesc() const;
};

class CLispInputStream:public CInputStream{
  typedef unsigned short ushort;
  typedef unsigned char uchar;
  typedef signed char byte;
  struct Frame{
    LispVal fd;
    bool start;
    byte enter_code;
    ushort i;
    Frame(LispVal _fd,bool _start,byte _enter_code,ushort _i=0):fd(_fd),start(_start),enter_code(_enter_code),i(_i){}
  };
  struct DataStack:public vector<Frame>{
    void push(LispVal fd,bool start,byte enter_code,ushort i=0);
    byte pop(LispVal& fd,bool& start,ushort& i);
    int area() const { return capacity() * sizeof(value_type); }
  };
  struct RetAddrStack:public vector<uchar>{
    void push(uchar ret){ push_back(ret); }
    uchar pop();
    int area() const { return capacity() * sizeof(value_type); }
  };
private:
  LispVal m_input;
  const char *m_buf;
  int m_pos;
  string m_str;
  DataStack m_ds;
  RetAddrStack m_ret;
  bool m_ungot;
  char m_dummy;
public:
  explicit CLispInputStream(LispVal input){ Init(input); }
  void Reset(LispVal input);
  int Get();
  void Unget();
  bool IsOK() { return !m_ret.empty(); }
private:
  void Dismember() throw (LispLexerFail *);
  void Init(LispVal input);
};

#endif
