#include "stdrun.h"

ScmAux::Cleanup ScmAux::s_defClean;
////////////////////////////////////////////////////////////////////////////////
// ScmAux methods
//
// The scheme object must be either an immediate integer number,
// a string or a symbol.
token_t ScmAux::scm2token(SCM scm_obj,int nArg,char *szCaller)
{
  if (NIMP(scm_obj)){
    if (STRINGP(scm_obj)){                      // the scm object is a zero terminated string
      string tmp;
      tmp << '"' << CHARS(scm_obj) << '"';      // create the respective string 
      return Symbols::insert(tmp.c_str());      // retrieve the token
    }else if (INEXP(scm_obj)){                  // the scm object is float number
      char buf[100];
      buf[iflo2str(scm_obj,buf)]='\0';          // create the respective string 
      return Symbols::insert(buf);              // retrieve the token
    }else if (SYMBOLP(scm_obj))                 // the scm object is a symbol
      return Symbols::insert(CHARS(scm_obj));   // retrieve the token
  }else if (IMP(scm_obj) && INUMP(scm_obj)){    // the scm object is an integer number
    char buf[20];
    buf[iint2str(INUM(scm_obj),10,buf)]='\0';   // create the respective string 
    return Symbols::insert(buf);                // retrieve the token
  }
  if (szCaller) ASSERT(false,scm_obj,nArg,szCaller);
  return 0;
}
void ScmAux::scm2elist(EdgeList& rLstTokens,SCM scm_lstTokens,int nArg,char *szCaller)
{
  if (scm_lstTokens == EOL) return;    // the list is empty - return
  
  SCM scm_lstSave = scm_lstTokens;
  token_t token;
  if (NIMP(scm_lstTokens) && CONSP(scm_lstTokens)){
    while (scm_lstTokens != EOL){
      token = scm2token(CAR(scm_lstTokens),0,0);
      if (!token) break;
      rLstTokens.push_back(token);
      scm_lstTokens = CDR(scm_lstTokens);
    }
    if (scm_lstTokens == EOL) return;
  }else{                        // the syntax is such, that a single element is allowed to appear not as a list ('a instead of '(a))
    token = scm2token(scm_lstTokens,0,0);
    if (token){
      rLstTokens.push_back(token);
      return;
    }
  }
  (&rLstTokens)->~EdgeList();
  ASSERT(false,scm_lstSave,nArg,szCaller);
}
// The token can represent either an immediate integer number,
// a string or a symbol.
SCM ScmAux::token2scm(token_t token)
{
  if (!token) return BOOL_F;
  
  char *endptr;
  long num = strtol(TOK2STR(token),&endptr,10);         // try to convert the token to an integer first
  if (*endptr){                                         // if the token is not an integer
    double num = strtod(TOK2STR(token),&endptr);        // next try to convert the token to a double
    if (*endptr){                                       // if the token is not a double either
      int len = strlen(TOK2STR(token));                 // get the token length
      if (len > 1 && TOK2STR(token)[0] == '"'){         // if it is a string
        assert(TOK2STR(token)[len-1] == '"');
        return makfromstr(TOK2STR(token)+1,len-2);      // return a scm representation of string
      }
      assert(TOK2STR(token)[0] != '"');                 // else the token is a symbol
      return CAR(intern(TOK2STR(token),len));           // return a scm symbol
    }
    return makdbl(num,0);                               // return a scm double number
  }
  return MAKINUM(num);                                  // return a scm integer number
}
SCM ScmAux::type2scm(CType *type)
{
  if (type->under()){
    CUnderType *utype = cast<CUnderType>(type);

    token_t tag = Predefined::under;
    if (cast<CStrictUnderType>(utype)) tag = Predefined::sunder;
    
    SCM under = make_vector(MAKINUM(2),BOOL_F);
    VELTS(under)[0] = token2scm(tag);
    VELTS(under)[1] = token2scm(utype->title());
    return under;
  }
  if (type->special()){
    static string str;
    (str = '\'') << type;
    return scm_evstr(const_cast<char *>(str.c_str()));
  }
  if (type->internal()) return token2scm(Predefined::internal);
  CNormalType *ntype = cast<CNormalType>(type);
  assert(ntype);
  char *title = TOK2STR(ntype->title());
  return (type->quoted() ? makfromstr(title,strlen(title)) : token2scm(title));
}
// Convert an arbitrary string to the corresponding scheme symbol.
SCM ScmAux::str2scm(string str)
{
  SCM lsym = makfromstr(const_cast<char *>(str.c_str()),str.length());
  DEFER_INTS;
  SETLENGTH(lsym, str.length(), tc7_msymbol);
  ALLOW_INTS;
  return lsym;
}
SCM ScmAux::elist2scm(const EdgeList& elist)
{
  SCM res = EOL;
  EdgeList::const_reverse_iterator it = elist.rbegin();
  for (uint i=elist.size();i;--i,++it) res = cons(token2scm(*it),res);
  return res;
}
SCM ScmAux::Failed(char *who,Fail *e,Cleanup& cleanup)
{
  bool bAbort;
  { // strFailDesc must be destroyed before ASSERT
    string strFailDesc(e->GetDesc()); 
    if (bAbort = e->IsAbort()) cerr << who << ": " << strFailDesc << ENDL;
    else (*g_pTheOutput) << who << ": " << strFailDesc << ENDL;
    delete e;
  }
  cleanup.Perform();
  if (bAbort){
    static SCM scmvoid = CAR(intern(" ",1));
    ASSERT(false,scmvoid, "", who); // abort execution and return to the read-eval-write loop
  }
  return BOOL_F;        // must return something to please compiler
}
