#include "stdrun.h"
#include "runfuf.hxx"
#include "cfuf.hxx"
#include "cfuf_fd.hxx"
#include "cfuf_grammar.hxx"
#include "cfuf_special.hxx"

DECLARE_STRING(FD,GetUCount);
DECLARE_STRING(FD,Compile);
DECLARE_STRING(FD,Copy);
DECLARE_STRING(FD,Cut);
DECLARE_STRING(FD,Delete);
DECLARE_STRING(FD,GetFirstChild);
DECLARE_STRING(FD,GetFirstParent);
DECLARE_STRING(FD,GetNextChild);
DECLARE_STRING(FD,GetNextParent);
DECLARE_STRING(FD,GetSpecialAttr);
DECLARE_STRING(FD,GetType);
DECLARE_STRING(FD,IsEmptyFD);
DECLARE_STRING(FD,IsFD);
DECLARE_STRING(FD,FindChild);
DECLARE_STRING(FD,IsEqual);
DECLARE_STRING(FD,Paste);
DECLARE_STRING(FD,ToList);
DECLARE_STRING(FD,Unify);
DECLARE_STRING(FD,PrettyPrint);

struct ScmFD{
  DECLARE_METHOD0(GetUCount);
  DECLARE_METHOD1(Compile);
  DECLARE_METHOD1(Copy);
  DECLARE_METHOD1(Cut);
  DECLARE_METHOD1(Delete);
  DECLARE_METHOD1(GetFirstChild);
  DECLARE_METHOD1(GetFirstParent);
  DECLARE_METHOD1(GetNextChild);
  DECLARE_METHOD1(GetNextParent);
  DECLARE_METHOD1(GetSpecialAttr);
  DECLARE_METHOD1(GetType);
  DECLARE_METHOD1(IsEmptyFD);
  DECLARE_METHOD1(IsFD);
  DECLARE_METHOD2(FindChild);
  DECLARE_METHOD2(IsEqual);
  DECLARE_METHOD2(Paste);
  DECLARE_METHOD2(ToList);
  DECLARE_METHOD2(Unify);
  DECLARE_METHOD3(PrettyPrint);
  
  static void Register();
private:
  static sizet Free(CELLPTR cellptr);
  static int Print(SCM scm_fd,SCM scm_port,int writing);
  static smobfuns s_smob;

  typedef pair<token_t,fd_t> (CFUFFD::*GetRelativeFunc)();
  static SCM GetRelative(SCM scm_fd,GetRelativeFunc f,char *szCaller);
  
  static SCM pattern2scm(CFUFSpecial::Pattern *p);
  static SCM cset2scm(CFUFSpecial::Cset *c);
  static SCM component2scm(CFUFSpecial::Cset *c,uint nComponent);
  static SCM fset2scm(CFUFSpecial::Fset *f);
public:
  static long s_tc16;
};

void PlugFD()
{
  ScmFD::Register();
  MAKE_SUBR(0,FD,GetUCount);
  MAKE_SUBR(1,FD,Compile);
  MAKE_SUBR(1,FD,Copy);
  MAKE_SUBR(1,FD,Cut);
  MAKE_SUBR(1,FD,Delete);
  MAKE_SUBR(1,FD,GetFirstChild);
  MAKE_SUBR(1,FD,GetFirstParent);
  MAKE_SUBR(1,FD,GetNextChild);
  MAKE_SUBR(1,FD,GetNextParent);
  MAKE_SUBR(1,FD,GetSpecialAttr);
  MAKE_SUBR(1,FD,GetType);
  MAKE_SUBR(1,FD,IsEmptyFD);
  MAKE_SUBR(1,FD,IsFD);
  MAKE_SUBR(2,FD,FindChild);
  MAKE_SUBR(2,FD,IsEqual);
  MAKE_SUBR(2,FD,Paste);
  MAKE_SUBR_TYPE(2,2o,FD,ToList);
  MAKE_SUBR(2,FD,Unify);
  MAKE_SUBR(3,FD,PrettyPrint);
}
smobfuns ScmFD::s_smob = {mark0,ScmFD::Free,ScmFD::Print,0};
long ScmFD::s_tc16 = 0;
/////////////////////////////////////////////////////////////////////////////////
// ScmFD methods
//
void ScmFD::Register()
{
  s_tc16 = newsmob(&s_smob);
}
sizet ScmFD::Free(CELLPTR cellptr)
{
  CFUFFD *pCFUFFD = ScmAux::scm2fd(reinterpret_cast<SCM>(cellptr),0,0);
  assert(pCFUFFD);
  pCFUFFD->Destroy();
  return 0;
}
int ScmFD::Print(SCM scm_fd,SCM scm_port,int)
{
  CFUFFD *pCFUFFD = ScmAux::scm2fd(scm_fd,0,0);
  assert(pCFUFFD);
  CFD *pFD = pCFUFFD->GetFD();
  char buf[100];
  sprintf(buf,"#<fuf compiled FD, (%p,%d)>",pFD,pFD->count());
  lputs(buf,scm_port);
  return 1;
}
/////////////////////////////////////////////////////////////////////////////////
struct FDCleanup:public ScmAux::Cleanup{
  FDCleanup(CFUFFD *pCFUFFD){ m_pCFUFFD = pCFUFFD; }
  void Perform(){ m_pCFUFFD->Destroy(); }
private:
  CFUFFD *m_pCFUFFD;
};
SCM ScmFD::Compile(SCM scm_fd)
{
  CScmInputStream istr(scm_fd);
  ufd_t fd = CFUFFD::Parse(istr);
  if (fd.fail){
    FDCleanup cleanup(fd.val);
    return ScmAux::Failed(szFDCompile,fd.fail,cleanup);
  }
  return ScmAux::fd2scm(fd.val);
}
SCM ScmFD::IsFD(SCM scm_obj)
{
  return ScmAux::scm2fd(scm_obj,0,0) ? BOOL_T : BOOL_F;
}
SCM ScmFD::IsEmptyFD(SCM scm_obj)
{
  CFUFFD *pFD = ScmAux::scm2fd(scm_obj,0,0);
  return (pFD && pFD->IsEmpty()) ? BOOL_T : BOOL_F;
}
SCM ScmFD::Unify(SCM scm_fd,SCM scm_gr)
{
  CFUFFD *pFD = ScmAux::scm2fd(scm_fd,ARG1,szFDUnify);
  CFUFGrammar *pGrammar = ScmAux::scm2gr(scm_gr,ARG2,szFDUnify);
  ret::ubool_t u = pFD->Unify(pGrammar);
  if (u.fail) return ScmAux::Failed(szFDUnify,u.fail);
  return u.val ? BOOL_T : BOOL_F; // return either #t or #f depending on the unification outcome
}
SCM ScmFD::Copy(SCM scm_fd)
{
  CFUFFD *pFD = ScmAux::scm2fd(scm_fd,ARG1,szFDCopy);
  return ScmAux::fd2scm(pFD->Copy());
}
SCM ScmFD::Cut(SCM scm_fd)
{
  CFUFFD *pFD = ScmAux::scm2fd(scm_fd,ARG1,szFDCut);
  return ScmAux::fd2scm(pFD->Cut());
}
SCM ScmFD::Delete(SCM scm_fd)
{
  CFUFFD *pFD = ScmAux::scm2fd(scm_fd,ARG1,szFDDelete);
  pFD->Delete();
  return UNSPECIFIED;
}
SCM ScmFD::Paste(SCM scm_fdDst,SCM scm_fdSrc)
{
  CFUFFD *pDstFD = ScmAux::scm2fd(scm_fdDst,ARG1,szFDPaste);
  CFUFFD *pSrcFD = ScmAux::scm2fd(scm_fdSrc,ARG2,szFDPaste);
  if (Fail *f = pDstFD->Paste(pSrcFD)) return ScmAux::Failed(szFDPaste,f);
  return BOOL_T;
}
SCM ScmFD::IsEqual(SCM scm_fd1,SCM scm_fd2)
{
  CFUFFD *pFD1 = ScmAux::scm2fd(scm_fd1,ARG1,szFDIsEqual);
  CFUFFD *pFD2 = ScmAux::scm2fd(scm_fd2,ARG2,szFDIsEqual);
  return pFD1->IsEqual(pFD2) ? BOOL_T : BOOL_F;
}
SCM ScmFD::GetType(SCM scm_fd)
{
  CFUFFD *pFD = ScmAux::scm2fd(scm_fd,ARG1,szFDGetType);
  CType *type = pFD->GetType();
  if (type->special()){
    token_t attr = pFD->GetSpecialAttr();
    if (attr == Predefined::pattern) return pattern2scm(CFUFSpecial::GetPattern(type));
    if (attr == Predefined::cset)    return cset2scm(CFUFSpecial::GetCset(type));
    if (attr == Predefined::fset)    return fset2scm(CFUFSpecial::GetFset(type));
    assert(false);
    return BOOL_F;
  }
  return ScmAux::type2scm(type);
}
SCM ScmFD::GetUCount()
{
  return MAKINUM(CFUFFD::GetUCount());
}
SCM ScmFD::GetSpecialAttr(SCM scm_fd)
{
  CFUFFD *pFD = ScmAux::scm2fd(scm_fd,ARG1,szFDGetSpecialAttr);
  return ScmAux::token2scm(pFD->GetSpecialAttr());
}
SCM ScmFD::FindChild(SCM scm_fd,SCM scm_lstEdges)
{
  CFUFFD *pFD = ScmAux::scm2fd(scm_fd,ARG1,szFDFindChild);
  EdgeList lstEdges;
  ScmAux::scm2elist(lstEdges,scm_lstEdges,ARG2,szFDFindChild);
  return ScmAux::fd2scm(pFD->FindDescendant(lstEdges));
}
SCM ScmFD::GetRelative(SCM scm_fd,GetRelativeFunc f,char *szCaller)
{
  CFUFFD *pFD = ScmAux::scm2fd(scm_fd,ARG1,szCaller);
  pair<token_t,fd_t> relative = (pFD->*f)();
  return cons(ScmAux::token2scm(relative.first),ScmAux::fd2scm(relative.second));
}
SCM ScmFD::GetFirstChild(SCM scm_fd)
{
  return GetRelative(scm_fd,&CFUFFD::GetFirstChild,szFDGetFirstChild);
}
SCM ScmFD::GetNextChild(SCM scm_fd)
{
  return GetRelative(scm_fd,&CFUFFD::GetNextChild,szFDGetNextChild);
}
SCM ScmFD::GetFirstParent(SCM scm_fd)
{
  return GetRelative(scm_fd,&CFUFFD::GetFirstParent,szFDGetFirstParent);
}
SCM ScmFD::GetNextParent(SCM scm_fd)
{
  return GetRelative(scm_fd,&CFUFFD::GetNextParent,szFDGetNextParent);
}
SCM ScmFD::PrettyPrint(SCM scm_fd,SCM scm_fdName,SCM scm_fileName)
{
  CFUFFD *pFD = ScmAux::scm2fd(scm_fd,ARG1,szFDPrettyPrint);
  ASSERT(NIMP(scm_fdName) && (SYMBOLP(scm_fdName) || STRINGP(scm_fdName)), scm_fdName, ARG2, szFDPrettyPrint);
  ASSERT(NIMP(scm_fileName) && (SYMBOLP(scm_fileName) || STRINGP(scm_fileName)), scm_fileName, ARG3, szFDPrettyPrint);
  pFD->PrettyPrint(CHARS(scm_fdName),CHARS(scm_fileName));
  return UNSPECIFIED;
}

class CFD2list:public CUnparseFD{
public:
  CFD2list(SCM scm_attr){
    m_scm_list = EOL;
    m_bDoingAll = (scm_attr != BOOL_F);
  }
  Fail *StartInternal(CFDNode *root);
  Fail *StartEndLeaf(CFDNode *root);
  Fail *EnterSubgraph(token_t edge,CFDNode *node,EdgeList *path);
  Fail *LeaveSubgraph();
  Fail *AddLeaf(token_t edge,CFDNode *node,EdgeList *path);
  Fail *AddPath(token_t edge,CFDNode *node,EdgeList *path);
  bool IsDoingAll() const { return m_bDoingAll; }
  int GetOrder() const    { return DESCENDING; }
  SCM GetScmList() const  { return m_scm_list; }
private:
  stack<pair<token_t,SCM> > m_nest;
  SCM m_scm_list;
  bool m_bDoingAll;
};

SCM ScmFD::ToList(SCM scm_fd,SCM scm_attr)
{
  CFUFFD *pFD = ScmAux::scm2fd(scm_fd,ARG1,szFDToList);
  EdgeList lstIgnored;
  if (scm_attr != BOOL_T && scm_attr != BOOL_F && scm_attr != UNDEFINED)
    ScmAux::scm2elist(lstIgnored,scm_attr,ARG2,szFDToList);
  
  CFD2list fd2list(scm_attr);
  CFDWalk walker(pFD->GetRoot());
  
  EdgeList::const_iterator it = lstIgnored.begin();
  for (uint i = lstIgnored.size();i;--i,++it)
    walker.ignore(*it);
  
  Fail *f = walker.walk(&fd2list);
  assert(!f);
  return fd2list.GetScmList();
}

/////////////////////////////////////////////////////////////////////////////////
// CFD2list methods
//
Fail *CFD2list::StartInternal(CFDNode *root)
{
  if (root->any())             // if the root is ANY, then output the pair ({} any)
    m_scm_list = cons(cons2(ScmAux::str2scm("{}"),ScmAux::token2scm(Predefined::any),EOL),m_scm_list);
  return 0;
}
Fail *CFD2list::StartEndLeaf(CFDNode *root)        // the fd is just a single node
{
  m_scm_list = ScmAux::type2scm(root->data().GetType());
  return 0;
}
Fail *CFD2list::EnterSubgraph(token_t edge,CFDNode *node,EdgeList *)
{
  if (node->any())             // special case when internal node is reported as ANY
    m_scm_list = cons(cons2(ScmAux::token2scm(edge),ScmAux::token2scm(Predefined::any),EOL),m_scm_list);
  m_nest.push(make_pair(edge,m_scm_list));
  m_scm_list = EOL;
  return 0;
}
Fail *CFD2list::LeaveSubgraph()
{
  pair<token_t,SCM> saved = m_nest.top();                         // get the parent feature and list
  SCM scm_edge = ScmAux::token2scm(saved.first);                  // convert the feature to a SCM object
  m_scm_list = cons(cons2(scm_edge,m_scm_list,EOL),saved.second); // cons the (scm_edge scm_value) pair to the result
  m_nest.pop();                 // return one level up
  return 0;
}
Fail *CFD2list::AddLeaf(token_t edge,CFDNode *node,EdgeList *)
{
  CType *type = node->data().GetType();        // get the type
  if (type->nil()) return 0;
  SCM scm_edge = ScmAux::token2scm(edge);       // convert the edge to a scheme attribute
  SCM scm_value = ScmAux::type2scm(type);       // convert the type to a scheme value
  m_scm_list = cons(cons2(scm_edge,scm_value,EOL),m_scm_list);  // cons the (scm_value scm_edge) pair to the result
  return 0;
}
Fail *CFD2list::AddPath(token_t edge,CFDNode *,EdgeList *path)
{
  SCM scm_edge = ScmAux::token2scm(edge);       // convert the edge to a scheme attribute
  string strPath;strPath.reserve(300);
  strPath << '{' << path->GetString() << '}';
  SCM scm_value = ScmAux::str2scm(strPath);     // convert the path to a scheme value
  m_scm_list = cons(cons2(scm_edge,scm_value,EOL),m_scm_list); // cons the (scm_value scm_edge) pair to the result
  return 0;
}

/////////////////////////////////////////////////////////////////////////////////
SCM ScmFD::pattern2scm(CFUFSpecial::Pattern *p)
{
  static SCM scm_dots = ScmAux::token2scm(Predefined::dots);
  SCM scm_acc=EOL;
  SCM scm_val;

  for (int i = p->m_nCount - 1; i >= 0; --i){
    if (fd_t fd = p->m_arrValues[i]){
      scm_val = ScmAux::fd2scm(fd);
      if (fd->IsMergeable()) scm_val = cons(scm_val,EOL);
    }else scm_val = scm_dots;
    scm_acc = cons(scm_val,scm_acc);
  }
  return scm_acc;
}
SCM ScmFD::cset2scm(CFUFSpecial::Cset *c)
{
  SCM scm_acc = EOL;
  for (int i = CFUFSpecial::Cset::DEC; i >= CFUFSpecial::Cset::ABS; --i)
    scm_acc = cons(component2scm(c,i),scm_acc);
  return scm_acc;
}
SCM ScmFD::component2scm(CFUFSpecial::Cset *c,uint nComponent)
{
  SCM scm_acc = EOL;
  for (int i = c->m_nCount[nComponent] - 1; i >= 0; --i)
    scm_acc = cons(ScmAux::fd2scm(c->m_arrValues[nComponent][i]),scm_acc);
  return scm_acc;
}
SCM ScmFD::fset2scm(CFUFSpecial::Fset *f)
{
  SCM scm_acc=EOL;

  for (int i = f->m_nCount - 1; i >= 0; --i)
    scm_acc = cons(ScmAux::token2scm(f->m_arrFeatures[i]),scm_acc);
    
  return scm_acc;
}
/////////////////////////////////////////////////////////////////////////////////
// ScmAux methods dealing with fd conversions
//
CFUFFD *ScmAux::scm2fd(SCM scm_fd,int nArg,char *szCaller)
{
  if (NIMP(scm_fd) && TYP16(scm_fd)==ScmFD::s_tc16){
    CFUFFD *pFD = reinterpret_cast<CFUFFD *>(CDR(scm_fd));
    assert(pFD);
    return pFD;
  }
  ASSERT(!szCaller, scm_fd, nArg, szCaller);
  return 0;
}
SCM ScmAux::fd2scm(CFUFFD *pFD)
{
  SCM lsym;
  NEWCELL(lsym);
  DEFER_INTS;
  CAR(lsym) = ScmFD::s_tc16;
  SETCDR(lsym, pFD);
  ALLOW_INTS;
  return lsym;
}
