#include "stdrun.h"
#include "runfuf.hxx"
#include "cfuf.hxx"
#include "cfuf_grammar.hxx"

DECLARE_STRING(Grammar,Compile);
DECLARE_STRING(Grammar,IsGrammar);
DECLARE_STRING(Grammar,DoingDemo);
DECLARE_STRING(Grammar,DoingWait);
DECLARE_STRING(Grammar,DoingIndex);
DECLARE_STRING(Grammar,DoingBKClass);

typedef bool (*IsParamEnabledFunc)();
typedef void (*EnableParamFunc)(bool bEnable);

struct ScmGrammar{
  DECLARE_METHOD1(Compile);
  DECLARE_METHOD1(IsGrammar);
  DECLARE_METHOD1(DoingDemo);
  DECLARE_METHOD1(DoingWait);
  DECLARE_METHOD1(DoingIndex);
  DECLARE_METHOD1(DoingBKClass);
  
  static void Register();
private:
  static SCM DoingParam(SCM scm_bYes,IsParamEnabledFunc IsParamEnabled,EnableParamFunc EnableParam);
  
  static sizet Free(CELLPTR cellptr);
  static int Print(SCM scm_gr,SCM scm_port,int writing);
  static smobfuns s_smob;
public:
  static long s_tc16;
};

void PlugGrammar()
{
  ScmGrammar::Register();
  MAKE_SUBR(1,Grammar,Compile);
  MAKE_SUBR(1,Grammar,IsGrammar);
  MAKE_SUBR_TYPE(1,1o,Grammar,DoingDemo);
  MAKE_SUBR_TYPE(1,1o,Grammar,DoingWait);
  MAKE_SUBR_TYPE(1,1o,Grammar,DoingIndex);
  MAKE_SUBR_TYPE(1,1o,Grammar,DoingBKClass);
}
smobfuns ScmGrammar::s_smob = {mark0,ScmGrammar::Free,ScmGrammar::Print,0};
long ScmGrammar::s_tc16 = 0;
/////////////////////////////////////////////////////////////////////////////////
// ScmGrammar methods
//
void ScmGrammar::Register()
{
  s_tc16 = newsmob(&s_smob);
}
sizet ScmGrammar::Free(CELLPTR cellptr)
{
  CFUFGrammar *pCFUFGrammar = ScmAux::scm2gr(reinterpret_cast<SCM>(cellptr),0,0);
  assert(pCFUFGrammar);
  pCFUFGrammar->Destroy();
  return 0;
}
int ScmGrammar::Print(SCM scm_gr,SCM scm_port,int)
{
  CFUFGrammar *pCFUFGrammar = ScmAux::scm2gr(scm_gr,0,0);
  assert(pCFUFGrammar);
  CGrammar *pGrammar = pCFUFGrammar->GetGrammar();
  char buf[100];
  sprintf(buf,"#<fuf compiled grammar, (%p,%d)>",pGrammar,pGrammar->root()->count());
  lputs(buf,scm_port);
  return 1;
}
/////////////////////////////////////////////////////////////////////////////////
struct GrammarCleanup:public ScmAux::Cleanup{
  GrammarCleanup(CFUFGrammar *pCFUFGrammar){ m_pCFUFGrammar = pCFUFGrammar; }
  void Perform(){ m_pCFUFGrammar->Destroy(); }
private:
  CFUFGrammar *m_pCFUFGrammar;
};
SCM ScmGrammar::Compile(SCM scm_gr)
{
  CScmInputStream istr(scm_gr);
  ugrammar_t gr = CFUFGrammar::Parse(istr);
  if (gr.fail){
    GrammarCleanup cleanup(gr.val);
    return ScmAux::Failed(szGrammarCompile,gr.fail,cleanup);
  }
  return ScmAux::gr2scm(gr.val);
}
SCM ScmGrammar::DoingParam(SCM scm_bYes,IsParamEnabledFunc IsParamEnabled,EnableParamFunc EnableParam)
{
  if (scm_bYes==UNDEFINED) return IsParamEnabled() ? BOOL_T : BOOL_F;
  EnableParam(scm_bYes != BOOL_F);
  return UNSPECIFIED;
}
SCM ScmGrammar::DoingDemo(SCM scm_bYes)
{
  return DoingParam(scm_bYes,CFUFGrammar::IsDemoEnabled,CFUFGrammar::EnableDemo);
}
SCM ScmGrammar::DoingWait(SCM scm_bYes)
{
  return DoingParam(scm_bYes,CFUFGrammar::IsWaitEnabled,CFUFGrammar::EnableWait);
}
SCM ScmGrammar::DoingIndex(SCM scm_bYes)
{
  return DoingParam(scm_bYes,CFUFGrammar::IsIndexEnabled,CFUFGrammar::EnableIndex);
}
SCM ScmGrammar::DoingBKClass(SCM scm_bYes)
{
  return DoingParam(scm_bYes,CFUFGrammar::IsBKClassEnabled,CFUFGrammar::EnableBKClass);
}
SCM ScmGrammar::IsGrammar(SCM scm_gr)
{
  return ScmAux::scm2gr(scm_gr,0,0) ? BOOL_T : BOOL_F;
}

/////////////////////////////////////////////////////////////////////////////////
// ScmAux methods dealing with grammar conversions
//
CFUFGrammar *ScmAux::scm2gr(SCM scm_gr,int nArg,char *szCaller)
{
  if (NIMP(scm_gr) && TYP16(scm_gr)==ScmGrammar::s_tc16){
    CFUFGrammar *pGrammar = reinterpret_cast<CFUFGrammar *>(CDR(scm_gr));
    assert(pGrammar);
    return pGrammar;
  }
  ASSERT(!szCaller, scm_gr, nArg, szCaller);
  return 0;
}
SCM ScmAux::gr2scm(CFUFGrammar *pGrammar)
{
  SCM lsym;
  NEWCELL(lsym);
  DEFER_INTS;
  CAR(lsym) = ScmGrammar::s_tc16;
  SETCDR(lsym, pGrammar);
  ALLOW_INTS;
  return lsym;
}
