#include "runfuf.hxx"
#include "cfuf.hxx"

extern "C" 
{
	void final_cfuf();
	void init_cfuf();
};

typedef void (*PlugInFunc)();
#define PLUG(Module) extern void Plug##Module()
#define BEGIN_MODULE_MAP PlugInFunc g_arrModules[]={
#define MODULE(Module) Plug##Module,
#define END_MODULE_MAP 0};
#include "modules.inl"

void final_cfuf()
{
	CFUF::Shutdown();
}
void init_cfuf()
{
	CFUF::Initialize(new CScmInterp,true);
	add_final(final_cfuf);

	// load cfuf modules
	for (PlugInFunc *fptr = g_arrModules;*fptr;++fptr)
	{
		(*fptr)();
	}

	add_feature("cfuf"); // make the feature fuf available through (require 'cfuf)
}

/////////////////////////////////////////////////////////////////////////////////
// CScmInterp methods
//
bool CScmInterp::EvalControl(const char *const exp)
{
	SCM res = scm_evstr(const_cast<char *>(exp));
	return res == BOOL_F ? false : true;
}
CInputStream *CScmInterp::EvalExtern(const char *const exp)
{
	return new CScmInputStream(scm_evstr(const_cast<char *>(exp)));
}

/////////////////////////////////////////////////////////////////////////////////
// ScmLexerFail methods
//
string ScmLexerFail::GetDesc() const
{
	const int BUFLEN=200;
	string str("Invalid fuf expression: ");
	SCM scm_str = makstr(BUFLEN);
	memset(CHARS(scm_str),0,BUFLEN);
	SCM strport = mkstrport(INUM0, scm_str,OPN | WRTNG,"ScmLexerFail::GetDesc()");
	lwrite(m_exp,strport);
	return str << CHARS(scm_str);
}

/////////////////////////////////////////////////////////////////////////////////
// CScmInputStream methods
//
void CScmInputStream::Reset(SCM input)
{
	m_ds.erase(m_ds.begin(),m_ds.end());
	m_ret.erase(m_ret.begin(),m_ret.end());
	Init(input);
}
void CScmInputStream::Init(SCM input)
{
	m_input = input;
	m_buf = &m_dummy;
	m_pos = -1;
	m_str = "";
	m_ds.push(m_input,true,0);
	m_ret.push(0);
	m_ungot = false;
	m_dummy = '\0';
}
int CScmInputStream::Get()
{
	m_ungot = false;
	if (m_buf[++m_pos])
	{
		return m_buf[m_pos];
	}
	Dismember();
	m_pos = 0;
	if (m_ret.empty())
	{
		m_ungot = true;
		return EOF;
	}
	m_buf = m_str.c_str();
	return *m_buf;
}
void CScmInputStream::Unget()
{
	if (m_ungot) 
	{
		return;
	}
	--m_pos;
	m_ungot = true;
}

// When making a recursive call there is no application to the enter_code field
// of the stack Frame.It is set to -2 then.
#define REC_CALL(arg_fd,arg_start,ret_code) \
m_ds.push(fd,start,-2,i); \
m_ret.push(ret_code); \
fd = arg_fd;start = arg_start; \
goto __dismember_start; \
__dismember_ret_##ret_code: \
m_ds.pop(fd,start,i)

#define RETURN(enter_code) \
m_ds.push(fd,start,enter_code,i); \
return; \
__dismember_enter_##enter_code:

#define END() \
m_ds.push(fd,start,-1,i); \
return
  
void CScmInputStream::Dismember() throw (ScmLexerFail *)
{
	SCM fd;
	bool start;
	ushort i;
	SCM *data;

	switch (m_ds.pop(fd,start,i))
	{
	case -1:goto __dismember_end;
	case 1:goto __dismember_enter_1;
	case 2:goto __dismember_enter_2;
	case 3:goto __dismember_enter_3;
	case 4:goto __dismember_enter_4;
	case -2:assert(false);
	}

__dismember_start:
	if (NIMP(fd) && CONSP(fd))
	{
		if (start)
		{
			m_str = '(';
			RETURN(1);
		}
		REC_CALL(CAR(fd),true,1);
		if (CDR(fd)!=EOL)
		{
			m_str = ' ';
			RETURN(2);
			REC_CALL(CDR(fd),false,2);
		}
		if (start)
		{
			m_str = ')';
			END();
		}
	}
	else if (IMP(fd))
	{
		if (INUMP(fd))
		{
			char buf[20];
			buf[iint2str(INUM(fd),10,buf)]='\0';
			m_str = buf;
		}
		else if (ICHRP(fd)) 
		{
			m_str = ICHR(fd);
		}
		else switch (fd)
		{
		case EOL:
			if (start) 
			{
				m_str = "()";
			}
			break;
		case BOOL_F:
			if (start) 
			{
				m_str = "#f";
			}
			break;
		case BOOL_T:
			if (start) 
			{
				m_str = "#t";
			}
			break;
		default:throw new ScmLexerFail(fd);
		}
		END();
	}
	else if (SYMBOLP(fd))
	{
		m_str = CHARS(fd);
		END();
	}
	else if (STRINGP(fd))
	{
		(m_str = '"') << CHARS(fd) << '"';
		END();
	}
	else if (VECTORP(fd))
	{
		m_str = "#(";
		RETURN(3);
		if (LENGTH(fd))
		{
			data=VELTS(fd);
			for (i=0;i<LENGTH(fd)-1;++i)
			{
				REC_CALL(data[i],true,3);
				m_str = ' ';
				RETURN(4);
				data = VELTS(fd);
			}
			REC_CALL(data[i],true,4);
		}
		m_str = ')';
		END();
	}
	else if (INEXP(fd))
	{
		char buf[100];
		buf[iflo2str(fd,buf)]='\0';
		m_str = buf;
		END();
	}
	else throw new ScmLexerFail(fd);

__dismember_end:
	switch (m_ret.pop())
	{
	case 1:goto __dismember_ret_1;
	case 2:goto __dismember_ret_2;
	case 3:goto __dismember_ret_3;
	case 4:goto __dismember_ret_4;
	}
	return;
}

/////////////////////////////////////////////////////////////////////
// CScmInputStream::DataStack methods
//
void CScmInputStream::DataStack::push(SCM fd,bool start,byte enter_code,ushort i)
{
	push_back(Frame(fd,start,enter_code,i));
}
CScmInputStream::byte CScmInputStream::DataStack::pop(SCM& fd,bool& start,ushort& i)
{
	Frame f(back());
	pop_back();
	fd = f.fd;
	start = f.start;
	i = f.i;
	return f.enter_code;
}

/////////////////////////////////////////////////////////////////////
// CScmInputStream::RetAddrStack methods
//
CScmInputStream::uchar CScmInputStream::RetAddrStack::pop()
{
	uchar tmp = back();
	pop_back();
	return tmp;
}
