#ifndef __SCM_CFUF__
#define __SCM_CFUF__

#include "scheme.h"
#include "b_objects.hxx"

class CScmInterp:public CInterp,public RCObject
{
public:
	bool EvalControl(const char *const exp);
	CInputStream *EvalExtern(const char *const exp);
};

class ScmLexerFail:public Fail
{
	SCM m_exp;
public:
	explicit ScmLexerFail(SCM exp)
	{
		m_exp = exp; 
	}
	string GetDesc() const;
};

class CScmInputStream:public CInputStream
{
public:
	typedef unsigned short ushort;
	typedef unsigned char uchar;
	typedef signed char byte;
	struct Frame
	{
		SCM fd;
		bool start;
		byte enter_code;
		ushort i;
		Frame(SCM _fd,bool _start,byte _enter_code,ushort _i=0)
		{
			fd = _fd;
			start = _start;
			enter_code = _enter_code;
			i = _i;
		}
	};
private:
	struct DataStack:public vector<Frame>
	{
		void push(SCM fd,bool start,byte enter_code,ushort i=0);
		byte pop(SCM& fd,bool& start,ushort& i);
		int area() const 
		{
			return capacity() * sizeof(value_type); 
		}
	};
	struct RetAddrStack:public vector<uchar>
	{
		void push(uchar ret)
		{
			push_back(ret); 
		}
		uchar pop();
		int area() const 
		{
			return capacity() * sizeof(value_type); 
		}
	};
private:
	SCM m_input;
	const char *m_buf;
	int m_pos;
	string m_str;
	DataStack m_ds;
	RetAddrStack m_ret;
	bool m_ungot;
	char m_dummy;
public:
	explicit CScmInputStream(SCM input)
	{
		Init(input); 
	}
	void Reset(SCM input);
	int Get();
	void Unget();
	bool IsOK() 
	{
		return !m_ret.empty(); 
	}
private:
	void Dismember() throw (ScmLexerFail *);
	void Init(SCM input);
};

#endif
