#include "stdrun.h"
#include "timer.hxx"

DECLARE_STRING(Timer,Create);
DECLARE_STRING(Timer,Reset);
DECLARE_STRING(Timer,Go);
DECLARE_STRING(Timer,Stop);
DECLARE_STRING(Timer,CancelLast);
DECLARE_STRING(Timer,GetLast);
DECLARE_STRING(Timer,GetTotal);

typedef void (CTimer::*MutatorFunc)();
struct ScmTimer{
  DECLARE_METHOD0(Create);
  DECLARE_METHOD1(Reset);
  DECLARE_METHOD1(Go);
  DECLARE_METHOD1(Stop);
  DECLARE_METHOD1(CancelLast);
  DECLARE_METHOD1(GetLast);
  DECLARE_METHOD1(GetTotal);

  static void Register();
private:
  static SCM CallMutator(SCM scm_timer,MutatorFunc Mutator,char *szCaller);
  
  static sizet Free(CELLPTR cellptr);
  static int Print(SCM scm_timer,SCM scm_port,int writing);
  static CTimer *scm2timer(SCM scm_timer,int nArg,char *szCaller);
  
  static smobfuns s_smob;
  static long s_tc16;
};

void PlugTimer()
{
  ScmTimer::Register();
  MAKE_SUBR(0,Timer,Create);
  MAKE_SUBR(1,Timer,Reset);
  MAKE_SUBR(1,Timer,Go);
  MAKE_SUBR(1,Timer,Stop);
  MAKE_SUBR(1,Timer,CancelLast);
  MAKE_SUBR(1,Timer,GetLast);
  MAKE_SUBR(1,Timer,GetTotal);
}
smobfuns ScmTimer::s_smob = {mark0,ScmTimer::Free,ScmTimer::Print,0};
long ScmTimer::s_tc16 = 0;
/////////////////////////////////////////////////////////////////////////////////
// ScmTimer methods
//
void ScmTimer::Register()
{
  s_tc16 = newsmob(&s_smob);
}
sizet ScmTimer::Free(CELLPTR cellptr)
{
  delete scm2timer(reinterpret_cast<SCM>(cellptr),0,0);
  return 0;
}
int ScmTimer::Print(SCM scm_timer,SCM scm_port,int)
{
  char buf[100];
  CTimer *pTimer = scm2timer(scm_timer,0,0);       // do not assert
  assert(pTimer);
  sprintf(buf,"#<timer real: (%.3f,%.3f) run: (%.3f,%.3f)>",
          pTimer->GetLastElapsedReal(),pTimer->GetTotalElapsedReal(),
          pTimer->GetLastElapsedRun(),pTimer->GetTotalElapsedRun());
  lputs(buf,scm_port);
  return 1;
}
CTimer *ScmTimer::scm2timer(SCM scm_timer,int nArg,char *szCaller)
{
  ASSERT(!szCaller || NIMP(scm_timer) && s_tc16==TYP16(scm_timer), scm_timer, nArg, szCaller);
  return reinterpret_cast<CTimer *>(CDR(scm_timer));
}
SCM ScmTimer::Create()
{
  SCM lsym;
  NEWCELL(lsym);
  DEFER_INTS;
  CAR(lsym) = s_tc16;
  SETCDR(lsym,new CTimer);                         
  ALLOW_INTS;
  return lsym;
}
SCM ScmTimer::CallMutator(SCM scm_timer,MutatorFunc Mutator,char *szCaller)
{
  CTimer *pTimer = scm2timer(scm_timer,ARG1,szCaller);
  (pTimer->*Mutator)();
  return UNSPECIFIED;
}
SCM ScmTimer::Reset(SCM scm_timer)
{
  return CallMutator(scm_timer,&CTimer::Reset,szTimerReset);
}
SCM ScmTimer::Go(SCM scm_timer)
{
  return CallMutator(scm_timer,&CTimer::Go,szTimerGo);
}
SCM ScmTimer::Stop(SCM scm_timer)
{
  return CallMutator(scm_timer,&CTimer::Stop,szTimerStop);
}
SCM ScmTimer::CancelLast(SCM scm_timer)
{
  return CallMutator(scm_timer,&CTimer::CancelLast,szTimerCancelLast);
}
SCM ScmTimer::GetLast(SCM scm_timer)
{
  CTimer *pTimer = scm2timer(scm_timer,ARG1,szTimerGetLast);
  return cons(makdbl(pTimer->GetLastElapsedReal(),0),makdbl(pTimer->GetLastElapsedRun(),0));
}
SCM ScmTimer::GetTotal(SCM scm_timer)
{
  CTimer *pTimer = scm2timer(scm_timer,ARG1,szTimerGetTotal);
  return cons(makdbl(pTimer->GetTotalElapsedReal(),0),makdbl(pTimer->GetTotalElapsedRun(),0));
}
